@@ -1,4 +1,4 @@
-/* $Id: ai.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: ai.hpp 12045 2006-06-11 20:32:40Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -13,6 +13,8 @@
 #ifndef AI_HPP_INCLUDED
 #define AI_HPP_INCLUDED
 
+#include "global.hpp"
+
 #include "actions.hpp"
 #include "ai_interface.hpp"
 
@@ -24,9 +26,6 @@ public:
 
 	virtual void play_turn();
 
-	virtual int choose_weapon(const location& att, const location& def,
-	                          battle_stats& cur_stats, gamemap::TERRAIN terrain, bool use_cache=false);
-
 	struct target {
 		enum TYPE { VILLAGE, LEADER, EXPLICIT, THREAT, BATTLE_AID, MASS, SUPPORT };
 
@@ -44,8 +43,9 @@ public:
 		double vulnerability, support;
 	};
 
-	const defensive_position& best_defensive_position(const location& unit, const move_map& dstsrc, const move_map& srcdst,
-	                                                  const move_map& enemy_dstsrc, const move_map& enemy_srcdst) const;
+	defensive_position const& best_defensive_position(const location& unit,
+			const move_map& dstsrc, const move_map& srcdst, const move_map& enemy_dstsrc) const;
+
 	void invalidate_defensive_position_cache();
 
 	bool leader_can_reach_keep() const;
@@ -60,21 +60,23 @@ protected:
 	virtual void do_move();
 
 	virtual bool do_combat(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc);
-	virtual bool get_villages(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc, unit_map::const_iterator leader);
-	virtual bool get_healing(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc);
-	virtual bool retreat_units(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc, unit_map::const_iterator leader);
-	virtual bool move_to_targets(std::map<gamemap::location,paths>& possible_moves, move_map& srcdst, move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc, unit_map::const_iterator leader);
+	virtual bool get_villages(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc, unit_map::iterator &leader);
+	virtual bool get_healing(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& enemy_dstsrc);
+	virtual bool retreat_units(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_dstsrc, unit_map::const_iterator leader);
+	virtual bool move_to_targets(std::map<gamemap::location,paths>& possible_moves, move_map& srcdst, move_map& dstsrc, const move_map& enemy_dstsrc, unit_map::const_iterator leader);
 
-	virtual bool should_retreat(const gamemap::location& loc, unit_map::const_iterator un, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc) const;
+	virtual bool should_retreat(const gamemap::location& loc, const unit_map::const_iterator un, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_dstsrc, double caution) const;
 
 	virtual void do_recruitment();
 
 	virtual void move_leader_to_keep(const move_map& enemy_dstsrc);
-	virtual void move_leader_after_recruit(const move_map& enemy_dstsrc);
-	virtual void move_leader_to_goals(const move_map& enemy_srcdst, const move_map& enemy_dstsrc);
+	virtual void move_leader_after_recruit(const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_dstsrc);
+	virtual void move_leader_to_goals(const move_map& enemy_dstsrc);
 
 	virtual bool recruit_usage(const std::string& usage);
 
+	virtual bool desperate_attack(const gamemap::location &loc);
+
 	void remove_unit_from_moves(const gamemap::location& u, move_map& srcdst, move_map& dstsrc);
 
 	//a function which will find enemy units that threaten our valuable assets
@@ -95,7 +97,7 @@ protected:
 	location move_unit(location from, location to, std::map<location,paths>& possible_moves);
 
 	//our own version of 'attack_enemy'. We record all attacks to support group attacking
-	void attack_enemy(const location& attacking_unit, const location& target, int weapon);
+	void attack_enemy(const location& attacking_unit, const location& target, int att_weapon, int def_weapon);
 
 	std::set<location> attacks_;
 
@@ -104,15 +106,17 @@ protected:
 
 	struct attack_analysis
 	{
-		void analyze(const gamemap& map, std::map<location,unit>& units, int sims,
-		             class ai& ai_obj, const move_map& dstsrc, const move_map& srcdst,
-		             const move_map& enemy_dstsrc, const move_map& enemy_srcdst);
+		void analyze(const gamemap& map, unit_map& units,
+					 const std::vector<team>& teams,
+					 const gamestatus& status, const game_data& gamedata,
+					 class ai& ai_obj,
+					 const move_map& dstsrc, const move_map& srcdst,
+					 const move_map& enemy_dstsrc, double aggression);
 
 		double rating(double aggression, class ai& ai_obj) const;
 
 		gamemap::location target;
 		std::vector<std::pair<gamemap::location,gamemap::location> > movements;
-		std::vector<int> weapons;
 
 		//the value of the unit being targeted
 		double target_value;
@@ -142,10 +146,6 @@ protected:
 		//(good for comparison to see just how good/bad 'terrain_quality' is)
 		double alternative_terrain_quality;
 
-		//the ratio of the attacks the unit being attacked will get to
-		//the strength of its most powerful attack
-		double counter_strength_ratio;
-
 		//the vulnerability is the power projection of enemy units onto the hex
 		//we're standing on. support is the power projection of friendly units.
 		double vulnerability, support;
@@ -155,6 +155,9 @@ protected:
 
 		//is true if this attack sequence makes use of the leader
 		bool uses_leader;
+
+		//is true if the units involved in this attack sequence are surrounded.
+		bool is_surrounded;
 	};
 
 	virtual void do_attack_analysis(
@@ -176,7 +179,7 @@ protected:
 	//Example: 'loc' can be reached by two units, one of whom has a 10-3 attack and has 48/48 hp, and
 	//can defend at 40% on the adjacent grassland. The other has a 8-2 attack, and has 30/40 hp, and
 	//can defend at 60% on the adjacent mountain. The rating will be 10*3*1.0*0.4 + 8*2*0.75*0.6 = 19.2
-	virtual double power_projection(const gamemap::location& loc, const move_map& srcdst, const move_map& dstsrc, bool use_terrain=true) const;
+	virtual double power_projection(const gamemap::location& loc, const move_map& dstsrc, bool use_terrain=true) const;
 
 	virtual std::vector<attack_analysis> analyze_targets(
 	             const move_map& srcdst, const move_map& dstsrc,
@@ -185,15 +188,15 @@ protected:
 
 	bool is_accessible(const location& loc, const move_map& dstsrc) const;
 
-	virtual std::vector<target> find_targets(unit_map::const_iterator leader, const move_map& enemy_srcdst, const move_map& enemy_dstsrc);
+	virtual std::vector<target> find_targets(unit_map::const_iterator leader, const move_map& enemy_dstsrc);
 
 	//function to form a group of units suitable for moving along the route, 'route'.
 	//returns the location which the group may reach this turn.
 	//stores the locations of the units in the group in 'units'
-	virtual location form_group(const std::vector<location>& route, const move_map& dstsrc, const move_map& srcdst, std::set<location>& units);
+	virtual location form_group(const std::vector<location>& route, const move_map& dstsrc, std::set<location>& units);
 
 	//function to return the group of enemies that threaten a certain path
-	virtual void enemies_along_path(const std::vector<location>& route, const move_map& dstsrc, const move_map& srcdst, std::set<location>& units);
+	virtual void enemies_along_path(const std::vector<location>& route, const move_map& dstsrc, std::set<location>& units);
 
 	virtual bool move_group(const location& dst, const std::vector<location>& route, const std::set<location>& units);
 
@@ -201,7 +204,7 @@ protected:
 
 	virtual double compare_groups(const std::set<location>& our_group, const std::set<location>& enemy_groups, const std::vector<location>& battlefield) const;
 
-	virtual std::pair<location,location> choose_move(std::vector<target>& targets,const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc);
+	virtual std::pair<location,location> choose_move(std::vector<target>& targets,const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_dstsrc);
 
 	//function which rates the value of moving onto certain terrain for a unit
 	virtual int rate_terrain(const unit& u, const location& loc);
@@ -211,7 +214,7 @@ protected:
 	const game_data& gameinfo_;
 	unit_map& units_;
 	std::vector<team>& teams_;
-	int team_num_;
+	unsigned int team_num_;
 	const gamestatus& state_;
 	bool consider_combat_;
 	std::vector<target> additional_targets_;
@@ -260,8 +263,12 @@ protected:
 
 	mutable std::set<location> avoid_;
 
+	// Weapon choice cache, to speed simulations.
+	std::map<std::pair<location,const unit_type *>,std::pair<battle_context::unit_stats,battle_context::unit_stats> > unit_stats_cache_;
+
 	int attack_depth() const;
 	mutable int attack_depth_;
+	friend struct attack_analysis;
 };
 
 #endif
