@@ -1,6 +1,6 @@
-/* $Id: ai_attack.cpp,v 1.63 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: ai_attack.cpp 12639 2006-07-19 11:54:19Z rusty $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,20 +13,12 @@
 
 #include "global.hpp"
 
-#include "actions.hpp"
 #include "ai.hpp"
+#include "attack_prediction.hpp"
 #include "game_config.hpp"
 #include "gamestatus.hpp"
-#include "log.hpp"
-#include "pathfind.hpp"
-#include "util.hpp"
 #include "wassert.hpp"
 
-#include <algorithm>
-#include <cstdlib>
-#include <iostream>
-#include <set>
-
 #define LOG_AI LOG_STREAM(info, ai)
 
 const int max_positions = 10000;
@@ -44,7 +36,7 @@ void ai::do_attack_analysis(
 	                )
 {
 	//this function is called fairly frequently, so interact with the user here.
-	user_interact();
+	raise_user_interact();
 
 	if(cur_analysis.movements.size() >= size_t(attack_depth()))
 		return;
@@ -76,20 +68,22 @@ void ai::do_attack_analysis(
 	for(size_t i = 0; i != units.size(); ++i) {
 		const location current_unit = units[i];
 
-		const unit_map::const_iterator unit_itor = units_.find(current_unit);
+		unit_map::iterator unit_itor = units_.find(current_unit);
 		wassert(unit_itor != units_.end());
 
 		//see if the unit has the backstab ability -- units with backstab
 		//will want to try to have a friendly unit opposite the position they move to
 		//see if the unit has the slow ability -- units with slow only attack first
 		bool backstab = false, slow = false;
-		const std::vector<attack_type>& attacks = unit_itor->second.attacks();
-		for(std::vector<attack_type>::const_iterator a = attacks.begin(); a != attacks.end(); ++a) {
-			if(a->backstab()) {
+		std::vector<attack_type>& attacks = unit_itor->second.attacks();
+		for(std::vector<attack_type>::iterator a = attacks.begin(); a != attacks.end(); ++a) {
+			a->set_specials_context(gamemap::location(),gamemap::location(),
+															&gameinfo_,&units_,&map_,&state_,&teams_,true,NULL);
+			if(a->get_special_bool("backstab")) {
 				backstab = true;
 			}
 
-			if(a->slow()) {
+			if(a->get_special_bool("slow")) {
 				slow = true;
 			}
 		}
@@ -98,6 +92,53 @@ void ai::do_attack_analysis(
 			continue;
 		}
 
+               //check if the friendly unit is surrounded, a unit is surrounded if
+               //it is flanked by enemy units and at least one other enemy unit is
+               //nearby or if the unit is totaly surrounded by enemies with max. one
+               //tile to escape.
+               bool is_surrounded = false;
+               bool is_flanked = false;
+               int enemy_units_around = 0;
+               int accessible_tiles = 0;
+               gamemap::location adj[6];
+               get_adjacent_tiles(current_unit, adj);
+
+               size_t tile;
+               for(tile = 0; tile != 3; ++tile) {
+
+                       const unit_map::const_iterator tmp_unit = units_.find(adj[tile]);
+                       bool possible_flanked = false;
+
+                       if(map_.on_board(adj[tile]))
+                       {
+                               accessible_tiles++;
+                               if(tmp_unit != units_.end() && team_num_ != tmp_unit->second.side())
+                               {
+                                       enemy_units_around++;
+                                       possible_flanked = true;
+                               }
+                       }
+
+                       const unit_map::const_iterator tmp_opposite_unit = units_.find(adj[tile + 3]);
+                        if(map_.on_board(adj[tile + 3]))
+                       {
+                               accessible_tiles++;
+                               if(tmp_opposite_unit != units_.end() && team_num_ != tmp_opposite_unit->second.side())
+                               {
+                                       enemy_units_around++;
+                                       if(possible_flanked)
+                                       {
+                                               is_flanked = true;
+                                       }
+                               }
+                       }
+               }
+
+               if(is_flanked && enemy_units_around > 2 || enemy_units_around >= accessible_tiles - 1)
+                       is_surrounded = true;
+
+
+
 		double best_vulnerability = 0.0, best_support = 0.0;
 		int best_rating = 0;
 		int cur_position = -1;
@@ -120,7 +161,7 @@ void ai::do_attack_analysis(
 			}
 
 			//if the unit can't move to this location
-			if(its.first == its.second) {
+			if(its.first == its.second || units_.find(tiles[j]) != units_.end()) {
 				continue;
 			}
 
@@ -136,7 +177,10 @@ void ai::do_attack_analysis(
 				//actually take backstab into account (too complicated), this could actually
 				//make our analysis look *worse* instead of better.
 				//so we only check for 'concrete' backstab opportunities.
-				if(itor != units_.end() && itor->second.side() == unit_itor->second.side()) {
+				//That would also break backstab_check, since it
+				//assumes the defender is in place.
+				if(itor != units_.end() &&
+					backstab_check(tiles[j], loc, units_, teams_)) {
 					if(backstab) {
 						backstab_bonus = 2;
 					}
@@ -154,12 +198,12 @@ void ai::do_attack_analysis(
 			}
 
 			//find out how vulnerable we are to attack from enemy units in this hex
-			const double vulnerability = power_projection(tiles[j],enemy_srcdst,enemy_dstsrc);
+			const double vulnerability = power_projection(tiles[j],enemy_dstsrc);
 
 			//calculate how much support we have on this hex from allies. Support does not
 			//take into account terrain, because we don't want to move into a hex that is
 			//surrounded by good defensive terrain
-			const double support = power_projection(tiles[j],fullmove_srcdst,fullmove_dstsrc,false);
+			const double support = power_projection(tiles[j],fullmove_dstsrc,false);
 
 			//if this is a position with equal defense to another position, but more vulnerability
 			//then we don't want to use it
@@ -175,14 +219,16 @@ void ai::do_attack_analysis(
 
 		if(cur_position != -1) {
 			units.erase(units.begin() + i);
-
+			
 			cur_analysis.movements.push_back(std::pair<location,location>(current_unit,tiles[cur_position]));
 
 			cur_analysis.vulnerability += best_vulnerability;
 
 			cur_analysis.support += best_support;
 
-			cur_analysis.analyze(map_, units_, 50, *this, dstsrc, srcdst, enemy_dstsrc, enemy_srcdst);
+			cur_analysis.is_surrounded = is_surrounded;
+
+			cur_analysis.analyze(map_, units_, teams_, state_, gameinfo_, *this, dstsrc, srcdst, enemy_dstsrc, current_team().aggression());
 
 			if(cur_analysis.rating(current_team().aggression(),*this) > rating_to_beat) {
 
@@ -215,7 +261,6 @@ struct battle_type {
 	const gamemap::location defender;
 	const gamemap::TERRAIN terrain;
 	int weapon;
-	battle_stats stats;
 };
 
 bool operator<(const battle_type& a, const battle_type& b)
@@ -232,84 +277,12 @@ bool operator==(const battle_type& a, const battle_type& b)
 	       a.terrain == b.terrain;
 }
 
-std::set<battle_type> weapon_choice_cache;
-
-int ai::choose_weapon(const location& att, const location& def,
-					  battle_stats& cur_stats, gamemap::TERRAIN terrain, bool use_cache)
-{
-	const std::map<location,unit>::const_iterator itor = units_.find(att);
-	if(itor == units_.end())
-		return -1;
-
-	static int cache_hits = 0;
-	static int cache_misses = 0;
-
-	if(use_cache == false) {
-		weapon_choice_cache.clear();
-	}
-
-	battle_type battle(att,def,terrain);
-	const std::set<battle_type>::const_iterator cache_itor = weapon_choice_cache.find(battle);
-
-	if(cache_itor != weapon_choice_cache.end()) {
-		wassert(*cache_itor == battle);
-
-		++cache_hits;
-		cur_stats = cache_itor->stats;
-
-		if(!(size_t(cache_itor->weapon) < itor->second.attacks().size())) {
-			LOG_STREAM(err, ai) << "cached illegal weapon: " << cache_itor->weapon
-			          << "/" << itor->second.attacks().size() << "\n";
-		}
-
-		wassert(size_t(cache_itor->weapon) < itor->second.attacks().size());
-		return cache_itor->weapon;
-	}
-
-	++cache_misses;
-
-	if((cache_misses%100) == 0) {
-		LOG_AI << "cache_stats: " << cache_hits << ":" << cache_misses << " " << weapon_choice_cache.size() << "\n";
-	}
-
-	int current_choice = -1;
-	double current_rating = 0.0;
-	const std::vector<attack_type>& attacks = itor->second.attacks();
-	wassert(!attacks.empty());
-
-	const unit_map::const_iterator d_itor = units_.find(def);
-	int d_hitpoints = d_itor->second.hitpoints();
-	int a_hitpoints = itor->second.hitpoints();
-
-	for(size_t a = 0; a != attacks.size(); ++a) {
-		const battle_stats stats = evaluate_battle_stats(map_, att, def, a, units_,
-		                                                 state_, terrain);
-
-		//TODO: improve this rating formula!
-		const double rating =
-		   (double(stats.chance_to_hit_defender)/100.0)*
-		               minimum<int>(stats.damage_defender_takes,d_hitpoints)*stats.nattacks -
-		   (double(stats.chance_to_hit_attacker)/100.0)*
-		               minimum<int>(stats.damage_attacker_takes,a_hitpoints)*stats.ndefends;
-		if(rating > current_rating || current_choice == -1) {
-			current_choice = a;
-			current_rating = rating;
-			cur_stats = stats;
-		}
-	}
-
-	wassert(size_t(current_choice) < attacks.size());
-
-	battle.stats = cur_stats;
-	battle.weapon = current_choice;
-	weapon_choice_cache.insert(battle);
-
-	return current_choice;
-}
-
-void ai::attack_analysis::analyze(const gamemap& map, unit_map& units, int num_sims, ai& ai_obj,
-                                  const ai::move_map& dstsrc, const ai::move_map& srcdst,
-                                  const ai::move_map& enemy_dstsrc, const ai::move_map& enemy_srcdst)
+void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
+								  const std::vector<team>& teams,
+								  const gamestatus& status, const game_data& gamedata,
+								  class ai& ai_obj,
+                                  const move_map& dstsrc, const move_map& srcdst,
+                                  const move_map& enemy_dstsrc, double aggression)
 {
 	const unit_map::const_iterator defend_it = units.find(target);
 	wassert(defend_it != units.end());
@@ -328,211 +301,173 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units, int num_s
 	leader_threat = (tile != 6);
 	uses_leader = false;
 
-	target_value = defend_it->second.type().cost();
+	target_value = defend_it->second.cost();
 	target_value += (double(defend_it->second.experience())/
 	                 double(defend_it->second.max_experience()))*target_value;
 	target_starting_damage = defend_it->second.max_hitpoints() -
 	                         defend_it->second.hitpoints();
-	chance_to_kill = 0.0;
-	avg_damage_inflicted = 0.0;
-	avg_damage_taken = 0.0;
-	resources_used = 0.0;
-	terrain_quality = 0.0;
-	counter_strength_ratio = 0.0;
-	avg_losses = 0.0;
-
-	const int target_max_hp = defend_it->second.max_hitpoints();
-	const int target_hp = defend_it->second.hitpoints();
-	static std::vector<int> hitpoints;
-	static std::vector<battle_stats> stats;
-
-	hitpoints.clear();
-	stats.clear();
-	weapons.clear();
-
-	std::vector<std::pair<location,location> >::const_iterator m;
-	for(m = movements.begin(); m != movements.end(); ++m) {
-		battle_stats bat_stats;
-		const int weapon = ai_obj.choose_weapon(m->first,target, bat_stats, map[m->second.x][m->second.y],true);
-
-		wassert(weapon != -1);
-		weapons.push_back(weapon);
-
-		stats.push_back(bat_stats);
-		hitpoints.push_back(units.find(m->first)->second.hitpoints());
-	}
-
-	for(int j = 0; j != num_sims; ++j) {
-
-		int defenderxp = 0;
-
-		bool defender_slowed = false;
-
-		int defhp = target_hp;
-		for(size_t i = 0; i != movements.size() && defhp; ++i) {
-			const battle_stats& stat = stats[i];
-			int atthp = hitpoints[i];
-
-			int attacks = stat.nattacks;
-			int defends = stat.ndefends;
-
-			if(defender_slowed && defends > 1) {
-				--defends;
-
-				//give an extra bonus based on slowing here
-				avg_damage_taken -= stat.damage_defender_takes;
-			}
-
-			unit_map::const_iterator att = units.find(movements[i].first);
-			double cost = att->second.type().cost();
-
-			if(att->second.can_recruit()) {
-				uses_leader = true;
-			}
-
-			const bool on_village = map.is_village(movements[i].second);
-
-			//up to double the value of a unit based on experience
-			cost += (double(att->second.experience())/
-			         double(att->second.max_experience()))*cost;
-
-			terrain_quality += (double(stat.chance_to_hit_attacker)/100.0)*cost * (on_village ? 0.5 : 1.0);
-			resources_used += cost;
-
-			bool defender_strikes_first = stat.defender_strikes_first;
-
-			while(attacks || defends) {
-				if(attacks && !defender_strikes_first) {
-					const int roll = rand()%100;
-					if(roll < stat.chance_to_hit_defender) {
-						defhp -= stat.damage_defender_takes;
-						if(defhp <= 0) {
-							defhp = 0;
-							break;
-						}
-
-						atthp += stat.amount_attacker_drains;
-						if(atthp > hitpoints[i]) {
-							atthp = hitpoints[i];
-						}
-
-						if(stat.attacker_slows && !defender_slowed && defend_it->second.has_flag("slowed") == false) {
-							defender_slowed = true;
-						}
-					}
-
-					--attacks;
-				}
-
-				defender_strikes_first = false;
-
-				if(defends) {
-					const int roll = rand()%100;
-					if(roll < stat.chance_to_hit_attacker) {
-						atthp -= stat.damage_attacker_takes;
-						if(atthp <= 0) {
-							atthp = 0;
-
-							//penalty for allowing plague is a 'negative' kill
-							if(stat.defender_plague) {
-								chance_to_kill -= 1.0;
-							}
-							break;
-						}
-
-						defhp += stat.amount_defender_drains;
-						if(defhp > target_max_hp) {
-							defhp = target_max_hp;
-						}
-					}
-
-					--defends;
-				}
-			}
-
-			const int xp = defend_it->second.type().level() * (defhp <= 0 ? game_config::kill_experience : 1);
-
-			const int xp_for_advance = att->second.max_experience() - att->second.experience();
-
-			//the reward for advancing a unit is to get a 'negative' loss of that unit
-			if(xp >= xp_for_advance) {
-				avg_losses -= att->second.type().cost();
-
-				//ignore any damage done to this unit
-				atthp = hitpoints[i];
-			} else {
-				//the reward for getting a unit closer to advancement is to get
-				//the proportion of remaining experience needed, and multiply
-				//it by a quarter of the unit cost. This will cause the AI
-				//to heavily favor getting xp for close-to-advance units.
-				avg_losses -= (att->second.type().cost()*xp)/(xp_for_advance*4);
-			}
-
-			if(defhp <= 0) {
-				//the reward for killing with a unit that
-				//plagues is to get a 'negative' loss of that unit
-				if(stat.attacker_plague) {
-					avg_losses -= att->second.type().cost();
-				}
-
-				break;
-			} else if(atthp == 0) {
-				avg_losses += cost;
-			}
-
-			//if the attacker moved onto a village, reward it for doing so
-			else if(on_village) {
-				atthp += game_config::cure_amount*2; //double reward to emphasize getting onto villages
-			}
-
-			defenderxp += (atthp == 0 ? game_config::kill_experience:1)*att->second.type().level();
-
-			avg_damage_taken += hitpoints[i] - atthp;
-		}
-
-		//penalty for allowing advancement is a 'negative' kill, and
-		//defender's hitpoints get restored to maximum
-		if(defend_it->second.experience() < defend_it->second.max_experience() &&
-		   defend_it->second.experience() + defenderxp >=
-		   defend_it->second.max_experience()) {
-			chance_to_kill -= 1.0;
-			defhp = defend_it->second.hitpoints();
-		} else if(defhp == 0) {
-			chance_to_kill += 1.0;
-		} else if(map.is_village(defend_it->first)) {
-			defhp += game_config::cure_amount;
-			if(defhp > target_hp)
-				defhp = target_hp;
-		}
-
-		avg_damage_inflicted += target_hp - defhp;
-	}
 
 	//calculate the 'alternative_terrain_quality' -- the best possible defensive values
 	//the attacking units could hope to achieve if they didn't attack and moved somewhere.
 	//this is could for comparative purposes to see just how vulnerable the AI is
 	//making itself
-
 	alternative_terrain_quality = 0.0;
 	double cost_sum = 0.0;
 	for(size_t i = 0; i != movements.size(); ++i) {
 		const unit_map::const_iterator att = units.find(movements[i].first);
-		const double cost = att->second.type().cost();
+		const double cost = att->second.cost();
 		cost_sum += cost;
-		alternative_terrain_quality += cost*ai_obj.best_defensive_position(att->first,dstsrc,srcdst,enemy_dstsrc,enemy_srcdst).chance_to_hit;
+		alternative_terrain_quality += cost*ai_obj.best_defensive_position(movements[i].first,dstsrc,srcdst,enemy_dstsrc).chance_to_hit;
 	}
-
 	alternative_terrain_quality /= cost_sum*100;
 
-	chance_to_kill /= num_sims;
-	avg_damage_inflicted /= num_sims;
-	avg_damage_taken /= num_sims;
-	terrain_quality /= resources_used;
-	resources_used /= num_sims;
-	avg_losses /= num_sims;
+	avg_damage_inflicted = 0.0;
+	avg_damage_taken = 0.0;
+	resources_used = 0.0;
+	terrain_quality = 0.0;
+	avg_losses = 0.0;
+	chance_to_kill = 0.0;
 
-	if(uses_leader) {
-		leader_threat = false;
+	double def_avg_experience = 0.0;
+	double first_chance_kill = 0.0;
+
+	double prob_dead_already = 0.0;
+	wassert(!movements.empty());
+	std::vector<std::pair<location,location> >::const_iterator m;
+
+	battle_context *prev_bc = NULL;
+	const combatant *prev_def = NULL;
+
+	for (m = movements.begin(); m != movements.end(); ++m) {
+		// We fix up units map to reflect what this would look like.
+		std::pair<gamemap::location,unit> *up = units.extract(m->first);
+		up->first = m->second;
+		units.add(up);
+
+		if (up->second.can_recruit()) {
+			uses_leader = true;
+			leader_threat = false;
+		}
+
+		int att_weapon = -1, def_weapon = -1;
+		bool from_cache = false;
+		battle_context *bc;
+
+		// This cache is only about 99% correct, but speeds up evaluation by about 1000 times.
+		// We recalculate when we actually attack.
+		std::map<std::pair<location, const unit_type *>,std::pair<battle_context::unit_stats,battle_context::unit_stats> >::iterator usc;
+		usc = ai_obj.unit_stats_cache_.find(std::pair<location, const unit_type *>(target, &up->second.type()));
+		// Just check this attack is valid for this attacking unit (may be modified)
+		if (usc != ai_obj.unit_stats_cache_.end() && usc->second.first.attack_num < (int)up->second.attacks().size()) {
+			from_cache = true;
+			bc = new battle_context(usc->second.first, usc->second.second);
+		} else {
+			bc = new battle_context(map, teams, units, status, gamedata, m->second, target, att_weapon, def_weapon, 1.0 - aggression, prev_def);
+		}
+		const combatant &att = bc->get_attacker_combatant(prev_def);
+		const combatant &def = bc->get_defender_combatant(prev_def);
+
+		delete prev_bc;
+		prev_bc = bc;
+		prev_def = &bc->get_defender_combatant(prev_def);
+
+		if (!from_cache) {
+			ai_obj.unit_stats_cache_.insert(std::pair<std::pair<location, const unit_type *>,std::pair<battle_context::unit_stats,battle_context::unit_stats> >
+											(std::pair<location, const unit_type *>(target, &up->second.type()),
+											 std::pair<battle_context::unit_stats,battle_context::unit_stats>(bc->get_attacker_stats(),
+																											  bc->get_defender_stats())));
+		}
+
+		// Note we didn't fight at all if defender already dead.
+		double prob_fought = (1.0 - prob_dead_already);
+
+		// FIXME: add combatant.prob_killed
+		double prob_killed = def.hp_dist[0] - prob_dead_already;
+		prob_dead_already = def.hp_dist[0];
+
+		double prob_died = att.hp_dist[0];
+		double prob_survived = (1.0 - prob_died) * prob_fought;
+
+		double cost = up->second.cost();
+		const bool on_village = map.is_village(m->second);
+		//up to double the value of a unit based on experience
+		cost += (double(up->second.experience())/double(up->second.max_experience()))*cost;
+		resources_used += cost;
+		avg_losses += cost * prob_died;
+
+		//double reward to emphasize getting onto villages if they survive
+		if (on_village) {
+			avg_damage_taken -= game_config::poison_amount*2 * prob_survived;
+		}
+
+		terrain_quality += (double(bc->get_defender_stats().chance_to_hit)/100.0)*cost * (on_village ? 0.5 : 1.0);
+
+		double advance_prob = 0.0;
+		//the reward for advancing a unit is to get a 'negative' loss of that unit
+		if (!up->second.advances_to().empty()) {
+			int xp_for_advance = up->second.max_experience() - up->second.experience();
+			int kill_xp, fight_xp;
+
+			// See bug #6272... in some cases unit has already got enough xp to advance,
+			// but hasn't (bug elsewhere?).  Can cause divide by zero.
+			if (xp_for_advance <= 0)
+				xp_for_advance = 1;
+
+			fight_xp = defend_it->second.level();
+			kill_xp = fight_xp * game_config::kill_experience;
+
+			if (fight_xp >= xp_for_advance)
+				advance_prob = prob_fought;
+			else if (kill_xp >= xp_for_advance)
+				advance_prob = prob_killed;
+			avg_losses -= up->second.cost() * advance_prob;
+
+			//the reward for getting a unit closer to advancement (if
+			//it didn't advance) is to get the proportion of remaining
+			//experience needed, and multiply it by a quarter of the
+			//unit cost. This will cause the AI to heavily favor
+			//getting xp for close-to-advance units.
+			avg_losses -= (up->second.cost()*fight_xp)/(xp_for_advance*4) * (prob_fought - prob_killed);
+			avg_losses -= (up->second.cost()*kill_xp)/(xp_for_advance*4) * prob_killed;
+
+			//the reward for killing with a unit that
+			//plagues is to get a 'negative' loss of that unit
+			if (bc->get_attacker_stats().plagues) {
+				avg_losses -= prob_killed * up->second.cost();
+			}
+		}
+
+		// If we didn't advance, we took this damage.
+		avg_damage_taken += (up->second.hitpoints() - att.average_hp()) * (1.0 - advance_prob);
+
+		// FIXME: attack_prediction.cpp should understand advancement directly.
+		// For each level of attacker def gets 1 xp or kill_experience.
+		def_avg_experience += up->second.level() *
+			(1.0 - att.hp_dist[0] + game_config::kill_experience * att.hp_dist[0]);
+		if (m == movements.begin()) {
+			first_chance_kill = def.hp_dist[0];
+		}
+	}
+
+	if (!defend_it->second.advances_to().empty() &&
+		def_avg_experience >= defend_it->second.max_experience() - defend_it->second.experience()) {
+		// It's likely to advance: only if we can kill with first blow.
+		chance_to_kill = first_chance_kill;
+		// Negative average damage (it will advance).
+		avg_damage_inflicted = defend_it->second.hitpoints() - defend_it->second.max_hitpoints();
+	} else {
+		chance_to_kill = prev_def->hp_dist[0];
+		avg_damage_inflicted = defend_it->second.hitpoints() - prev_def->average_hp(map.gives_healing(defend_it->first));
+	}
+
+	delete prev_bc;
+	terrain_quality /= resources_used;
+
+	// Restore the units to their original positions.
+	for (m = movements.begin(); m != movements.end(); ++m) {
+		std::pair<gamemap::location,unit> *up = units.extract(m->second);
+		up->first = m->first;
+		units.add(up);
 	}
 }
 
@@ -576,11 +511,16 @@ double ai::attack_analysis::rating(double aggression, ai& ai_obj) const
 	//prefer to attack already damaged targets
 	value += ((target_starting_damage/3 + avg_damage_inflicted) - (1.0-aggression)*avg_damage_taken)/10.0;
 
-	//sanity check: if we're putting ourselves at major risk, and have no chance to kill, and we're not
-	//aiding our allies who are also attacking, then don't do it
-	if(vulnerability > 50.0 && vulnerability > support*2.0 && chance_to_kill < 0.02 && aggression < 0.75 && !ai_obj.attack_close(target)) {
-		return -1.0;
-	}
+       //if the unit is surrounded and there is no support or if the unit is surrounded and the average damage
+       //is 0, the unit skips its sanity check and tries to break free as good as possible
+       if(!is_surrounded || (support != 0 && avg_damage_taken != 0))
+       {
+               //sanity check: if we're putting ourselves at major risk, and have no chance to kill, and we're not
+               //aiding our allies who are also attacking, then don't do it
+               if(vulnerability > 50.0 && vulnerability > support*2.0 && chance_to_kill < 0.02 && aggression < 0.75 && !ai_obj.attack_close(target)) {
+                       return -1.0;
+               }
+        }
 
 	if(!leader_threat && vulnerability*terrain_quality > 0.0) {
 		value *= support/(vulnerability*terrain_quality);
@@ -608,8 +548,6 @@ std::vector<ai::attack_analysis> ai::analyze_targets(
 {
 	log_scope2(ai, "analyzing targets...");
 
-	weapon_choice_cache.clear();
-
 	std::vector<attack_analysis> res;
 
 	std::vector<location> unit_locs;
@@ -626,13 +564,13 @@ std::vector<ai::attack_analysis> ai::analyze_targets(
 	move_map fullmove_srcdst, fullmove_dstsrc;
 	calculate_possible_moves(dummy_moves,fullmove_srcdst,fullmove_dstsrc,false,true);
 
+	unit_stats_cache_.clear();
+
 	for(unit_map::const_iterator j = units_.begin(); j != units_.end(); ++j) {
 
 		//attack anyone who is on the enemy side, and who is not invisible or turned to stone
-		if(current_team().is_enemy(j->second.side()) && j->second.stone() == false &&
-		   j->second.invisible(map_.underlying_terrain(map_[j->first.x][j->first.y]),
-				state_.get_time_of_day().lawful_bonus,j->first,
-				units_,teams_) == false) {
+		if(current_team().is_enemy(j->second.side()) && !j->second.incapacitated() &&
+		   j->second.invisible(j->first,units_,teams_) == false) {
 			location adjacent[6];
 			get_adjacent_tiles(j->first,adjacent);
 			attack_analysis analysis;
@@ -655,7 +593,7 @@ std::vector<ai::attack_analysis> ai::analyze_targets(
 	return res;
 }
 
-double ai::power_projection(const gamemap::location& loc, const move_map& srcdst, const move_map& dstsrc, bool use_terrain) const
+double ai::power_projection(const gamemap::location& loc,  const move_map& dstsrc, bool use_terrain) const
 {
 	static gamemap::location used_locs[6];
 	static double ratings[6];
@@ -701,9 +639,9 @@ double ai::power_projection(const gamemap::location& loc, const move_map& srcdst
 				const unit& un = u->second;
 
 				int tod_modifier = 0;
-				if(un.type().alignment() == unit_type::LAWFUL) {
+				if(un.alignment() == unit_type::LAWFUL) {
 					tod_modifier = lawful_bonus;
-				} else if(un.type().alignment() == unit_type::CHAOTIC) {
+				} else if(un.alignment() == unit_type::CHAOTIC) {
 					tod_modifier = -lawful_bonus;
 				}
 
@@ -721,7 +659,7 @@ double ai::power_projection(const gamemap::location& loc, const move_map& srcdst
 				const bool village = map_.is_village(terrain);
 				const double village_bonus = (use_terrain && village) ? 1.5 : 1.0;
 
-				const double defense = use_terrain ? double(100 - un.defense_modifier(map_,terrain))/100.0 : 0.5;
+				const double defense = use_terrain ? double(100 - un.defense_modifier(terrain))/100.0 : 0.5;
 				const double rating = village_bonus*hp*defense*double(most_damage);
 				if(rating > best_rating) {
 					best_rating = rating;
@@ -764,3 +702,85 @@ double ai::power_projection(const gamemap::location& loc, const move_map& srcdst
 
 	return res;
 }
+
+// There is no real hope for us: we should try to do some damage to the enemy.
+// We can spend some cycles here, since it's rare.
+bool ai::desperate_attack(const gamemap::location &loc)
+{
+	const unit &u = units_.find(loc)->second;
+	std::cerr << "desperate attack by '" << u.id() << "' " << loc << "\n";
+
+	gamemap::location adj[6];
+	get_adjacent_tiles(loc, adj);
+
+	double best_kill_prob = 0.0;
+	unsigned int best_weapon = 0;
+	int best_def_weapon = -1;
+	unsigned best_dir = 0;
+
+	{
+		for(unsigned int n = 0; n != 6; ++n) {
+			const unit_map::iterator enemy = find_visible_unit(units_,adj[n], map_, teams_, current_team());
+			if (enemy != units_.end() &&
+			   current_team().is_enemy(enemy->second.side()) && !enemy->second.incapacitated()) {
+				const std::vector<attack_type>& attacks = u.attacks();
+				for (unsigned int i = 0; i != attacks.size(); ++i) {
+					// skip weapons with attack_weight=0
+					if (attacks[i].attack_weight() > 0) {
+						battle_context bc(map_, teams_, units_, state_, gameinfo_, loc, adj[n], i);
+						combatant att(bc.get_attacker_stats());
+						combatant def(bc.get_defender_stats());
+						att.fight(def);
+						if (def.hp_dist[0] > best_kill_prob) {
+							best_kill_prob = def.hp_dist[0];
+							best_weapon = i;
+							best_def_weapon = bc.get_defender_stats().attack_num;
+							best_dir = n;
+						}
+					}
+				}
+			}
+		}
+	}
+
+	if (best_kill_prob > 0.0) {
+		attack_enemy(loc, adj[best_dir], best_weapon, best_def_weapon);
+		return true;
+	}
+
+	double least_hp = u.hitpoints() + 1;
+
+	{
+		// Who would do most damage to us when they attack?  (approximate: may be different ToD)
+		for (unsigned int n = 0; n != 6; ++n) {
+			const unit_map::iterator enemy = find_visible_unit(units_,adj[n], map_, teams_, current_team());
+			if (enemy != units_.end() &&
+			   current_team().is_enemy(enemy->second.side()) && !enemy->second.incapacitated()) {
+
+				const std::vector<attack_type>& attacks = units_.find(adj[n])->second.attacks();
+				for (unsigned int i = 0; i != attacks.size(); ++i) {
+					// skip weapons with attack_weight=0
+					if (attacks[i].attack_weight() > 0) {
+						battle_context bc(map_, teams_, units_, state_, gameinfo_, adj[n], loc, i);
+						combatant att(bc.get_attacker_stats());
+						combatant def(bc.get_defender_stats());
+						att.fight(def);
+						if (def.average_hp() < least_hp) {
+							least_hp = def.average_hp();
+							best_dir = n;
+						}
+					}
+				}
+			}
+		}
+	}
+
+	// It's possible that there were no adjacent units to attack...
+	if (least_hp != u.hitpoints() + 1) {
+		battle_context bc(map_, teams_, units_, state_, gameinfo_, loc, adj[best_dir], -1, -1, 0.5);
+		attack_enemy(loc, adj[best_dir], bc.get_attacker_stats().attack_num,
+					 bc.get_defender_stats().attack_num);
+		return true;
+	}
+	return false;
+}
