@@ -1,4 +1,4 @@
-/* $Id: animated.hpp 7120 2005-06-04 19:16:06Z ott $ */
+/* $Id: animated.hpp 11550 2006-05-20 11:16:33Z boucman $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -32,9 +32,11 @@ public:
 	{
 	public:
 		virtual T operator()(const std::string& s) const { return T(s); }
+		virtual ~string_initializer(){};
 	};
 
 	animated();
+	virtual ~animated(){};
 
 	//if T can be constructed from a string, you may use this constructor
 	// animated(const std::string& cfg);
@@ -56,17 +58,22 @@ public:
 
 	int get_first_frame_time() const;
 	int get_last_frame_time() const;
-	int get_duration() const;
 
 	//inlined for performance
-	void update_current_frame() { if(does_not_change_) return; update_current_frame_internal(); };
+	void update_current_frame();
 	bool frame_changed() const;
 
 	//True if the current animation was finished
 	bool animation_finished() const;
 	int get_animation_time() const;
-	int get_frame_time() const;
+	int get_cycle_time() const;
 	const T& get_current_frame() const;
+	const T& get_first_frame() const;
+	const T& get_last_frame() const;
+	int get_frames_count() const;
+	const bool does_not_change() const {return does_not_change_;}
+
+
 
 private:
 	struct frame
@@ -84,7 +91,6 @@ private:
 		bool has_value;
 		T value;
 	};
-	void update_current_frame_internal();
 
 	static const T void_value_;
 
