@@ -1,4 +1,4 @@
-/* $Id: campaign_server.cpp 7534 2005-07-20 08:56:52Z ott $ */
+/* $Id: campaign_server.cpp 12009 2006-06-10 18:59:00Z torangan $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -62,7 +62,7 @@ int campaign_server::load_config()
 {
 	scoped_istream stream = istream_file(file_);
 	read(cfg_, *stream);
-	return lexical_cast_default<int>(cfg_["port"], 15002);
+	return lexical_cast_default<int>(cfg_["port"], 15003);
 }
 
 campaign_server::campaign_server(const std::string& cfgfile)
@@ -73,6 +73,20 @@ campaign_server::campaign_server(const std::string& cfgfile)
 	}
 }
 
+void find_translations(const config& cfg, config& campaign)
+{
+        const config::child_list& dirs = cfg.get_children("dir");
+        for(config::child_list::const_iterator i = dirs.begin(); i != dirs.end(); ++i) {
+		if((const t_string)(**i)["name"] == "LC_MESSAGES") {
+			config* language = &campaign.add_child("translation");
+			(*language)["language"] = cfg["name"];
+		}
+		else {
+			find_translations(**i, campaign);
+		}
+        }
+}
+
 void campaign_server::run()
 {
 	for(int increment = 0; ; ++increment) {
@@ -92,9 +106,53 @@ void campaign_server::run()
 
 			config data;
 			while((sock = network::receive_data(data)) != network::null_connection) {
-				if(data.child("request_campaign_list") != NULL) {
-					config campaign_list = campaigns();
-					config::child_list cmps = campaign_list.get_children("campaign");
+				if(const config* req = data.child("request_campaign_list")) {
+					time_t epoch = time(NULL);
+					config campaign_list;
+					(campaign_list)["timestamp"] = lexical_cast<std::string>(epoch);
+					if((const t_string)(*req)["times_relative_to"] != "now") {
+						epoch = 0;
+					}
+					int before_flag = 0;
+					time_t before = epoch;
+					if((const t_string)(*req)["before"] != "") {
+						try {
+							before = before + lexical_cast<time_t>((*req)["before"]);
+							before_flag = 1;
+						}
+						catch(bad_lexical_cast) {
+						}
+					}
+					int after_flag = 0;
+					time_t after = epoch;
+					if((const t_string)(*req)["after"] != "") {
+						try {
+							after = after + lexical_cast<time_t>((*req)["after"]);
+							after_flag = 1;
+						}
+						catch(bad_lexical_cast) {
+						}
+					}
+					config::child_list cmps = campaigns().get_children("campaign");
+					for(config::child_list::iterator i = cmps.begin(); i != cmps.end(); ++i) {
+						if((const t_string)(*req)["name"] != "" && (*req)["name"] != (**i)["name"]) continue;
+						if(before_flag && ((const t_string)(**i)["timestamp"] == "" || lexical_cast_default<time_t>((**i)["timestamp"],0) >= before)) continue;
+						if(after_flag && ((const t_string)(**i)["timestamp"] == "" || lexical_cast_default<time_t>((**i)["timestamp"],0) <= after)) continue;
+						int found = 1;
+						if((const t_string)(*req)["language"] != "") {
+							found = 0;
+							config::child_list translation = (**i).get_children("translation");
+							for(config::child_list::iterator j = translation.begin(); j != translation.end(); ++j) {
+								if((*req)["language"] == (**j)["language"]) {
+									found = 1;
+									break;
+								}
+							}
+						}
+						if(found == 0) continue;
+						campaign_list.add_child("campaign", (**i));
+					}
+					cmps = campaign_list.get_children("campaign");
 					for(config::child_list::iterator i = cmps.begin(); i != cmps.end(); ++i) {
 						(**i)["passphrase"] = "";
 					}
@@ -105,7 +163,7 @@ void campaign_server::run()
 				} else if(const config* req = data.child("request_campaign")) {
 					config* const campaign = campaigns().find_child("campaign","name",(*req)["name"]);
 					if(campaign == NULL) {
-						network::send_data(construct_error("Campaign not found."),sock);
+						network::send_data(construct_error("Add-on not found."),sock);
 					} else {
 						config cfg;
 						scoped_istream stream = istream_file((*campaign)["filename"]);
@@ -117,13 +175,18 @@ void campaign_server::run()
 					}
 
 				} else if(data.child("request_terms") != NULL) {
-					network::send_data(construct_message("All campaigns uploaded to this server must be licensed under the terms of the GNU General Public License (GPL). By uploading content to this server, you certify that you have the right to place the content under the conditions of the GPL, and choose to do so."),sock);
-				} else if(const config* upload = data.child("upload")) {
+					network::send_data(construct_message("All add-ons uploaded to this server must be licensed under the terms of the GNU General Public License (GPL). By uploading content to this server, you certify that you have the right to place the content under the conditions of the GPL, and choose to do so."),sock);
+				} else if(config* upload = data.child("upload")) {
+					config* data = upload->child("data");
 					config* campaign = campaigns().find_child("campaign","name",(*upload)["name"]);
-					if(campaign != NULL && (*campaign)["passphrase"] != (*upload)["passphrase"]) {
-						network::send_data(construct_error("The campaign already exists, and your passphrase was incorrect."),sock);
+					if(data == NULL) {
+						network::send_data(construct_error("No add-on data was supplied."),sock);
+					} else if(campaign != NULL && (*campaign)["passphrase"] != (*upload)["passphrase"]) {
+						network::send_data(construct_error("The add-on already exists, and your passphrase was incorrect."),sock);
 					} else if(campaign_name_legal((*upload)["name"]) == false) {
-						network::send_data(construct_error("The name of the campaign is invalid"),sock);
+						network::send_data(construct_error("The name of the add-on is invalid"),sock);
+					} else if(check_names_legal(*data) == false) {
+						network::send_data(construct_error("The add-on contains an illegal file or directory name."),sock);
 					} else {
 						if(campaign == NULL) {
 							campaign = &campaigns().add_child("campaign");
@@ -141,26 +204,34 @@ void campaign_server::run()
 						if((*campaign)["downloads"].empty()) {
 							(*campaign)["downloads"] = "0";
 						}
+						(*campaign)["timestamp"] = lexical_cast<std::string>(time(NULL));
 
-						const config* const data = upload->child("data");
-						if(data != NULL) {
-							std::string filename = (*campaign)["filename"];
-							{
-								scoped_ostream campaign_file = ostream_file(filename);
-								write_compressed(*campaign_file, *data);
-							}
-							(*campaign)["size"] = lexical_cast<std::string>(
-								file_size(filename));
+						std::string filename = (*campaign)["filename"];
+						(*data)["title"] = (*campaign)["title"];
+						(*data)["name"] = "";
+						(*data)["campaign_name"] = (*campaign)["name"];
+						(*data)["author"] = (*campaign)["author"];
+						(*data)["description"] = (*campaign)["description"];
+						(*data)["version"] = (*campaign)["version"];
+						(*data)["timestamp"] = (*campaign)["timestamp"];
+						(*data)["icon"] = (*campaign)["icon"];
+						(*campaign).clear_children("translation");
+						find_translations(*data, *campaign);
+						{
+							scoped_ostream campaign_file = ostream_file(filename);
+							write_compressed(*campaign_file, *data);
 						}
+						(*campaign)["size"] = lexical_cast<std::string>(
+							file_size(filename));
 
 						scoped_ostream cfgfile = ostream_file(file_);
 						write(*cfgfile, cfg_);
-						network::send_data(construct_message("Campaign accepted."),sock);
+						network::send_data(construct_message("Add-on accepted."),sock);
 					}
 				} else if(const config* erase = data.child("delete")) {
 					config* const campaign = campaigns().find_child("campaign","name",(*erase)["name"]);
 					if(campaign == NULL) {
-						network::send_data(construct_error("The campaign does not exist."),sock);
+						network::send_data(construct_error("The add-on does not exist."),sock);
 						continue;
 					}
 
@@ -171,13 +242,28 @@ void campaign_server::run()
 
 					//erase the campaign
 					write_file((*campaign)["filename"],"");
+					remove((*campaign)["filename"].c_str());
 
 					const config::child_list& campaigns_list = campaigns().get_children("campaign");
 					const size_t index = std::find(campaigns_list.begin(),campaigns_list.end(),campaign) - campaigns_list.begin();
 					campaigns().remove_child("campaign",index);
 					scoped_ostream cfgfile = ostream_file(file_);
 					write(*cfgfile, cfg_);
-					network::send_data(construct_message("Campaign erased."),sock);
+					network::send_data(construct_message("Add-on deleted."),sock);
+				} else if(const config* cpass = data.child("change_passphrase")) {
+					config* campaign = campaigns().find_child("campaign","name",(*cpass)["name"]);
+					if(campaign == NULL) {
+						network::send_data(construct_error("No add-on with that name exists."),sock);
+					} else if((*campaign)["passphrase"] != (*cpass)["passphrase"]) {
+						network::send_data(construct_error("Your old passphrase was incorrect."),sock);
+					} else if((const t_string)(*cpass)["new_passphrase"] == "") {
+						network::send_data(construct_error("No new passphrase was supplied."),sock);
+					} else {
+						(*campaign)["passphrase"] = (*cpass)["new_passphrase"];
+						scoped_ostream cfgfile = ostream_file(file_);
+						write(*cfgfile, cfg_);
+						network::send_data(construct_message("Passphrase changed."),sock);
+					}
 				}
 			}
 		} catch(network::error& e) {
@@ -197,7 +283,7 @@ void campaign_server::run()
 
 }
 
-int main(int argc, char** argv)
+int main(int, char**)
 {
 	lg::timestamps(true);
 	try {
