@@ -1,4 +1,4 @@
-/* $Id: cavegen.cpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: cavegen.cpp 11309 2006-05-06 15:21:53Z rusty $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -66,7 +66,7 @@ size_t cave_map_generator::translate_y(size_t y) const
 
 bool cave_map_generator::allow_user_config() const { return true; }
 
-void cave_map_generator::user_config(display& disp) { return; }
+void cave_map_generator::user_config(display& /*disp*/) { return; }
 
 std::string cave_map_generator::name() const { return "cave"; }
 
@@ -76,7 +76,7 @@ std::string cave_map_generator::create_map(const std::vector<std::string>& args)
 	return res["map_data"];
 }
 
-config cave_map_generator::create_scenario(const std::vector<std::string>& args)
+config cave_map_generator::create_scenario(const std::vector<std::string>& /*args*/)
 {
 	map_ = std::vector<std::vector<gamemap::TERRAIN> >(width_,std::vector<gamemap::TERRAIN>(height_,wall_));
 	chambers_.clear();
@@ -129,7 +129,7 @@ void cave_map_generator::build_chamber(gamemap::location loc, std::set<gamemap::
 	gamemap::location adj[6];
 	get_adjacent_tiles(loc,adj);
 	for(size_t n = 0; n != 6; ++n) {
-		if((rand()%100) < (100-jagged)) {
+		if((rand()%100) < (100l-(long)jagged)) {
 			build_chamber(adj[n],locs,size-1,jagged);
 		}
 	}
@@ -238,7 +238,7 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 			object_filter = object->child("filter");
 		}
 
-		if(cfg["same_location_as_previous"] != "yes") {
+		if(!utils::string_bool(cfg["same_location_as_previous"])) {
 			index = rand()%c.locs.size();
 		}
 
@@ -246,7 +246,7 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 		std::advance(loc,index);
 
 		char buf[50];
-		sprintf(buf,"%d",loc->x+1);
+		snprintf(buf,sizeof(buf),"%d",loc->x+1);
 		cfg.values["x"] = buf;
 		if(filter != NULL) {
 			(*filter)["x"] = buf;
@@ -256,7 +256,7 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 			(*object_filter)["x"] = buf;
 		}
 
-		sprintf(buf,"%d",loc->y+1);
+		snprintf(buf,sizeof(buf),"%d",loc->y+1);
 		cfg.values["y"] = buf;
 		if(filter != NULL) {
 			(*filter)["y"] = buf;
@@ -267,7 +267,7 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 		}
 
 		//if this is a side, place a castle for the side
-		if(key == "side" && cfg["no_castle"] != "yes") {
+		if(key == "side" && !utils::string_bool(cfg["no_castle"])) {
 			place_castle(cfg["side"],*loc);
 		}
 
@@ -283,7 +283,7 @@ struct passage_path_calculator : cost_calculator
 		        : map_(mapdata), wall_(wall), laziness_(laziness), windiness_(windiness)
 	{}
 
-	virtual double cost(const gamemap::location& loc, const double so_far, const bool is_dest) const;
+	virtual double cost(const gamemap::location& src,const gamemap::location& loc, const double so_far, const bool is_dest) const;
 private:
 	const std::vector<std::vector<gamemap::TERRAIN> >& map_;
 	gamemap::TERRAIN wall_;
@@ -291,7 +291,7 @@ private:
 	size_t windiness_;
 };
 
-double passage_path_calculator::cost(const gamemap::location& loc, const double, const bool) const
+double passage_path_calculator::cost(const gamemap::location& /*src*/,const gamemap::location& loc, const double, const bool) const
 {
 	wassert(loc.x >= 0 && loc.y >= 0 && size_t(loc.x) < map_.size() &&
 	        !map_.empty() && size_t(loc.y) < map_.front().size());
@@ -338,13 +338,13 @@ void cave_map_generator::place_passage(const passage& p)
 
 bool cave_map_generator::on_board(const gamemap::location& loc) const
 {
-	return loc.x >= 0 && loc.y >= 0 && loc.x < width_ && loc.y < height_;
+	return loc.x >= 0 && loc.y >= 0 && loc.x < (long)width_ && loc.y < (long)height_;
 }
 
 void cave_map_generator::set_terrain(gamemap::location loc, gamemap::TERRAIN t)
 {
 	if(on_board(loc)) {
-		if(t == clear_ && (rand()%1000) < village_density_) {
+		if(t == clear_ && (rand()%1000) < (long)village_density_) {
 			t = village_;
 		}
 
