@@ -1,4 +1,4 @@
-/* $Id: checksum.hpp,v 1.3 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: checksum.hpp 7863 2005-08-12 10:32:12Z j_daniel $ */
 /*
    Copyright (C) 2005 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -21,7 +21,7 @@ class checksumstreambuf : public std::basic_streambuf<char>
 {
 public:
 	checksumstreambuf();
-	unsigned long checksum();
+	unsigned int checksum();
 
 protected:
 	virtual int overflow(int c);
@@ -43,7 +43,7 @@ class checksumstream : private checksumstreambuf, public std::basic_ostream<char
 {
 public:
 	checksumstream();
-	unsigned long checksum();
+	unsigned int checksum();
 private:
 	checksumstreambuf& sbuf;
 };
