@@ -1,4 +1,4 @@
-/* $Id: config.cpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: config.cpp 12374 2006-07-03 13:21:51Z brunowolff $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -17,15 +17,18 @@
 #include <algorithm>
 #include <sstream>
 #include "config.hpp"
-#include "log.hpp"
-#include "wassert.hpp"
 #include "gettext.hpp"
+#include "log.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <iostream>
 
 #define ERR_CF LOG_STREAM(err, config)
 
+// Note this isn't a real copy. The children are still in both config trees
+// and updating them will affect both trees.
 config::config(const config& cfg)
 {
 	append(cfg);
@@ -36,6 +39,8 @@ config::~config()
 	clear();
 }
 
+// Note this isn't a real copy. The children are still in both config trees
+// and updating them will affect both trees.
 config& config::operator=(const config& cfg)
 {
 	if(this == &cfg) {
@@ -49,6 +54,8 @@ config& config::operator=(const config& cfg)
 	return *this;
 }
 
+// Note this isn't a real copy. The children are still in both config trees
+// and updating them will affect both trees.
 void config::append(const config& cfg)
 {
 	for(all_children_iterator i = cfg.ordered_begin(); i != cfg.ordered_end(); ++i) {
@@ -527,6 +534,38 @@ void config::apply_diff(const config& diff)
 	}
 }
 
+// Create a new config tree as a copy of 'this' overridden by 'c'.
+// Nodes are matched up by name and with name by order. Nodes in 'c',
+// but not in 'this' are added at the end in the order they appeared in 'c'.
+// Note that some nodes in the copied tree are still in both 'this' and 'c'.
+// It worked like this before. This is more efficent, but can have surprising
+// effects if a node is changed in one of these config trees.
+config config::merge_with(const config& c) const
+{
+	config n;
+	for(string_map::const_iterator j = this->values.begin();
+	    j != this->values.end(); ++j) {
+		n.values[j->first] = j->second;
+	}
+	// This ends up copying values twice (in config initialization and
+	// append), but is simpler than dealing with the guts of m.
+	config m(c);
+	for(all_children_iterator i = this->ordered_begin();
+	    i != this->ordered_end(); ++i) {
+		const std::pair<const std::string*,const config*>& value = *i;
+		config *mc = m.child(*value.first);
+		if (mc == NULL) {
+			n.add_child(*value.first, *value.second);
+		}
+		else {
+			n.add_child(*value.first, value.second->merge_with(*mc));
+			m.remove_child(*value.first, 0);
+		}
+	}
+	n.append(m);
+	return n;
+}
+
 void config::reset_translation() const
 {
 	for(string_map::const_iterator val = values.begin(); val != values.end(); ++val) {
@@ -541,6 +580,85 @@ void config::reset_translation() const
 	}
 }
 
+void config::debug() const{
+	static int i = 0;
+	i++;
+	for(string_map::const_iterator val = values.begin(); val != values.end(); ++val) {
+		for (int j = 0; j < i; j++){ std::cerr << char(9); }
+		std::cerr << val->first << ", " << val->second << "\n";
+	}
+
+	for(child_map::const_iterator list = children.begin(); list != children.end(); ++list) {
+		{ for (int j = 0; j < i-1; j++){ std::cerr << char(9); } }
+		std::cerr << "[" << list->first << "]\n";
+		for(child_list::const_iterator child = list->second.begin(); child != list->second.end(); ++child) {
+			(*child)->debug();
+		}
+		{ for (int j = 0; j < i-1; j++){ std::cerr << char(9); } }
+		std::cerr << "[/" << list->first << "]\n";
+	}
+	i--;
+}
+
+std::string config::hash() const
+{
+	static const unsigned int hash_length = 128;
+	static const char hash_string[] =
+		"+-,.<>0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+	char hash_str[hash_length + 1];
+	std::string::const_iterator c;
+
+	unsigned int i;
+	for(i = 0; i != hash_length; ++i) {
+		hash_str[i] = 'a';
+	}
+	hash_str[hash_length] = 0;
+
+	i = 0;
+	for(string_map::const_iterator val = values.begin(); val != values.end(); ++val) {
+		if(val->first.size() && val->second.size()) {
+			for(c = val->first.begin(); c != val->first.end(); ++c) {
+				if(utils::portable_isspace(*c)) {
+					continue;
+				}
+				hash_str[i] ^= *c;
+				++i;
+				if(i == hash_length) {
+					i = 0;
+				}
+			}
+			for(c = val->second.value().begin(); c != val->second.value().end(); ++c) {
+				if(utils::portable_isspace(*c)) {
+					continue;
+				}
+				hash_str[i] ^= *c;
+				++i;
+				if(i == hash_length) {
+					i = 0;
+				}
+			}
+		}
+	}
+	for(child_map::const_iterator list = children.begin(); list != children.end(); ++list) {
+		for(child_list::const_iterator child = list->second.begin(); child != list->second.end(); ++child) {
+			std::string child_hash = (*child)->hash();
+			for(c = child_hash.begin(); c != child_hash.end(); ++c) {
+				hash_str[i] ^= *c;
+				++i;
+				if(i == hash_length) {
+					i = 0;
+				}
+			}
+		}
+	}
+
+	for(i = 0; i != hash_length; ++i) {
+		hash_str[i] = hash_string[(unsigned)hash_str[i]%strlen(hash_string)];
+	}
+
+	return std::string(hash_str);
+}
+
 bool operator==(const config& a, const config& b)
 {
 	if (a.values != b.values)
