@@ -1,4 +1,4 @@
-/* $Id: config.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: config.hpp 11877 2006-06-05 20:02:25Z soliton $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -13,6 +13,8 @@
 #ifndef CONFIG_HPP_INCLUDED
 #define CONFIG_HPP_INCLUDED
 
+#include "global.hpp"
+
 #include <map>
 #include <string>
 #include <vector>
@@ -26,8 +28,6 @@
 //sent across the network in this format. It is thus used extensively
 //throughout the game.
 
-class t_string;
-
 typedef std::map<std::string,t_string> string_map;
 
 //a config object defines a single node in a WML file, with access to
@@ -79,6 +79,9 @@ public:
 	void clear();
 	bool empty() const;
 
+	void debug() const;
+	std::string hash() const;
+
 	struct error {
 		error(const std::string& msg) : message(msg) {}
 		std::string message;
@@ -120,6 +123,8 @@ public:
 
 	void apply_diff(const config& diff); //throw error
 
+	config merge_with(const config& c) const;
+
 	//append data from another config object to this one. attributes in the
 	//latter config object will clobber attributes in this one.
 	void append(const config& cfg);
