@@ -1,4 +1,4 @@
-/* $Id: dialogs.cpp 8269 2005-09-17 15:06:28Z ott $ */
+/* $Id: dialogs.cpp 12768 2006-07-25 16:32:50Z jhinrichs $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -16,7 +16,6 @@
 #include "dialogs.hpp"
 #include "events.hpp"
 #include "filesystem.hpp"
-#include "font.hpp"
 #include "game_config.hpp"
 #include "game_errors.hpp"
 #include "gettext.hpp"
@@ -24,7 +23,9 @@
 #include "language.hpp"
 #include "log.hpp"
 #include "map.hpp"
-#include "playturn.hpp"
+#include "marked-up_text.hpp"
+#include "menu_events.hpp"
+#include "minimap.hpp"
 #include "preferences.hpp"
 #include "replay.hpp"
 #include "show_dialog.hpp"
@@ -46,40 +47,45 @@
 #define LOG_DP LOG_STREAM(info, display)
 #define ERR_G  LOG_STREAM(err, general)
 
+
 namespace dialogs
 {
 
 void advance_unit(const game_data& info,
 				  const gamemap& map,
-                  std::map<gamemap::location,unit>& units,
+                  unit_map& units,
                   gamemap::location loc,
                   display& gui, bool random_choice)
 {
-	std::map<gamemap::location,unit>::iterator u = units.find(loc);
+	unit_map::iterator u = units.find(loc);
 	if(u == units.end() || u->second.advances() == false)
 		return;
 
-	LOG_DP << "advance_unit: " << u->second.type().id() << "\n";
+	LOG_DP << "advance_unit: " << u->second.id() << "\n";
 
-	const std::vector<std::string>& options = u->second.type().advances_to();
+	const std::vector<std::string>& options = u->second.advances_to();
 
 	std::vector<std::string> lang_options;
 
 	std::vector<unit> sample_units;
 	for(std::vector<std::string>::const_iterator op = options.begin(); op != options.end(); ++op) {
 		sample_units.push_back(::get_advanced_unit(info,units,loc,*op));
-		const unit_type& type = sample_units.back().type();
-		lang_options.push_back(IMAGE_PREFIX + type.image() + COLUMN_SEPARATOR + type.language_name());
+		const unit& type = sample_units.back();
+		lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + "~TC(" + lexical_cast_default<std::string>(u->second.side()) + "," + type.team_color() + ")" + COLUMN_SEPARATOR + type.language_name());
 		preferences::encountered_units().insert(*op);
 	}
 
 	const config::child_list& mod_options = u->second.get_modification_advances();
 
 	for(config::child_list::const_iterator mod = mod_options.begin(); mod != mod_options.end(); ++mod) {
-		sample_units.push_back(::get_advanced_unit(info,units,loc,u->second.type().id()));
+		sample_units.push_back(::get_advanced_unit(info,units,loc,u->second.id()));
 		sample_units.back().add_modification("advance",**mod);
-		const unit_type& type = sample_units.back().type();
-		lang_options.push_back(IMAGE_PREFIX + type.image() + COLUMN_SEPARATOR + (**mod)["description"].str());
+		const unit& type = sample_units.back();
+		if((**mod)["image"].str().size()){
+		  lang_options.push_back(IMAGE_PREFIX + (**mod)["image"].str() + COLUMN_SEPARATOR + (**mod)["description"].str());
+		}else{
+		  lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + "~TC(" + lexical_cast_default<std::string>(u->second.side()) + "," + type.team_color() + ")" + COLUMN_SEPARATOR + (**mod)["description"].str());
+		}
 	}
 
 	LOG_DP << "options: " << options.size() << "\n";
@@ -92,12 +98,11 @@ void advance_unit(const game_data& info,
 		res = rand()%lang_options.size();
 	} else if(lang_options.size() > 1) {
 
-		const events::event_context dialog_events_context;
 		unit_preview_pane unit_preview(gui,&map,sample_units);
 		std::vector<gui::preview_pane*> preview_panes;
 		preview_panes.push_back(&unit_preview);
 
-		res = gui::show_dialog(gui,NULL,_("Advance Unit"),
+		res = gui::show_dialog2(gui,NULL,_("Advance Unit"),
 		                       _("What should our victorious unit become?"),
 		                       gui::OK_ONLY, &lang_options, &preview_panes);
 	}
@@ -110,14 +115,14 @@ void advance_unit(const game_data& info,
 
 bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::location loc, display& gui, size_t choice)
 {
-	const command_disabler cmd_disabler;
+	const events::command_disabler cmd_disabler;
 
-	std::map<gamemap::location,unit>::iterator u = units.find(loc);
+	unit_map::iterator u = units.find(loc);
 	if(u == units.end() || u->second.advances() == false) {
 		return false;
 	}
 
-	const std::vector<std::string>& options = u->second.type().advances_to();
+	const std::vector<std::string>& options = u->second.advances_to();
 	const config::child_list& mod_options = u->second.get_modification_advances();
 
 	if(choice >= options.size() + mod_options.size()) {
@@ -128,40 +133,42 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 	//new unit, then fades back to the normal colour
 
 	if(!gui.update_locked()) {
-		for(double intensity = 1.0; intensity >= 0.0; intensity -= 0.05) {
-			gui.set_advancing_unit(loc,intensity);
+		u->second.set_leveling_out(gui,u->first);
+		while(!u->second.get_animation()->animation_finished()) {
+			gui.invalidate(loc);
+			gui.draw();
 			events::pump();
-			gui.draw(false);
-			gui.update_display();
-			SDL_Delay(30);
+			gui.non_turbo_delay();
 		}
 	}
 
 	if(choice < options.size()) {
-		const std::string& chosen_unit = choice < options.size() ? options[choice] : u->second.type().id();
+		const std::string& chosen_unit = choice < options.size() ? options[choice] : u->second.id();
 		::advance_unit(info,units,loc,chosen_unit);
 	}
 
 	u = units.find(loc);
 	if(u != units.end() && choice >= options.size()) {
-		u->second.get_experience(-u->second.experience()); //reset xp to 0
+		u->second.get_experience(-u->second.max_experience()); //subtract xp required
 		u->second.add_modification("advance",*mod_options[choice - options.size()]);
 	}
 
 	gui.invalidate_unit();
 
 	if(!gui.update_locked()) {
-		for(double intensity = 0.0; intensity <= 1.0; intensity += 0.05) {
-			gui.set_advancing_unit(loc,intensity);
+		u->second.set_leveling_in(gui,u->first);
+		while(!u->second.get_animation()->animation_finished()) {
+			gui.invalidate(loc);
+			gui.draw();
 			events::pump();
-			gui.draw(false);
-			gui.update_display();
-			SDL_Delay(30);
+			gui.non_turbo_delay();
 		}
+		u->second.set_standing(gui,u->first);
+		gui.invalidate(loc);
+		gui.draw();
+		events::pump();
 	}
 
-	gui.set_advancing_unit(gamemap::location(),0.0);
-
 	gui.invalidate_all();
 	gui.draw();
 
@@ -285,7 +292,12 @@ void save_preview_pane::draw_contents()
 		return;
 	}
 
-	const config& summary = *(*summaries_)[index_];
+	std::string dummy;
+	config& summary = *(*summaries_)[index_];
+	if (summary["label"] == ""){
+		load_game_summary((*info_)[index_].name, summary, &dummy);
+		*(*summaries_)[index_] = summary;
+	}
 
 	surface const screen = video().getSurface();
 
@@ -299,7 +311,7 @@ void save_preview_pane::draw_contents()
 
 	const game_data::unit_type_map::const_iterator leader = data_->unit_types.find(summary["leader"]);
 	if(leader != data_->unit_types.end()) {
-		const surface image(image::get_image(leader->second.image(),image::UNSCALED));
+		const surface image(image::get_image(leader->second.image() + "~TC(1," + leader->second.flag_rgb() + ")",image::UNSCALED));
 		if(image != NULL) {
 			SDL_Rect image_rect = {area.x,area.y,image->w,image->h};
 			ypos += image_rect.h + save_preview_border;
@@ -354,12 +366,17 @@ void save_preview_pane::draw_contents()
 		SDL_BlitSurface(map_surf,NULL,screen,&map_rect);
 	}
 
+	char* old_locale= setlocale(LC_TIME, get_locale().localename.c_str());
 	char time_buf[256];
 	const size_t res = strftime(time_buf,sizeof(time_buf),_("%a %b %d %H:%M %Y"),localtime(&((*info_)[index_].time_modified)));
 	if(res == 0) {
 		time_buf[0] = 0;
 	}
 
+	if(old_locale) {
+		setlocale(LC_TIME, old_locale);
+	}
+
 	std::stringstream str;
 
 	// escape all special characters in filenames
@@ -461,15 +478,10 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 		return "";
 	}
 
-	std::vector<size_t> no_summary; //all items that aren't in the summary table
 	std::vector<config*> summaries;
 	std::vector<save_info>::const_iterator i;
 	for(i = games.begin(); i != games.end(); ++i) {
 		config& cfg = save_summary(i->name);
-		if(cfg["campaign_type"].empty() && cfg["corrupt"] != "yes" || lexical_cast<int>(cfg["mod_time"]) != static_cast<int>(i->time_modified)) {
-			no_summary.push_back(i - games.begin());
-		}
-
 		summaries.push_back(&cfg);
 	}
 
@@ -478,76 +490,8 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	std::vector<gui::dialog_button> buttons;
 	buttons.push_back(delete_button);
 
-	bool generate_summaries = !no_summary.empty();
-
-	//if there are more than 5 saves without a summary, it may take a substantial
-	//amount of time to convert them over. Ask the user if they want to do this
-	if(no_summary.size() > 5) {
-
-		if(preferences::cache_saves() == preferences::CACHE_SAVES_NEVER) {
-			generate_summaries = false;
-		} else if(preferences::cache_saves() == preferences::CACHE_SAVES_ALWAYS) {
-			generate_summaries = true;
-		} else {
-			char* caption = N_("Import Saved Games");
-			char* message = N_("Your saves directory contains some files from an old version of Battle for Wesnoth. Would you like to update these to the latest version? This may take some time.");
-
-			//if there are already some cached games, then we assume that the user is importing new
-			//games, and it's not a total import, so tailor the message accordingly
-			if(no_summary.size() < games.size()) {
-				message = N_("Your saves directory contains some files that don't appear to have been generated by this version of Battle for Wesnoth. Would you like to register these files with the game?");
-			}
-
-			std::vector<gui::check_item> options;
-			options.push_back(gui::check_item(_("Don't ask me again!"),false));
-			const int res = gui::show_dialog(disp,NULL,_(caption),_(message),
-			                                 gui::YES_NO,NULL,NULL,"",NULL,-1,NULL,&options);
-
-			generate_summaries = res == 0;
-			if(options.front().checked) {
-				preferences::set_cache_saves(generate_summaries ? preferences::CACHE_SAVES_ALWAYS : preferences::CACHE_SAVES_NEVER);
-			}
-		}
-	}
-
 	const events::event_context context;
 
-	if(generate_summaries) {
-		gui::progress_bar bar(disp.video());
-		const SDL_Rect bar_area = {disp.x()/2 - 100, disp.y()/2 - 20, 200, 40};
-		bar.set_location(bar_area);
-
-		for(std::vector<size_t>::const_iterator s = no_summary.begin(); s != no_summary.end(); ++s) {
-			bar.set_progress_percent(((s - no_summary.begin())*100)/no_summary.size());
-			events::raise_draw_event();
-			events::pump();
-			disp.update_display();
-
-			log_scope("load");
-			game_state state;
-
-			config& summary = save_summary(games[*s].name);
-
-			try {
-				summary["mod_time"] = str_cast(lexical_cast<int>(games[*s].time_modified));
-				std::string dummy;
-				load_game(data,games[*s].name,state,&dummy);
-				extract_summary_data_from_save(state,summary);
-			} catch(io_exception&) {
-				summary["corrupt"] = "yes";
-				ERR_G << "save '" << games[*s].name << "' could not be loaded (io_exception)\n";
-			} catch(config::error&) {
-				summary["corrupt"] = "yes";
-				ERR_G << "save '" << games[*s].name << "' could not be loaded (config parse error)\n";
-			} catch(game::load_game_failed&) {
-				summary["corrupt"] = "yes";
-				ERR_G << "save '" << games[*s].name << "' could not be loaded (load_game_failed exception)\n";
-			}
-		}
-
-		write_save_index();
-	}
-
 	std::vector<std::string> items;
 	std::ostringstream heading;
 	heading << HEADING_PREFIX << _("Name") << COLUMN_SEPARATOR << _("Date");
@@ -578,11 +522,13 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	if(show_replay != NULL)
 		options.push_back(gui::check_item(_("Show replay"),false));
 
-	const int res = gui::show_dialog(disp,NULL,
+	const int res = gui::show_dialog2(disp,NULL,
 					 _("Load Game"),
 					 _("Choose the game to load"),
 			         gui::OK_CANCEL,&items,&preview_panes,"",NULL,-1,NULL,&options,-1,-1,NULL,&buttons,"",&sorter);
 
+	write_save_index();
+
 	if(res == -1)
 		return "";
 
@@ -601,9 +547,10 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 void unit_speak(const config& message_info, display& disp, const unit_map& units)
 {
 	for(unit_map::const_iterator i = units.begin(); i != units.end(); ++i) {
-		if(i->second.matches_filter(message_info)) {
+		if(i->second.matches_filter(message_info,i->first)) {
+
 			disp.scroll_to_tile(i->first.x,i->first.y);
-			const surface surface(image::get_image(i->second.type().image_profile(),image::UNSCALED));
+			const surface surface(image::get_image(i->second.profile(),image::UNSCALED));
 			gui::show_dialog(disp,surface,i->second.underlying_description(),message_info["message"],gui::MESSAGE);
 		}
 	}
@@ -626,7 +573,7 @@ unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const un
 	unit_store_.push_back(u);
 }
 
-unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const std::vector<unit>& units, TYPE type, bool on_left_side)
+unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, std::vector<unit>& units, TYPE type, bool on_left_side)
                                     : gui::preview_pane(disp.video()), disp_(disp),
 				      details_button_(disp.video(),_("Profile"),gui::button::TYPE_PRESS,"lite_small",gui::button::MINIMUM_SPACE),
 				      map_(map), units_(&units), index_(0), left_(on_left_side),
@@ -635,6 +582,12 @@ unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const st
 	set_measurements(font::relative_size(200), font::relative_size(370));
 }
 
+void unit_preview_pane::join()
+{
+	widget::join();
+	details_button_.join();
+}
+
 bool unit_preview_pane::show_above() const
 {
 	return !weapons_;
@@ -663,7 +616,7 @@ void unit_preview_pane::draw_contents()
 		return;
 	}
 
-	const unit& u = (*units_)[index_];
+	unit& u = (*units_)[index_];
 
 	const bool right_align = left_side();
 
@@ -675,10 +628,7 @@ void unit_preview_pane::draw_contents()
 	SDL_Rect clip_area = area;
 	const clip_rect_setter clipper(screen,clip_area);
 
-	surface unit_image(image::get_image(u.type().image(),image::UNSCALED));
-	if(left_side() == false && unit_image != NULL) {
-		unit_image.assign(image::reverse_image(unit_image));
-	}
+	surface unit_image = u.still_image();
 
 	SDL_Rect image_rect = {area.x,area.y,0,0};
 
@@ -707,18 +657,19 @@ void unit_preview_pane::draw_contents()
 	}
 
 	std::stringstream details;
-	details << u.type().language_name()
+	details << u.language_name()
 			<< "\n" << _("level") << " "
-			<< u.type().level() << "\n"
-			<< unit_type::alignment_description(u.type().alignment()) << "\n"
+			<< u.level() << "\n"
+			<< unit_type::alignment_description(u.alignment()) << "\n"
 			<< u.traits_description() << " \n";
 
-	const std::vector<std::string>& abilities = u.type().abilities();
+	const std::vector<std::string>& abilities = u.unit_ability_tooltips();
 	for(std::vector<std::string>::const_iterator a = abilities.begin(); a != abilities.end(); ++a) {
 		details << gettext(a->c_str());
-		if(a+1 != abilities.end()) {
+		if(a+2 != abilities.end()) {
 			details << ", ";
 		}
+		++a;
 	}
 
 	details << " \n";
@@ -750,17 +701,18 @@ void unit_preview_pane::draw_contents()
 				<< u.total_movement()
 				<< "\n";
 
-		const std::vector<attack_type>& attacks = u.attacks();
-		for(std::vector<attack_type>::const_iterator at_it = attacks.begin();
+		std::vector<attack_type>& attacks = u.attacks();
+		for(std::vector<attack_type>::iterator at_it = attacks.begin();
 		    at_it != attacks.end(); ++at_it) {
+			at_it->set_specials_context(gamemap::location(),u);
 
 			details << "\n" << at_it->name()
 			        << " (" << gettext(at_it->type().c_str()) << ")\n";
-			if (!at_it->special().empty())
-				details << gettext(at_it->special().c_str());
+
+			details << at_it->weapon_specials(true);
 			details << "\n"
 			        << at_it->damage() << "-" << at_it->num_attacks() << " -- "
-			        << (at_it->range() == attack_type::SHORT_RANGE ? _("melee") : _("ranged"));
+			        << _(at_it->range().c_str());
 		}
 	}
 
@@ -796,7 +748,7 @@ void unit_preview_pane::process_event()
 
 void show_unit_description(display &disp, const unit& u)
 {
-	help::show_help(disp,"unit_" + u.type().id());
+	help::show_help(disp,"unit_" + u.id());
 }
 
 
