@@ -1,4 +1,4 @@
-/* $Id: display.hpp 7949 2005-08-21 16:22:15Z gruikya $ */
+/* $Id: display.hpp 11673 2006-05-28 05:21:57Z rusty $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -50,15 +50,13 @@ class gamestatus;
 class display
 {
 public:
-	typedef std::map<gamemap::location,unit> unit_map;
-
 	display(unit_map& units, CVideo& video,
 			const gamemap& map, const gamestatus& status,
 			const std::vector<team>& t, const config& theme_cfg,
 			const config& cfg, const config& level);
 	~display();
 
-	Uint32 rgb(Uint8 red, Uint8 green, Uint8 blue);
+	static Uint32 rgb(Uint8 red, Uint8 green, Uint8 blue);
 
 	//new_turn should be called on every new turn, to update
 	//lighting settings.
@@ -68,11 +66,6 @@ public:
 	//the map. Used for special effects like flashes.
 	void adjust_colours(int r, int g, int b);
 
-	//function to make it so a unit is 'hidden' - not displayed
-	//when the tile it is on is drawn. Only one unit may be hidden
-	//at a time. The previously hidden unit will be returned.
-	//'hide_energy' determines whether the unit's energy bar should be hidden too.
-	gamemap::location hide_unit(const gamemap::location& loc, bool hide_energy=false);
 
 	//function which scrolls the display by xmov,ymov. Invalidation and
 	//redrawing will be scheduled.
@@ -96,16 +89,21 @@ public:
 	//(i.e. not entirely from tip to tip -- use hex_size() to get the distance from tip to tip)
 	int hex_width() const;
 
-	enum SCROLL_TYPE { SCROLL, WARP };
+	enum SCROLL_TYPE { SCROLL, WARP, ONSCREEN };
 
 	//function which will scroll such that location x,y is on-screen.
-	void scroll_to_tile(int x, int y, SCROLL_TYPE scroll_type=SCROLL, bool check_fogged=true);
+	// WARP jumps to x,y; SCROLL uses scroll speed;
+	// ONSCREEN only scrolls if x,y is offscreen
+	void scroll_to_tile(int x, int y, SCROLL_TYPE scroll_type=ONSCREEN, bool check_fogged=true);
 
 	//function which will scroll such that location x1,y1 is on-screen.
 	//it will also try to make it such that x2,y2 is on-screen but this
 	//is not guaranteed.
 	void scroll_to_tiles(int x1, int y1, int x2, int y2,
-	                     SCROLL_TYPE scroll_type=SCROLL, bool check_fogged=true);
+	                     SCROLL_TYPE scroll_type=ONSCREEN, bool check_fogged=true);
+
+	//scrolls to the leader of a certain side. This will normally be the playing team.
+	void scroll_to_leader(unit_map& units, int side);
 
 	//invalidates entire screen, including all tiles and sidebar.
 	void redraw_everything();
@@ -126,13 +124,19 @@ public:
 
 	const SDL_Rect& map_area() const;
 	const SDL_Rect& minimap_area() const;
+	const SDL_Rect& unit_image_area() const;
 
 	SDL_Rect screen_area() const;
 
+	//check if pixel x,y is outside specified area
+	bool outside_area(const SDL_Rect& area, const int x, const int y) const;
+
 	//function to display a location as selected. If a unit is in the location,
 	//and there is no unit in the currently highlighted hex, the unit will be
 	//displayed in the sidebar.
 	void select_hex(gamemap::location hex);
+	const gamemap::location& selected_hex() { return selectedHex_; }
+	const gamemap::location& mouseover_hex() { return mouseoverHex_; }
 
 	//function to highlight a location. If a unit is in the location, it will
 	//be displayed in the sidebar. Selection is used when a unit has been
@@ -155,11 +159,20 @@ public:
 	//location if the mouse isn't over the minimap.
 	gamemap::location minimap_location_on(int x, int y);
 
+	// given x,y co-ordinates of the mouse, returns whether the
+	// mouse is over the unit image or not
+	bool unit_image_on(int x, int y);
+
 	//sets the paths that are currently displayed as available for the unit
-	//to move along. All other paths will be greyed out. If NULL, no paths
-	//will be displayed as selected.
-	//paths_list must remain valid until it is set again
-	void set_paths(const paths* paths_list);
+	//to move along.  All other paths will be greyed out.
+	void highlight_reach(const paths &paths_list);
+
+	//add more paths to highlight.  Print numbers where they overlap.
+	//Used only by Show Enemy Moves.
+	void highlight_another_reach(const paths &paths_list);
+
+	//reset highlighting of paths.
+	void unhighlight_reach();
 
 	//sets the route along which footsteps are drawn to show movement of a
 	//unit. If NULL, no route is displayed.
@@ -176,30 +189,24 @@ public:
 	//function to remove a footstep from a specific location
 	void remove_footstep(const gamemap::location& loc);
 
-	//function to draw the tile at location (x,y). If unit_image is not NULL,
-	//then it will be used, otherwise the unit's default image will be used.
-	//alpha controls how faded the unit is. If blend_to is not 0, then the
-	//unit will be alpha-blended to blend_to instead of the background colour
-	void draw_tile(int x, int y, surface unit_image=surface(NULL),
-	               fixed_t alpha=ftofxp(1.0), Uint32 blend_to=0);
-
 	//function to float a label above a tile
 	void float_label(const gamemap::location& loc, const std::string& text,
 	                 int red, int green, int blue);
 
+	const gamemap& get_map()const { return map_;}
 private:
 	enum ADJACENT_TERRAIN_TYPE { ADJACENT_BACKGROUND, ADJACENT_FOREGROUND, ADJACENT_FOGSHROUD };
 
 	//composes and draws the terrains on a tile
 	void draw_terrain_on_tile(int x, int y, image::TYPE image_type, ADJACENT_TERRAIN_TYPE type);
 
-	void draw_unit_on_tile(int x, int y, surface unit_image=surface(NULL),
-	                       fixed_t alpha=ftofxp(1.0), Uint32 blend_to=0);
 
-	void draw_halo_on_tile(int x, int y);
+
+	gui::button::TYPE string_to_button_type(std::string type);
 
 	// void draw_tile_adjacent(int x, int y, image::TYPE image_type, ADJACENT_TERRAIN_TYPE type);
 
+	void draw_enemies_reach(unsigned int num, int xloc, int yloc);
 
 public:
 	//function to draw a footstep for the given location, on screen at
@@ -217,11 +224,12 @@ public:
 	//function to invalidate all tiles.
 	void invalidate_all();
 
-	//function to invalidate a specific tile
+	//function to invalidate a specific tile for redrawing
 	void invalidate(const gamemap::location& loc);
 
 	//function to invalidate the game status displayed on the sidebar.
 	void invalidate_game_status();
+	const gamestatus &get_game_status() {return status_;};
 
 	//function to invalidate that unit status displayed on the sidebar.
 	void invalidate_unit();
@@ -237,6 +245,10 @@ public:
 	//have moved about on the map
 	void redraw_minimap();
 
+	//temporarily place a unit on map (moving: can overlap others)
+	void place_temporary_unit(unit &u, const gamemap::location& loc);
+	void remove_temporary_unit();
+
 	//functions to add and remove overlays from locations. An overlay is an
 	//image that is displayed on top of the tile. One tile may have multiple
 	//overlays. remove_overlay will remove all overlays on a tile.
@@ -269,11 +281,9 @@ public:
 	//set_playing_team sets the team whose turn it currently is
 	void set_team(size_t team);
 	void set_playing_team(size_t team);
+	const std::vector<team>& get_teams() {return teams_;};
 
-	//makes it so that the unit at the given location will be displayed
-	//in an advancing-highlight with intensity between 0.0 and 1.0 given
-	//by amount.
-	void set_advancing_unit(const gamemap::location& loc, double amount);
+	unit_map& get_units() {return units_;};
 
 	//compat methods to be dropped after full migration
 	void lock_updates(bool value) {screen_.lock_updates(value); };
@@ -284,6 +294,10 @@ public:
 	bool turbo() const;
 	void set_turbo(bool turbo);
 
+	//Delay routines: use these not SDL_Delay (for --nogui).
+	void non_turbo_delay() const;
+	void delay(unsigned int milliseconds) const;
+
 	//function which determines whether a grid should be overlayed on the
 	//game board to more clearly show where hexes are.
 	void set_grid(bool grid);
@@ -302,8 +316,9 @@ public:
 	//is the team whose turn it is
 	size_t viewing_team() const;
 	size_t playing_team() const;
+	bool team_valid() const;
 
-	const theme& get_theme() const;
+	theme& get_theme();
 
 	const theme::menu* menu_pressed();
 
@@ -357,19 +372,17 @@ public:
 	void begin_game();
 
 	bool in_game() const { return in_game_; }
+	void draw_bar(const std::string& image, int xpos, int ypos, size_t height, double filled, const SDL_Color& col, fixed_t alpha);
 
 private:
 	display(const display&);
 	void operator=(const display&);
 
+	void draw_tile(const gamemap::location &loc, const SDL_Rect &clip_rect);
 	void draw_sidebar();
 	void draw_minimap(int x, int y, int w, int h);
 	void draw_game_status();
 
-	SDL_Rect gameStatusRect_;
-	SDL_Rect unitDescriptionRect_;
-	SDL_Rect unitProfileRect_;
-
 	void draw_image_for_report(surface& img, SDL_Rect& rect);
 	void draw_report(reports::TYPE report_num);
 	SDL_Rect reportRects_[reports::NUM_REPORTS];
@@ -398,7 +411,8 @@ private:
 
 	unit_map& units_;
 
-	void draw_bar(const std::string& image, int xpos, int ypos, size_t height, double filled, const SDL_Color& col, fixed_t alpha);
+	unit *temp_unit_;
+	gamemap::location temp_unit_loc_;
 
 	//function which finds the start and end rows on the energy bar image
 	//where white pixels are substituted for the colour of the energy
@@ -408,12 +422,10 @@ private:
 	surface minimap_;
 	bool redrawMinimap_;
 
-	const paths* pathsList_;
 	paths::route route_;
 
 	const gamestatus& status_;
 
-	bool team_valid() const;
 
 	const std::vector<team>& teams_;
 
@@ -444,18 +456,8 @@ private:
 
 	size_t currentTeam_, activeTeam_;
 
-	//used to store a unit that is not drawn, because it's currently
-	//being moved or otherwise changed
-	gamemap::location hiddenUnit_;
-	bool hideEnergy_;
 
-	//used to store any unit that is dying
-	gamemap::location deadUnit_;
-	fixed_t deadAmount_;
 
-	//used to store any unit that is advancing
-	gamemap::location advancingUnit_;
-	double advancingAmount_;
 
 	bool turbo_, grid_;
 	double sidebarScaling_;
@@ -490,8 +492,9 @@ private:
 	//then we will use these two masks on top of all hexes when we blit
 	surface tod_hex_mask1, tod_hex_mask2;
 
-	typedef std::map<gamemap::location,int> halo_map;
-	halo_map haloes_;
+	//tiles lit for showing where unit(s) can reach
+	typedef std::map<gamemap::location,unsigned int> reach_map;
+	reach_map reach_map_;
 
 	//for debug mode
 	static std::map<gamemap::location,fixed_t> debugHighlights_;
@@ -501,6 +504,7 @@ private:
 	int diagnostic_label_;
 
 	//animated flags for each team
+	//
 	std::vector<animated<image::locator> > flags_;
 
 	//the handle for the label which displays fps
