@@ -24,7 +24,7 @@
 #include "../language.hpp"
 #include "../widgets/menu.hpp"
 #include "../pathfind.hpp"
-#include "../playlevel.hpp"
+
 #include "../preferences.hpp"
 #include "../sdl_utils.hpp"
 #include "../tooltips.hpp"
@@ -48,7 +48,7 @@
 #include <cmath>
 
 namespace {
-	const int num_players = 10;
+	const int num_players = 9;
 	// Milliseconds to sleep in every iteration of the main loop.
 	const unsigned int sdl_delay = 20;
 	const std::string prefs_filename = get_dir(get_user_data_dir() + "/editor")
@@ -92,8 +92,8 @@ map_editor::map_editor(display &gui, gamemap &map, config &theme, config &game_c
 	: gui_(gui), map_(map), abort_(DONT_ABORT),
 	  theme_(theme), game_config_(game_config), map_dirty_(false), l_button_palette_dirty_(true),
 	  everything_dirty_(false), palette_(gui, size_specs_, map), brush_(gui, size_specs_),
-	  l_button_held_func_(NONE), highlighted_locs_cleared_(false), prefs_disp_manager_(&gui_),
-	  all_hexes_selected_(false), tooltip_manager_(gui_.video()), floating_label_manager_() {
+	  l_button_held_func_(NONE), tooltip_manager_(gui_.video()), floating_label_manager_(),
+	  highlighted_locs_cleared_(false), prefs_disp_manager_(&gui_), all_hexes_selected_(false) {
 
 	// Set size specs.
 	adjust_sizes(gui_, size_specs_);
@@ -359,7 +359,9 @@ void map_editor::right_click(const gamemap::location hex_clicked ) {
 	}
 }
 
-// Set the language...
+/**
+ * Change the language (effectively reload the editor).
+ */
 void map_editor::change_language() {
 	std::vector<language_def> langdefs = get_languages();
 
@@ -378,9 +380,10 @@ void map_editor::change_language() {
 	const int res = gui::show_dialog(gui_,NULL,_("Language"),
 	                         _("Choose your preferred language:"),
 	                         gui::OK_CANCEL,&langs);
-	if(size_t(res) < langs.size()) {
-		::set_language(known_languages[res]);
-		preferences::set_language(known_languages[res].localename);
+	const std::vector<language_def>& languages = get_languages();
+	if(size_t(res) < languages.size()) {
+		::set_language(languages[res]);
+		preferences::set_language(languages[res].localename);
 
 		game_config_.reset_translation();
 
@@ -388,8 +391,18 @@ void map_editor::change_language() {
 		load_tooltips();
 	}
 
+	// Update the frame title
+	SDL_WM_SetCaption(_("Battle for Wesnoth Map Editor"), NULL);
+
 	font::load_font_config();
 	hotkey::load_descriptions();
+
+	// To reload the terrain names, we need to reload the configuration file
+	gamemap new_map(game_config_, map_.write());
+	map_ = new_map;
+
+	// Update the selected terrain strings
+	palette_.update_selected_terrains();
 }
 
 
@@ -437,7 +450,7 @@ void map_editor::perform_set_starting_pos() {
 	std::vector<std::string> players;
 	for (int i = 0; i < num_players; i++) {
 		std::stringstream str;
-		str << _("Player ") << i + 1;
+		str << _("Player") << " " << i + 1;
 		players.push_back(str.str());
 	}
 	int res = gui::show_dialog(gui_, NULL, _("Which Player?"),
@@ -445,7 +458,6 @@ void map_editor::perform_set_starting_pos() {
 	                                 gui::OK_CANCEL, &players);
 	if (res >= 0) {
 		res++;
-		if (res == num_players) res = 0;
 		set_starting_position(res, selected_hex_);
 	}
 }
@@ -497,7 +509,7 @@ void map_editor::edit_load_map() {
 	if (res == 0) {
 		// Check if the mapname contains any illegal characters
 		if(!verify_filename(fn, false))	{
-			gui::show_dialog(gui_, NULL, "", _("Warning: Illegal characters found in the map name. You've to save under a different name."), gui::OK_ONLY);
+			gui::show_dialog(gui_, NULL, "", _("Warning: Illegal characters found in the map name. Please save under a different name."), gui::OK_ONLY);
 		}
 
 		std::string new_map;
@@ -715,6 +727,7 @@ bool map_editor::can_execute_command(hotkey::HOTKEY_COMMAND command) const {
 	case hotkey::HOTKEY_FULLSCREEN:
 	case hotkey::HOTKEY_SCREENSHOT:
 	case hotkey::HOTKEY_TOGGLE_GRID:
+	case hotkey::HOTKEY_MOUSE_SCROLL:
 	case hotkey::HOTKEY_PREFERENCES:
 	case hotkey::HOTKEY_EDIT_SAVE_MAP:
 	case hotkey::HOTKEY_EDIT_SAVE_AS:
@@ -1099,8 +1112,8 @@ void map_editor::invalidate_all_and_adjacent(const std::set<gamemap::location> &
 	invalidate_all_and_adjacent(v);
 }
 
-void map_editor::right_button_down(const int mousex, const int mousey) {
-	// Draw with the bacground terrain on rightclick, no matter what
+void map_editor::right_button_down(const int /*mousex*/, const int /*mousey*/) {
+	// Draw with the background terrain on rightclick, no matter what
 	// operations are wanted with the left button.
 	// TODO evaluate if this is what is the smartest thing to do.
 	draw_on_mouseover_hexes(palette_.selected_bg_terrain());
@@ -1171,7 +1184,7 @@ bool map_editor::verify_filename(const std::string& filename, bool show_error) c
 {
 	static const std::string allowed_characters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_/\\.";
 	static const std::string prefix = "\\/";
-	std::size_t start_pos = filename.find_last_of(prefix);
+	size_t start_pos = filename.find_last_of(prefix);
 
 	if(filename.find_first_not_of(allowed_characters, start_pos) != std::string::npos) {
 		if(show_error) {
@@ -1227,11 +1240,11 @@ void map_editor::recalculate_starting_pos_labels() {
 	}
 	starting_positions_.clear();
 	// Set new labels.
-	for (int i = 0; i < 10; i++) {
+	for (int i = 1; i < 10; i++) {
 		gamemap::location loc = map_.starting_position(i);
 		if (loc.valid()) {
 			std::stringstream ss;
-			ss << _("Player") << i;
+			ss << _("Player") << " " << i;
 			gui_.labels().set_label(loc, ss.str());
 			starting_positions_.push_back(loc);
 		}
@@ -1424,8 +1437,6 @@ void map_editor::main_loop() {
 		mouse_moved_ = false;
 	}
 }
-
-
 }
 
 
