@@ -146,10 +146,18 @@ void terrain_palette::select_bg_terrain(gamemap::TERRAIN terrain) {
 	}
 }
 
+/**
+ * After the language is changed, the selected terrains needs an update.
+ */
+void terrain_palette::update_selected_terrains(void) {
+	set_dirty();
+	update_report();
+}
+
 std::string terrain_palette::get_terrain_string(const gamemap::TERRAIN t) {
 	std::stringstream str;
 	const std::string& name = map_.get_terrain_info(t).name();
-	const std::string& underlying = map_.underlying_terrain(t);
+	const std::string& underlying = map_.underlying_union_terrain(t);
 	str << name;
 	if(underlying.size() != 1 || underlying[0] != t) {
 		str << " (";
