@@ -1,7 +1,7 @@
-/* $Id: font.cpp,v 1.145 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: font.cpp 12603 2006-07-17 09:53:55Z isaac $ */
 /* vim:set encoding=utf-8: */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,13 +15,12 @@
 #include "wesconfig.h"
 #include "global.hpp"
 
-#include "font.hpp"
+#include "config.hpp"
 #include "filesystem.hpp"
+#include "font.hpp"
 #include "game_config.hpp"
-#include "gettext.hpp"
 #include "log.hpp"
 #include "sdl_utils.hpp"
-#include "team.hpp"
 #include "tooltips.hpp"
 #include "util.hpp"
 #include "video.hpp"
@@ -53,6 +52,13 @@
 #error Please use the SDL_ttf files in the sdl_ttf directory, and not the original SDL_ttf library.
 #endif
 
+#ifdef	HAVE_FRIBIDI
+#include <fribidi/fribidi.h>
+
+#else
+
+#endif
+
 namespace {
 
 // Signed int. Negative values mean "no subset".
@@ -141,52 +147,36 @@ std::vector<text_chunk> split_text(std::string const & utf8_text) {
 TTF_Font* open_font(const std::string& fname, int size)
 {
 	std::string name;
-
-	LOG_FT << "Opening font '" << fname << "' ...\n";
-#ifndef USE_ZIPIOS
-	if(game_config::path.empty() == false) {
+	if(!game_config::path.empty()) {
 		name = game_config::path + "/fonts/" + fname;
-		LOG_FT << "Trying file '" << name << "' ...\n";
-
-		if (!file_exists(name)) {
+		if(!file_exists(name)) {
 			name = "fonts/" + fname;
-			WRN_FT << "Failed opening '" << name << "'; now trying '" << name << "' ...\n";
-			if (!file_exists(name)) {
-				ERR_FT << "Failed opening font: " << fname << "\n";
-				return NULL;
+			if(!file_exists(name)) {
+				name = fname;
+				if(!file_exists(name)) {
+					ERR_FT << "Failed opening font: '" << name << "': No such file or directory\n";
+					return NULL;
+				}
 			}
 		}
+
 	} else {
 		name = "fonts/" + fname;
-		LOG_FT << "Trying file '" << name << "' ...\n";
-
 		if(!file_exists(name)) {
-			ERR_FT << "Failed opening font '" << fname << "'\n";
-			return NULL;
+			if(!file_exists(fname)) {
+				ERR_FT << "Failed opening font: '" << name << "': No such file or directory\n";
+				return NULL;
+			}
+			name = fname;
 		}
 	}
 
-	LOG_FT << "Opening font file '" << name << "', font size is " << size << "\n";
-
 	TTF_Font* font = TTF_OpenFont(name.c_str(),size);
-#else
-	std::string tmp = read_file("fonts/" + fname);
-	if (tmp.empty()) {
-		ERR_FT << "Failed opening font file '" << fname << "'\n";
-		return NULL;
-	}
-	// the following statement would leak memory if fonts were closed
-	std::string *s = new std::string(tmp);
-	SDL_RWops* ops = SDL_RWFromMem((void*)s->c_str(), s->size());
-	TTF_Font* font = TTF_OpenFontRW(ops, 0, size);
-#endif
 	if(font == NULL) {
-		ERR_FT << "Failed opening font file '" << name << "'\n";
+		ERR_FT << "Failed opening font: TTF_OpenFont: " << TTF_GetError() << "\n";
 		return NULL;
 	}
 
-	LOG_FT << "Opened font okay\n";
-
 	return font;
 }
 
@@ -286,12 +276,6 @@ manager::~manager()
 	TTF_Quit();
 }
 
-void set_font()
-{
-	clear_fonts();
-	font_names.push_back(_("Vera.ttf"));
-}
-
 void set_font_list(const std::vector<subset_descriptor>& fontlist)
 {
 	clear_fonts();
@@ -299,7 +283,25 @@ void set_font_list(const std::vector<subset_descriptor>& fontlist)
 
 	std::vector<subset_descriptor>::const_iterator itor;
 	for(itor = fontlist.begin(); itor != fontlist.end(); ++itor) {
-		subset_id subset = font_names.size();
+		// Insert fonts only if the font file exists
+		if(game_config::path.empty() == false) {
+			if(!file_exists(game_config::path + "/fonts/" + itor->name)) {
+				if(!file_exists("fonts/" + itor->name)) {
+					if(!file_exists(itor->name)) {
+					WRN_FT << "Failed opening font file '" << itor->name << "': No such file or directory\n";
+					continue;
+					}
+				}
+			}
+		} else {
+			if(!file_exists("fonts/" + itor->name)) {
+				if(!file_exists(itor->name)) {
+					WRN_FT << "Failed opening font file '" << itor->name << "': No such file or directory\n";
+					continue;
+				}
+			}
+		}
+		const subset_id subset = font_names.size();
 		font_names.push_back(itor->name);
 
 		std::vector<std::pair<size_t,size_t> >::const_iterator cp_range;
@@ -328,9 +330,7 @@ const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
                 BUTTON_COLOUR = {0xBC,0xB0,0x88,0},
                 STONED_COLOUR = {0xA0,0xA0,0xA0,0},
                 TITLE_COLOUR  = {0xBC,0xB0,0x88,0};
-
-const char LARGE_TEXT='*', SMALL_TEXT='`', GOOD_TEXT='@', BAD_TEXT='#',
-           NORMAL_TEXT='{', BLACK_TEXT='}', BOLD_TEXT='~', IMAGE='&', NULL_MARKUP='^';
+const SDL_Color DISABLED_COLOUR = inverse(STONED_COLOUR);
 
 namespace {
 
@@ -346,6 +346,9 @@ public:
 	void measure() const;
 	size_t width() const;
 	size_t height() const;
+#ifdef	HAVE_FRIBIDI
+	bool is_rtl() const { return is_rtl_; }	// Right-To-Left alignment
+#endif
 	std::vector<surface> const & get_surfaces() const;
 
 	bool operator==(text_surface const &t) const {
@@ -363,13 +366,42 @@ private:
 	mutable bool initialized_;
 	mutable std::vector<text_chunk> chunks_;
 	mutable std::vector<surface> surfs_;
+#ifdef	HAVE_FRIBIDI
+	bool is_rtl_;
+	void bidi_cvt();
+#endif
 	void hash();
 };
 
+#ifdef	HAVE_FRIBIDI
+void text_surface::bidi_cvt()
+{
+	char		*c_str = const_cast<char *>(str_.c_str());	// fribidi forgot const...
+	int		len = str_.length();
+	FriBidiChar	*bidi_logical = new FriBidiChar[len + 2];
+	FriBidiChar	*bidi_visual = new FriBidiChar[len + 2];
+	char		*utf8str = new char[4*len + 1];	//assume worst case here (all 4 Byte characters)
+	FriBidiCharType	base_dir = FRIBIDI_TYPE_ON;
+	int n;
+
+	n = fribidi_utf8_to_unicode (c_str, len, bidi_logical);
+	fribidi_log2vis(bidi_logical, n, &base_dir, bidi_visual, NULL, NULL, NULL);
+	fribidi_unicode_to_utf8 (bidi_visual, n, utf8str);
+	is_rtl_ = base_dir == FRIBIDI_TYPE_RTL;
+	str_ = std::string(utf8str);
+	delete[] bidi_logical;
+	delete[] bidi_visual;
+	delete[] utf8str;
+}
+#endif
+
 text_surface::text_surface(std::string const &str, int size, SDL_Color color, int style)
   : font_size_(size), color_(color), style_(style), w_(-1), h_(-1), str_(str),
   initialized_(false)
 {
+#ifdef	HAVE_FRIBIDI
+	bidi_cvt();
+#endif
 	hash();
 }
 
@@ -384,6 +416,9 @@ void text_surface::set_text(std::string const &str)
 	w_ = -1;
 	h_ = -1;
 	str_ = str;
+#ifdef	HAVE_FRIBIDI
+	bidi_cvt();
+#endif
 	hash();
 }
 
@@ -556,74 +591,6 @@ surface render_text(const std::string& text, int fontsize, const SDL_Color& colo
 	}
 }
 
-namespace {
-
-//function which will parse the markup tags at the front of a string
-std::string::const_iterator parse_markup(std::string::const_iterator i1, std::string::const_iterator i2,
-					 int* font_size, SDL_Color* colour, int* style)
-{
-	if(font_size == NULL || colour == NULL) {
-		return i1;
-	}
-
-	while(i1 != i2) {
-		switch(*i1) {
-		case '\\':
-			// this must either be a quoted special character or a
-			// quoted backslash - either way, remove leading backslash
-			break;
-		case BAD_TEXT:
-			*colour = BAD_COLOUR;
-			break;
-		case GOOD_TEXT:
-			*colour = GOOD_COLOUR;
-			break;
-		case NORMAL_TEXT:
-			*colour = NORMAL_COLOUR;
-			break;
-		case BLACK_TEXT:
-			*colour = BLACK_COLOUR;
-			break;
-		case LARGE_TEXT:
-			*font_size += 2;
-			break;
-		case SMALL_TEXT:
-			*font_size -= 2;
-			break;
-		case BOLD_TEXT:
-			*style |= TTF_STYLE_BOLD;
-			break;
-		case NULL_MARKUP:
-			return i1+1;
-		// semi ANSI colour escape sequences at the start of the line for now only
-		case '\033':
-			if(i2 - i1 >= 4) {
-				++i1;
-				if(*i1 == '[') {
-					++i1;
-					if(*i1 == '3') {
-						++i1;
-						if(*i1 >= '0' && *i1 <= '9' && *(i1 + 1) == 'm')
-						{
-							if(*i1 != '0')
-								*colour = team::get_side_colour(lexical_cast<int, char>(*i1));
-							++i1;
-						}
-					}
-				}
-			}
-			break;
-		default:
-			return i1;
-		}
-
-		++i1;
-	}
-
-	return i1;
-}
-
-}
 
 surface get_rendered_text(const std::string& str, int size, const SDL_Color& colour, int style)
 {
@@ -668,7 +635,10 @@ SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::
 
 			font_style_setter const style_setter(ttfont, style);
 
-			TTF_SizeUNICODE(ttfont, (Uint16 const *)&chunk.front(), (int*)&rect.x, (int*)&rect.y);
+			int x, y;
+			TTF_SizeUNICODE(ttfont, (Uint16 const *)&chunk.front(), &x, &y);
+			rect.x = x;
+			rect.y = y;
 
 			rect.w += rect.x;
 			rect.h = maximum<Sint16>(rect.h, rect.y);
@@ -690,7 +660,10 @@ SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::
 
 		font_style_setter const style_setter(ttfont, style);
 
-		TTF_SizeUNICODE(ttfont, (Uint16 const *)&chunk.front(), (int*)&rect.x, (int*)&rect.y);
+		int x, y;
+		TTF_SizeUNICODE(ttfont, (Uint16 const *)&chunk.front(), &x, &y);
+		rect.x = x;
+		rect.y = y;
 
 		rect.w += rect.x;
 		rect.h = maximum<Sint16>(rect.h, rect.y);
@@ -722,9 +695,17 @@ SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 	}
 
 	SDL_Rect dest;
-	if(x!=-1)
+	if(x!=-1) {
 		dest.x = x;
-	else
+#ifdef	HAVE_FRIBIDI
+		// Oron -- Conditional, until all draw_text_line calls have fixed area parameter
+		if(getenv("NO_RTL") == NULL) {
+			bool is_rtl = text_cache::find(text_surface(text, size, colour, style)).is_rtl();
+			if(is_rtl)
+				dest.x = area.x + area.w - surface->w - (x - area.x);
+		}
+#endif
+	} else
 		dest.x = (area.w/2)-(surface->w/2);
 	if(y!=-1)
 		dest.y = y;
@@ -774,61 +755,6 @@ SDL_Rect draw_text_line(CVideo* gui, const SDL_Rect& area, int size,
 	return draw_text_line(surface, area, size, colour, text, x, y, use_tooltips, style);
 }
 
-SDL_Rect text_area(const std::string& text, int size, int style)
-{
-	const SDL_Rect area = {0,0,10000,10000};
-	return draw_text(NULL, area, size, font::NORMAL_COLOUR, text, 0, 0, false, style);
-}
-
-SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
-                   const SDL_Color& colour, const std::string& txt,
-                   int x, int y, bool use_tooltips, int style)
-{
-	//make sure there's always at least a space, so we can ensure
-	//that we can return a rectangle for height
-	static const std::string blank_text(" ");
-	const std::string& text = txt.empty() ? blank_text : txt;
-
-	SDL_Rect res;
-	res.x = x;
-	res.y = y;
-	res.w = 0;
-	res.h = 0;
-
-	std::string::const_iterator i1 = text.begin();
-	std::string::const_iterator i2 = std::find(i1,text.end(),'\n');
-	for(;;) {
-		SDL_Color col = colour;
-		int sz = size;
-		int text_style = style;
-
-		i1 = parse_markup(i1,i2,&sz,&col,&text_style);
-
-		if(i1 != i2) {
-			std::string new_string(i1,i2);
-
-			utils::unescape(new_string);
-
-			const SDL_Rect rect = draw_text_line(gui, area, sz, col, new_string, x, y, use_tooltips, text_style);
-			if(rect.w > res.w) {
-				res.w = rect.w;
-			}
-
-			res.h += rect.h;
-			y += rect.h;
-		}
-
-		if(i2 == text.end()) {
-			break;
-		}
-
-		i1 = i2+1;
-		i2 = std::find(i1,text.end(),'\n');
-	}
-
-	return res;
-}
-
 int get_max_height(int size)
 {
 	// Only returns the maximal size of the first font
@@ -838,29 +764,6 @@ int get_max_height(int size)
 	return TTF_FontHeight(font);
 }
 
-bool is_format_char(char c)
-{
-	//side coloring
-	if(c > 0 && c <= 10) {
-		return true;
-	}
-
-	switch(c) {
-	case LARGE_TEXT:
-	case SMALL_TEXT:
-	case GOOD_TEXT:
-	case BAD_TEXT:
-	case NORMAL_TEXT:
-	case BLACK_TEXT:
-	case BOLD_TEXT:
-	case NULL_MARKUP:
-		return true;
-	default:
-		return false;
-	}
-}
-
-
 std::string remove_first_space(const std::string& text)
 {
   if (text.length() > 0 && text[0] == ' ') {
@@ -894,6 +797,7 @@ SDL_Rect line_size(const std::string& line, int font_size, int style)
 
 	res.w = s.width();
 	res.h = s.height();
+	res.x = res.y = 0;
 
 	if(line.size() < max_cache_size) {
 		cache.insert(std::pair<std::string,SDL_Rect>(line,res));
@@ -902,198 +806,6 @@ SDL_Rect line_size(const std::string& line, int font_size, int style)
 	return res;
 }
 
-namespace {
-
-void cut_word(std::string& line, std::string& word, int size, int max_width)
-{
-	std::string tmp = line;
-	utils::utf8_iterator tc(word);
-	bool first = true;
-
-	for(;tc != utils::utf8_iterator::end(word); ++tc) {
-		tmp.append(tc.substr().first, tc.substr().second);
-		SDL_Rect tsize = line_size(tmp, size);
-		if(tsize.w > max_width) {
-			const std::string& w = word;
-			if(line.empty() && first) {
-				line += std::string(w.begin(), tc.substr().second);
-				word = std::string(tc.substr().second, w.end());
-			} else {
-				line += std::string(w.begin(), tc.substr().first);
-				word = std::string(tc.substr().first, w.end());
-			}
-			break;
-		}
-		first = false;
-	}
-}
-
-
-/*
- * According to Kinsoku-Shori, Japanese rules about line-breaking:
- *
- * * the following characters cannot begin a line (so we will never break before them):
- * 、。，．）〕］｝〉》」』】’”ゝゞヽヾ々？！：；ぁぃぅぇぉゃゅょゎァィゥェォャュョヮっヵッヶ・…ー
- *
- * * the following characters cannot end a line (so we will never break after them):
- * （〔［｛〈《「『【‘“
- */
-inline bool no_break_after(wchar_t ch)
-{
-	return
-		ch == 0x2018 || ch == 0x201c || ch == 0x3008 || ch == 0x300a || ch == 0x300c ||
-		ch == 0x300e || ch == 0x3010 || ch == 0x3014 || ch == 0xff08 || ch == 0xff3b ||
-		ch == 0xff5b;
-
-}
-
-inline bool no_break_before(wchar_t ch)
-{
-	return
-		ch == 0x2019 || ch == 0x201d || ch == 0x2026 || ch == 0x3001 || ch == 0x3002 ||
-		ch == 0x3005 || ch == 0x3009 || ch == 0x300b || ch == 0x300d || ch == 0x300f ||
-		ch == 0x3011 || ch == 0x3015 || ch == 0x3041 || ch == 0x3043 || ch == 0x3045 ||
-		ch == 0x3047 || ch == 0x3049 || ch == 0x3063 || ch == 0x3083 || ch == 0x3085 ||
-		ch == 0x3087 || ch == 0x308e || ch == 0x309d || ch == 0x309e || ch == 0x30a1 ||
-		ch == 0x30a3 || ch == 0x30a5 || ch == 0x30a7 || ch == 0x30a9 || ch == 0x30c3 ||
-		ch == 0x30e3 || ch == 0x30e5 || ch == 0x30e7 || ch == 0x30ee || ch == 0x30f5 ||
-		ch == 0x30f6 || ch == 0x30fb || ch == 0x30fc || ch == 0x30fd || ch == 0x30fe ||
-		ch == 0xff01 || ch == 0xff09 || ch == 0xff0c || ch == 0xff0e || ch == 0xff1a ||
-		ch == 0xff1b || ch == 0xff1f || ch == 0xff3d || ch == 0xff5d;
-}
-
-inline bool break_before(wchar_t ch)
-{
-	if(no_break_before(ch))
-		return false;
-
-	return ch == ' ' ||
-		// CKJ characters
-		(ch >= 0x3000 && ch < 0xa000) ||
-		(ch >= 0xf900 && ch < 0xfb00) ||
-		(ch >= 0xff00 && ch < 0xfff0);
-}
-
-inline bool break_after(wchar_t ch)
-{
-	if(no_break_after(ch))
-		return false;
-
-	return ch == ' ' ||
-		// CKJ characters
-		(ch >= 0x3000 && ch < 0xa000) ||
-		(ch >= 0xf900 && ch < 0xfb00) ||
-		(ch >= 0xff00 && ch < 0xfff0);
-}
-}
-
-std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height, int max_lines)
-{
-	assert(max_width > 0);
-
-	utils::utf8_iterator ch(unwrapped_text);
-	std::string current_word;
-	std::string current_line;
-	size_t line_width = 0;
-	size_t current_height = 0;
-	bool line_break = false;
-	bool first = true;
-	bool start_of_line = true;
-	std::string wrapped_text;
-	std::string format_string;
-	utils::utf8_iterator end = utils::utf8_iterator::end(unwrapped_text);
-
-	while(1) {
-		if(start_of_line) {
-			line_width = 0;
-			format_string = "";
-			while(ch != end && *ch < 0x100U && is_format_char(*ch)) {
-				format_string.append(ch.substr().first, ch.substr().second);
-				++ch;
-			}
-			current_line = format_string;
-			start_of_line = false;
-		}
-
-		// If there is no current word, get one
-		if(current_word.empty() && ch == end) {
-			break;
-		} else if(current_word.empty()) {
-			if(*ch == ' ' || *ch == '\n') {
-				current_word = *ch;
-				++ch;
-			} else {
-				wchar_t previous = 0;
-				for(;ch != utils::utf8_iterator::end(unwrapped_text) &&
-						*ch != ' ' && *ch != '\n'; ++ch) {
-
-					if(!current_word.empty() &&
-							break_before(*ch) &&
-							!no_break_after(previous))
-						break;
-
-					if(!current_word.empty() &&
-							break_after(previous) &&
-							!no_break_before(*ch))
-						break;
-
-					current_word.append(ch.substr().first, ch.substr().second);
-
-					previous = *ch;
-				}
-			}
-		}
-
-		if(current_word == "\n") {
-			line_break = true;
-			current_word = "";
-			start_of_line = true;
-		} else {
-
-			const std::string word = format_string + current_word;
-
-			const size_t word_width = line_size(word,font_size).w;
-
-			line_width += word_width;
-
-			if(line_width > max_width) {
-				if(word_width > max_width) {
-					cut_word(current_line, current_word, font_size, max_width);
-				}
-				if(current_word == " ")
-					current_word = "";
-				line_break = true;
-			} else {
-				current_line += current_word;
-				current_word = "";
-			}
-		}
-
-		if(line_break || current_word.empty() && ch == end) {
-			SDL_Rect size = line_size(current_line, font_size);
-			if(max_height > 0 && current_height + size.h >= size_t(max_height)) {
-				return wrapped_text;
-			}
-
-			if(!first) {
-				wrapped_text += '\n';
-			}
-
-			wrapped_text += current_line;
-			current_line = format_string;
-			line_width = 0;
-			current_height += size.h;
-			line_break = false;
-			first = false;
-
-			if(--max_lines == 0) {
-				return wrapped_text;
-			}
-		}
-	}
-	return wrapped_text;
-}
-
 std::string make_text_ellipsis(const std::string &text, int font_size, int max_width)
 {
 	static const std::string ellipsis = "...";
@@ -1122,14 +834,6 @@ std::string make_text_ellipsis(const std::string &text, int font_size, int max_w
 	return text; // Should not happen
 }
 
-SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
-		     const SDL_Color& colour, const std::string& text,
-		     int x, int y, int max_width)
-{
-	std::string wrapped_text = word_wrap_text(text, font_size, max_width);
-	return font::draw_text(gui, area, font_size, colour, wrapped_text, x, y, false);
-}
-
 }
 
 //floating labels
@@ -1547,50 +1251,5 @@ bool load_font_config()
 }
 
 
-size_t text_to_lines(std::string& message, size_t max_length)
-{
-	std::string starting_markup;
-	bool at_start = true;
-
-	size_t cur_line = 0, longest_line = 0;
-	for(std::string::iterator i = message.begin(); i != message.end(); ++i) {
-		if(at_start) {
-			if(font::is_format_char(*i)) {
-				push_back(starting_markup,*i);
-			} else {
-				at_start = false;
-			}
-		}
-
-		if(*i == '\n') {
-			at_start = true;
-			starting_markup = "";
-		}
-
-		if(*i == ' ' && cur_line > max_length) {
-			*i = '\n';
-			const size_t index = i - message.begin();
-			message.insert(index+1,starting_markup);
-			i = message.begin() + index + starting_markup.size();
-
-			if(cur_line > longest_line)
-				longest_line = cur_line;
-
-			cur_line = 0;
-		}
-
-		if(*i == '\n' || i+1 == message.end()) {
-			if(cur_line > longest_line)
-				longest_line = cur_line;
-
-			cur_line = 0;
-
-		} else {
-			++cur_line;
-		}
-	}
-
-	return longest_line;
-}
 
 }
