@@ -1,4 +1,4 @@
-/* $Id: game.cpp 8821 2005-11-12 19:23:56Z brunowolff $ */
+/* $Id: game.cpp 12798 2006-07-26 06:54:36Z isaac $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -16,52 +16,39 @@
 #include "SDL.h"
 
 #include "about.hpp"
-#include "actions.hpp"
-#include "ai_interface.hpp"
 #include "config.hpp"
 #include "cursor.hpp"
 #include "dialogs.hpp"
 #include "display.hpp"
-#include "events.hpp"
 #include "filesystem.hpp"
 #include "font.hpp"
 #include "game_config.hpp"
 #include "game_errors.hpp"
-#include "game_events.hpp"
 #include "gamestatus.hpp"
 #include "gettext.hpp"
+#include "help.hpp"
 #include "hotkeys.hpp"
 #include "intro.hpp"
-#include "key.hpp"
 #include "language.hpp"
-#include "log.hpp"
-#include "mapgen.hpp"
+#include "loadscreen.hpp"
 #include "multiplayer.hpp"
 #include "network.hpp"
-#include "pathfind.hpp"
 #include "playcampaign.hpp"
-#include "preferences.hpp"
+#include "preferences_display.hpp"
 #include "publish_campaign.hpp"
 #include "replay.hpp"
-#include "show_dialog.hpp"
 #include "sound.hpp"
 #include "statistics.hpp"
-#include "team.hpp"
 #include "thread.hpp"
 #include "titlescreen.hpp"
 #include "util.hpp"
-#include "unit_types.hpp"
-#include "unit.hpp"
-#include "video.hpp"
+#include "upload_log.hpp"
 #include "wassert.hpp"
 #include "wml_separators.hpp"
 #include "serialization/binary_or_text.hpp"
 #include "serialization/binary_wml.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include "serialization/string_utils.hpp"
-#include "widgets/button.hpp"
-#include "widgets/menu.hpp"
 
 #include "wesconfig.h"
 
@@ -103,12 +90,18 @@ public:
 
 	bool new_campaign();
 	bool play_multiplayer();
+	void download_campaigns();
 	bool change_language();
 
+	void show_help();
 	void show_preferences();
+	void show_upload_begging();
 
 	enum RELOAD_GAME_DATA { RELOAD_DATA, NO_RELOAD_DATA };
 	void play_game(RELOAD_GAME_DATA reload=RELOAD_DATA);
+	void play_replay();
+  const config& game_config(){return game_config_;};
+  const game_data& units_data(){return units_data_;};
 
 private:
 	game_controller(const game_controller&);
@@ -117,7 +110,6 @@ private:
 	void read_game_cfg(const preproc_map& defines, config& cfg, bool use_cache);
 	void refresh_game_cfg(bool reset_translations=false);
 
-	void download_campaigns();
 	void upload_campaign(const std::string& campaign, network::connection sock);
 	void delete_campaign(const std::string& campaign, network::connection sock);
 	void remove_campaign(const std::string& campaign);
@@ -138,6 +130,7 @@ private:
 	const image::manager image_manager_;
 	const events::event_context main_event_context_;
 	const hotkey::manager hotkey_manager_;
+	const upload_log::manager upload_log_manager_;
 	binary_paths_manager paths_manager_;
 
 	bool test_mode_, multiplayer_mode_, no_gui_;
@@ -192,6 +185,11 @@ game_controller::game_controller(int argc, char** argv)
 				++arg_;
 				force_bpp_ = lexical_cast_default<int>(argv_[arg_],-1);
 			}
+		} else if(val == "--load") {
+			if(arg_+1 != argc_) {
+				++arg_;
+				loaded_game_ = argv_[arg_];
+			}
 		} else if(val == "--nogui") {
 			no_gui_ = true;
 		} else if(val == "--windowed" || val == "-w") {
@@ -205,6 +203,9 @@ game_controller::game_controller(int argc, char** argv)
 			test_mode_ = true;
 		} else if(val == "--debug" || val == "-d") {
 			game_config::debug = true;
+			game_config::mp_debug = true;
+		} else if(val == "--no-delay") {
+			game_config::no_delay = true;
 		} else if (val.substr(0, 6) == "--log-") {
 		} else if(val == "--nosound") {
 			preferences::set_sound(false);
@@ -245,7 +246,7 @@ display& game_controller::disp()
 			throw CVideo::error();
 		}
 
-		static display::unit_map dummy_umap;
+		static unit_map dummy_umap;
 		static config dummy_cfg;
 		static gamemap dummy_map(dummy_cfg, "1");
 		static gamestatus dummy_status(dummy_cfg, 0);
@@ -265,6 +266,7 @@ bool game_controller::init_video()
 			return false;
 		}
 		video_.make_fake();
+		game_config::no_delay = true;
 		return true;
 	}
 
@@ -379,22 +381,18 @@ bool game_controller::init_config()
 	hotkey::load_hotkeys(game_config_);
 	paths_manager_.set_paths(game_config_);
 	::init_textdomains(game_config_);
+	about::set_about(game_config_);
 
 	return true;
 }
 
 bool game_controller::init_language()
 {
-	const bool lang_res = ::set_language(get_locale());
-	if(!lang_res) {
-		std::cerr << "No translation for locale '" << get_locale().language
-		          << "', default to system locale\n";
+	if(!::load_language_list())
+		return false;
 
-		const bool lang_res = ::set_language(known_languages[0]);
-		if(!lang_res) {
-			std::cerr << "Language data not found\n";
-		}
-	}
+	if (!::set_language(get_locale()))
+		return false;
 
 	if(!no_gui_) {
 		SDL_WM_SetCaption(_("The Battle for Wesnoth"), NULL);
@@ -421,7 +419,8 @@ bool game_controller::play_test()
 	state_.scenario = "test";
 
 	try {
-		::play_game(disp(),state_,game_config_,units_data_,video_);
+		upload_log nolog(false);
+		::play_game(disp(),state_,game_config_,units_data_,video_,nolog);
 	} catch(game::load_game_exception& e) {
 		loaded_game_ = e.game;
 		loaded_game_show_replay_ = e.show_replay;
@@ -441,9 +440,10 @@ bool game_controller::play_multiplayer_mode()
 	}
 
 	std::string era = "era_default";
-	std::string scenario = "multiplayer_test";
+	std::string scenario = "multiplayer_Charge";
 	std::map<int,std::string> side_types, side_controllers, side_algorithms;
 	std::map<int,string_map> side_parameters;
+	std::string turns = "50";
 
 	size_t sides_counted = 0;
 
@@ -473,6 +473,8 @@ bool game_controller::play_multiplayer_mode()
 
 			if(name == "--scenario") {
 				scenario = value;
+			} else if(name == "--turns") {
+				turns = value;
 			} else if(name == "--era") {
 				era = value;
 			} else if(last_digit && name_head == "--controller") {
@@ -493,6 +495,10 @@ bool game_controller::play_multiplayer_mode()
 				std::cerr << "unrecognized option: '" << name << "'\n";
 				return false;
 			}
+		} else {
+			if (val == "--exit-at-end") {
+				game_config::exit_at_end = true;
+			}
 		}
 	}
 
@@ -515,6 +521,8 @@ bool game_controller::play_multiplayer_mode()
 		return false;
 	}
 
+	level["turns"] = turns;
+
 	const config* const side = era_cfg->child("multiplayer_side");
 	if(side == NULL) {
 		std::cerr << "Could not find multiplayer side\n";
@@ -537,7 +545,6 @@ bool game_controller::play_multiplayer_mode()
 			era_cfg->find_child("multiplayer_side", "id", type->second);
 
 		if (side == NULL) {
-			unknown_side_id:
 			std::string side_name = (type == side_types.end() ? "default" : type->second);
 			std::cerr << "Could not find side '" << side_name << "' for side " << side_num << "\n";
 			return false;
@@ -545,11 +552,19 @@ bool game_controller::play_multiplayer_mode()
 
 		if ((*side)["random_faction"] == "yes") {
 			const config::child_list& eras = era_cfg->get_children("multiplayer_side");
-			for(int i = 0; i != 100 && (*side)["random_faction"] == "yes"; ++i) {
-				side = eras[rand()%eras.size()];
+			for(unsigned int i = 0, j = 0; i < eras.size(); ++i) {
+				if ((*eras[i])["random_faction"] != "yes") {
+					j++;
+					if (rand()%j == 0) {
+						side = eras[i];
+					}
+				}
+			}
+			if ((*side)["random_faction"] == "yes") {
+				std::string side_name = (type == side_types.end() ? "default" : type->second);
+				std::cerr << "Could not find any non-random faction for side " << side_num << "\n";
+				return false;
 			}
-			if ((*side)["random_faction"] == "yes")
-				goto unknown_side_id;
 		}
 
 		char buf[20];
@@ -558,6 +573,7 @@ bool game_controller::play_multiplayer_mode()
 
 		(*itors.first)->values["canrecruit"] = "1";
 
+
 		for(string_map::const_iterator i = side->values.begin(); i != side->values.end(); ++i) {
 			(*itors.first)->values[i->first] = i->second;
 		}
@@ -580,8 +596,9 @@ bool game_controller::play_multiplayer_mode()
 	}
 
 	try {
+		upload_log nolog(false);
 		state_.snapshot = level;
-		::play_game(disp(),state_,game_config_,units_data_,video_);
+		::play_game(disp(),state_,game_config_,units_data_,video_,nolog);
 		//play_level(units_data_,game_config_,&level,video_,state_,story);
 	} catch(game::error& e) {
 		std::cerr << "caught error: '" << e.message << "'\n";
@@ -667,7 +684,7 @@ bool game_controller::load_game()
 	}
 	recorder = replay(state_.replay_data);
 	recorder.start_replay();
-	recorder.set_skip(0);
+	recorder.set_skip(false);
 
 	std::cerr << "has snapshot: " << (state_.snapshot.child("side") ? "yes" : "no") << "\n";
 
@@ -679,7 +696,7 @@ bool game_controller::load_game()
 			std::cerr << "replaying (start of scenario)\n";
 		} else {
 			std::cerr << "skipping...\n";
-			recorder.set_skip(-1);
+			recorder.set_skip(false);
 		}
 	} else {
 		// We have a snapshot. But does the user want to see a replay?
@@ -746,17 +763,17 @@ bool game_controller::new_campaign()
 		campaign_desc.push_back(std::pair<std::string,std::string>(desc,image));
 	}
 
-	campaign_names.push_back(std::string(1, COLUMN_SEPARATOR) + _("Get More Campaigns..."));
-	campaign_desc.push_back(std::pair<std::string,std::string>(_("Download more campaigns from a server on Internet."),game_config::download_campaign_image));
-
 	int res = 0;
 
 	dialogs::campaign_preview_pane campaign_preview(disp().video(),&campaign_desc);
 	std::vector<gui::preview_pane*> preview_panes;
 	preview_panes.push_back(&campaign_preview);
 
-	wassert(campaign_names.size() > 0);
-	res = gui::show_dialog(disp(),NULL,_("Campaign"),
+	if(campaign_names.size() <= 0) {
+		return false;
+	}
+
+	res = gui::show_dialog2(disp(),NULL,_("Campaign"),
 			_("Choose the campaign you want to play:"),
 			gui::OK_CANCEL,&campaign_names,&preview_panes);
 
@@ -764,14 +781,9 @@ bool game_controller::new_campaign()
 		return false;
 	}
 
-	//get more campaigns from server
-	if(res == int(campaign_names.size()-1)) {
-		download_campaigns();
-		return new_campaign();
-	}
-
 	const config& campaign = *campaigns[res];
 
+	state_.campaign = campaign["id"];
 	state_.scenario = campaign["first_scenario"];
 
 	const std::string difficulty_descriptions = campaign["difficulty_descriptions"];
@@ -785,7 +797,7 @@ bool game_controller::new_campaign()
 			std::copy(difficulties.begin(),difficulties.end(),difficulty_options.begin());
 		}
 
-		const int res = gui::show_dialog(disp(),NULL,_("Difficulty"),
+		const int res = gui::show_dialog2(disp(),NULL,_("Difficulty"),
 		                            _("Select difficulty level:"),
 		                            gui::OK_CANCEL,&difficulty_options);
 		if(res == -1) {
@@ -845,7 +857,7 @@ void game_controller::download_campaigns()
 	std::string host = preferences::campaign_server();
 
 	const int res = gui::show_dialog(disp(),NULL,_("Connect to Server"),
-	        _("You will now connect to a campaign server to download campaigns."),
+	        _("You will now connect to a server to download add-ons."),
 	        gui::OK_CANCEL,NULL,NULL,_("Server: "),&host);
 	if(res != 0) {
 		return;
@@ -857,7 +869,7 @@ void game_controller::download_campaigns()
 
 	try {
 		const network::manager net_manager;
-		const network::connection sock = network::connect(items.front(),lexical_cast_default<int>(items.back(),15002));
+		const network::connection sock = network::connect(items.front(),lexical_cast_default<int>(items.back(),15003));
 		if(!sock) {
 			gui::show_error_message(disp(), _("Could not connect to host."));
 			return;
@@ -867,7 +879,7 @@ void game_controller::download_campaigns()
 		cfg.add_child("request_campaign_list");
 		network::send_data(cfg,sock);
 
-		network::connection res = gui::network_data_dialog(disp(),_("Awaiting response from server"),cfg,sock);
+		network::connection res = gui::network_receive_dialog(disp(),_("Asking for list of add-ons"),cfg,sock);
 		if(!res) {
 			return;
 		}
@@ -951,23 +963,23 @@ void game_controller::download_campaigns()
 		options.push_back(heading.str());
 
 		for(std::vector<std::string>::const_iterator j = publish_options.begin(); j != publish_options.end(); ++j) {
-			options.push_back(sep + _("Publish campaign: ") + *j);
+			options.push_back(sep + _("Publish add-on: ") + *j);
 		}
 
 		for(std::vector<std::string>::const_iterator d = delete_options.begin(); d != delete_options.end(); ++d) {
-			options.push_back(sep + _("Delete campaign: ") + *d);
+			options.push_back(sep + _("Delete add-on: ") + *d);
 		}
 
 		if(campaigns.empty() && publish_options.empty()) {
-			gui::show_error_message(disp(), _("There are no campaigns available for download from this server."));
+			gui::show_error_message(disp(), _("There are no add-ons available for download from this server."));
 			return;
 		}
 
 		gui::menu::basic_sorter sorter;
 		sorter.set_alpha_sort(1).set_alpha_sort(2).set_alpha_sort(3).set_numeric_sort(4).set_position_sort(5,sizes);
 
-		const int index = gui::show_dialog(disp(),NULL,_("Get Campaign"),_("Choose the campaign to download."),gui::OK_CANCEL,&options,
-			                               NULL,"",NULL,0,NULL,NULL,-1,-1,NULL,NULL,"",&sorter);
+		const int index = gui::show_dialog2(disp(),NULL,_("Get Add-ons"),_("Choose the add-on to download."),gui::OK_CANCEL,&options,
+			NULL, "", NULL, 256, NULL, NULL, -1, -1, NULL, NULL, "", &sorter);
 		if(index < 0) {
 			return;
 		}
@@ -986,7 +998,7 @@ void game_controller::download_campaigns()
 		request.add_child("request_campaign")["name"] = campaigns[index];
 		network::send_data(request,sock);
 
-		res = gui::network_data_dialog(disp(),_("Downloading campaign..."),cfg,sock);
+		res = gui::network_receive_dialog(disp(),_("Downloading add-on..."),cfg,sock);
 		if(!res) {
 			return;
 		}
@@ -997,7 +1009,7 @@ void game_controller::download_campaigns()
 		}
 
 		if(!check_names_legal(cfg)) {
-			gui::show_error_message(disp(), "The campaign has an invalid file or directory name and can not be installed.");
+			gui::show_error_message(disp(), "The add-on has an invalid file or directory name and can not be installed.");
 			return;
 		}
 
@@ -1025,7 +1037,7 @@ void game_controller::download_campaigns()
 
 		clear_binary_paths_cache();
 
-		gui::show_dialog(disp(),NULL,_("Campaign Installed"),_("The campaign has been installed."),gui::OK_ONLY);
+		gui::show_dialog(disp(),NULL,_("Add-on Installed"),_("The add-on has been installed."),gui::OK_ONLY);
 	} catch(config::error&) {
 		gui::show_error_message(disp(), _("Network communication error."));
 	} catch(network::error&) {
@@ -1080,7 +1092,7 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 	std::cerr << "uploading campaign...\n";
 	network::send_data(data,sock);
 
-	sock = gui::network_data_dialog(disp(),_("Awaiting response from server"),data,sock);
+	sock = gui::network_send_dialog(disp(),_("Sending add-on"),data,sock);
 	if(!sock) {
 		gui::show_error_message(disp(), _("Connection timed out"));
 	} else if(data.child("error")) {
@@ -1151,12 +1163,19 @@ bool game_controller::play_multiplayer()
 
 	std::string login = preferences::login();
 
-	int res = gui::show_dialog(disp(), NULL, _("Multiplayer"), "",
-	                           gui::OK_CANCEL, &host_or_join, NULL,
-	                           _("Login: "), &login);
+	int res;
+	do {
+		res = gui::show_dialog2(disp(), NULL, _("Multiplayer"), "",
+					   gui::OK_CANCEL, &host_or_join, NULL,
+					   _("Login: "), &login);
+		if(login.size() > 18) {
+			gui::show_error_message(disp(), _("The login name you chose is too long, please use a login with less than 18 characters"));
+		}
+	}while(login.size() > 18) ;
 	if (res < 0)
 		return false;
 
+
 	preferences::set_login(login);
 
 	try {
@@ -1210,26 +1229,28 @@ bool game_controller::play_multiplayer()
 
 bool game_controller::change_language()
 {
-	std::vector<language_def> langdefs = get_languages();
+	const std::vector<language_def>& languages = get_languages();
+	std::vector<std::string> langs;
 
-	// this only works because get_languages() returns a fresh vector at each calls
-	// unless show_gui cleans the "*" flag
-	const std::vector<language_def>::iterator current = std::find(langdefs.begin(),langdefs.end(),get_language());
-	if(current != langdefs.end()) {
-		(*current).language = "*" + (*current).language;
+	for (std::vector<language_def>::const_iterator itor = languages.begin();
+			itor != languages.end(); ++itor) {
+		if (*itor == get_language()) {
+			langs.push_back("*" + itor->language);
+		} else {
+			langs.push_back(itor->language);
+		}
 	}
 
-	// prepare a copy with just the labels for the list to be displayed
-	std::vector<std::string> langs;
-	langs.reserve(langdefs.size());
-	std::transform(langdefs.begin(),langdefs.end(),std::back_inserter(langs),languagedef_name);
-
-	const int res = gui::show_dialog(disp(),NULL,_("Language"),
+	const int res = gui::show_dialog2(disp(),NULL,_("Language"),
 	                         _("Choose your preferred language:"),
 	                         gui::OK_CANCEL,&langs);
 	if(size_t(res) < langs.size()) {
-		::set_language(known_languages[res]);
-		preferences::set_language(known_languages[res].localename);
+		::set_language(languages[res]);
+		preferences::set_language(languages[res].localename);
+
+		if(!no_gui_) {
+			SDL_WM_SetCaption(_("The Battle for Wesnoth"), NULL);
+		}
 
 		refresh_game_cfg(true);
 	}
@@ -1248,6 +1269,13 @@ void game_controller::show_preferences()
 	disp().redraw_everything();
 }
 
+void game_controller::show_upload_begging()
+{
+	upload_log_dialog::show_beg_dialog(disp());
+
+	disp().redraw_everything();
+}
+
 //this function reads the game configuration, searching for valid cached copies first
 void game_controller::read_game_cfg(const preproc_map& defines, config& cfg, bool use_cache)
 {
@@ -1312,6 +1340,11 @@ void game_controller::read_game_cfg(const preproc_map& defines, config& cfg, boo
 				//read the file and then write to the cache
 				scoped_istream stream = preprocess_file("data/game.cfg", &defines_map);
 
+				//reset the parse counter before reading the game files
+				if (loadscreen::global_loadscreen) {
+					loadscreen::global_loadscreen->parser_counter = 0;
+				}
+
 				std::string error_log, user_error_log;
 
 				read(cfg, *stream, &error_log);
@@ -1366,6 +1399,11 @@ void game_controller::read_game_cfg(const preproc_map& defines, config& cfg, boo
 
 				cfg.merge_children("units");
 
+				config& hashes = cfg.add_child("multiplayer_hashes");
+				for(config::child_list::const_iterator ch = cfg.get_children("multiplayer").begin(); ch != cfg.get_children("multiplayer").end(); ++ch) {
+					hashes[(**ch)["id"]] = (*ch)->hash();
+				}
+
 				if(!error_log.empty()) {
 					gui::show_error_message(disp(),
 							_("Warning: Errors occurred while loading game configuration files: '") +
@@ -1411,8 +1449,7 @@ void game_controller::refresh_game_cfg(bool reset_translations)
 
 			const config* const units = game_config_.child("units");
 			if(units != NULL) {
-				const bool allow_advancefrom = defines_map_.find("MULTIPLAYER") == defines_map_.end();
-				units_data_.set_config(*units,allow_advancefrom);
+				units_data_.set_config(*units);
 			}
 
 			old_defines_map_ = defines_map_;
@@ -1461,10 +1498,17 @@ void game_controller::play_game(RELOAD_GAME_DATA reload)
 	const binary_paths_manager bin_paths_manager(game_config_);
 
 	try {
-		const LEVEL_RESULT result = ::play_game(disp(),state_,game_config_,units_data_,video_);
-		if(result == VICTORY) {
+		// Only record log for single-player games & tutorial.
+		upload_log log(state_.campaign_type.empty()
+					   || state_.campaign_type == "scenario"
+					   || state_.campaign_type == "tutorial");
+
+		const LEVEL_RESULT result = ::play_game(disp(),state_,game_config_,units_data_,video_, log);
+		// don't show The End for multiplayer scenario
+		// change this if MP campaigns are implemented
+		if(result == VICTORY && (state_.campaign_type.empty() || state_.campaign_type != "multiplayer")) {
 			the_end(disp());
-			about::show_about(disp());
+			about::show_about(disp(),state_.campaign);
 		}
 	} catch(game::load_game_exception& e) {
 
@@ -1476,6 +1520,21 @@ void game_controller::play_game(RELOAD_GAME_DATA reload)
 
 } //end anon namespace
 
+void game_controller::play_replay()
+{
+	const binary_paths_manager bin_paths_manager(game_config_);
+
+	try {
+		::play_replay(disp(),state_,game_config_,units_data_,video_);
+
+	} catch(game::load_game_exception& e) {
+
+		//this will make it so next time through the title screen loop, this game is loaded
+		loaded_game_ = e.game;
+		loaded_game_show_replay_ = e.show_replay;
+	}
+}
+
 int play_game(int argc, char** argv)
 {
 	const int start_ticks = SDL_GetTicks();
@@ -1490,23 +1549,46 @@ int play_game(int argc, char** argv)
 
 		if(val == "--help" || val == "-h") {
 			std::cout << "usage: " << argv[0]
-		    << " [options] [data-directory]\n"
-			<< "  -d, --debug       Shows debugging information in-game\n"
-			<< "  -f, --fullscreen  Runs the game in full-screen\n"
-			<< "  -h, --help        Prints this message and exits\n"
-			<< "  --path            Prints the name of the game data directory and exits\n"
-			<< "  -t, --test        Runs the game in a small example scenario\n"
-			<< "  -w, --windowed    Runs the game in windowed mode\n"
-			<< "  -v, --version     Prints the game's version number and exits\n"
+			<< " [OPTIONS] [DATA-DIRECTORY]\n"
+			<< "  --bpp number                 sets BitsPerPixel value. Example: --bpp 32\n"
+			<< "  --compress INFILE OUTFILE    compresses a savefile (INFILE) that is in text WML\n"
+			<< "                               format into binary WML format (OUTFILE).\n"
+			<< "  -d, --debug                  shows extra debugging information and enables\n"
+			<< "                               additional command mode options in-game.\n"
+			<< "  --decompress INFILE OUTFILE  decompresses a savefile (INFILE) that is in binary\n"
+			<< "                               WML format into text WML format (OUTFILE).\n"
+			<< "  -f, --fullscreen             runs the game in full screen mode.\n"
+			<< "  --fps                        displays the number of frames per second the game\n"
+			<< "                               is currently running at, in a corner of the screen.\n"
+			<< "  -h, --help                   prints this message and exits.\n"
+			<< "  --load SAVEGAME              loads the file SAVEGAME from the standard save\n"
+			<< "                               game directory.\n"
 			<< "  --log-error=\"domain1,domain2,...\", --log-warning=..., --log-info=...\n"
-			<< "                    Set the severity level of the debug domains\n"
-			<< "                    \"all\" can be used to match any debug domain\n"
-			<< "  --nocache         Disables caching of game data\n"
-			<< "  --nosound         Disables sounds\n"
-			<< "  --compress file1 file2 Compresses the text-WML file file1 into the\n"
-			<< "                    binary-WML file file2\n"
-			<< "  --decompress file1 file2 Uncompresses the binary-WML file file2 into\n"
-			<< "                    the text-WML file file2\n"
+			<< "                               sets the severity level of the debug domains.\n"
+			<< "                               \"all\" can be used to match any debug domain.\n"
+			<< "  --nocache                    disables caching of game data.\n"
+			<< "  --nosound                    runs the game without sounds and music.\n"
+			<< "  --path                       prints the name of the game data directory and exits.\n"
+			<< "  -r, --resolution XxY         sets the screen resolution. Example: -r 800x600\n"
+			<< "  -t, --test                   runs the game in a small test scenario.\n"
+			<< "  -v, --version                prints the game's version number and exits.\n"
+			<< "  -w, --windowed               runs the game in windowed mode.\n"
+			<< "  --no-delay                   run the game without any delays.\n"
+			<< "  --multiplayer                runs a multiplayer game. There are additional\n"
+			<< "                               options that can be used as explained below:\n"
+			<< "  --algorithm<number>=value    selects a non-standard algorithm to be used by the\n"
+			<< "                               AI controller for this side.\n"
+			<< "  --controller<number>=value   selects the controller for this side.\n"
+			<< "  --era=value                  selects the era to be played in by its id.\n"
+			<< "  --nogui                      runs the game without the GUI. Must appear before\n"
+			<< "                               --multiplayer to have the desired effect.\n"
+			<< "  --parm<number>=name:value    sets additional parameters for this side.\n"
+			<< "  --scenario=value             selects a multiplayer scenario. The default\n"
+			<< "                               scenario is \"multiplayer_Charge\".\n"
+			<< "  --side<number>=value         selects a faction of the current era for this side\n"
+			<< "                               by id.\n"
+			<< "  --turns=value                sets the number of turns. The default is \"50\".\n"
+			<< "  --exit-at-end                exit Wesnoth at end of scenario.\n"
 			;
 			return 0;
 		} else if(val == "--version" || val == "-v") {
@@ -1621,13 +1703,8 @@ int play_game(int argc, char** argv)
 		return 0;
 	}
 
-#ifdef WIN32
-	res = game.init_config();
-	if(res == false) {
-		std::cerr << "could not initialize game config\n";
-		return 0;
-	}
-#endif
+	loadscreen::global_loadscreen = new loadscreen(game.disp().video());
+	loadscreen::global_loadscreen->clear_screen();
 
 	res = game.init_language();
 	if(res == false) {
@@ -1635,21 +1712,19 @@ int play_game(int argc, char** argv)
 		return 0;
 	}
 
-	res = font::load_font_config();
-	if(res == false) {
-		std::cerr << "could not re-initialize fonts for the current language\n";
-		return 0;
-	}
-
-#ifndef WIN32
-	// it is better for gettext-native platforms to read the config
-	// files after having pre-initialized the language, maybe...
+	loadscreen::global_loadscreen->increment_progress(5, "Loading game configuration.");
 	res = game.init_config();
 	if(res == false) {
 		std::cerr << "could not initialize game config\n";
 		return 0;
 	}
-#endif
+	loadscreen::global_loadscreen->increment_progress(10, "Re-initialize fonts for the current language.");
+
+	res = font::load_font_config();
+	if(res == false) {
+		std::cerr << "could not re-initialize fonts for the current language\n";
+		return 0;
+	}
 
 	const cursor::manager cursor_manager;
 #if defined(_X11) && !defined(__APPLE__)
@@ -1659,13 +1734,17 @@ int play_game(int argc, char** argv)
 	int ntip = -1;
 	config tips_of_day;
 
+	loadscreen::global_loadscreen->set_progress(100, _("Loading title screen."));
+	delete loadscreen::global_loadscreen;
+	loadscreen::global_loadscreen = 0;
+
 	for(;;) {
 		//make sure the game config is always set to how it should be at the title screen
 		game.reset_game_cfg();
 
 		statistics::fresh_stats();
 
-		sound::play_music(game_config::title_music);
+		sound::play_music_repeatedly(game_config::title_music);
 
 		std::cerr << "started music\n";
 		std::cerr << (SDL_GetTicks() - start_ticks) << "\n";
@@ -1721,9 +1800,26 @@ int play_game(int argc, char** argv)
 		} else if(res == gui::SHOW_ABOUT) {
 			about::show_about(game.disp());
 			continue;
+		} else if(res == gui::SHOW_HELP) {
+			config dummy_help_cfg;
+			gamemap dummy_help_map(dummy_help_cfg, "1");
+			help::help_manager help_manager(&game.game_config(), &game.units_data(), &dummy_help_map);
+			help::show_help(game.disp());
+			continue;
+		} else if(res == gui::GET_ADDONS) {
+			game.download_campaigns();
+			continue;
+		} else if(res == gui::BEG_FOR_UPLOAD) {
+			game.show_upload_begging();
+			continue;
 		}
 
-		game.play_game(should_reload);
+		if (recorder.at_end()){
+			game.play_game(should_reload);
+		}
+		else{
+			game.play_replay();
+		}
 		ntip = -1; // Change tip when a game is played
 	}
 
@@ -1732,6 +1828,12 @@ int play_game(int argc, char** argv)
 
 int main(int argc, char** argv)
 {
+#ifdef OS2 /* required for SDL_GetTicks to work on OS/2 */
+        if ( SDL_Init(SDL_INIT_TIMER) < 0 ) {
+		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
+		return(1);
+	}
+#endif
 	try {
 		std::cerr << "Battle for Wesnoth v" << VERSION << "\n";
 		time_t t = time(NULL);
@@ -1740,6 +1842,9 @@ int main(int argc, char** argv)
 		std::cerr << "started game: " << SDL_GetTicks() << "\n";
 		const int res = play_game(argc,argv);
 		std::cerr << "exiting with code " << res << "\n";
+#ifdef OS2 /* required to correctly shutdown SDL on OS/2 */
+	        SDL_Quit();
+#endif
 		return res;
 	} catch(CVideo::error&) {
 		std::cerr << "Could not initialize video. Exiting.\n";
