@@ -1,6 +1,6 @@
-/* $Id: game_events.hpp,v 1.28 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: game_events.hpp 11673 2006-05-28 05:21:57Z rusty $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,6 @@ class display;
 //movement, keyboard events, etc. See events.hpp for how they are handled.
 namespace game_events
 {
-
 //the game event manager loads the scenario configuration object, and
 //ensures that events are handled according to the scenario configuration
 //for its lifetime.
@@ -48,7 +47,7 @@ struct manager {
 	//note that references will be maintained, and must remain valid
 	//for the life of the object.
 	manager(const config& scenario_cfg, display& disp, gamemap& map,
-			std::map<gamemap::location,unit>& units, std::vector<team>& teams,
+			unit_map& units, std::vector<team>& teams,
 			game_state& state_of_game, gamestatus& status, const game_data& data);
 	~manager();
 
@@ -57,17 +56,23 @@ struct manager {
 
 void write_events(config& cfg);
 
-bool unit_matches_filter(const unit& u, const vconfig filter);
+bool matches_special_filter(const config* cfg, const vconfig filter);
+bool unit_matches_filter(const unit& u, const vconfig filter,const gamemap::location& loc);
 bool unit_matches_filter(unit_map::const_iterator itor, const vconfig filter);
-bool unit_matches_filter(const unit& u, const vconfig filter);
 
 //function to fire an event. Events may have up to two arguments, both of
 //which must be locations.
 bool fire(const std::string& event,
           const gamemap::location& loc1=gamemap::location::null_location,
-          const gamemap::location& loc2=gamemap::location::null_location);
+          const gamemap::location& loc2=gamemap::location::null_location,
+		  const config& data=config());
 
-bool conditional_passed(const std::map<gamemap::location,unit>* units,
+void raise(const std::string& event,
+          const gamemap::location& loc1=gamemap::location::null_location,
+          const gamemap::location& loc2=gamemap::location::null_location,
+		  const config& data=config());
+
+bool conditional_passed(const unit_map* units,
                         const vconfig cond);
 bool pump();
 
