@@ -1,4 +1,4 @@
-/* $Id: gamestatus.cpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: gamestatus.cpp 12768 2006-07-25 16:32:50Z jhinrichs $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -49,8 +49,9 @@ player_info* game_state::get_player(const std::string& id) {
 
 time_of_day::time_of_day(const config& cfg)
                  : lawful_bonus(atoi(cfg["lawful_bonus"].c_str())),
+                   bonus_modified(0),
                    image(cfg["image"]), name(cfg["name"]), id(cfg["id"]),
-		   image_mask(cfg["mask"]),
+			       image_mask(cfg["mask"]),
                    red(atoi(cfg["red"].c_str())),
                    green(atoi(cfg["green"].c_str())),
                    blue(atoi(cfg["blue"].c_str()))
@@ -60,16 +61,16 @@ time_of_day::time_of_day(const config& cfg)
 void time_of_day::write(config& cfg) const
 {
 	char buf[50];
-	sprintf(buf,"%d",lawful_bonus);
+	snprintf(buf,sizeof(buf),"%d",lawful_bonus);
 	cfg["lawful_bonus"] = buf;
 
-	sprintf(buf,"%d",red);
+	snprintf(buf,sizeof(buf),"%d",red);
 	cfg["red"] = buf;
 
-	sprintf(buf,"%d",green);
+	snprintf(buf,sizeof(buf),"%d",green);
 	cfg["green"] = buf;
 
-	sprintf(buf,"%d",blue);
+	snprintf(buf,sizeof(buf),"%d",blue);
 	cfg["blue"] = buf;
 
 	cfg["image"] = image;
@@ -80,7 +81,7 @@ void time_of_day::write(config& cfg) const
 
 namespace {
 
-void parse_times(const config& cfg, std::vector<time_of_day>& normal_times, std::vector<time_of_day>& illuminated_times)
+void parse_times(const config& cfg, std::vector<time_of_day>& normal_times)
 {
 	const config::child_list& times = cfg.get_children("time");
 	config::child_list::const_iterator t;
@@ -88,12 +89,6 @@ void parse_times(const config& cfg, std::vector<time_of_day>& normal_times, std:
 		normal_times.push_back(time_of_day(**t));
 	}
 
-	const config::child_list& times_illum = cfg.get_children("illuminated_time");
-	const config::child_list& illum = times_illum.empty() ? times : times_illum;
-
-	for(t = illum.begin(); t != illum.end(); ++t) {
-		illuminated_times.push_back(time_of_day(**t));
-	}
 }
 
 }
@@ -106,7 +101,7 @@ gamestatus::gamestatus(const config& time_cfg, int num_turns) :
 		turn_ = atoi(turn_at.c_str());
 	}
 
-	parse_times(time_cfg,times_,illuminatedTimes_);
+	parse_times(time_cfg,times_);
 
 	const config::child_list& times_range = time_cfg.get_children("time_area");
 	for(config::child_list::const_iterator t = times_range.begin(); t != times_range.end(); ++t) {
@@ -115,8 +110,7 @@ gamestatus::gamestatus(const config& time_cfg, int num_turns) :
 		area.xsrc = (**t)["x"];
 		area.ysrc = (**t)["y"];
 		std::copy(locs.begin(),locs.end(),std::inserter(area.hexes,area.hexes.end()));
-		parse_times(**t,area.times,area.illuminated_times);
-
+		parse_times(**t,area.times);
 		areas_.push_back(area);
 	}
 }
@@ -135,9 +129,6 @@ void gamestatus::write(config& cfg) const
 		t->write(cfg.add_child("time"));
 	}
 
-	for(t = illuminatedTimes_.begin(); t != illuminatedTimes_.end(); ++t) {
-		t->write(cfg.add_child("illuminated_time"));
-	}
 
 	for(std::vector<area_time_of_day>::const_iterator i = areas_.begin(); i != areas_.end(); ++i) {
 		config& area = cfg.add_child("time_area");
@@ -147,13 +138,10 @@ void gamestatus::write(config& cfg) const
 			t->write(area.add_child("time"));
 		}
 
-		for(t = i->illuminated_times.begin(); t != i->illuminated_times.end(); ++t) {
-			t->write(area.add_child("illuminated_time"));
-		}
 	}
 }
 
-const time_of_day& gamestatus::get_time_of_day_turn(int nturn) const
+time_of_day gamestatus::get_time_of_day_turn(int nturn) const
 {
 	if(times_.empty() == false) {
 		return times_[(nturn-1)%times_.size()];
@@ -164,37 +152,52 @@ const time_of_day& gamestatus::get_time_of_day_turn(int nturn) const
 	}
 }
 
-const time_of_day& gamestatus::get_time_of_day() const
+time_of_day gamestatus::get_time_of_day() const
 {
 	return get_time_of_day_turn(turn());
 }
 
-const time_of_day& gamestatus::get_previous_time_of_day() const
+time_of_day gamestatus::get_previous_time_of_day() const
 {
 	return get_time_of_day_turn(turn()-1);
 }
 
-const time_of_day& gamestatus::get_time_of_day(bool illuminated, const gamemap::location& loc) const
+
+time_of_day gamestatus::get_time_of_day(int illuminated, const gamemap::location& loc, int n_turn) const
 {
 	for(std::vector<area_time_of_day>::const_iterator i = areas_.begin(); i != areas_.end(); ++i) {
 		if(i->hexes.count(loc) == 1) {
-			if(illuminated && i->illuminated_times.empty() == false) {
-				return i->illuminated_times[(turn()-1)%i->illuminated_times.size()];
-			} else if(i->times.empty() == false) {
-				return i->times[(turn()-1)%i->times.size()];
+			if (i->times.empty()) { // prevent division by zero if no time is available
+				config dummy_cfg;
+				static time_of_day const default_time(dummy_cfg);
+				return default_time;
 			}
+			time_of_day res = i->times[(n_turn-1)%i->times.size()];
+			if(illuminated) {
+				res.bonus_modified=illuminated;
+				res.lawful_bonus += illuminated;
+			}
+			return res;
 		}
 	}
 
-	if(illuminated && illuminatedTimes_.empty() == false) {
-		return illuminatedTimes_[(turn()-1)%illuminatedTimes_.size()];
-	} else if(times_.empty() == false) {
-		return times_[(turn()-1)%times_.size()];
+	if (times_.empty()) { // prevent division by zero if no time is available
+	    config dummy_cfg;
+	    static time_of_day const default_time(dummy_cfg);
+	    return default_time;
 	}
 
-	config dummy_cfg;
-	static time_of_day const default_time(dummy_cfg);
-	return default_time;
+	time_of_day res = times_[(n_turn-1)%times_.size()];
+	if(illuminated) {
+		res.bonus_modified=illuminated;
+		res.lawful_bonus += illuminated;
+	}
+	return res;
+}
+
+time_of_day gamestatus::get_time_of_day(int illuminated, const gamemap::location& loc) const
+{
+	return get_time_of_day(illuminated,loc,turn());
 }
 
 size_t gamestatus::turn() const
@@ -206,6 +209,15 @@ int gamestatus::number_of_turns() const
 {
 	return numTurns_;
 }
+void gamestatus::modify_turns(const std::string& mod)
+{
+	numTurns_ = maximum<int>(utils::apply_modifier(numTurns_,mod,0),-1);
+}
+void gamestatus::add_turns(int num)
+{
+	numTurns_ = maximum<int>(numTurns_ + num,-1);
+}
+
 
 bool gamestatus::next_turn()
 {
@@ -242,6 +254,7 @@ game_state read_game(const game_data& data, const config* cfg)
 	res.label = (*cfg)["label"];
 	res.version = (*cfg)["version"];
 	res.scenario = (*cfg)["scenario"];
+	res.campaign = (*cfg)["campaign"];
 
 	const config::child_list& players = cfg->get_children("player");
 
@@ -323,7 +336,7 @@ game_state read_game(const game_data& data, const config* cfg)
 void write_player(const player_info& player, config& cfg)
 {
 	char buf[50];
-	sprintf(buf,"%d",player.gold);
+	snprintf(buf,sizeof(buf),"%d",player.gold);
 
 	cfg["gold"] = buf;
 
@@ -346,7 +359,7 @@ void write_player(const player_info& player, config& cfg)
 	cfg["can_recruit"] = can_recruit_str;
 }
 
-void write_game(const game_state& game, config& cfg, WRITE_GAME_MODE mode)
+void write_game(const game_state& game, config& cfg/*, WRITE_GAME_MODE mode*/)
 {
 	log_scope("write_game");
 	cfg["label"] = game.label;
@@ -354,6 +367,8 @@ void write_game(const game_state& game, config& cfg, WRITE_GAME_MODE mode)
 
 	cfg["scenario"] = game.scenario;
 
+	cfg["campaign"] = game.campaign;
+
 	cfg["campaign_type"] = game.campaign_type;
 
 	cfg["difficulty"] = game.difficulty;
@@ -370,7 +385,7 @@ void write_game(const game_state& game, config& cfg, WRITE_GAME_MODE mode)
 		cfg.add_child("player", new_cfg);
 	}
 
-	if(mode == WRITE_FULL_GAME) {
+//	if(mode == WRITE_FULL_GAME) {
 		if(game.replay_data.child("replay") == NULL) {
 			cfg.add_child("replay",game.replay_data);
 		}
@@ -379,7 +394,7 @@ void write_game(const game_state& game, config& cfg, WRITE_GAME_MODE mode)
 		cfg.add_child("replay_start",game.starting_pos);
 		cfg.add_child("statistics",statistics::write_stats());
 	}
-}
+//}
 
 //a structure for comparing to save_info objects based on their modified time.
 //if the times are equal, will order based on the name
@@ -390,9 +405,9 @@ struct save_info_less_time {
 	}
 };
 
-std::vector<save_info> get_saves_list()
+std::vector<save_info> get_saves_list(const std::string *dir)
 {
-	const std::string& saves_dir = get_saves_dir();
+	const std::string& saves_dir = (dir) ? *dir : get_saves_dir();
 
 	std::vector<std::string> saves;
 	get_files_in_dir(saves_dir,&saves);
@@ -456,6 +471,15 @@ void load_game(const game_data& data, const std::string& name, game_state& state
 	state = read_game(data,&cfg);
 }
 
+void load_game_summary(const std::string& name, config& cfg_summary, std::string* error_log){
+	log_scope("load_game_summary");
+
+	config cfg;
+	read_save_file(name,cfg,error_log);
+
+	extract_summary_from_config(cfg, cfg_summary);
+}
+
 //throws game::save_game_failed
 void save_game(const game_state& state)
 {
@@ -555,6 +579,7 @@ void extract_summary_data_from_save(const game_state& state, config& out)
 	out["replay"] = has_replay ? "yes" : "no";
 	out["snapshot"] = has_snapshot ? "yes" : "no";
 
+	out["label"] = state.label;
 	out["campaign_type"] = state.campaign_type;
 	out["scenario"] = state.scenario;
 	out["difficulty"] = state.difficulty;
@@ -578,7 +603,7 @@ void extract_summary_data_from_save(const game_state& state, config& out)
 	    p!=state.players.end(); ++p) {
 		for(std::vector<unit>::const_iterator u = p->second.available_units.begin(); u != p->second.available_units.end(); ++u) {
 			if(u->can_recruit()) {
-				leader = u->type().id();
+				leader = u->id();
 			}
 		}
 	}
@@ -601,7 +626,7 @@ void extract_summary_data_from_save(const game_state& state, config& out)
 			const config::child_list& units = (**s).get_children("unit");
 			for(config::child_list::const_iterator u = units.begin(); u != units.end(); ++u) {
 				if((**u)["canrecruit"] == "1") {
-					leader = (**u)["type"];
+					leader = (**u)["id"];
 					break;
 				}
 			}
@@ -624,6 +649,88 @@ void extract_summary_data_from_save(const game_state& state, config& out)
 	}
 }
 
+void extract_summary_from_config(config& cfg_save, config& cfg_summary)
+{
+	const config* cfg_snapshot = cfg_save.child("snapshot");
+	const config* cfg_replay_start = cfg_save.child("replay_start");
+
+	const bool has_replay = cfg_save.child("replay") != NULL;
+	const bool has_snapshot = (cfg_snapshot != NULL) && (cfg_snapshot->child("side") != NULL);
+
+	cfg_summary["replay"] = has_replay ? "yes" : "no";
+	cfg_summary["snapshot"] = has_snapshot ? "yes" : "no";
+
+	cfg_summary["label"] = cfg_save["label"];
+	cfg_summary["campaign_type"] = cfg_save["campaign_type"];
+	cfg_summary["scenario"] = cfg_save["scenario"];
+	cfg_summary["difficulty"] = cfg_save["difficulty"];
+	cfg_summary["version"] = cfg_save["version"];
+	cfg_summary["corrupt"] = "";
+
+	if(has_snapshot) {
+		cfg_summary["turn"] = (*cfg_snapshot)["turn_at"];
+		if((*cfg_snapshot)["turns"] != "-1") {
+			cfg_summary["turn"] = cfg_summary["turn"].str() + "/" + (*cfg_snapshot)["turns"].str();
+		}
+	}
+
+	//find the first human leader so we can display their icon in the load menu
+
+	//ideally we should grab all leaders if there's more than 1
+	//human player?
+	std::string leader;
+
+	const config::child_list& players = cfg_save.get_children("player");
+
+	for(config::child_list::const_iterator i = cfg_save.get_children("player").begin(); i != cfg_save.get_children("player").end(); ++i) {
+		if ((**i)["canrecruit"] == "1"){
+			leader = (**i)["save_id"];
+		}
+	}
+
+	bool shrouded = false;
+
+	if(leader == "") {
+		const config* snapshot = has_snapshot ? cfg_snapshot : cfg_replay_start;
+		if (snapshot != NULL){
+			const config::child_list& sides = snapshot->get_children("side");
+			for(config::child_list::const_iterator s = sides.begin(); s != sides.end() && leader.empty(); ++s) {
+
+				if((**s)["controller"] != "human") {
+					continue;
+				}
+
+				if((**s)["shroud"] == "yes") {
+					shrouded = true;
+				}
+
+				const config::child_list& units = (**s).get_children("unit");
+				for(config::child_list::const_iterator u = units.begin(); u != units.end(); ++u) {
+					if((**u)["canrecruit"] == "1") {
+						leader = (**u)["id"];
+						break;
+					}
+				}
+			}
+		}
+	}
+
+	cfg_summary["leader"] = leader;
+	cfg_summary["map_data"] = "";
+
+	if(!shrouded) {
+		if(has_snapshot) {
+			if(cfg_snapshot->find_child("side","shroud","yes") == NULL) {
+				cfg_summary["map_data"] = (*cfg_snapshot)["map_data"];
+			}
+		} else if(has_replay) {
+			if(cfg_replay_start->find_child("side","shroud","yes") == NULL) {
+				cfg_summary["map_data"] = (*cfg_replay_start)["map_data"];
+			}
+		}
+	}
+}
+
 namespace {
 const size_t MaxLoop = 1024;
 }
@@ -636,7 +743,7 @@ void game_state::get_variable_internal(const std::string& key, config& cfg,
 	const std::string::const_iterator itor = std::find(key.begin(),key.end(),'.');
 	if(itor != key.end()) {
 		std::string element(key.begin(),itor);
-		const std::string sub_key(itor+1,key.end());
+		std::string sub_key(itor+1,key.end());
 
 		size_t index = 0;
 		const std::string::iterator index_start = std::find(element.begin(),element.end(),'[');
@@ -727,4 +834,70 @@ void game_state::set_variable(const std::string& key, const t_string& value)
 	variables[key] = value;
 }
 
+void game_state::clear_variable(const std::string& varname)
+{
+	config* vars = &variables;
+	std::string key(varname);
+	std::string::const_iterator itor = std::find(key.begin(),key.end(),'.');
+	int dot_index = key.find('.');
+	// "mover.modifications.trait[0]"
+	while(itor != key.end()) { // subvar access
+		std::string element=key.substr(0,dot_index);
+		key = key.substr(dot_index+1);
+
+		size_t index = 0;
+		const std::string::iterator index_start = std::find(element.begin(),element.end(),'[');
+		const bool explicit_index = index_start != element.end();
+		if(explicit_index) {
+			const std::string::iterator index_end = std::find(index_start,element.end(),']');
+			const std::string index_str(index_start+1,index_end);
+			index = static_cast<size_t>(lexical_cast_default<int>(index_str));
+			if(index > MaxLoop) {
+				LOG_NG << "clear_variable: index greater than " << MaxLoop
+				       << ", truncated\n";
+				index = MaxLoop;
+			}
+			element = std::string(element.begin(),index_start);
+		}
+
+		if(vars->get_children(element).size() <= index) {
+			return;
+		}
+		//std::cerr << "Entering " << element << "[" << index << "] of [" << vars->get_children(element).size() << "]\n";
+		vars = vars->get_children(element)[index];
+		if(!vars) {
+			return;
+		}
+		itor = std::find(key.begin(),key.end(),'.');
+		dot_index = key.find('.');
+	}
+	size_t index = 0;
+	const std::string::iterator index_start = std::find(key.begin(),key.end(),'[');
+	const bool explicit_index = index_start != key.end();
+	if(explicit_index) {
+		const std::string::iterator index_end = std::find(index_start,key.end(),']');
+		const std::string index_str(index_start+1,index_end);
+		index = static_cast<size_t>(lexical_cast_default<int>(index_str));
+		if(index > MaxLoop) {
+			LOG_NG << "clear_variable: index greater than " << MaxLoop
+				   << ", truncated\n";
+			index = MaxLoop;
+		}
+		key = std::string(key.begin(),index_start);
+	}
+
+	if(explicit_index) {
+		if(vars->get_children(key).size() <= index) {
+			return;
+		}
+		//std::cerr << "Removing " << key << "\n";
+		vars->remove_child(key,index);
+	} else {
+		vars->values.erase(key);
+		vars->clear_children(key);
+	}
+}
+
+
+
 
