@@ -1,4 +1,4 @@
-/* $Id: gamestatus.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: gamestatus.hpp 12768 2006-07-25 16:32:50Z jhinrichs $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -29,6 +29,7 @@ struct time_of_day
 	//the % bonus lawful units receive. chaotic units will
 	//receive -lawful_bonus.
 	int lawful_bonus;
+	int bonus_modified;
 
 	//the image to be displayed in the game status.
 	std::string image;
@@ -52,24 +53,27 @@ public:
 	gamestatus(const config& time_cfg, int num_turns);
 	void write(config& cfg) const;
 
-	const time_of_day& get_time_of_day() const;
-	const time_of_day& get_previous_time_of_day() const;
-	const time_of_day& get_time_of_day(bool illuminated, const gamemap::location& loc) const;
+	time_of_day get_time_of_day() const;
+	time_of_day get_previous_time_of_day() const;
+	time_of_day get_time_of_day(int illuminated, const gamemap::location& loc) const;
+	time_of_day get_time_of_day(int illuminated, const gamemap::location& loc, int n_turn) const;
 	size_t turn() const;
 	int number_of_turns() const;
+	void modify_turns(const std::string& mod);
+	void add_turns(int num);
 
 	//function to move to the next turn. Returns true iff time
 	//has expired.
 	bool next_turn();
 
 private:
-	const time_of_day& get_time_of_day_turn(int nturn) const;
+	time_of_day get_time_of_day_turn(int nturn) const;
 
-	std::vector<time_of_day> times_, illuminatedTimes_;
+	std::vector<time_of_day> times_;
 
 	struct area_time_of_day {
 		std::string xsrc, ysrc;
-		std::vector<time_of_day> times, illuminated_times;
+		std::vector<time_of_day> times;
 		std::set<gamemap::location> hexes;
 	};
 
@@ -102,6 +106,7 @@ struct game_state : public variable_set
 
 	std::string campaign_define; //if there is a define the campaign uses to customize data
 
+	std::string campaign; //the campaign being played
 	std::string scenario; //the scenario being played
 
 	// information about campaign players who carry resources from
@@ -122,6 +127,8 @@ struct game_state : public variable_set
 	config& get_variable_cfg(const std::string& varname);
 	void set_variable(const std::string& varname, const t_string& value);
 
+	void clear_variable(const std::string& varname);
+
 	std::string difficulty; //the difficulty level the game is being played on.
 
 	//if the game is saved mid-level, we have a series of replay steps to
@@ -148,20 +155,21 @@ struct save_info {
 };
 
 //function to get a list of available saves.
-std::vector<save_info> get_saves_list();
+std::vector<save_info> get_saves_list(const std::string *dir = NULL);
 
-enum WRITE_GAME_MODE { WRITE_SNAPSHOT_ONLY, WRITE_FULL_GAME };
+//enum WRITE_GAME_MODE { WRITE_SNAPSHOT_ONLY, WRITE_FULL_GAME };
 
 void read_save_file(const std::string& name, config& cfg, std::string* error_log);
 
 game_state read_game(const game_data& data, const config* cfg);
-void write_game(const game_state& game, config& cfg, WRITE_GAME_MODE mode=WRITE_FULL_GAME);
+void write_game(const game_state& game, config& cfg/*, WRITE_GAME_MODE mode=WRITE_FULL_GAME*/);
 
 // function returns true iff there is already savegame with that name
 bool save_game_exists(const std::string & name);
 
 //functions to load/save games.
 void load_game(const game_data& data, const std::string& name, game_state& state, std::string* error_log);
+void load_game_summary(const std::string& name, config& cfg_summary, std::string* error_log);
 //throws gamestatus::save_game_failed
 void save_game(const game_state& state);
 
@@ -175,5 +183,6 @@ void delete_save_summary(const std::string& save);
 void write_save_index();
 
 void extract_summary_data_from_save(const game_state& state, config& out);
+void extract_summary_from_config(config& cfg_save, config& cfg_summary);
 
 #endif
