@@ -1,4 +1,4 @@
-/* $Id: global.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: global.hpp 9082 2005-12-07 11:37:11Z ott $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -14,8 +14,19 @@
 #ifndef DISABLE_4786_HPP_INCLUDED
 #define DISABLE_4786_HPP_INCLUDED
 
+//for windows compilers
+#ifdef __MSVCRT__
+#ifndef __GNUC__
+	#undef snprintf
+	#define snprintf _snprintf
+#endif
+#endif
+
 #ifdef _MSC_VER
 
+#undef snprintf
+#define snprintf _snprintf
+
 //disable the warning to let us know about 'this' being used in
 //initializer list, since it's a common thing to want to do
 //for callbacks, and there is no other way to easily workaround the warning
