@@ -16,12 +16,12 @@
 #include "cursor.hpp"
 #include "display.hpp"
 #include "events.hpp"
-#include "font.hpp"
 #include "game_config.hpp"
 #include "gettext.hpp"
 #include "help.hpp"
 #include "image.hpp"
 #include "language.hpp"
+#include "marked-up_text.hpp"
 #include "log.hpp"
 #include "preferences.hpp"
 #include "sdl_utils.hpp"
@@ -257,7 +257,7 @@ public:
 	std::string ref_at(const int x, const int y);
 
 protected:
-	virtual void scroll(int pos);
+	virtual void scroll(unsigned int pos);
 	virtual void set_inner_location(const SDL_Rect& rect);
 
 private:
@@ -420,20 +420,20 @@ private:
 
 /// Dispatch generators to their appropriate functions.
 std::vector<section> generate_sections(const std::string &generator);
-std::vector<topic> generate_topics(const std::string &generator);
+std::vector<topic> generate_topics(const bool sort_topics,const std::string &generator);
 std::string generate_topic_text(const std::string &generator);
 std::string generate_about_text();
 std::string generate_traits_text();
-std::vector<topic> generate_unit_topics();
+std::vector<topic> generate_unit_topics(const bool);
 enum UNIT_DESCRIPTION_TYPE {FULL_DESCRIPTION, NO_DESCRIPTION, NON_REVEALING_DESCRIPTION};
 /// Return the type of description that should be shown for a unit of
 /// the given kind. This method is intended to filter out information
 /// about units that should not be shown, for example due to not being
 /// encountered.
 UNIT_DESCRIPTION_TYPE description_type(const unit_type &type);
-std::vector<topic> generate_ability_topics();
-std::vector<topic> generate_weapon_special_topics();
-std::vector<topic> generate_terrains_topics();
+std::vector<topic> generate_ability_topics(const bool);
+std::vector<topic> generate_weapon_special_topics(const bool);
+std::vector<topic> generate_terrains_topics(const bool);
 
 /// Parse a help config, return the top level section. Return an empty
 /// section if cfg is NULL.
@@ -815,10 +815,30 @@ void parse_config_internal(const config *help_cfg, const config *section_cfg,
 		const std::vector<section> generated_sections =
 			generate_sections((*section_cfg)["generator"]);
 		std::transform(generated_sections.begin(), generated_sections.end(),
-					   std::back_inserter(sec.sections), create_section());
-		const std::vector<std::string> topics = utils::quoted_split((*section_cfg)["topics"]);
+			       std::back_inserter(sec.sections), create_section());
+		bool sort_topics = false;
+		bool sort_generated = true;
+
+		if ((*section_cfg)["sort_topics"] == "yes") {
+		  sort_topics = true;
+		  sort_generated = false;
+		} else if ((*section_cfg)["sort_topics"] == "no") {
+		  sort_topics = false;
+		  sort_generated = false;
+		} else if ((*section_cfg)["sort_topics"] == "generated") {
+		  sort_topics = false;
+		  sort_generated = true;
+		} else if ((*section_cfg)["sort_topics"] != "") {
+		  std::stringstream ss;
+		  ss << "Invalid sort option: '" << (*section_cfg)["sort_topics"] << "'";
+		  throw parse_error(ss.str());
+		}
+
+		const std::vector<std::string> topics_id = utils::quoted_split((*section_cfg)["topics"]);
+		std::vector<topic> topics;
+
 		// Find all topics in this section.
-		for (it = topics.begin(); it != topics.end(); it++) {
+		for (it = topics_id.begin(); it != topics_id.end(); it++) {
 			config const *topic_cfg = help_cfg->find_child("topic", "id", *it);
 			if (topic_cfg != NULL) {
 				std::string text = (*topic_cfg)["text"];
@@ -829,7 +849,7 @@ void parse_config_internal(const config *help_cfg, const config *section_cfg,
 					ss << "Invalid ID, used for internal purpose: '" << id << "'";
 					throw parse_error(ss.str());
 				}
-				sec.topics.push_back(child_topic);
+				topics.push_back(child_topic);
 			}
 			else {
 				std::stringstream ss;
@@ -838,10 +858,25 @@ void parse_config_internal(const config *help_cfg, const config *section_cfg,
 				throw parse_error(ss.str());
 			}
 		}
-		const std::vector<topic> generated_topics =
-			generate_topics((*section_cfg)["generator"]);
-		std::copy(generated_topics.begin(), generated_topics.end(),
-				  std::back_inserter(sec.topics));
+
+		std::vector<topic> generated_topics =
+		  generate_topics(sort_generated,(*section_cfg)["generator"]);
+
+		if (sort_topics) {
+		  	std::sort(topics.begin(),topics.end(), title_less());
+			std::sort(generated_topics.begin(),
+			  generated_topics.end(), title_less());
+			std::merge(generated_topics.begin(),
+			  generated_topics.end(),topics.begin(),topics.end()
+			  ,std::back_inserter(sec.topics),title_less());
+		}
+		else {
+			std::copy(topics.begin(), topics.end(),
+			  std::back_inserter(sec.topics));
+			std::copy(generated_topics.begin(),
+			  generated_topics.end(),
+			  std::back_inserter(sec.topics));
+		}
 	}
 }
 
@@ -865,23 +900,23 @@ std::vector<section> generate_sections(const std::string &generator)
 	return empty_vec;
 }
 
-std::vector<topic> generate_topics(const std::string &generator)
+std::vector<topic> generate_topics(const bool sort_generated,const std::string &generator)
 {
 	std::vector<topic> res;
 	if (generator == "units") {
-		res = generate_unit_topics();
+		res = generate_unit_topics(sort_generated);
 	}
 	else if (generator == "abilities") {
-		res = generate_ability_topics();
+		res = generate_ability_topics(sort_generated);
 	}
 	else if (generator == "weapon_specials") {
-		res = generate_weapon_special_topics();
+		res = generate_weapon_special_topics(sort_generated);
 	}
 //  terrain topics removed for now, since they aren't release-quality
 //	else if (generator == "terrains") {
-//		res = generate_terrains_topics();
+//		res = generate_terrains_topics(sort_generated);
 //	}
-	std::sort(res.begin(), res.end(), title_less());
+
 	return res;
 }
 
@@ -932,8 +967,9 @@ topic_text::operator std::vector< std::string > const &() const
 	return parsed_text_;
 }
 
-std::vector<topic> generate_weapon_special_topics()
+std::vector<topic> generate_weapon_special_topics(const bool sort_generated)
 {
+
 	std::vector<topic> topics;
 	if (game_info == NULL) {
 		return topics;
@@ -948,34 +984,43 @@ std::vector<topic> generate_weapon_special_topics()
 			std::vector<attack_type> attacks = type.attacks();
 			for (std::vector<attack_type>::const_iterator it = attacks.begin();
 				 it != attacks.end(); it++) {
-				std::string special = (*it).special();
 
-				//some abilities like plague can be in the form ability(argument)
-				//make sure we cut off the argument
-				special.erase(std::find(special.begin(),special.end(),'('),special.end());
-				if (special != "") {
-					if (checked_specials.find(special) == checked_specials.end()) {
-						std::string lang_special = gettext(special.c_str());
-						lang_special = utils::capitalize(lang_special);
-						std::string description
-							= string_table["weapon_special_" + special + "_description"];
-						const size_t colon_pos = description.find(':');
-						if (colon_pos != std::string::npos) {
-							// Remove the first colon and the following newline.
-							description.erase(0, colon_pos + 2);
+				std::vector<std::string> specials = (*it).special_tooltips(true);
+				std::vector<std::string>::iterator sp_it;
+				for (sp_it = specials.begin(); sp_it != specials.end(); ++sp_it)
+				{
+					std::string special = *sp_it;
+					++sp_it;
+
+					//some abilities like plague can be in the form ability(argument)
+					//make sure we cut off the argument
+					special.erase(std::find(special.begin(),special.end(),'('),special.end());
+					if (special != "") {
+						if (checked_specials.find(special) == checked_specials.end()) {
+							std::string lang_special = gettext(special.c_str());
+							lang_special = utils::capitalize(lang_special);
+							std::string description;
+							description = *sp_it;
+							const size_t colon_pos = description.find(':');
+							if (colon_pos != std::string::npos) {
+								// Remove the first colon and the following newline.
+								description.erase(0, colon_pos + 2);
+							}
+							topic t(lang_special, "weaponspecial_" + special, description);
+							topics.push_back(t);
+							checked_specials.insert(special);
 						}
-						topic t(lang_special, "weaponspecial_" + special, description);
-						topics.push_back(t);
-						checked_specials.insert(special);
 					}
 				}
 			}
 		}
 	}
+	if (sort_generated)
+		std::sort(topics.begin(), topics.end(), title_less());
 	return topics;
 }
 
-std::vector<topic> generate_ability_topics()
+std::vector<topic> generate_ability_topics(const bool sort_generated)
 {
 	std::vector<topic> topics;
 	if (game_info == NULL) {
@@ -987,15 +1032,23 @@ std::vector<topic> generate_ability_topics()
 	// for display. We do not want to show abilities that the user has
 	// not encountered yet.
 	for(game_data::unit_type_map::const_iterator i = game_info->unit_types.begin();
-	    i != game_info->unit_types.end(); i++) {
+	    i != game_info->unit_types.end(); ++i) {
 		const unit_type &type = (*i).second;
 		if (description_type(type) == FULL_DESCRIPTION) {
+			std::vector<std::string> descriptions = type.ability_tooltips();
+			std::vector<std::string>::const_iterator desc_it = descriptions.begin();
 			for (std::vector<std::string>::const_iterator it = type.abilities().begin();
-				 it != type.abilities().end(); it++) {
+				 it != type.abilities().end(); ++it) {
 				if (checked_abilities.find(*it) == checked_abilities.end()) {
 					const std::string id = "ability_" + *it;
-					std::string lang_ability = utils::capitalize(string_table[id]);
-					std::string description = string_table[*it + "_description"];
+					std::string lang_ability = utils::capitalize(gettext(it->c_str()));
+					std::string description;
+					if(desc_it != descriptions.end())
+					{
+						description = *desc_it;
+					} else {
+						description = string_table[*it + "_description"];
+					}
 					const size_t colon_pos = description.find(':');
 					if (colon_pos != std::string::npos) {
 						// Remove the first colon and the following newline.
@@ -1005,10 +1058,14 @@ std::vector<topic> generate_ability_topics()
 					topics.push_back(t);
 					checked_abilities.insert(*it);
 				}
+				// Description ptr needs to pace ability ptr
+				++desc_it;
 			}
 		}
 
 	}
+	if (sort_generated)
+		std::sort(topics.begin(), topics.end(), title_less());
 	return topics;
 }
 
@@ -1029,10 +1086,10 @@ public:
 		const unit_type& male_type = type_.get_gender_unit_type(unit_race::MALE);
 
 		// Show the unit's image and its level.
-		ss << "<img>src='" << male_type.image() << "'</img> ";
+		ss << "<img>src='" << male_type.image() << "~TC(1," << male_type.flag_rgb() << ")" << "'</img> ";
 
 		if (&female_type != &male_type)
-			ss << "<img>src='" << female_type.image() << "'</img> ";
+			ss << "<img>src='" << female_type.image() << "~TC(1," << female_type.flag_rgb() << ")" << "'</img> ";
 
 		ss << "<format>font_size=" << font::relative_size(11) << " text=' " << escape(_("level"))
 		   << " " << type_.level() << "'</format>";
@@ -1080,7 +1137,7 @@ public:
 				 ability_end = type_.abilities().end();
 				 ability_it != ability_end; ++ability_it) {
 				const std::string ref_id = std::string("ability_") + *ability_it;
-				std::string lang_ability = string_table[ref_id];
+				std::string lang_ability = gettext(ability_it->c_str());
 				ss << "<ref>dst='" << escape(ref_id) << "' text='" << escape(lang_ability)
 				   << "'</ref>";
 				if (ability_it + 1 != ability_end)
@@ -1089,11 +1146,12 @@ public:
 			ss << "\n";
 		}
 
-		if (!next_units.empty() || !type_.abilities().empty())
+		if (!next_units.empty() || !type_.ability_tooltips().empty())
 			ss << "\n";
 		// Print some basic information such as HP and movement points.
 		ss << _("HP: ") << type_.hitpoints() << jump(30)
 		   << _("Moves: ") << type_.movement() << jump(30)
+		   << _("Cost: ") << type_.cost() << jump(30)
 		   << _("Alignment: ")
 		   << "<ref>dst='time_of_day' text='"
 		   << type_.alignment_description(type_.alignment())
@@ -1139,20 +1197,30 @@ public:
 				attack_ss << attack_it->damage() << '-' << attack_it->num_attacks();
 				push_tab_pair(row, attack_ss.str());
 				attack_ss.str(clear_stringstream);
-				push_tab_pair(row, (*attack_it).range() == attack_type::SHORT_RANGE ?
-							  _("melee") : _("ranged"));
+				push_tab_pair(row, _((*attack_it).range().c_str()));
 				// Show this attack's special, if it has any. Cross
 				// reference it to the section describing the
 				// special.
-				if (!attack_it->special().empty()) {
-					const std::string ref_id = std::string("weaponspecial_")
-						+ (*attack_it).special();
-					std::string lang_special = gettext(attack_it->special().c_str());
-					attack_ss << "<ref>dst='" << escape(ref_id)
-					          << "' text='" << escape(lang_special) << "'</ref>";
+				std::vector<std::string> specials = attack_it->special_tooltips(true);
+				if(!specials.empty())
+				{
+					std::string lang_special = "";
+					std::vector<std::string>::iterator sp_it;
+					for (sp_it = specials.begin(); sp_it != specials.end(); sp_it++) {
+						const std::string ref_id = std::string("weaponspecial_")
+							+ (*sp_it);
+						lang_special = gettext(sp_it->c_str());
+						attack_ss << "<ref>dst='" << escape(ref_id)
+								  << "' text='" << escape(lang_special) << "'</ref>";
+						if((sp_it + 1) != specials.end() && (sp_it + 2) != specials.end())
+						{
+							attack_ss << ", "; //comma placed before next special
+						}
+						sp_it++; //skip description
+					}
 					row.push_back(std::make_pair(attack_ss.str(),
-								     font::line_width(lang_special,
-										      normal_font_size)));
+						font::line_width(lang_special, normal_font_size)));
+
 				}
 				table.push_back(row);
 			}
@@ -1196,7 +1264,7 @@ public:
 			table_spec table;
 			push_header(first_row, _("Terrain"));
 			push_header(first_row, _("Defense"));
-			push_header(first_row, _("Movement"));
+			push_header(first_row, _("Movement Cost"));
 
 			table.push_back(first_row);
 			for (std::set<std::string>::const_iterator terrain_it =
@@ -1208,7 +1276,7 @@ public:
 				if (terrain == gamemap::FOGGED || terrain == gamemap::VOID_TERRAIN)
 					continue;
 				const terrain_type& info = map->get_terrain_info(terrain);
-				if (!info.is_alias() && info.is_nonnull()) {
+				if (info.union_type().size() == 1 && info.union_type()[0] == info.letter() && info.is_nonnull()) {
 					std::vector<item> row;
 					const std::string& name = info.name();
 					const std::string id = info.id();
@@ -1244,8 +1312,18 @@ public:
 	}
 };
 
-std::vector<topic> generate_unit_topics()
+struct unit_topic_less {
+	bool operator() (std::pair<const unit_type*,topic> a,
+	    std::pair<const unit_type*,topic> b) {
+		if (a.first->race() == b.first->race())
+			return a.second.title < b.second.title;
+		return a.first->race() < b.first->race();
+	}
+};
+
+std::vector<topic> generate_unit_topics(const bool sort_generated)
 {
+	std::vector<std::pair<const unit_type*,topic> > unit_topics;
 	std::vector<topic> topics;
 	if (game_info == NULL) {
 		return topics;
@@ -1266,7 +1344,13 @@ std::vector<topic> generate_unit_topics()
 		} else {
 			wassert(false);
 		}
-		topics.push_back(unit_topic);
+		unit_topics.push_back(std::pair<const unit_type*,topic>(&type,unit_topic));
+	}
+	if (sort_generated)
+		std::sort(unit_topics.begin(), unit_topics.end(), unit_topic_less());
+	for(std::vector<std::pair<const unit_type*,topic> >::const_iterator j = unit_topics.begin();
+	    j != unit_topics.end(); j++) {
+		topics.push_back((*j).second);
 	}
 	return topics;
 }
@@ -1291,8 +1375,8 @@ struct terrain_topic_generator: topic_generator
 	virtual std::string operator()() const {
 		std::stringstream ss;
 		ss << "<img>src='terrain/" << type.symbol_image() << ".png'</img>\n\n";
-		if (type.is_alias()) {
-			const std::string aliased_terrains = type.type();
+		if (type.mvt_type().size() != 1 || type.mvt_type()[0] != type.letter()) {
+			const std::string aliased_terrains = type.mvt_type();
 			std::stringstream alias_ss;
 			for (std::string::const_iterator it = aliased_terrains.begin();
 				  it != aliased_terrains.end(); it++) {
@@ -1308,22 +1392,48 @@ struct terrain_topic_generator: topic_generator
 			utils::string_map sm;
 			sm["terrains"] = alias_ss.str();
 			ss << utils::interpolate_variables_into_string(
-				_("This terrain acts as $terrains for movement and defense purposes."), &sm);
-			if (aliased_terrains.size() > 1)
+				_("This terrain acts as $terrains for movement purposes."), &sm);
+			if (aliased_terrains.size() > 1 && aliased_terrains[0] != '-')
 				ss << " " << _("The terrain with the best modifier is chosen automatically.");
+			else
+				ss << " " << _("The terrain with the worst modifier is chosen automatically.");
+			ss << "\n\n";
+		}
+		if (type.def_type().size() != 1 || type.def_type()[0] != type.letter()) {
+			const std::string aliased_terrains = type.def_type();
+			std::stringstream alias_ss;
+			for (std::string::const_iterator it = aliased_terrains.begin();
+				  it != aliased_terrains.end(); it++) {
+				const gamemap::TERRAIN t = *it;
+				const std::string &alias_name = map->get_terrain_info(t).name();
+				alias_ss << "<ref>text='" << escape(alias_name) << "' dst='"
+					 << escape(std::string("terrain_") + t) << "'</ref>";
+				if (it + 2 == aliased_terrains.end())
+					alias_ss << " " << _("or") << " ";
+				else if (it + 1 != aliased_terrains.end())
+					alias_ss << ", ";
+			}
+			utils::string_map sm;
+			sm["terrains"] = alias_ss.str();
+			ss << utils::interpolate_variables_into_string(
+				_("This terrain acts as $terrains for defense purposes."), &sm);
+			if (aliased_terrains.size() > 1 && aliased_terrains[0] != '-')
+				ss << " " << _("The terrain with the best modifier is chosen automatically.");
+			else
+				ss << " " << _("The terrain with the worst modifier is chosen automatically.");
 			ss << "\n\n";
 		}
 		if (type.is_keep())
-			ss << _("This terrain acts as keep, i.e., you can recruit units when a leader is in a location with this terrain.") << "\n\n";
+			ss << _("This terrain acts as a keep, i.e., you can recruit units when a leader is in a location with this terrain.") << "\n\n";
 		if (type.is_castle())
-			ss << _("This terrain acts as castle, i.e., you can recruit units onto a location with this terrain.") << "\n\n";
+			ss << _("This terrain acts as a castle, i.e., you can recruit units onto a location with this terrain.") << "\n\n";
 		if (type.gives_healing())
 			ss << _("This terrain gives healing.") << "\n\n";
 		return ss.str();
 	}
 };
 
-std::vector<topic> generate_terrains_topics()
+std::vector<topic> generate_terrains_topics(const bool sort_generated)
 {
 	std::vector<topic> res;
 	std::vector<gamemap::TERRAIN> show_info_about;
@@ -1351,6 +1461,8 @@ std::vector<topic> generate_terrains_topics()
 		topic t(name, std::string("terrain_") + *terrain_it, new terrain_topic_generator(info));
 		res.push_back(t);
 	}
+	if (sort_generated)
+		std::sort(res.begin(), res.end(), title_less());
 	return res;
 }
 
@@ -1463,7 +1575,7 @@ void section::clear()
 }
 
 help_menu::help_menu(CVideo &video, section const &toplevel, int max_height)
-	: gui::menu(video, empty_string_vector, true, max_height),
+: gui::menu(video, empty_string_vector, true, max_height, -1, NULL, &gui::menu::bluebg_style),
 	  toplevel_(toplevel), chosen_topic_(NULL), selected_item_(&toplevel, "")
 {
 	update_visible_items(toplevel_);
@@ -2140,7 +2252,7 @@ void help_text_area::draw_contents()
 	update_rect(loc);
 }
 
-void help_text_area::scroll(int)
+void help_text_area::scroll(unsigned int)
 {
 	// Nothing will be done on the actual scroll event. The scroll
 	// position is checked when drawing instead and things drawn
@@ -2174,7 +2286,7 @@ std::string help_text_area::ref_at(const int x, const int y)
 help_browser::help_browser(display &disp, const section &toplevel)
 	: gui::widget(disp.video()), disp_(disp), menu_(disp.video(), toplevel),
 	  text_area_(disp.video(), toplevel), toplevel_(toplevel), ref_cursor_(false),
-	  back_button_(disp.video(), _("< Back"), gui::button::TYPE_PRESS),
+	  back_button_(disp.video(), _(" < Back"), gui::button::TYPE_PRESS),
 	  forward_button_(disp.video(), _("Forward >"), gui::button::TYPE_PRESS),
 	  shown_topic_(NULL)
 {
@@ -2581,22 +2693,16 @@ std::string escape(const std::string &s)
 
 std::string get_first_word(const std::string &s)
 {
-	if (s == "") {
+	size_t first_word_start = s.find_first_not_of(' ');
+	if (first_word_start == std::string::npos) {
 		return s;
 	}
-	size_t first_word_start = s.find_first_not_of(" ");
-	if (first_word_start == std::string::npos) {
-		first_word_start = 0;
+ 	size_t first_word_end = s.find_first_of(" \n", first_word_start);
+	if( first_word_end == first_word_start ) {
+		// This word is '\n'.
+		first_word_end = first_word_start+1;
 	}
-	size_t first_word_end = s.find_first_of(" \n", first_word_start);
-	if (first_word_end == std::string::npos || first_word_end == first_word_start) {
-		// Either this word contains no spaces/newlines, or it consists
-		// of only spaces and newlines. In either case, use the whole
-		// chunk as a word.
-		first_word_end = s.size();
-	}
-	const std::string first_word = s.substr(0, first_word_end);
-	return first_word;
+	return s.substr(0, first_word_end);
 }
 
 void show_help(display &disp, std::string show_topic, int xloc, int yloc)
@@ -2666,7 +2772,8 @@ void show_help(display &disp, const section &toplevel_sec, const std::string sho
 					 NULL, &buttons_ptr, &restorer);
 
 	if (preferences::encountered_units().size() != size_t(last_num_encountered_units) ||
-	    preferences::encountered_terrains().size() != size_t(last_num_encountered_terrains)) {
+	    preferences::encountered_terrains().size() != size_t(last_num_encountered_terrains) ||
+		last_num_encountered_units < 0) {
 		// More units or terrains encountered, update the contents.
 		last_num_encountered_units = preferences::encountered_units().size();
 		last_num_encountered_terrains = preferences::encountered_terrains().size();
@@ -2704,7 +2811,7 @@ void show_help(display &disp, const section &toplevel_sec, const std::string sho
 				}
 			}
 			disp.flip();
-			SDL_Delay(10);
+			disp.delay(10);
 		}
 	}
 	catch (parse_error e) {
