@@ -1,4 +1,4 @@
-/* $Id: hotkeys.cpp 8303 2005-09-18 23:39:42Z ott $ */
+/* $Id: hotkeys.cpp 12024 2006-06-10 23:37:20Z rusty $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -19,13 +19,16 @@
 #include "display.hpp"
 #include "events.hpp"
 #include "hotkeys.hpp"
+#include "game_config.hpp"
+#include "game_errors.hpp"
 #include "gettext.hpp"
-#include "playlevel.hpp"
-#include "preferences.hpp"
+#include "menu_events.hpp"
+#include "preferences_display.hpp"
 #include "show_dialog.hpp"
 #include "util.hpp"
 #include "video.hpp"
 #include "wesconfig.h"
+#include "wml_separators.hpp"
 #include "SDL.h"
 
 #include <algorithm>
@@ -33,6 +36,7 @@
 #include <map>
 
 #define ERR_G LOG_STREAM(err, general)
+#define INFO_C LOG_STREAM(info, config)
 
 namespace {
 
@@ -42,8 +46,8 @@ const struct {
 	const char* description;
 	bool hidden;
 } hotkey_list_[] = {
-	{ hotkey::HOTKEY_CYCLE_UNITS, "cycle", N_("Next unit"), false },
-	{ hotkey::HOTKEY_CYCLE_BACK_UNITS, "cycleback", N_("Previous unit"), false },
+	{ hotkey::HOTKEY_CYCLE_UNITS, "cycle", N_("Next Unit"), false },
+	{ hotkey::HOTKEY_CYCLE_BACK_UNITS, "cycleback", N_("Previous Unit"), false },
 	{ hotkey::HOTKEY_UNIT_HOLD_POSITION, "holdposition", N_("Hold Position"), false},
 	{ hotkey::HOTKEY_END_UNIT_TURN, "endunitturn", N_("End Unit Turn"), false },
 	{ hotkey::HOTKEY_LEADER, "leader", N_("Leader"), false },
@@ -64,6 +68,7 @@ const struct {
 	{ hotkey::HOTKEY_RECALL, "recall", N_("Recall"), false },
 	{ hotkey::HOTKEY_ENDTURN, "endturn", N_("End Turn"), false },
 	{ hotkey::HOTKEY_TOGGLE_GRID, "togglegrid", N_("Toggle Grid"), false },
+	{ hotkey::HOTKEY_MOUSE_SCROLL, "mousescroll", N_("Mouse Scrolling"), false },
 	{ hotkey::HOTKEY_STATUS_TABLE, "statustable", N_("Status Table"), false },
 	{ hotkey::HOTKEY_MUTE, "mute", N_("Mute"), false },
 	{ hotkey::HOTKEY_SPEAK, "speak", N_("Speak"), false },
@@ -75,8 +80,17 @@ const struct {
 	{ hotkey::HOTKEY_STATISTICS, "statistics", N_("Statistics"), false },
 	{ hotkey::HOTKEY_QUIT_GAME, "quit", N_("Quit Game"), false },
 	{ hotkey::HOTKEY_LABEL_TERRAIN, "labelterrain", N_("Set Label"), false },
+	{ hotkey::HOTKEY_CLEAR_LABELS, "clearlabels", N_("Clear Labels"), false },
 	{ hotkey::HOTKEY_SHOW_ENEMY_MOVES, "showenemymoves", N_("Show Enemy Moves"), false },
 	{ hotkey::HOTKEY_BEST_ENEMY_MOVES, "bestenemymoves", N_("Best Possible Enemy Moves"), false },
+	{ hotkey::HOTKEY_PLAY_REPLAY, "playreplay", N_("Play"), false },
+	{ hotkey::HOTKEY_RESET_REPLAY, "resetreplay", N_("Reset"), false },
+	{ hotkey::HOTKEY_STOP_REPLAY, "stopreplay", N_("Stop"), false },
+	{ hotkey::HOTKEY_REPLAY_NEXT_TURN, "replaynextturn", N_("Next Turn"), false },
+	{ hotkey::HOTKEY_REPLAY_NEXT_SIDE, "replaynextside", N_("Next Side"), false },
+	{ hotkey::HOTKEY_REPLAY_SHROUD, "replayswitchshroud", N_("Shroud"), false },
+	{ hotkey::HOTKEY_REPLAY_FOG, "replayswitchfog", N_("Fog"), false },
+	{ hotkey::HOTKEY_REPLAY_SKIP_ANIMATION, "replayskipanimation", N_("Skip animation"), false },
 
 	{ hotkey::HOTKEY_EDIT_SET_TERRAIN, "editsetterrain", N_("Set Terrain"),true },
 	{ hotkey::HOTKEY_EDIT_QUIT, "editquit", N_("Quit Editor"),true },
@@ -120,38 +134,76 @@ hotkey::hotkey_item null_hotkey_;
 namespace hotkey {
 
 hotkey_item::hotkey_item(HOTKEY_COMMAND id, const std::string& command, const std::string& description, bool hidden)
-	: id_(id), command_(command), description_(description), keycode_(0),
-	  shift_(false), ctrl_(false), alt_(false), cmd_(false), hidden_(hidden)
+	: id_(id), command_(command), description_(description), type_(UNBOUND),
+	  ctrl_(false), alt_(false), cmd_(false), shift_(false), hidden_(hidden)
 {
 }
 
+// There are two kinds of "key" values.  One refers to actual keys, like
+// F1 or SPACE.  The other refers to characters produced, eg 'M' or ':'.
+// For the latter, specifying shift+; doesn't make sense, because ; is
+// already shifted on French keyboards, for example.  You really want to
+// say ':', however that is typed.  However, when you say shift+SPACE,
+// you're really referring to the space bar, as shift+SPACE usually just
+// produces a SPACE character.
 void hotkey_item::load_from_config(const config& cfg)
 {
-	const std::string& code = cfg["key"];
-	if(code.empty()) {
-		keycode_ = 0;
-	} else {
-		keycode_ = sdl_keysym_from_name(code);
-		if (keycode_ == SDLK_UNKNOWN) {
-			if(code.size() >= 2  && code.size() <= 3 && tolower(code[0]) == 'f') {
-				const int num = lexical_cast_default<int>(std::string(code.begin() + 1, code.end()), 1);
-				keycode_ = num + SDLK_F1 - 1;
-			} else if (code.size() == 1) {
-				keycode_ = code[0];
-			}
-		}
-	}
+	const std::string& key = cfg["key"];
 
 	alt_ = (cfg["alt"] == "yes");
+	cmd_ = (cfg["cmd"] == "yes");
 	ctrl_ = (cfg["ctrl"] == "yes");
 	shift_ = (cfg["shift"] == "yes");
-	cmd_ = (cfg["cmd"] == "yes");
+
+	if (!key.empty()) {
+		// They may really want a specific key on the keyboard: we assume
+		// that any single character keyname is a character.
+		if (key.size() > 1) {
+			type_ = BY_KEYCODE;
+
+			keycode_ = sdl_keysym_from_name(key);
+			if (keycode_ == SDLK_UNKNOWN) {
+				if (tolower(key[0]) != 'f') {
+					LOG_STREAM(err, config)
+						<< "hotkey key '" << key << "' invalid\n";
+				} else {
+					int num = lexical_cast_default<int>
+						(std::string(key.begin()+1,	key.end()), 1);
+					keycode_ = num + SDLK_F1 - 1;
+				}
+			}
+		} else if (key == " " || shift_
+#ifdef __APPLE__
+			   || alt_
+#endif
+			   ) {
+			// Space must be treated as a key because shift-space
+			// isn't a different character from space, and control key
+			// makes it go weird.  shift=yes should never be specified
+			// on single characters (eg. key=m, shift=yes would be
+			// key=M), but we don't want to break old preferences
+			// files.
+			type_ = BY_KEYCODE;
+			keycode_ = key[0];
+		} else {
+			type_ = BY_CHARACTER;
+			character_ = key[0];
+		}
+	}
 }
 
 std::string hotkey_item::get_name() const
 {
-	if (keycode_ != 0) {
-		std::stringstream str;
+	std::stringstream str;
+	if (type_ == BY_CHARACTER) {
+		if (alt_)
+			str << "alt+";
+		if (cmd_)
+			str << "cmd+";
+		if (ctrl_)
+			str << "ctrl+";
+		str << (char)character_;
+	} else if (type_ == BY_KEYCODE) {
 		if (alt_)
 			str << "alt+";
 		if (ctrl_)
@@ -160,12 +212,9 @@ std::string hotkey_item::get_name() const
 			str << "shift+";
 		if (cmd_)
 			str << "cmd+";
-
 		str << SDL_GetKeyName(SDLKey(keycode_));
-		return str.str();
-	} else {
-		return "";
 	}
+	return str.str();
 }
 
 void hotkey_item::set_description(const std::string& description)
@@ -173,13 +222,44 @@ void hotkey_item::set_description(const std::string& description)
 	description_ = description;
 }
 
-void hotkey_item::set_key(int keycode, bool shift, bool ctrl, bool alt, bool cmd)
+void hotkey_item::set_key(int character, int keycode, bool shift, bool ctrl, bool alt, bool cmd)
 {
-	keycode_ = keycode;
-	shift_ = shift;
-	ctrl_ = ctrl;
-	alt_ = alt;
-	cmd_ = cmd;
+	const std::string keyname = SDL_GetKeyName(SDLKey(keycode));
+
+	INFO_C << "setting hotkey: char=" << lexical_cast<std::string>(character)
+		   << " keycode="  << lexical_cast<std::string>(keycode) << " "
+		   << (shift ? "shift," : "")
+		   << (ctrl ? "ctrl," : "")
+		   << (alt ? "alt," : "")
+		   << (cmd ? "cmd," : "")
+		   << "\n";
+
+	// Sometimes control modifies by -64, ie ^A == 1.
+	if (character < 64 && ctrl) {
+		if (shift)
+			character += 64;
+		else
+			character += 96;
+		INFO_C << "Mapped to character " << lexical_cast<std::string>(character) << "\n";
+	}
+
+	// We handle simple cases by character, others by the actual key.
+	if (isprint(character) && !isspace(character)) {
+		type_ = BY_CHARACTER;
+		character_ = character;
+		ctrl_ = ctrl;
+		alt_ = alt;
+		cmd_ = cmd;
+		INFO_C << "type = BY_CHARACTER\n";
+	} else {
+		type_ = BY_KEYCODE;
+		keycode_ = keycode;
+		shift_ = shift;
+		ctrl_ = ctrl;
+		alt_ = alt;
+		cmd_ = cmd;
+		INFO_C << "type = BY_KEYCODE\n";
+	}
 }
 
 manager::manager()
@@ -198,7 +278,7 @@ manager::~manager()
 
 void load_descriptions()
 {
-	for (int i = 0; hotkey_list_[i].command; ++i) {
+	for (size_t i = 0; hotkey_list_[i].command; ++i) {
 		if (i >= hotkeys_.size()) {
 			ERR_G << "Hotkey list too short: " << hotkeys_.size() << "\n";
 		}
@@ -223,16 +303,20 @@ void save_hotkeys(config& cfg)
 	cfg.clear_children("hotkey");
 
 	for(std::vector<hotkey_item>::iterator i = hotkeys_.begin(); i != hotkeys_.end(); ++i) {
-		if (i->hidden() || i->get_keycode() == 0)
+		if (i->hidden() || i->get_type() == hotkey_item::UNBOUND)
 			continue;
 
 		config& item = cfg.add_child("hotkey");
-
 		item["command"] = i->get_command();
-		item["key"] = SDL_GetKeyName(SDLKey(i->get_keycode()));
+
+		if (i->get_type() == hotkey_item::BY_KEYCODE) {
+			item["key"] = SDL_GetKeyName(SDLKey(i->get_keycode()));
+			item["shift"] = i->get_shift() ? "yes" : "no";
+		} else if (i->get_type() == hotkey_item::BY_CHARACTER) {
+			item["key"] = std::string(1, (char)i->get_character());
+		}
 		item["alt"] = i->get_alt() ? "yes" : "no";
 		item["ctrl"] = i->get_ctrl() ? "yes" : "no";
-		item["shift"] = i->get_shift() ? "yes" : "no";
 		item["cmd"] = i->get_cmd() ? "yes" : "no";
 	}
 }
@@ -267,26 +351,49 @@ hotkey_item& get_hotkey(const std::string& command)
 	return *itor;
 }
 
-hotkey_item& get_hotkey(int keycode, bool shift, bool ctrl, bool alt, bool cmd, bool mods)
+hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl, bool alt, bool cmd)
 {
-
 	std::vector<hotkey_item>::iterator itor;
 
+	INFO_C << "getting hotkey: char=" << lexical_cast<std::string>(character)
+		   << " keycode="  << lexical_cast<std::string>(keycode) << " "
+		   << (shift ? "shift," : "")
+		   << (ctrl ? "ctrl," : "")
+		   << (alt ? "alt," : "")
+		   << (cmd ? "cmd," : "")
+		   << "\n";
+
+	// Sometimes control modifies by -64, ie ^A == 1.
+	if (character < 64 && ctrl) {
+		if (shift)
+			character += 64;
+		else
+			character += 96;
+		INFO_C << "Mapped to character " << lexical_cast<std::string>(character) << "\n";
+	}
+
 	for (itor = hotkeys_.begin(); itor != hotkeys_.end(); ++itor) {
-		if(mods) {
-			if(itor->get_keycode() == keycode
-					&& (shift == itor->get_shift() || shift == true)
-					&& (ctrl == itor->get_ctrl() || ctrl == true)
-					&& (alt == itor->get_alt() || alt == true)
-					&& (cmd == itor->get_cmd() || cmd == true))
-				break;
-		} else {
-			if(itor->get_keycode() == keycode
-					&& shift == itor->get_shift()
+		if (itor->get_type() == hotkey_item::BY_CHARACTER) {
+			if (character == itor->get_character()) {
+				if (ctrl == itor->get_ctrl()
+					&& alt == itor->get_alt()
+					&& cmd == itor->get_cmd()) {
+					INFO_C << "Could match by character..." << "yes\n";
+					break;
+				}
+				INFO_C << "Could match by character..." << "but modifiers different\n";
+			}
+		} else if (itor->get_type() == hotkey_item::BY_KEYCODE) {
+			if (keycode == itor->get_keycode()) {
+				if (shift == itor->get_shift()
 					&& ctrl == itor->get_ctrl()
 					&& alt == itor->get_alt()
-					&& cmd == itor->get_cmd())
-				break;
+					&& cmd == itor->get_cmd()) {
+					INFO_C << "Could match by keycode..." << "yes\n";
+					break;
+				}
+				INFO_C << "Could match by keycode..." << "but modifiers different\n";
+			}
 		}
 	}
 
@@ -296,9 +403,9 @@ hotkey_item& get_hotkey(int keycode, bool shift, bool ctrl, bool alt, bool cmd,
 	return *itor;
 }
 
-hotkey_item& get_hotkey(const SDL_KeyboardEvent& event, bool mods)
+hotkey_item& get_hotkey(const SDL_KeyboardEvent& event)
 {
-	return get_hotkey(event.keysym.sym,
+	return get_hotkey(event.keysym.unicode, event.keysym.sym,
 			(event.keysym.mod & KMOD_SHIFT) != 0,
 			(event.keysym.mod & KMOD_CTRL) != 0,
 			(event.keysym.mod & KMOD_ALT) != 0,
@@ -306,7 +413,7 @@ hotkey_item& get_hotkey(const SDL_KeyboardEvent& event, bool mods)
 #ifdef __APPLE__
 			|| (event.keysym.mod & KMOD_RMETA) != 0
 #endif
-			, mods);
+			);
 }
 
 hotkey_item& get_visible_hotkey(int index)
@@ -371,10 +478,13 @@ void key_event_execute(display& disp, const SDL_KeyboardEvent& event, command_ex
 {
 	const hotkey_item* hk = &get_hotkey(event);
 
+#if 0
+	// This is not generally possible without knowing keyboard layout.
 	if(hk->null()) {
 		//no matching hotkey was found, but try an in-exact match.
 		hk = &get_hotkey(event, true);
 	}
+#endif
 
 	if(hk->null())
 		return;
@@ -409,7 +519,9 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 			preferences::set_turbo(!preferences::turbo());
 			break;
 		case HOTKEY_MUTE:
-			preferences::mute(!preferences::is_muted());
+			// Toggle both sound & music, based on music (the more obvious cue)
+			preferences::set_sound(!preferences::music_on());
+			preferences::set_music(!preferences::music_on());
 			break;
 		case HOTKEY_CYCLE_UNITS:
 			if(executor)
@@ -463,6 +575,9 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 			if(executor)
 				executor->toggle_grid();
 			break;
+		case HOTKEY_MOUSE_SCROLL:
+			preferences::enable_mouse_scroll(!preferences::mouse_scroll_enabled());
+			break;
 		case HOTKEY_STATUS_TABLE:
 			if(executor)
 				executor->status_table();
@@ -523,6 +638,10 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 			if(executor)
 				executor->label_terrain();
 			break;
+		case HOTKEY_CLEAR_LABELS:
+			if(executor)
+				executor->clear_labels();
+			break;
 		case HOTKEY_SHOW_ENEMY_MOVES:
 			if(executor)
 				executor->show_enemy_moves(false);
@@ -650,12 +769,72 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 			if(executor)
 				executor->change_language();
 			break;
+		 case HOTKEY_PLAY_REPLAY:
+			 if (executor)
+				 executor->play_replay();
+			 break;
+		 case HOTKEY_RESET_REPLAY:
+			 if (executor)
+				 executor->reset_replay();
+			 break;
+		 case HOTKEY_STOP_REPLAY:
+			 if (executor)
+				executor->stop_replay();
+			 break;
+		 case HOTKEY_REPLAY_NEXT_TURN:
+			 if (executor)
+				 executor->replay_next_turn();
+			 break;
+		 case HOTKEY_REPLAY_NEXT_SIDE:
+			 if (executor)
+				 executor->replay_next_side();
+			 break;
+		 case HOTKEY_REPLAY_SHROUD:
+			 if (executor)
+				 executor->replay_switch_shroud();
+			 break;
+		 case HOTKEY_REPLAY_FOG:
+			 if (executor)
+				 executor->replay_switch_fog();
+			 break;
+		 case HOTKEY_REPLAY_SKIP_ANIMATION:
+			 if (executor)
+				 executor->replay_skip_animation();
+			 break;
 		default:
 			std::cerr << "command_executor: unknown command number " << command << ", ignoring.\n";
 			break;
 	}
 }
 
+void command_executor::show_menu(const std::vector<std::string>& items_arg, int xloc, int yloc, bool /*context_menu*/, display& gui, const std::vector<std::string>& savenames)
+{
+	std::vector<std::string> items = items_arg;
+	if (can_execute_command(hotkey::get_hotkey(items.front()).get_id())){
+		//if just one item is passed in, that means we should execute that item
+		if(items.size() == 1 && items_arg.size() == 1) {
+			hotkey::execute_command(gui,hotkey::get_hotkey(items.front()).get_id(),this);
+			return;
+		}
+
+		std::vector<std::string> menu = get_menu_images(items);
+
+		static const std::string style = "menu2";
+		const int res = gui::show_dialog(gui,NULL,"","",
+				gui::MESSAGE,&menu,NULL,"",NULL,-1,NULL,NULL,xloc,yloc,&style);
+		if (size_t(res) >= items.size())
+			return;
+
+		const hotkey::HOTKEY_COMMAND cmd = hotkey::get_hotkey(items[res]).get_id();
+		// Fake entries: they want us to load a specific autosave.
+		if (cmd == hotkey::HOTKEY_NULL && (unsigned)res < savenames.size() && !savenames[(unsigned)res].empty()) {
+			throw game::load_game_exception(savenames[(unsigned)res],false);
+		}
+
+		hotkey::execute_command(gui,cmd,this);
+	}
+}
+
 std::string command_executor::get_menu_image(hotkey::HOTKEY_COMMAND command) const {
 	switch(get_action_state(command)) {
 		case ACTION_ON: return game_config::checked_menu_image;
@@ -664,4 +843,35 @@ std::string command_executor::get_menu_image(hotkey::HOTKEY_COMMAND command) con
 	}
 }
 
+std::vector<std::string> command_executor::get_menu_images(const std::vector<std::string>& items){
+	std::vector<std::string> result;
+	bool has_image = false;
+
+	for(std::vector<std::string>::const_iterator i = items.begin(); i != items.end(); ++i) {
+		const hotkey::hotkey_item hk = hotkey::get_hotkey(*i);
+
+		std::stringstream str;
+		//see if this menu item has an associated image
+		std::string img(get_menu_image(hk.get_id()));
+		if(img.empty() == false) {
+			has_image = true;
+			str << IMAGE_PREFIX << img << COLUMN_SEPARATOR;
+		}
+
+		if (hk.get_id() == hotkey::HOTKEY_NULL) {
+			str << *i << COLUMN_SEPARATOR;
+		} else {
+			str << hk.get_description() << COLUMN_SEPARATOR << hk.get_name();
+		}
+
+		result.push_back(str.str());
+	}
+	//If any of the menu items have an image, create an image column
+	if(has_image)
+		for(std::vector<std::string>::iterator i = result.begin(); i != result.end(); ++i)
+			if(*(i->begin()) != IMAGE_PREFIX)
+				i->insert(i->begin(), COLUMN_SEPARATOR);
+	return result;
+}
+
 }
