@@ -1,4 +1,4 @@
-/* $Id: map.cpp 8821 2005-11-12 19:23:56Z brunowolff $ */
+/* $Id: map.cpp 11877 2006-06-05 20:02:25Z soliton $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -13,11 +13,14 @@
 
 #include "global.hpp"
 
+#include "actions.hpp"
 #include "config.hpp"
+#include "gamestatus.hpp"
 #include "log.hpp"
 #include "map.hpp"
 #include "pathutils.hpp"
 #include "util.hpp"
+#include "variable.hpp"
 #include "wassert.hpp"
 #include "serialization/string_utils.hpp"
 
@@ -30,6 +33,13 @@
 #define ERR_CF LOG_STREAM(err, config)
 #define LOG_G LOG_STREAM(info, general)
 
+const gamemap::TERRAIN gamemap::VOID_TERRAIN = ' ';
+const gamemap::TERRAIN gamemap::FOGGED = '~';
+const gamemap::TERRAIN gamemap::KEEP = 'K';
+const gamemap::TERRAIN gamemap::CASTLE = 'C';
+const gamemap::TERRAIN gamemap::VILLAGE = 't';
+const gamemap::TERRAIN gamemap::FOREST = 'f';
+
 std::ostream &operator<<(std::ostream &s, gamemap::location const &l) {
 	s << (l.x + 1) << ',' << (l.y + 1);
 	return s;
@@ -37,7 +47,7 @@ std::ostream &operator<<(std::ostream &s, gamemap::location const &l) {
 
 gamemap::location gamemap::location::null_location;
 
-const std::string& gamemap::underlying_terrain(TERRAIN terrain) const
+const std::string& gamemap::underlying_mvt_terrain(TERRAIN terrain) const
 {
 	const std::map<TERRAIN,terrain_type>::const_iterator i = letterToTerrain_.find(terrain);
 	if(i == letterToTerrain_.end()) {
@@ -46,7 +56,31 @@ const std::string& gamemap::underlying_terrain(TERRAIN terrain) const
 		res[0] = terrain;
 		return res;
 	} else {
-		return i->second.type();
+		return i->second.mvt_type();
+	}
+}
+const std::string& gamemap::underlying_def_terrain(TERRAIN terrain) const
+{
+	const std::map<TERRAIN,terrain_type>::const_iterator i = letterToTerrain_.find(terrain);
+	if(i == letterToTerrain_.end()) {
+		static std::string res;
+		res.resize(1);
+		res[0] = terrain;
+		return res;
+	} else {
+		return i->second.def_type();
+	}
+}
+const std::string& gamemap::underlying_union_terrain(TERRAIN terrain) const
+{
+	const std::map<TERRAIN,terrain_type>::const_iterator i = letterToTerrain_.find(terrain);
+	if(i == letterToTerrain_.end()) {
+		static std::string res;
+		res.resize(1);
+		res[0] = terrain;
+		return res;
+	} else {
+		return i->second.union_type();
 	}
 }
 
@@ -55,7 +89,7 @@ bool gamemap::is_village(gamemap::TERRAIN terrain) const
 	return get_terrain_info(terrain).is_village();
 }
 
-bool gamemap::gives_healing(gamemap::TERRAIN terrain) const
+int gamemap::gives_healing(gamemap::TERRAIN terrain) const
 {
 	return get_terrain_info(terrain).gives_healing();
 }
@@ -75,9 +109,11 @@ bool gamemap::is_village(const gamemap::location& loc) const
 	return on_board(loc) && is_village(get_terrain(loc));
 }
 
-bool gamemap::gives_healing(const gamemap::location& loc) const
+int gamemap::gives_healing(const gamemap::location& loc) const
 {
-	return on_board(loc) && gives_healing(get_terrain(loc));
+	if (!on_board(loc))
+		return 0;
+	return gives_healing(get_terrain(loc));
 }
 
 bool gamemap::is_castle(const gamemap::location& loc) const
@@ -90,6 +126,18 @@ bool gamemap::is_keep(const gamemap::location& loc) const
 	return on_board(loc) && is_keep(get_terrain(loc));
 }
 
+bool gamemap::filter_location(const gamemap::location &loc,const config & /*con*/) const
+{ //need to fill this in
+  return on_board(loc);
+}
+
+void gamemap::write_terrain(const gamemap::location &loc, config& cfg) const
+{
+	std::string loc_terrain;
+	loc_terrain += get_terrain(loc);
+	cfg["terrain"] = loc_terrain;
+}
+
 gamemap::location::DIRECTION gamemap::location::parse_direction(const std::string& str)
 {
 	if(str == "n") {
@@ -109,11 +157,29 @@ gamemap::location::DIRECTION gamemap::location::parse_direction(const std::strin
 	}
 }
 
-gamemap::location::location(const config& cfg) : x(-1), y(-1)
+std::string gamemap::location::write_direction(gamemap::location::DIRECTION dir)
 {
-	const std::string& xstr = cfg["x"];
-	const std::string& ystr = cfg["y"];
+	switch(dir) {
+		case NORTH:
+			return std::string("n");
+		case NORTH_EAST:
+			return std::string("ne");
+		case NORTH_WEST:
+			return std::string("nw");
+		case SOUTH:
+			return std::string("s");
+		case SOUTH_EAST:
+			return std::string("se");
+		case SOUTH_WEST:
+			return std::string("sw");
+		default:
+			return std::string();
 
+	}
+}
+
+void gamemap::location::init(const std::string &xstr, const std::string &ystr)
+{
 	//the co-ordinates in config files will be 1-based, while we
 	//want them as 0-based
 	if(xstr.empty() == false)
@@ -123,6 +189,16 @@ gamemap::location::location(const config& cfg) : x(-1), y(-1)
 		y = atoi(ystr.c_str()) - 1;
 }
 
+gamemap::location::location(const config& cfg) : x(-1), y(-1)
+{
+	init(cfg["x"], cfg["y"]);
+}
+
+gamemap::location::location(const vconfig& cfg) : x(-1), y(-1)
+{
+	init(cfg["x"], cfg["y"]);
+}
+
 void gamemap::location::write(config& cfg) const
 {
 	char buf[50];
@@ -189,6 +265,23 @@ gamemap::location gamemap::location::get_direction(
 	}
 }
 
+gamemap::location::DIRECTION gamemap::location::get_relative_dir(gamemap::location loc) const {
+	location diff = loc -*this;
+	if(diff == location(0,0)) return NDIRECTIONS;
+	if( diff.y < 0 && diff.x >= 0 && abs(diff.x) >= abs(diff.y)) return NORTH_EAST;
+	if( diff.y < 0 && diff.x < 0 && abs(diff.x) >= abs(diff.y)) return NORTH_WEST;
+	if( diff.y < 0 && abs(diff.x) < abs(diff.y)) return NORTH;
+
+	if( diff.y >= 0 && diff.x >= 0 && abs(diff.x) >= abs(diff.y)) return SOUTH_EAST;
+	if( diff.y >= 0 && diff.x < 0 && abs(diff.x) >= abs(diff.y)) return SOUTH_WEST;
+	if( diff.y >= 0 && abs(diff.x) < abs(diff.y)) return SOUTH;
+
+	//impossible
+	wassert(false);
+	return NDIRECTIONS;
+
+
+}
 gamemap::gamemap(const config& cfg, const std::string& data) : tiles_(1)
 {
 	LOG_G << "loading map: '" << data << "'\n";
@@ -283,12 +376,20 @@ void gamemap::overlay(const gamemap& m, const config& rules_cfg, const int xpos,
 {
 	const config::child_list& rules = rules_cfg.get_children("rule");
 
-	const int xend = minimum<int>(xpos+m.x(),x());
-	const int yend = minimum<int>(ypos+m.y(),y());
-	for(int x = xpos; x < xend; ++x) {
-		for(int y = ypos; y < yend; ++y) {
-			const TERRAIN t = m[x-xpos][y-ypos];
-			const TERRAIN current = (*this)[x][y];
+	const int xstart = maximum<int>(0, -xpos);
+	const int ystart = maximum<int>(0, -ypos-((xpos & 1) ? 1 : 0));
+	const int xend = minimum<int>(m.x(),x()-xpos);
+	const int yend = minimum<int>(m.y(),y()-ypos);
+	for(int x1 = xstart; x1 < xend; ++x1) {
+		for(int y1 = ystart; y1 < yend; ++y1) {
+			const int x2 = x1 + xpos;
+			const int y2 = y1 + ypos +
+				((xpos & 1) && (x1 & 1) ? 1 : 0);
+			if (y2 < 0 || y2 >= y()) {
+				continue;
+			}
+			const TERRAIN t = m[x1][y1];
+			const TERRAIN current = (*this)[x2][y2];
 
 			if(t == FOGGED || t == VOID_TERRAIN) {
 				continue;
@@ -328,12 +429,12 @@ void gamemap::overlay(const gamemap& m, const config& rules_cfg, const int xpos,
 				const config& cfg = **rule;
 				const std::string& terrain = cfg["terrain"];
 				if(terrain != "") {
-					set_terrain(location(x,y),terrain[0]);
+					set_terrain(location(x2,y2),terrain[0]);
 				} else if(cfg["use_old"] != "yes") {
-					set_terrain(location(x,y),t);
+					set_terrain(location(x2,y2),t);
 				}
 			} else {
-				set_terrain(location(x,y),t);
+				set_terrain(location(x2,y2),t);
 			}
 		}
 	}
@@ -450,6 +551,72 @@ const terrain_type& gamemap::get_terrain_info(const gamemap::location &loc) cons
 {
 	return get_terrain_info(get_terrain(loc));
 }
+bool gamemap::terrain_matches_filter(const gamemap::location& loc, const config& cfg, const gamestatus& game_status, const unit_map& units,bool flat_tod) const
+{
+	const std::string& terrain = cfg["terrain"];
+	const std::string& tod_type = cfg["time_of_day"];
+	const std::string& tod_id = cfg["time_of_day_id"];
+	// Any of these may be a CSV
+	std::string terrain_letter;
+	terrain_letter += get_terrain_info(loc).letter();
+	if(!terrain.empty()) {
+		if(terrain != terrain_letter) {
+			if(std::find(terrain.begin(),terrain.end(),',') != terrain.end() &&
+				std::search(terrain.begin(),terrain.end(),
+				terrain_letter.begin(),terrain_letter.end()) != terrain.end()) {
+				const std::vector<std::string>& vals = utils::split(terrain);
+				if(std::find(vals.begin(),vals.end(),terrain_letter) == vals.end()) {
+					return false;
+				}
+			} else {
+				return false;
+			}
+		}
+	}
+
+	static config const dummy_cfg;
+	time_of_day tod(dummy_cfg);
+	if(!tod_type.empty() || !tod_id.empty()) {
+		if(flat_tod) {
+			tod = game_status.get_time_of_day(0,loc);
+		} else {
+			tod = timeofday_at(game_status,units,loc,*this);
+		}
+	}
+	if(!tod_type.empty()) {
+		const std::vector<std::string>& vals = utils::split(tod_type);
+		if(tod.lawful_bonus<0) {
+			if(std::find(vals.begin(),vals.end(),"chaotic") == vals.end()) {
+				return false;
+			}
+		} else if(tod.lawful_bonus>0) {
+			if(std::find(vals.begin(),vals.end(),"lawful") == vals.end()) {
+				return false;
+			}
+		} else {
+			if(std::find(vals.begin(),vals.end(),"neutral") == vals.end()) {
+				return false;
+			}
+		}
+	}
+	if(!tod_id.empty()) {
+		if(tod_id != tod.id) {
+			if(std::find(tod_id.begin(),tod_id.end(),',') != tod_id.end() &&
+				std::search(tod_id.begin(),tod_id.end(),
+				tod.id.begin(),tod.id.end()) != tod_id.end()) {
+				const std::vector<std::string>& vals = utils::split(tod_id);
+				if(std::find(vals.begin(),vals.end(),tod.id) == vals.end()) {
+					return false;
+				}
+			} else {
+				return false;
+			}
+		}
+	}
+
+	return true;
+}
+
 
 const std::vector<gamemap::TERRAIN>& gamemap::get_terrain_list() const
 {
