@@ -1,6 +1,6 @@
-/* $Id: map.hpp,v 1.39 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: map.hpp 11677 2006-05-28 11:57:45Z rusty $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,6 +15,10 @@
 #define MAP_H_INCLUDED
 
 class config;
+class gamestatus;
+class unit;
+class vconfig;
+class unit_map;
 
 #include "terrain.hpp"
 
@@ -36,13 +40,20 @@ public:
 	//in dynamically because they're special. It's asserted that there will
 	//be corresponding entries for these types of terrain in the terrain
 	//configuration file.
-	enum { FOGGED = '~', VOID_TERRAIN = ' ', KEEP = 'K', CASTLE = 'C', VILLAGE = 't', FOREST = 'f' };
+	static const TERRAIN VOID_TERRAIN;
+	static const TERRAIN FOGGED;
+	static const TERRAIN KEEP;
+	static const TERRAIN CASTLE;
+	static const TERRAIN VILLAGE;
+	static const TERRAIN FOREST;
 
 	//the name of the terrain is the terrain itself, the underlying terrain
 	//is the name of the terrain for game-logic purposes. I.e. if the terrain
 	//is simply an alias, the underlying terrain name is the name of the
 	//terrain that it's aliased to
-	const std::string& underlying_terrain(TERRAIN terrain) const;
+	const std::string& underlying_mvt_terrain(TERRAIN terrain) const;
+	const std::string& underlying_def_terrain(TERRAIN terrain) const;
+	const std::string& underlying_union_terrain(TERRAIN terrain) const;
 
 	//exception thrown if the map file is not in the correct format.
 	struct incorrect_format_exception {
@@ -57,10 +68,12 @@ public:
 		                 SOUTH_WEST, NORTH_WEST, NDIRECTIONS };
 
 		static DIRECTION parse_direction(const std::string& str);
+		static std::string write_direction(DIRECTION dir);
 
 		location() : x(-1), y(-1) {}
 		location(int x, int y) : x(x), y(y) {}
 		explicit location(const config& cfg);
+		explicit location(const vconfig& cfg);
 
 		void write(config& cfg) const;
 
@@ -86,23 +99,33 @@ public:
 		location &operator-=(const location &a);
 
 		location get_direction(DIRECTION d) const;
+		DIRECTION get_relative_dir(location loc) const;
 
 		static location null_location;
+	private:
+		void init(const std::string &x, const std::string &y);
 	};
 
-	const std::string& underlying_terrain(const location& loc) const
-	{ return underlying_terrain(get_terrain(loc)); }
+	const std::string& underlying_mvt_terrain(const location& loc) const
+	{ return underlying_mvt_terrain(get_terrain(loc)); }
+	const std::string& underlying_def_terrain(const location& loc) const
+	{ return underlying_def_terrain(get_terrain(loc)); }
+	const std::string& underlying_union_terrain(const location& loc) const
+	{ return underlying_union_terrain(get_terrain(loc)); }
 
 	bool is_village(TERRAIN terrain) const;
-	bool gives_healing(TERRAIN terrain) const;
+	int gives_healing(TERRAIN terrain) const;
 	bool is_castle(TERRAIN terrain) const;
 	bool is_keep(TERRAIN terrain) const;
 
 	bool is_village(const location& loc) const;
-	bool gives_healing(const location& loc) const;
+	int gives_healing(const location& loc) const;
 	bool is_castle(const location& loc) const;
 	bool is_keep(const location& loc) const;
 
+        //function to filter whether a location matches a set of criteria
+        bool filter_location(const location &loc,const config &con) const;
+
 	//loads a map, with the given terrain configuration.
 	//data should be a series of lines, with each character representing
 	//one hex on the map. Starting locations are represented by numbers,
@@ -127,6 +150,10 @@ public:
 	//this allows proper drawing of the edges of the map
 	TERRAIN get_terrain(const location& loc) const;
 
+	//writes the terrain at loc to cfg
+	void write_terrain(const gamemap::location &loc, config& cfg) const;
+
+
 	//functions to manipulate starting positions of the different sides.
 	const location& starting_position(int side) const;
 	int is_starting_position(const location& loc) const;
@@ -156,6 +183,8 @@ public:
 
 	//shortcut to get_terrain_info(get_terrain(loc))
 	const terrain_type& get_terrain_info(const location &loc) const;
+	//
+	bool terrain_matches_filter(const location& loc, const config& cfg, const gamestatus& game_status, const unit_map& units,bool flat_tod=false) const;
 
 	//gets the list of terrains
 	const std::vector<TERRAIN>& get_terrain_list() const;
