@@ -1,4 +1,4 @@
-/* $Id: map_label.cpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: map_label.cpp 9980 2006-02-05 00:34:03Z boucman $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -86,7 +86,7 @@ const std::string& map_labels::get_label(const gamemap::location& loc) const
 	}
 }
 
-void map_labels::set_label(const gamemap::location& loc, const std::string& str)
+void map_labels::set_label(const gamemap::location& loc, const std::string& str, const SDL_Color colour)
 {
 	std::string text = str;
 	if(text.size() > max_label_size) {
@@ -103,15 +103,7 @@ void map_labels::set_label(const gamemap::location& loc, const std::string& str)
 		return;
 	}
 
-	SDL_Color colour = font::NORMAL_COLOUR;
 
-#if 0
-	if(map_.get_terrain_info(map_.get_terrain(loc)).is_light()) {
-		colour.r = 0;
-		colour.g = 0;
-		colour.b = 0;
-	}
-#endif
 
 	const gamemap::location loc_nextx(loc.x+1,loc.y);
 	const gamemap::location loc_nexty(loc.x,loc.y+1);
