@@ -1,4 +1,4 @@
-/* $Id: multiplayer.cpp 8172 2005-09-06 13:53:14Z ott $ */
+/* $Id: multiplayer.cpp 12730 2006-07-24 00:20:36Z sapient $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -24,12 +24,15 @@
 #include "multiplayer_create.hpp"
 #include "network.hpp"
 #include "playcampaign.hpp"
+#include "playmp_controller.hpp"
 #include "preferences.hpp"
+#include "preferences_display.hpp"
 #include "random.hpp"
 #include "replay.hpp"
 #include "video.hpp"
 #include "statistics.hpp"
 #include "serialization/string_utils.hpp"
+#include "upload_log.hpp"
 
 #define LOG_NW LOG_STREAM(info, network)
 
@@ -73,7 +76,7 @@ void run_lobby_loop(display& disp, mp::ui& ui)
 		ui.process_network();
 
 		disp.flip();
-		SDL_Delay(20);
+		disp.delay(20);
 	}
 }
 
@@ -129,8 +132,8 @@ server_type open_connection(display& disp, const std::string& original_host)
 		}
 
 		data.clear();
-		network::connection data_res = gui::network_data_dialog(
-				disp,_("Connecting to remote host..."),data);
+		network::connection data_res = gui::network_receive_dialog(
+				disp,_("Reading from Server..."),data);
 		mp::check_response(data_res, data);
 
 		// Backwards-compatibility "version" attribute
@@ -188,6 +191,10 @@ server_type open_connection(display& disp, const std::string& original_host)
 					if(res != 0 || login.empty()) {
 						return ABORT_SERVER;
 					}
+					if(login.size() > 18) {
+						gui::show_error_message(disp, _("The login name you chose is too long, please use a login with less than 18 characters"));
+						return ABORT_SERVER;
+					}
 
 					preferences::set_login(login);
 				}
@@ -236,6 +243,7 @@ void enter_wait_mode(display& disp, const config& game_config, game_data& data,
 	mp::ui::result res;
 	game_state state;
 	network_game_manager m;
+	upload_log nolog(false);
 
 	gamelist.clear();
 	statistics::fresh_stats();
@@ -250,14 +258,20 @@ void enter_wait_mode(display& disp, const config& game_config, game_data& data,
 
 		if (res == mp::ui::PLAY) {
 			ui.start_game();
-
-			state = ui.get_state();
+			if (preferences::skip_mp_replay()){
+				//FIXME implement true skip replay
+				//state = ui.request_snapshot();
+				state = ui.get_state();
+			}
+			else{
+				state = ui.get_state();
+			}
 		}
 	}
 
 	switch (res) {
 	case mp::ui::PLAY:
-		play_game(disp, state, game_config, data, disp.video(), IO_CLIENT);
+		play_game(disp, state, game_config, data, disp.video(), nolog, IO_CLIENT, preferences::skip_mp_replay());
 		recorder.clear();
 
 		break;
@@ -278,6 +292,7 @@ void enter_connect_mode(display& disp, const config& game_config, game_data& dat
 			network::server_manager::TRY_CREATE_SERVER :
 			network::server_manager::NO_SERVER);
 	network_game_manager m;
+	upload_log nolog(false);
 
 	gamelist.clear();
 	statistics::fresh_stats();
@@ -298,7 +313,7 @@ void enter_connect_mode(display& disp, const config& game_config, game_data& dat
 
 	switch (res) {
 	case mp::ui::PLAY:
-		play_game(disp, state, game_config, data, disp.video(), IO_SERVER);
+		play_game(disp, state, game_config, data, disp.video(), nolog, IO_SERVER);
 		recorder.clear();
 
 		break;
@@ -326,6 +341,10 @@ void enter_create_mode(display& disp, const config& game_config, game_data& data
 		break;
 	case mp::ui::QUIT:
 	default:
+		//update lobby content
+		config request;
+		request.add_child("refresh_lobby");
+		network::send_data(request);
 		break;
 	}
 }
@@ -383,6 +402,14 @@ void enter_lobby_mode(display& disp, const config& game_config, game_data& data,
 			}
 			break;
 		case mp::ui::QUIT:
+			return;
+		case mp::ui::PREFERENCES:
+			{
+				const preferences::display_manager disp_manager(&disp);
+				preferences::show_preferences_dialog(disp,game_config);
+				disp.redraw_everything();
+			}
+			break;
 		default:
 			return;
 		}
@@ -399,6 +426,7 @@ void start_server(display& disp, const config& game_config, game_data& data,
 	const set_random_generator generator_setter(&recorder);
 	mp::chat chat;
 	config gamelist;
+	playmp_controller::set_replay_last_turn(0);
 
 	enter_create_mode(disp, game_config, data, chat, gamelist, default_controller, is_server);
 }
@@ -418,6 +446,7 @@ void start_client(display& disp, const config& game_config, game_data& data,
 		enter_lobby_mode(disp, game_config, data, chat, gamelist);
 		break;
 	case SIMPLE_SERVER:
+		playmp_controller::set_replay_last_turn(0);
 		enter_wait_mode(disp, game_config, data, chat, gamelist, false);
 		break;
 	case ABORT_SERVER:
