@@ -1,4 +1,4 @@
-/* $Id: multiplayer_connect.cpp 8470 2005-10-04 09:34:44Z j_daniel $ */
+/* $Id: multiplayer_connect.cpp 12675 2006-07-20 18:00:46Z xan $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -24,6 +24,7 @@
 #include "show_dialog.hpp"
 #include "wassert.hpp"
 #include "serialization/string_utils.hpp"
+#include "ai.hpp"
 
 #define LOG_NW LOG_STREAM(info, network)
 #define ERR_NW LOG_STREAM(err, network)
@@ -52,17 +53,20 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	               font::SIZE_LARGE, font::LOBBY_COLOUR),
 	combo_controller_(parent.disp(), parent.player_types_),
 	orig_controller_(parent.video(), cfg["description"], font::SIZE_SMALL),
+	combo_ai_algorithm_(parent.disp(), std::vector<std::string>()),
 	combo_faction_(parent.disp(), parent.player_factions_),
 	combo_leader_(parent.disp(), std::vector<std::string>()),
 	combo_team_(parent.disp(), parent.player_teams_),
 	combo_colour_(parent.disp(), parent.player_colours_),
 	slider_gold_(parent.video()),
+	slider_income_(parent.video()),
 	label_gold_(parent.video(), "100", font::SIZE_SMALL, font::LOBBY_COLOUR),
+	label_income_(parent.video(), _("Normal"), font::SIZE_SMALL, font::LOBBY_COLOUR),
 	enabled_(!parent_->params_.saved_game),
 	changed_(false),
 	llm_(parent.era_sides_, &parent.game_data_, enabled_ ? &combo_leader_ : NULL)
 {
-	if(enabled_) {
+	if(cfg_["allow_player"] != "no" && enabled_) {
 		controller_ = parent_->default_controller_;
 	} else {
 		for(size_t i = CNTR_NETWORK; i != CNTR_LAST; ++i) {
@@ -74,16 +78,23 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	}
 
 	slider_gold_.set_min(20);
-	slider_gold_.set_max(1000);
+	slider_gold_.set_max(800);
 	slider_gold_.set_increment(25);
 	slider_gold_.set_value(lexical_cast_default<int>(cfg_["gold"], 100));
-	slider_gold_.set_measurements(100, 16);
+	slider_gold_.set_measurements(80, 16);
+
+	slider_income_.set_min(-2);
+	slider_income_.set_max(18);
+	slider_income_.set_increment(1);
+	slider_income_.set_value(lexical_cast_default<int>(cfg_["income"], 0));
+	slider_income_.set_measurements(50, 16);
 
 	combo_faction_.enable(enabled_);
 	combo_leader_.enable(enabled_);
 	combo_team_.enable(enabled_);
 	combo_colour_.enable(enabled_);
 	slider_gold_.hide(!enabled_);
+	slider_income_.hide(!enabled_);
 
 	id_ = ""; // Id is reset, and not imported from loading savegames
 	save_id_ = cfg_["save_id"];
@@ -97,6 +108,13 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	}
 	colour_ = lexical_cast_default<int>(cfg_["colour"], index_ + 1) - 1;
 	gold_ = lexical_cast_default<int>(cfg_["gold"], 100);
+	income_ = lexical_cast_default<int>(cfg_["income"], 0);
+	config *ai = cfg_.child("ai");
+	if (ai)
+        ai_algorithm_ = lexical_cast_default<std::string>((*ai)["ai_algorithm"], "default");
+    else
+        ai_algorithm_ = "default";
+	init_ai_algorithm_combo();
 
 	// "Faction name" hack
 	if (!enabled_) {
@@ -141,12 +159,15 @@ connect::side::side(const side& a) :
 	index_(a.index_), id_(a.id_),  save_id_(a.save_id_),
 	controller_(a.controller_),
 	faction_(a.faction_), team_(a.team_), colour_(a.colour_),
-	gold_(a.gold_), leader_(a.leader_), /* taken_(a.taken_), */
+	gold_(a.gold_), income_(a.income_), leader_(a.leader_), /* taken_(a.taken_), */
+	ai_algorithm_(a.ai_algorithm_),
 	player_number_(a.player_number_), combo_controller_(a.combo_controller_),
 	orig_controller_(a.orig_controller_),
+	combo_ai_algorithm_(a.combo_ai_algorithm_),
 	combo_faction_(a.combo_faction_), combo_leader_(a.combo_leader_),
 	combo_team_(a.combo_team_), combo_colour_(a.combo_colour_),
-	slider_gold_(a.slider_gold_), label_gold_(a.label_gold_),
+	slider_gold_(a.slider_gold_), slider_income_(a.slider_income_),
+	label_gold_(a.label_gold_), label_income_(a.label_income_),
 	enabled_(a.enabled_), changed_(a.changed_), llm_(a.llm_)
 {
 	llm_.set_combo(enabled_ ? &combo_leader_ : NULL);
@@ -157,12 +178,16 @@ void connect::side::add_widgets_to_scrollpane(gui::scrollpane& pane, int pos)
 	pane.add_widget(&player_number_, 0, 5 + pos);
 	pane.add_widget(&combo_controller_, 20, 5 + pos);
 	pane.add_widget(&orig_controller_, 20 + (combo_controller_.width() - orig_controller_.width()) / 2, 35 + pos + (combo_leader_.height() - orig_controller_.height()) / 2);
+	pane.add_widget(&combo_ai_algorithm_, 20, 35 + pos);
 	pane.add_widget(&combo_faction_, 135, 5 + pos);
 	pane.add_widget(&combo_leader_, 135, 35 + pos);
 	pane.add_widget(&combo_team_, 250, 5 + pos);
 	pane.add_widget(&combo_colour_, 365, 5 + pos);
-	pane.add_widget(&slider_gold_, 480, 5 + pos);
-	pane.add_widget(&label_gold_, 490 + slider_gold_.width(), 8 + pos);
+	pane.add_widget(&slider_gold_, 475, 5 + pos);
+	//pane.add_widget(&label_gold_, 490 + slider_gold_.width(), 8 + pos);
+	pane.add_widget(&label_gold_, 475, 35 + pos);
+	pane.add_widget(&slider_income_, 475 + slider_gold_.width(), 5 + pos);
+	pane.add_widget(&label_income_, 475 + slider_gold_.width(), 35 + pos);
 }
 
 void connect::side::process_event()
@@ -205,6 +230,7 @@ void connect::side::process_event()
 				changed_ = true;
 			}
 		}
+		update_ai_algorithm_combo();
 	}
 
 	if(!enabled_)
@@ -215,6 +241,10 @@ void connect::side::process_event()
 		llm_.update_leader_list(faction_);
 		changed_ = true;
 	}
+	if (combo_ai_algorithm_.changed() && combo_ai_algorithm_.selected() >= 0) {
+		ai_algorithm_ = parent_->ai_algorithms_[combo_ai_algorithm_.selected()];
+		changed_ = true;
+	}
 	if (combo_leader_.changed() && combo_leader_.selected() >= 0) {
 		changed_ = true;
 	}
@@ -231,6 +261,20 @@ void connect::side::process_event()
 		label_gold_.set_text(lexical_cast_default<std::string>(gold_, "0"));
 		changed_ = true;
 	}
+	if (slider_income_.value() != income_) {
+
+		income_ = slider_income_.value();
+		std::stringstream buf;
+		if(income_ < 0) {
+			buf << _("(") << income_ << _(")");
+		} else if(income_ > 0) {
+			buf << _("+") << income_;
+		} else {
+			buf << _("Normal");
+		}
+		label_income_.set_text(buf.str());
+		changed_ = true;
+	}
 }
 
 bool connect::side::changed()
@@ -259,6 +303,44 @@ void connect::side::update_controller_ui()
 		}
 	}
 
+    update_ai_algorithm_combo();
+}
+
+void connect::side::hide_ai_algorithm_combo(bool invis)
+{
+	if(!invis)
+	{
+		if(controller_ == CNTR_COMPUTER)
+		{
+			//computer selected, show AI combo
+			orig_controller_.hide(true);
+			combo_ai_algorithm_.hide(false);
+		} else {
+			//computer de-selected, hide AI combo
+			combo_ai_algorithm_.hide(true);
+			orig_controller_.hide(false);
+		}
+	} else {
+		combo_ai_algorithm_.hide(true);
+	}
+}
+
+void connect::side::init_ai_algorithm_combo()
+{
+	wassert(parent_->ai_algorithms_.empty() == false);
+
+	int sel = 0;
+	std::vector<std::string> ais = parent_->ai_algorithms_;
+	for (unsigned int i = 0; i < ais.size(); i++) {
+		if (ais[i] == ai_algorithm_) {
+			sel = i;
+		}
+		if (ais[i] == "default") {
+			ais[i] = _("Default AI");
+		}
+	}
+	combo_ai_algorithm_.set_items(ais);
+	combo_ai_algorithm_.set_selected(sel);
 }
 
 void connect::side::update_ui()
@@ -273,6 +355,16 @@ void connect::side::update_ui()
 	combo_colour_.set_selected(colour_);
 	slider_gold_.set_value(gold_);
 	label_gold_.set_text(lexical_cast_default<std::string>(gold_, "0"));
+	slider_income_.set_value(income_);
+	std::stringstream buf;
+	if(income_ < 0) {
+		buf << _("(") << income_ << _(")");
+	} else if(income_ > 0) {
+		buf << _("+") << income_;
+	} else {
+		buf << _("Normal");
+	}
+	label_income_.set_text(buf.str());
 }
 
 config connect::side::get_config() const
@@ -307,6 +399,21 @@ config connect::side::get_config() const
 			if(enabled_ && cfg_["save_id"].empty()) {
 				res["save_id"] = "ai" + res["side"].str();
 			}
+			{
+                config *ai = res.child("ai");
+                if (!ai) ai = &res.add_child("ai");
+#ifdef HAVE_PYTHON
+                if (ai_algorithm_.substr(ai_algorithm_.length() - 3) == ".py") {
+                    (*ai)["ai_algorithm"] = "python_ai";
+                    (*ai)["python_script"] = ai_algorithm_;
+                }
+                else
+#endif
+                {
+                    if (ai_algorithm_ != "default")
+                        (*ai)["ai_algorithm"] = ai_algorithm_;
+                }
+			}
 			description = N_("Computer player");
 			break;
 		case CNTR_EMPTY:
@@ -336,6 +443,7 @@ config connect::side::get_config() const
 		res["team_name"] = parent_->team_names_[team_];
 		res["colour"] = lexical_cast<std::string>(colour_ + 1);
 		res["gold"] = lexical_cast<std::string>(gold_);
+		res["income"] = lexical_cast<std::string>(income_);
 
 		if(!parent_->params_.use_map_settings || res["fog"].empty() || (res["fog"] != "yes" && res["fog"] != "no")) {
 			res["fog"] = parent_->params_.fog_game ? "yes" : "no";
@@ -345,6 +453,24 @@ config connect::side::get_config() const
 			res["shroud"] = parent_->params_.shroud_game ? "yes" : "no";
 		}
 
+
+		if(!parent_->params_.use_map_settings || res["mp_countdown"].empty() || (res["mp_countdown"] != "yes" && res["mp_countdown"] != "no")) {
+			res["mp_countdown"] = parent_->params_.mp_countdown ? "yes" : "no";;
+		}
+
+		if(!parent_->params_.use_map_settings || res["mp_countdown_init_time"].empty()) {
+			res["mp_countdown_init_time"] = lexical_cast<std::string>(parent_->params_.mp_countdown_init_time);
+		}
+		if(!parent_->params_.use_map_settings || res["mp_countdown_turn_bonus"].empty()) {
+			res["mp_countdown_turn_bonus"] = lexical_cast<std::string>(parent_->params_.mp_countdown_turn_bonus);
+		}
+		if(!parent_->params_.use_map_settings || res["mp_countdown_reservoir_time"].empty()) {
+			res["mp_countdown_reservoir_time"] = lexical_cast<std::string>(parent_->params_.mp_countdown_reservoir_time);
+		}
+		if(!parent_->params_.use_map_settings || res["mp_countdown_action_bonus"].empty()) {
+			res["mp_countdown_action_bonus"] = lexical_cast<std::string>(parent_->params_.mp_countdown_action_bonus);
+		}
+
 		res["share_maps"] = parent_->params_.share_maps ? "yes" : "no";
 		res["share_view"] =  parent_->params_.share_view ? "yes" : "no";
 		if(!parent_->params_.use_map_settings || res["village_gold"].empty())
@@ -469,14 +595,21 @@ void connect::side::resolve_random()
 	if (llm_.get_leader() == "random") {
 		// Choose a random leader type.
 		const config& fact = *parent_->era_sides_[faction_];
-		std::vector<std::string> types = utils::split(fact["leader"]);
+		std::vector<std::string> types = utils::split(fact["random_leader"]);
 		if (!types.empty()) {
 			const int lchoice = rand() % types.size();
 			leader_ = types[lchoice];
 		} else {
-			utils::string_map i18n_symbols;
-			i18n_symbols["faction"] = fact["name"];
-			throw config::error(vgettext("Unable to find a leader type for faction $faction", i18n_symbols));
+			// if random_leader= doesn't exists, we use leader=
+			types = utils::split(fact["leader"]);
+			if (!types.empty()) {
+				const int lchoice = rand() % types.size();
+				leader_ = types[lchoice];
+			} else {
+				utils::string_map i18n_symbols;
+				i18n_symbols["faction"] = fact["name"];
+				throw config::error(vgettext("Unable to find a leader type for faction $faction", i18n_symbols));
+			}
 		}
 	}
 }
@@ -502,6 +635,7 @@ connect::connect(display& disp, const config& game_config, const game_data& data
 	team_title_label_(video(), _("Team"), font::SIZE_SMALL, font::LOBBY_COLOUR),
 	colour_title_label_(video(), _("Color"), font::SIZE_SMALL, font::LOBBY_COLOUR),
 	gold_title_label_(video(), _("Gold"), font::SIZE_SMALL, font::LOBBY_COLOUR),
+	income_title_label_(video(), _("Income"), font::SIZE_SMALL, font::LOBBY_COLOUR),
 
 	ai_(video(), _("Computer vs Computer")),
 	launch_(video(), _("I'm Ready")),
@@ -511,9 +645,20 @@ connect::connect(display& disp, const config& game_config, const game_data& data
 	config response;
 	config& create_game = response.add_child("create_game");
 	create_game["name"] = params.name;
-	network::send_data(response);
 
 	load_game();
+	//The number of human-controlled sides is important to know to let the server decide,
+	//how many players can join this game
+	int human_sides = 0;
+	const config::child_list cfg_sides = level_.get_children("side");
+	for (config::child_list::const_iterator side = cfg_sides.begin(); side != cfg_sides.end(); side++){
+		if ((**side)["controller"] == "human"){
+			human_sides++;
+		}
+	}
+	create_game["human_sides"] = lexical_cast<std::string>(human_sides);
+	network::send_data(response);
+
 	if(get_result() == QUIT)
 		return;
 	lists_init();
@@ -622,11 +767,14 @@ void connect::hide_children(bool hide)
 	waiting_label_.hide(hide);
 	// Hiding the scrollpane automatically hides its contents
 	scroll_pane_.hide(hide);
-	type_title_label_.hide(hide);
+	for (side_list::iterator itor = sides_.begin(); itor != sides_.end(); ++itor) {
+		itor->hide_ai_algorithm_combo(hide);
+	}
 	faction_title_label_.hide(hide);
 	team_title_label_.hide(hide);
 	colour_title_label_.hide(hide);
 	gold_title_label_.hide(hide);
+	income_title_label_.hide(hide);
 
 	ai_.hide(hide);
 	launch_.hide(hide);
@@ -661,13 +809,29 @@ void connect::process_network_data(const config& data, const network::connection
 
 		// Checks if the connecting user has a valid and unique name.
 		const std::string name = data["name"];
-		if (name.empty() || find_player(name) != users_.end()) {
+		if(name.empty()) {
 			config response;
 			response.values["failed"] = "yes";
 			network::send_data(response,sock);
 			return;
 		}
 
+		connected_user_list::iterator player = find_player(name);
+		if(player != users_.end()) {
+			if(find_player_side(name) != -1) {
+				config response;
+				response.values["failed"] = "yes";
+				response["message"] = "The nick '" + name + "' is already in use.";
+				network::send_data(response,sock);
+				return;
+			} else {
+				users_.erase(player);
+				config observer_quit;
+				observer_quit.add_child("observer_quit").values["name"] = name;
+				network::send_data(observer_quit);
+				update_user_combos();
+			}
+		}
 
 		// Assigns this user to a side
 		if(side_taken >= 0 && side_taken < int(sides_.size())) {
@@ -717,6 +881,31 @@ void connect::process_network_data(const config& data, const network::connection
 			network::send_data(response, sock);
 		}
 	}
+
+	if(data.child("observer") != NULL) {
+		const t_string& observer_name = data.child("observer")->get_attribute("name");
+		if(!observer_name.empty()) {
+			connected_user_list::iterator player = find_player(observer_name);
+			if(player == users_.end()) {
+				users_.push_back(connected_user(observer_name, CNTR_NETWORK, sock));
+				update_user_combos();
+				update_playerlist_state();
+				update_and_send_diff();
+			}
+		}
+	}
+	if(data.child("observer_quit") != NULL) {
+		const t_string& observer_name = data.child("observer_quit")->get_attribute("name");
+		if(!observer_name.empty()) {
+			connected_user_list::iterator player = find_player(observer_name);
+			if(player != users_.end() && find_player_side(observer_name) == -1) {
+				users_.erase(player);
+				update_user_combos();
+				update_playerlist_state();
+				update_and_send_diff();
+			}
+		}
+	}
 }
 
 void connect::process_network_error(network::error& error)
@@ -810,6 +999,7 @@ void connect::layout_children(const SDL_Rect& rect)
 	team_title_label_.set_location((left+260), top+35);
 	colour_title_label_.set_location((left+375), top+35);
 	gold_title_label_.set_location((left+493), top+35);
+	income_title_label_.set_location((left+560), top+35);
 
 	SDL_Rect scroll_pane_rect;
 	scroll_pane_rect.x = ca.x;
@@ -832,6 +1022,9 @@ void connect::lists_init()
 		player_factions_.push_back((**faction)["name"]);
 	}
 
+	//AI algorithms
+    ai_algorithms_ = get_available_ais();
+
 	//Factions
 	const config::child_itors sides = level_.child_range("side");
 
@@ -848,7 +1041,7 @@ void connect::lists_init()
 			std::vector<std::string>::const_iterator itor = std::find(team_names_.begin(), team_names_.end(), team_name);
 			if(itor == team_names_.end()) {
 				team_names_.push_back(team_name);
-				player_teams_.push_back(team_prefix_ + team_name.str());
+				player_teams_.push_back(team_name.str());
 			}
 		}
 	} else {
@@ -884,10 +1077,15 @@ void connect::lists_init()
 	for(sd = sides.first; sd != sides.second; ++sd, ++index) {
 		sides_.push_back(side(*this, **sd, index));
 	}
+	int offset=0;
 	// This function must be called after the sides_ vector is fully populated.
 	for(side_list::iterator s = sides_.begin(); s != sides_.end(); ++s) {
 		const int side_num = s - sides_.begin();
-		const int spos = 60 * side_num;
+		const int spos = 60 * (side_num-offset);
+		if(s->get_config()["allow_player"] == "no") {
+			offset++;
+			continue;
+		}
 
 		s->add_widgets_to_scrollpane(scroll_pane_, spos);
 	}
@@ -950,7 +1148,16 @@ void connect::load_game()
 
 	} else {
 		level_ = params_.scenario_data;
+
+		level_["hash"] = level_.hash();
 		level_["turns"] = lexical_cast_default<std::string>(params_.num_turns, "20");
+		level_["mp_countdown"] = params_.mp_countdown ? "yes" : "no";
+		level_["mp_countdown_init_time"] = lexical_cast_default<std::string>(params_.mp_countdown_init_time, "270");
+		level_["mp_countdown_turn_bonus"] = lexical_cast_default<std::string>(params_.mp_countdown_turn_bonus, "35");
+		level_["mp_countdown_reservoir_time"] = lexical_cast_default<std::string>(params_.mp_countdown_reservoir_time, "330");
+		level_["mp_countdown_action_bonus"] = lexical_cast_default<std::string>(params_.mp_countdown_action_bonus, "13");
+
+
 
 		const std::string& era = params_.era;
 
@@ -977,7 +1184,7 @@ void connect::load_game()
 	level_["observer"] = params_.allow_observers ? "yes" : "no";
 
 	if(level_["objectives"].empty()) {
-		level_["objectives"] = t_string(N_("Victory\n\
+		level_["objectives"] = t_string(N_("Victory:\n\
 @Defeat enemy leader(s)"), "wesnoth");
 	}
 }
