@@ -1,4 +1,4 @@
-/* $Id: multiplayer_connect.hpp 7120 2005-06-04 19:16:06Z ott $ */
+/* $Id: multiplayer_connect.hpp 11626 2006-05-25 02:12:37Z sapient $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -85,8 +85,10 @@ public:
 
 		// Resolves the random leader / factions.
 		void resolve_random();
-
+		void hide_ai_algorithm_combo(bool invis);
 	private:
+		void init_ai_algorithm_combo();
+		void update_ai_algorithm_combo() {hide_ai_algorithm_combo(parent_->hidden());}
 		void update_controller_ui();
 		void update_ui();
 
@@ -105,19 +107,24 @@ public:
 		int team_;
 		int colour_;
 		int gold_;
+		int income_;
 		std::string leader_;
+		std::string ai_algorithm_;
 		//bool taken_;
 
 		// Widgets for this side
 		gui::label player_number_;
 		gui::combo combo_controller_;
 		gui::label orig_controller_;
+		gui::combo combo_ai_algorithm_;
 		gui::combo combo_faction_;
 		gui::combo combo_leader_;
 		gui::combo combo_team_;
 		gui::combo combo_colour_;
 		gui::slider slider_gold_;
+		gui::slider slider_income_;
 		gui::label label_gold_;
+		gui::label label_income_;
 
 		bool enabled_;
 		bool changed_;
@@ -205,6 +212,7 @@ private:
 	std::vector<std::string> player_factions_;
 	std::vector<std::string> player_teams_;
 	std::vector<std::string> player_colours_;
+	std::vector<std::string> ai_algorithms_;
 
 	// team_name list and "Team" prefix
 	std::vector<std::string> team_names_;
@@ -226,6 +234,7 @@ private:
 	gui::label team_title_label_;
 	gui::label colour_title_label_;
 	gui::label gold_title_label_;
+	gui::label income_title_label_;
 
 	gui::button ai_;
 	gui::button launch_;
