@@ -1,4 +1,4 @@
-/* $Id: multiplayer_create.cpp 8269 2005-09-17 15:06:28Z ott $ */
+/* $Id: multiplayer_create.cpp 12603 2006-07-17 09:53:55Z isaac $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -20,6 +20,7 @@
 #include "image.hpp"
 #include "show_dialog.hpp"
 #include "map_create.hpp"
+#include "minimap.hpp"
 #include "multiplayer_create.hpp"
 #include "filesystem.hpp"
 #include "log.hpp"
@@ -38,10 +39,21 @@ create::create(display& disp, const config &cfg, chat& c, config& gamelist) :
 
 	tooltip_manager_(disp.video()),
 	map_selection_(-1),
+	mp_countdown_init_time_(270),
+	mp_countdown_reservoir_time_(330),
 
 	maps_menu_(disp.video(), std::vector<std::string>()),
 	turns_slider_(disp.video()),
 	turns_label_(disp.video(), "", font::SIZE_SMALL, font::LOBBY_COLOUR),
+	countdown_game_(disp.video(), _("Time limit"), gui::button::TYPE_CHECK),
+	countdown_init_time_slider_(disp.video()),
+	countdown_init_time_label_(disp.video(), "", font::SIZE_SMALL, font::LOBBY_COLOUR),
+	countdown_reservoir_time_slider_(disp.video()),
+	countdown_reservoir_time_label_(disp.video(), "", font::SIZE_SMALL, font::LOBBY_COLOUR),
+	countdown_turn_bonus_label_(disp.video(), "", font::SIZE_SMALL, font::LOBBY_COLOUR),
+	countdown_turn_bonus_slider_(disp.video()),
+	countdown_action_bonus_label_(disp.video(), "", font::SIZE_SMALL, font::LOBBY_COLOUR),
+	countdown_action_bonus_slider_(disp.video()),
 	village_gold_slider_(disp.video()),
 	village_gold_label_(disp.video(), "", font::SIZE_SMALL, font::LOBBY_COLOUR),
 	xp_modifier_slider_(disp.video()),
@@ -66,16 +78,21 @@ create::create(display& disp, const config &cfg, chat& c, config& gamelist) :
 	generator_(NULL)
 {
 	//build the list of scenarios to play
+
+	//add the 'load game' option
+	map_options_.push_back(_("Load Game") + std::string("..."));
+
+	//user maps
 	get_files_in_dir(get_user_data_dir() + "/editor/maps",&user_maps_,NULL,FILE_NAME_ONLY);
 
-	map_options_ = user_maps_;
+	for(unsigned int i = 0; i < user_maps_.size(); i++)
+		map_options_.push_back(user_maps_[i]);
 
+	//standard maps
 	const config::child_list& levels = cfg.get_children("multiplayer");
 	for(config::child_list::const_iterator j = levels.begin(); j != levels.end(); ++j){
 		map_options_.push_back((**j)["name"]);
 	}
-	//add the 'load game' option
-	map_options_.push_back(_("Load Game") + std::string("..."));
 
 	//create the scenarios menu
 	maps_menu_.set_items(map_options_);
@@ -86,7 +103,34 @@ create::create(display& disp, const config &cfg, chat& c, config& gamelist) :
 	turns_slider_.set_min(20);
 	turns_slider_.set_max(100);
 	turns_slider_.set_value(preferences::turns());
-	turns_slider_.set_help_string(_("The maximum turns the game will go for"));
+	turns_slider_.set_help_string(_("The maximum number of turns the game can last"));
+
+	countdown_game_.set_check(preferences::countdown());
+	countdown_game_.set_help_string(_("Enables user time limit"));
+
+	countdown_init_time_slider_.set_min(0);
+	countdown_init_time_slider_.set_max(1500);
+	countdown_init_time_slider_.set_increment(30);
+	countdown_init_time_slider_.set_value(preferences::countdown_init_time());
+	countdown_init_time_slider_.set_help_string(_("Longest time allowed for first turn (seconds)"));
+
+	countdown_reservoir_time_slider_.set_min(30);
+	countdown_reservoir_time_slider_.set_max(1500);
+	countdown_reservoir_time_slider_.set_increment(30);
+	countdown_reservoir_time_slider_.set_value(preferences::countdown_reservoir_time());
+	countdown_reservoir_time_slider_.set_help_string(_("Longest time possible for any turn (seconds)"));
+
+	countdown_turn_bonus_slider_.set_min(10);
+	countdown_turn_bonus_slider_.set_max(300);
+	countdown_turn_bonus_slider_.set_increment(5);
+	countdown_turn_bonus_slider_.set_value(preferences::countdown_turn_bonus());
+	countdown_turn_bonus_slider_.set_help_string(_("Time for general tasks each turn (seconds)"));
+
+	countdown_action_bonus_slider_.set_min(0);
+	countdown_action_bonus_slider_.set_max(30);
+	countdown_action_bonus_slider_.set_increment(1);
+	countdown_action_bonus_slider_.set_value(preferences::countdown_action_bonus());
+	countdown_action_bonus_slider_.set_help_string(_("Time for each attack, recruit, and capture"));
 
 	village_gold_slider_.set_min(1);
 	village_gold_slider_.set_max(5);
@@ -155,6 +199,11 @@ create::~create()
 	preferences::set_fog(parameters_.fog_game);
 	preferences::set_shroud(parameters_.shroud_game);
 	preferences::set_turns(parameters_.num_turns);
+	preferences::set_countdown(parameters_.mp_countdown);
+	preferences::set_countdown_init_time(parameters_.mp_countdown_init_time);
+	preferences::set_countdown_turn_bonus(parameters_.mp_countdown_turn_bonus);
+	preferences::set_countdown_reservoir_time(parameters_.mp_countdown_reservoir_time);
+	preferences::set_countdown_action_bonus(parameters_.mp_countdown_action_bonus);
 	preferences::set_village_gold(parameters_.village_gold);
 	preferences::set_xp_modifier(parameters_.xp_modifier);
 	preferences::set_era(era_combo_.selected()); // FIXME: may be broken if new eras are added
@@ -168,6 +217,17 @@ create::parameters& create::get_parameters()
 	const int turns = turns_slider_.value() < turns_slider_.max_value() ?
 		turns_slider_.value() : -1;
 
+	const int mp_countdown_turn_bonus_val = countdown_turn_bonus_slider_.value() <= countdown_turn_bonus_slider_.max_value() ?
+		countdown_turn_bonus_slider_.value() : -1;
+	const int mp_countdown_action_bonus_val = countdown_action_bonus_slider_.value() <= countdown_action_bonus_slider_.max_value() ?
+		countdown_action_bonus_slider_.value() : -1;
+	const int mp_countdown_reservoir_time_val = countdown_reservoir_time_slider_.value() <= countdown_reservoir_time_slider_.max_value() ?
+		countdown_reservoir_time_slider_.value() : -1;
+	int mp_countdown_init_time_val = countdown_init_time_slider_.value() <= countdown_init_time_slider_.max_value() ?
+		countdown_init_time_slider_.value() : -1;
+	if(mp_countdown_reservoir_time_val > 0 && mp_countdown_init_time_val > mp_countdown_reservoir_time_val)
+		mp_countdown_init_time_val = mp_countdown_reservoir_time_val;
+
 	// Updates the values in the "parameters_" member to match the values
 	// selected by the user with the widgets:
 	parameters_.name = name_entry_.text();
@@ -176,6 +236,12 @@ create::parameters& create::get_parameters()
 	}
 	parameters_.era = (*era_list[era_combo_.selected()])["id"];
 	parameters_.num_turns = turns;
+	//CHECK
+	parameters_.mp_countdown_init_time = mp_countdown_init_time_val;
+	parameters_.mp_countdown_turn_bonus = mp_countdown_turn_bonus_val;
+	parameters_.mp_countdown_reservoir_time = mp_countdown_reservoir_time_val;
+	parameters_.mp_countdown_action_bonus = mp_countdown_action_bonus_val;
+	parameters_.mp_countdown = countdown_game_.checked();
 	parameters_.village_gold = village_gold_slider_.value();
 	parameters_.xp_modifier = xp_modifier_slider_.value();
 	parameters_.use_map_settings = use_map_settings_.checked();
@@ -219,6 +285,49 @@ void create::process_event()
 	}
 	turns_label_.set_text(buf.str());
 
+	countdown_init_time_label_.enable(countdown_game_.checked());
+	countdown_init_time_slider_.enable(countdown_game_.checked());
+	countdown_turn_bonus_label_.enable(countdown_game_.checked());
+	countdown_turn_bonus_slider_.enable(countdown_game_.checked());
+
+	countdown_reservoir_time_label_.enable(countdown_game_.checked());
+	countdown_reservoir_time_slider_.enable(countdown_game_.checked());
+	countdown_action_bonus_label_.enable(countdown_game_.checked());
+	countdown_action_bonus_slider_.enable(countdown_game_.checked());
+
+	if(mp_countdown_init_time_ != countdown_init_time_slider_.value()
+		&& countdown_init_time_slider_.value() > countdown_reservoir_time_slider_.value())
+	{
+		countdown_reservoir_time_slider_.set_value(countdown_init_time_slider_.value());
+	}
+	if(mp_countdown_reservoir_time_ != countdown_reservoir_time_slider_.value()
+		&& countdown_reservoir_time_slider_.value() < countdown_init_time_slider_.value())
+	{
+		countdown_init_time_slider_.set_value(countdown_reservoir_time_slider_.value());
+	}
+	mp_countdown_init_time_ = countdown_init_time_slider_.value();
+	mp_countdown_reservoir_time_ = countdown_reservoir_time_slider_.value();
+
+
+	buf.str("");
+	buf <<  _("Init. Limit: ") << mp_countdown_init_time_; // << _(" sec.");
+	countdown_init_time_label_.set_text(buf.str());
+
+	const int mp_countdown_turn_bonus_val = countdown_turn_bonus_slider_.value();
+	buf.str("");
+	buf <<  _("Turn Bonus: ") << mp_countdown_turn_bonus_val; // << _(" sec.");
+	countdown_turn_bonus_label_.set_text(buf.str());
+
+	buf.str("");
+	buf <<  _("Reservoir: ") << mp_countdown_reservoir_time_; // << _(" sec.");
+	countdown_reservoir_time_label_.set_text(buf.str());
+
+	const int mp_countdown_action_bonus_val = countdown_action_bonus_slider_.value();
+	buf.str("");
+	buf <<  _("Action Bonus: ") << mp_countdown_action_bonus_val; // << _(" sec.");
+	countdown_action_bonus_label_.set_text(buf.str());
+
+
 	//Villages can produce between 1 and 10 gold a turn
 	const int village_gold = village_gold_slider_.value();
 	buf.str("");
@@ -241,17 +350,17 @@ void create::process_event()
 
 		const size_t select = size_t(maps_menu_.selection());
 
-		if(select < user_maps_.size()) {
+		if(select > 0 && select <= user_maps_.size()) {
 			parameters_.saved_game = false;
 			const config* const generic_multiplayer = game_config().child("generic_multiplayer");
 			if(generic_multiplayer != NULL) {
 				parameters_.scenario_data = *generic_multiplayer;
-				parameters_.scenario_data["map_data"] = read_map(user_maps_[select]);
+				parameters_.scenario_data["map_data"] = read_map(user_maps_[select-1]);
 			}
 
-		} else if(select != maps_menu_.nitems()-1) {
+		} else if(select > user_maps_.size() && select <= maps_menu_.nitems()-1) {
 			parameters_.saved_game = false;
-			const size_t index = select - user_maps_.size();
+			const size_t index = select - user_maps_.size() - 1;
 			const config::child_list& levels = game_config().get_children("multiplayer");
 
 			if(index < levels.size()) {
@@ -306,7 +415,7 @@ void create::process_event()
 
 		std::auto_ptr<gamemap> map(NULL);
 		try {
-			map.reset(new gamemap(game_config(), map_data));
+			map = std::auto_ptr<gamemap>(new gamemap(game_config(), map_data));
 		} catch(gamemap::incorrect_format_exception& e) {
 			LOG_STREAM(err,general) << "map could not be loaded: " << e.msg_ << "\n";
 			tooltips::clear_tooltips(minimap_rect_);
@@ -328,11 +437,6 @@ void create::process_event()
 			side["controller"] = "human";
 		}
 
-		//if there are too many sides, remove some
-		while(int(parameters_.scenario_data.get_children("side").size()) > map_positions) {
-			 parameters_.scenario_data.remove_child("side",
-					  parameters_.scenario_data.get_children("side").size()-1);
-		}
 
 		if(map.get() != NULL) {
 			const surface mini(image::getMinimap(minimap_rect_.w,minimap_rect_.h,*map,0));
@@ -343,7 +447,14 @@ void create::process_event()
 			}
 		}
 
-		const int nsides = parameters_.scenario_data.get_children("side").size();
+		int nsides = parameters_.scenario_data.get_children("side").size();
+		const config::child_list& player_sides = parameters_.scenario_data.get_children("side");
+		for(config::child_list::const_iterator k = player_sides.begin(); k != player_sides.end(); ++k) {
+			if((**k)["allow_player"] == "no") {
+				nsides--;
+			}
+		}
+
 		std::stringstream players;
 		if(map.get() != NULL) {
 			players << _("Players: ") << nsides;
@@ -369,6 +480,17 @@ void create::hide_children(bool hide)
 	maps_menu_.hide(hide);
 	turns_slider_.hide(hide);
 	turns_label_.hide(hide);
+
+	countdown_init_time_slider_.hide(hide);
+	countdown_init_time_label_.hide(hide);
+	countdown_turn_bonus_slider_.hide(hide);
+	countdown_turn_bonus_label_.hide(hide);
+	countdown_reservoir_time_slider_.hide(hide);
+	countdown_reservoir_time_label_.hide(hide);
+	countdown_action_bonus_slider_.hide(hide);
+	countdown_action_bonus_label_.hide(hide);
+	countdown_game_.hide(hide);
+
 	village_gold_slider_.hide(hide);
 	village_gold_label_.hide(hide);
 	xp_modifier_slider_.hide(hide);
@@ -454,6 +576,11 @@ void create::layout_children(const SDL_Rect& rect)
 	era_combo_.set_location(xpos + era_label_.width() + border_size, ypos);
 	ypos += era_combo_.height() + border_size;
 
+#ifdef MP_VISION_OPTIONAL
+	vision_combo_.set_location(xpos, ypos);
+	ypos += vision_combo_.height() + border_size;
+#endif
+
 	// Second column: map menu
 	ypos = ypos_columntop;
 	xpos += minimap_width + column_border_size;
@@ -477,6 +604,7 @@ void create::layout_children(const SDL_Rect& rect)
 	turns_slider_.set_location(xpos, ypos);
 	ypos += turns_slider_.height() + border_size;
 
+
 	village_gold_label_.set_location(xpos, ypos);
 	ypos += village_gold_label_.height() + border_size;
 	village_gold_slider_.set_width(ca.w - xpos);
@@ -489,22 +617,39 @@ void create::layout_children(const SDL_Rect& rect)
 	xp_modifier_slider_.set_location(xpos, ypos);
 	ypos += xp_modifier_slider_.height() + border_size;
 
-
 	use_map_settings_.set_location(xpos, ypos);
 	ypos += use_map_settings_.height() + border_size;
 
-	fog_game_.set_location(xpos, ypos);
+	countdown_init_time_label_.set_location(xpos, ypos);
+	countdown_turn_bonus_label_.set_location(xpos + (ca.w - xpos)/2 + 5 , ypos);
+	ypos += countdown_init_time_label_.height() + border_size;
+	countdown_init_time_slider_.set_width(((ca.w - xpos)/2)-5);
+	countdown_turn_bonus_slider_.set_width(((ca.w - xpos)/2)-5);
+	countdown_init_time_slider_.set_location(xpos, ypos);
+	countdown_turn_bonus_slider_.set_location(xpos + (ca.w - xpos)/2 + 5, ypos);
+	ypos += countdown_init_time_slider_.height() + border_size;
+
+	countdown_reservoir_time_label_.set_location(xpos, ypos);
+	countdown_action_bonus_label_.set_location(xpos + (ca.w - xpos)/2 + 5 , ypos);
+	ypos += countdown_reservoir_time_label_.height() + border_size;
+	countdown_reservoir_time_slider_.set_width(((ca.w - xpos)/2)-5);
+	countdown_action_bonus_slider_.set_width(((ca.w - xpos)/2)-5);
+	countdown_reservoir_time_slider_.set_location(xpos, ypos);
+	countdown_action_bonus_slider_.set_location(xpos + (ca.w - xpos)/2 + 5, ypos);
+	ypos += countdown_reservoir_time_slider_.height() + border_size;
+
+	countdown_game_.set_location(xpos, ypos);
+//	ypos += countdown_game_.height() + border_size;
+
+	fog_game_.set_location(xpos + (ca.w - xpos)/2 + 5, ypos);
 	ypos += fog_game_.height() + border_size;
 
 	shroud_game_.set_location(xpos, ypos);
-	ypos += shroud_game_.height() + border_size;
+//	ypos += shroud_game_.height() + border_size;
 
-	observers_game_.set_location(xpos, ypos);
+	observers_game_.set_location(xpos + (ca.w - xpos)/2 + 5, ypos);
 	ypos += observers_game_.height() + border_size;
 
-	vision_combo_.set_location(xpos, ypos);
-	ypos += vision_combo_.height() + border_size;
-
 	// OK / Cancel buttons
 	gui::button* left_button = &launch_game_;
 	gui::button* right_button = &cancel_game_;
