@@ -1,4 +1,4 @@
-/* $Id: multiplayer_create.hpp,v 1.4 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: multiplayer_create.hpp 10794 2006-03-25 23:41:10Z soliton $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -35,7 +35,12 @@ public:
 			num_turns = 0;
 			village_gold = 0;
 			xp_modifier = 0;
-			fog_game = shroud_game = allow_observers = share_view = share_maps = false;
+			mp_countdown=false;
+			mp_countdown_init_time=0;
+			mp_countdown_turn_bonus=0;
+			mp_countdown_reservoir_time=0;
+			mp_countdown_action_bonus=0;
+			use_map_settings = fog_game = shroud_game = allow_observers = share_view = share_maps = false;
 
 			scenario_data.clear();
 		}
@@ -48,6 +53,12 @@ public:
 		int num_turns;
 		int village_gold;
 		int xp_modifier;
+		int mp_countdown_init_time;
+		int mp_countdown_reservoir_time;
+		int mp_countdown_turn_bonus;
+		int mp_countdown_action_bonus;
+		bool mp_countdown;
+		bool use_map_settings;
 		bool fog_game;
 		bool shroud_game;
 		bool allow_observers;
@@ -76,6 +87,9 @@ private:
 
 	tooltips::manager tooltip_manager_;
 	int map_selection_;
+	int mp_countdown_init_time_;
+	int mp_countdown_reservoir_time_;
+
 
 	std::vector<std::string> user_maps_;
 	std::vector<std::string> map_options_;
@@ -83,6 +97,15 @@ private:
 	gui::menu maps_menu_;
 	gui::slider turns_slider_;
 	gui::label turns_label_;
+	gui::button countdown_game_;
+	gui::slider countdown_init_time_slider_;
+	gui::label countdown_init_time_label_;
+	gui::slider countdown_reservoir_time_slider_;
+	gui::label countdown_reservoir_time_label_;
+	gui::label countdown_turn_bonus_label_;
+	gui::slider countdown_turn_bonus_slider_;
+	gui::label countdown_action_bonus_label_;
+	gui::slider countdown_action_bonus_slider_;
 	gui::slider village_gold_slider_;
 	gui::label village_gold_label_;
 	gui::slider xp_modifier_slider_;
@@ -93,6 +116,7 @@ private:
 	gui::label era_label_;
 	gui::label map_label_;
 
+	gui::button use_map_settings_;
 	gui::button fog_game_;
 	gui::button shroud_game_;
 	gui::button observers_game_;
