@@ -1,4 +1,4 @@
-/* $Id: multiplayer_lobby.cpp 8285 2005-09-18 19:08:31Z ott $ */
+/* $Id: multiplayer_lobby.cpp 12730 2006-07-24 00:20:36Z sapient $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -12,14 +12,18 @@
 */
 
 #include "global.hpp"
-#include "multiplayer_lobby.hpp"
 #include "filesystem.hpp"
-#include "font.hpp"
+#include "marked-up_text.hpp"
+#include "minimap.hpp"
+#include "multiplayer_lobby.hpp"
+#include "replay.hpp"
 #include "wassert.hpp"
 #include "wml_separators.hpp"
 #include "game_config.hpp"
 #include "gettext.hpp"
+#include "playmp_controller.hpp"
 #include "show_dialog.hpp"
+#include "sound.hpp"
 
 namespace {
 
@@ -34,13 +38,15 @@ std::string games_menu_heading()
 }
 
 namespace mp {
-gamebrowser::gamebrowser(CVideo& video) : scrollarea(video),
+	gamebrowser::gamebrowser(CVideo& video, const config* map_hashes) :
+	menu(video, empty_string_vector, false, -1, -1, NULL, &menu::bluebg_style),
 	gold_icon_locator_("misc/gold.png"),
 	xp_icon_locator_("misc/units.png"),
 	vision_icon_locator_("misc/invisible.png"),
-	observer_icon_locator_("misc/eye.png"), header_height_(20),
+	time_limit_icon_locator_("misc/sand-clock.png"),
+	observer_icon_locator_("misc/eye.png"), map_hashes_(map_hashes),
 	item_height_(100), margin_(5), h_padding_(5),
-	v_padding_(5), selected_(0), visible_range_(std::pair<size_t,size_t>(0,0)),
+	v_padding_(5), header_height_(20), selected_(0), visible_range_(std::pair<size_t,size_t>(0,0)),
 	double_clicked_(false), ignore_next_doubleclick_(false), last_was_doubleclick_(false)
 {
 }
@@ -48,16 +54,16 @@ gamebrowser::gamebrowser(CVideo& video) : scrollarea(video),
 void gamebrowser::set_inner_location(const SDL_Rect& rect)
 {
 	set_full_size(games_.size());
-	set_shown_size(rect.h / item_height_);
+	set_shown_size(rect.h / row_height());
 	bg_register(rect);
 	scroll(get_position());
 }
 
-void gamebrowser::scroll(int pos)
+void gamebrowser::scroll(unsigned int pos)
 {
-	if(pos >= 0 && pos < games_.size()) {
+	if(pos < games_.size()) {
 		visible_range_.first = pos;
-		visible_range_.second = minimum<size_t>(pos + inner_location().h / item_height_, games_.size());
+		visible_range_.second = minimum<size_t>(pos + inner_location().h / row_height(), games_.size());
 		set_dirty();
 	}
 }
@@ -68,7 +74,7 @@ SDL_Rect gamebrowser::get_item_rect(size_t index) const {
 		return res;
 	}
 	const SDL_Rect& loc = inner_location();
-	const SDL_Rect res = { loc.x, loc.y + (index - visible_range_.first) * item_height_, loc.w, item_height_ };
+	const SDL_Rect res = { loc.x, loc.y + (index - visible_range_.first) * row_height(), loc.w, row_height()};
 	return res;
 }
 
@@ -87,48 +93,45 @@ void gamebrowser::draw()
 	}
 }
 
-void gamebrowser::draw_contents() const
+void gamebrowser::draw_contents()
 {
 	if(!games_.empty()) {
 		for(size_t i = visible_range_.first; i != visible_range_.second; ++i) {
-			draw_item(i);
+			style_->draw_row(*this,i,get_item_rect(i),(i==selected_)? SELECTED_ROW : NORMAL_ROW);
 		}
 	} else {
 		const SDL_Rect rect = inner_location();
-		font::draw_text(&video(), rect, font::SIZE_NORMAL, font::NORMAL_COLOUR, _("<no games open>"), rect.x + margin_, rect.y + margin_);
+		font::draw_text(&video(), rect, font::SIZE_NORMAL, font::NORMAL_COLOUR, _("--no games open--"), rect.x + margin_, rect.y + margin_);
 	}
 }
 
-void gamebrowser::draw_item(size_t index) const {
+void gamebrowser::draw_row(const size_t index, const SDL_Rect& item_rect, ROW_TYPE /*type*/) {
 	const game_item& game = games_[index];
-	SDL_Rect item_rect = get_item_rect(index);
 	int xpos = item_rect.x + margin_;
 	int ypos = item_rect.y + margin_;
 
-	bg_restore(item_rect);
-	draw_solid_tinted_rectangle(item_rect.x, item_rect.y, item_rect.w, item_rect.h, 0, 0, 0, 0.2, video().getSurface());
+	//bg_restore(item_rect);
+	//draw_solid_tinted_rectangle(item_rect.x, item_rect.y, item_rect.w, item_rect.h, 0, 0, 0, 0.2, video().getSurface());
 
 	// draw mini map
 	video().blit_surface(xpos, ypos, game.mini_map);
 
 	xpos += item_height_ + margin_;
 
+	// draw game name
 	const surface name_surf(font::get_rendered_text(font::make_text_ellipsis(game.name, font::SIZE_PLUS, (item_rect.x + item_rect.w) - xpos - margin_), font::SIZE_PLUS, game.vacant_slots > 0 ? font::GOOD_COLOUR : game.observers ? font::NORMAL_COLOUR : font::BAD_COLOUR));
 	video().blit_surface(xpos, ypos, name_surf);
 
-	ypos += v_padding_;
-
-	// draw map info
-	const surface map_info_surf(font::get_rendered_text(font::make_text_ellipsis(game.map_info, font::SIZE_PLUS, (item_rect.x + item_rect.w) - xpos - margin_), font::SIZE_PLUS, font::GOOD_COLOUR));
-	video().blit_surface(xpos, ypos, map_info_surf);
-
-
 	// draw gold icon
 	const surface gold_icon(image::get_image(gold_icon_locator_, image::UNSCALED));
 	ypos = item_rect.y + item_rect.h  - margin_ - gold_icon->h;
 
 	video().blit_surface(xpos, ypos, gold_icon);
 
+	// draw map info
+	const surface map_info_surf(font::get_rendered_text(font::make_text_ellipsis(game.map_info, font::SIZE_NORMAL, (item_rect.x + item_rect.w) - xpos - margin_), font::SIZE_NORMAL, font::NORMAL_COLOUR));
+	video().blit_surface(xpos, ypos - map_info_surf->h - 4 * v_padding_, map_info_surf);
+
 	xpos += gold_icon->w + h_padding_;
 
 	// draw gold text
@@ -152,6 +155,22 @@ void gamebrowser::draw_item(size_t index) const {
 
 	xpos += xp_text->w + 2 * h_padding_;
 
+	if(!game.time_limit.empty()) {
+		// draw time icon
+		const surface time_icon(image::get_image(time_limit_icon_locator_, image::UNSCALED));
+		ypos = item_rect.y + item_rect.h  - margin_ - time_icon->h;
+		video().blit_surface(xpos, ypos, time_icon);
+
+		xpos += time_icon->w + h_padding_;
+
+		// draw time text
+		const surface time_text(font::get_rendered_text(game.time_limit, font::SIZE_NORMAL, font::NORMAL_COLOUR));
+		ypos -= abs(time_icon->h - time_text->h) / 2;
+		video().blit_surface(xpos, ypos, time_text);
+
+		xpos += time_text->w + 2 * h_padding_;
+	}
+
 	// draw vision icon
 	const surface vision_icon(image::get_image(vision_icon_locator_, image::UNSCALED));
 	video().blit_surface(xpos, ypos, vision_icon);
@@ -167,8 +186,8 @@ void gamebrowser::draw_item(size_t index) const {
 	xpos = item_rect.x + item_rect.w - margin_ - status_text->w;
 	video().blit_surface(xpos, ypos, status_text);
 
-	if(selected_ == index)
-		draw_solid_tinted_rectangle(item_rect.x, item_rect.y, item_rect.w, item_rect.h, 153, 0, 0, 0.3, video().getSurface());
+	//if(selected_ == index)
+	//	draw_solid_tinted_rectangle(item_rect.x, item_rect.y, item_rect.w, item_rect.h, 153, 0, 0, 0.3, video().getSurface());
 }
 
 void gamebrowser::handle_event(const SDL_Event& event)
@@ -239,7 +258,6 @@ void gamebrowser::handle_event(const SDL_Event& event)
 
 				if(point_in_rect(x, y, item_rect)) {
 					set_focus(true);
-					set_dirty();
 					selected_ = i;
 					break;
 				}
@@ -274,7 +292,13 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 
 	for(game = games.begin(); game != games.end(); ++game) {
 		games_.push_back(game_item());
+		if((**game)["mp_era"] != "") {
+			const config* const era_cfg = game_config.find_child("era", "id", (**game)["mp_era"]);
+			games_.back().map_info = era_cfg != NULL ? era_cfg->get_attribute("name") : _("Unknown era");
 
+		} else {
+			games_.back().map_info = _("Unknown era");
+		}
 		games_.back().map_data = (**game)["map_data"];
 		if(games_.back().map_data == "")
 			games_.back().map_data = read_map((**game)["map"]);
@@ -283,9 +307,35 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 			try {
 				gamemap map(game_config, games_.back().map_data);
 				games_.back().mini_map = image::getMinimap(item_height_ - margin_, item_height_ - 2 * margin_, map, 0);
+				games_.back().map_info += " - " + lexical_cast_default<std::string, int>(map.x(), "??") + std::string("x") + lexical_cast_default<std::string, int>(map.y(), "??");
 			} catch(gamemap::incorrect_format_exception &e) {
 				std::cerr << "illegal map: " << e.msg_ << "\n";
 			}
+		} else {
+			games_.back().map_info += " - ??x??";
+		}
+		games_.back().map_info += " ";
+		if((**game)["mp_scenario"] != "" && map_hashes_) {
+			const config* level_cfg = game_config.find_child("generic_multiplayer", "id", (**game)["mp_scenario"]);
+			if(level_cfg == NULL)
+				level_cfg = game_config.find_child("multiplayer", "id", (**game)["mp_scenario"]);
+			games_.back().map_info += level_cfg != NULL ? level_cfg->get_attribute("name") : _("Unknown scenario");
+			if(level_cfg) {
+				const std::string& hash = (**game)["hash"];
+				bool hash_found = false;
+				for(string_map::const_iterator i = map_hashes_->values.begin(); i != map_hashes_->values.end(); ++i) {
+					if(i->first == (**game)["mp_scenario"] && i->second == hash) {
+						hash_found = true;
+						break;
+					}
+				}
+				if(!hash_found) {
+					games_.back().map_info += " - ";
+					games_.back().map_info += _("Remote scenario");
+				}
+			}
+		} else {
+			games_.back().map_info += _("Unknown scenario");
 		}
 		games_.back().name = (**game)["name"];
 		const std::string& turn = (**game)["turn"];
@@ -294,7 +344,7 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 		if(turn != "")
 			games_.back().status = _("Turn") + (" " + turn);
 		else if(slots != "")
-			games_.back().status = slots + " " + ngettext(_("Vacant Slot"), _("Vacant Slots"), games_.back().vacant_slots);
+			games_.back().status = std::string(ngettext(_("Vacant Slot:"), _("Vacant Slots:"), games_.back().vacant_slots)) + " " + slots;
 
 		if((**game)["mp_use_map_settings"] == "yes") {
 			games_.back().gold = _("Use map settings");
@@ -323,11 +373,17 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 				games_.back().shroud = false;
 			}
 		}
+		if((**game)["mp_countdown"] == "yes" ) {
+			games_.back().time_limit = (**game)["mp_countdown_init_time"] + " / +" + (**game)["mp_countdown_turn_bonus"] + " " + (**game)["mp_countdown_action_bonus"];
+		} else {
+			games_.back().time_limit = "";
+		}
+
 		games_.back().xp = (**game)["experience_modifier"] + "%";
 		games_.back().observers = (**game)["observer"] != "no" ? true : false;
 	}
 	set_full_size(games_.size());
-	set_shown_size(inner_location().h / item_height_);
+	set_shown_size(inner_location().h / row_height());
 	scroll(get_position());
 	if(selected_ >= games_.size())
 		selected_ = maximum<long>(static_cast<long>(games_.size()) - 1, 0);
@@ -406,13 +462,17 @@ lobby::lobby(display& disp, const config& cfg, chat& c, config& gamelist) :
 	observe_game_(disp.video(), _("Observe Game")),
 	join_game_(disp.video(), _("Join Game")),
 	create_game_(disp.video(), _("Create Game")),
+	skip_replay_(disp.video(), _("Quick Replays"), gui::button::TYPE_CHECK),
+	game_preferences_(disp.video(), _("Preferences")),
 	quit_game_(disp.video(), _("Quit")),
 	sorter_(gamelist),
-	games_menu_(disp.video()),
-	current_game_(0)
+	games_menu_(disp.video(),cfg.child("multiplayer_hashes"))
 {
+	skip_replay_.set_check(preferences::skip_mp_replay());
+	skip_replay_.set_help_string(_("Skip quickly to the active turn when observing"));
 	game_config::debug = false;
 	gamelist_updated();
+	sound::play_music_repeatedly(game_config::title_music);
 }
 
 void lobby::hide_children(bool hide)
@@ -423,6 +483,8 @@ void lobby::hide_children(bool hide)
 	observe_game_.hide(hide);
 	join_game_.hide(hide);
 	create_game_.hide(hide);
+	skip_replay_.hide(hide);
+	game_preferences_.hide(hide);
 	quit_game_.hide(hide);
 }
 
@@ -433,7 +495,9 @@ void lobby::layout_children(const SDL_Rect& rect)
 	join_game_.set_location(xscale(12),yscale(7));
 	observe_game_.set_location(join_game_.location().x + join_game_.location().w + 5,yscale(7));
 	create_game_.set_location(observe_game_.location().x + observe_game_.location().w + 5,yscale(7));
-	quit_game_.set_location(create_game_.location().x + create_game_.location().w + 5,yscale(7));
+	skip_replay_.set_location(create_game_.location().x + 2*(create_game_.location().w),yscale(7));
+	game_preferences_.set_location(skip_replay_.location().x + skip_replay_.location().w + 5,yscale(7));
+	quit_game_.set_location(game_preferences_.location().x + game_preferences_.location().w + 5,yscale(7));
 
 	games_menu_.set_location(client_area().x, client_area().y + title().height());
 	games_menu_.set_measurements(client_area().w, client_area().h
@@ -462,6 +526,8 @@ void lobby::process_event()
 
 	const bool observe = (observe_game_.pressed() || (games_menu_.selected() && !games_menu_.selection_is_joinable())) && games_menu_.selection_is_observable();
 	const bool join = (join_game_.pressed() || games_menu_.selected()) && games_menu_.selection_is_joinable();
+	preferences::set_skip_mp_replay(skip_replay_.checked());
+	playmp_controller::set_replay_last_turn(0);
 
 	if(join || observe) {
 		const config* game = gamelist().child("gamelist");
@@ -477,9 +543,21 @@ void lobby::process_event()
 			config response;
 			config& join = response.add_child("join");
 			join["id"] = id;
+			if (observe){
+				join["observe"] = "yes";
+			}
+			else{
+				join["observe"] = "no";
+			}
 			network::send_data(response);
 
 			if(observe) {
+				const std::string& str_current_turn = game_cfg["turn"];
+				int index = str_current_turn.find_first_of('/');
+				if (index > -1){
+					const std::string strTest = str_current_turn.substr(0, index);
+					playmp_controller::set_replay_last_turn(lexical_cast<unsigned int>(strTest));
+				}
 				set_result(OBSERVE);
 			} else {
 				set_result(JOIN);
@@ -493,7 +571,13 @@ void lobby::process_event()
 		return;
 	}
 
+	if(game_preferences_.pressed()) {
+		set_result(PREFERENCES);
+		return;
+	}
+
 	if(quit_game_.pressed()) {
+		recorder.set_skip(false);
 		set_result(QUIT);
 		return;
 	}
