@@ -1,4 +1,4 @@
-/* $Id: multiplayer_lobby.hpp 8493 2005-10-08 11:18:23Z j_daniel $ */
+/* $Id: multiplayer_lobby.hpp 12375 2006-07-03 18:52:20Z jhinrichs $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -25,7 +25,7 @@
 // This module controls the multiplayer lobby. A section on the server which
 // allows players to chat, create games, and join games.
 namespace mp {
-class gamebrowser : public gui::scrollarea {
+class gamebrowser : public gui::menu {
 public:
 	struct game_item {
 		surface mini_map;
@@ -36,21 +36,22 @@ public:
 		std::string xp;
 		std::string vision;
 		std::string status;
+		std::string time_limit;
 		size_t vacant_slots;
 		bool fog;
 		bool shroud;
 		bool observers;
 		bool use_map_settings;
 	};
-	gamebrowser(CVideo& video);
-	void scroll(int pos);
+	gamebrowser(CVideo& video,const config* map_hashes);
+	void scroll(unsigned int pos);
 	void handle_event(const SDL_Event& event);
 	void set_inner_location(const SDL_Rect& rect);
 	void set_item_height(unsigned int height);
 	void set_game_items(const config& cfg, const config& game_config);
 	void draw();
-	void draw_contents() const;
-	void draw_item(size_t index) const;
+	void draw_contents();
+	void draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
 	SDL_Rect get_item_rect(size_t index) const;
 	bool empty() const { return games_.empty(); }
 	bool selection_is_joinable() const { return empty() ? false : games_[selected_].vacant_slots > 0; }
@@ -58,12 +59,16 @@ public:
 	bool selected() const { return double_clicked_ && !empty(); }
 	int selection() const { return selected_; }
 protected:
+	unsigned int row_height() const { return item_height_ + (2 * style_->get_thickness()); }
 private:
 	image::locator gold_icon_locator_;
 	image::locator xp_icon_locator_;
 	image::locator vision_icon_locator_;
+	image::locator time_limit_icon_locator_;
 	image::locator observer_icon_locator_;
 
+	const config* map_hashes_;
+
 	unsigned int item_height_;
 	int margin_;
 	int h_padding_;
@@ -73,6 +78,7 @@ private:
 	std::pair<size_t, size_t> visible_range_;
 	std::vector<game_item> games_;
 	std::vector<size_t> redraw_items_;
+	mutable std::vector<int> widths_;
 	bool double_clicked_;
 	bool ignore_next_doubleclick_;
 	bool last_was_doubleclick_;
@@ -111,11 +117,12 @@ private:
 	gui::button observe_game_;
 	gui::button join_game_;
 	gui::button create_game_;
+	gui::button skip_replay_;
+	gui::button game_preferences_;
 	gui::button quit_game_;
 
 	lobby_sorter sorter_;
 	gamebrowser games_menu_;
-	int current_game_;
 
 	std::map<std::string,std::string> minimaps_;
 };
