@@ -1,4 +1,4 @@
-/* $Id: multiplayer_ui.cpp 8626 2005-10-21 13:49:07Z j_daniel $ */
+/* $Id: multiplayer_ui.cpp 11877 2006-06-05 20:02:25Z soliton $ */
 /*
    Copyright (C) 2005
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -52,7 +52,7 @@ void level_to_gamestate(config& level, game_state& state)
 		LOG_NW << "setting replay\n";
 		recorder = replay(replay_data_store);
 		if(!recorder.empty()) {
-			recorder.set_skip(-1);
+			recorder.set_skip(false);
 		}
 
 		level.clear_children("replay");
@@ -72,28 +72,11 @@ std::string get_colour_string(int id)
 {
 	std::string prefix("\033[3 m");
 	prefix[3] = lexical_cast<char, int>(id + 1);
-
-	switch(id) {
-	case 0:
-		return prefix + _("Red");
-	case 1:
-		return prefix + _("Blue");
-	case 2:
-		return prefix + _("Green");
-	case 3:
-		return prefix + _("Yellow");
-	case 4:
-		return prefix + _("Purple");
-	case 5:
-		return prefix + _("Orange");
-	case 6:
-		return prefix + _("Grey");
-	case 7:
-		return prefix + _("White");
-	case 8:
-		return prefix + _("Brown");
-	default:
-		return _("Invalid colour");
+	std::map<int, std::string>::iterator name = game_config::team_rgb_name.find(id+1);
+	if(name != game_config::team_rgb_name.end()){
+	  return prefix + _(name->second.c_str());
+	}else{
+	  return prefix + _("Invalid Color");
 	}
 }
 
@@ -136,15 +119,18 @@ void chat::update_textbox(gui::textbox& textbox)
 		s.append(format_message(*itor));
 	}
 
-	textbox.append_text(s);
-	textbox.scroll_to_bottom();
+	textbox.append_text(s,true);
 
 	last_update_ = message_history_.size();
 }
 
 std::string chat::format_message(const msg& message)
 {
-	return "<" + message.user + ">" + message.message + "\n";
+	if(message.message.substr(0,3) == "/me") {
+		return "<" + message.user + " " + message.message.substr(3) + ">\n";
+	} else {
+		return "<" + message.user + ">" + message.message + "\n";
+	}
 }
 
 ui::ui(display& disp, const std::string& title, const config& cfg, chat& c, config& gamelist) :
@@ -163,7 +149,7 @@ ui::ui(display& disp, const std::string& title, const config& cfg, chat& c, conf
 	title_(disp.video(), title, font::SIZE_LARGE, font::TITLE_COLOUR),
 	chat_textbox_(disp.video(), 100, "", false),
 	entry_textbox_(disp.video(), 100),
-	users_menu_(disp.video(), std::vector<std::string>()),
+	users_menu_(disp.video(), std::vector<std::string>(), false, -1, -1, NULL, &gui::menu::slateborder_style),
 
 	result_(CONTINUE)
 {
@@ -271,52 +257,132 @@ void ui::handle_event(const SDL_Event& event)
 	}
 }
 
+void ui::send_chat_query(const std::string& args)
+{
+	config data;
+	data.add_child("query")["type"] = args;
+	network::send_data(data);
+}
+
+void ui::add_chat_message(const std::string& speaker, int /*side*/, const std::string& message, display::MESSAGE_TYPE /*type*/)
+{
+	chat_.add_message(speaker,message);
+	chat_.update_textbox(chat_textbox_);
+}
+
+void ui::send_chat_message(const std::string& message, bool /*allies_only*/)
+{
+	config data, msg;
+	msg["message"] = message;
+	msg["sender"] = preferences::login();
+	data.add_child("message", msg);
+
+	add_chat_message(preferences::login(),0, message);	//local echo
+	network::send_data(data);
+}
+
+
 void ui::handle_key_event(const SDL_KeyboardEvent& event)
 {
 	//On enter, adds the current chat message to the chat textbox.
 	if(event.keysym.sym == SDLK_RETURN && !entry_textbox_.text().empty()) {
 
-		const std::string& text = entry_textbox_.text();
+		chat_handler::do_speak(entry_textbox_.text());
+		entry_textbox_.clear();
 
-		//if the text starts with '/query' it's a query to the server.
-		//otherwise it's just a chat message
-		static const std::string query = "/query ";
+	} else if(event.keysym.sym == SDLK_TAB ) {
+		std::string text = entry_textbox_.text();
+		std::string semiword;
+		bool beginning;
 
-		config data;
-
-		if(text.size() >= query.size() && std::equal(query.begin(),query.end(),text.begin())) {
-			const std::string args = text.substr(query.size());
-
-			data.add_child("query")["type"] = args;
-
-		} else {
-
-			// Sends the message to the network
-			config msg;
-			msg["message"] = text;
-			msg["sender"] = preferences::login();
-			data.add_child("message", msg);
-
-			chat_.add_message(preferences::login(), entry_textbox_.text());
-			chat_.update_textbox(chat_textbox_);
+		const size_t last_space = text.rfind(" ");
 
+		//if last character is a space return
+		if(last_space == text.size() -1) {
+			return;
+		}
+
+		if(last_space == std::string::npos) {
+			beginning = true;
+			semiword = text;
+		}else{
+			beginning = false;
+			semiword.assign(text,last_space+1,text.size());
+		}
+
+
+		std::vector<std::string> matches;
+		std::string best_match = semiword;
+		std::vector<std::string>& users = user_list_;
+		std::sort<std::vector<std::string>::iterator>(users.begin(), users.end());
+		for(std::vector<std::string>::const_iterator i = users.begin(); i != users.end(); ++i) {
+			if( i->size() >= semiword.size() &&
+					std::equal(semiword.begin(),semiword.end(),i->begin(),chars_equal_insensitive)) {
+				if(matches.empty()) {
+					best_match = *i;
+				} else {
+					int j= 0;;
+					while(toupper(best_match[j]) == toupper((*i)[j])) j++;
+					best_match.erase(best_match.begin()+j,best_match.end());
+				}
+				matches.push_back(*i);
+			}
+		}
+
+		if(!matches.empty()) {
+			std::string add = beginning ? ": " : " ";
+			text.replace(last_space+1, semiword.size(), best_match);
+			if(matches.size() == 1) {
+				text.append(add);
+			} else {
+				std::string completion_list;
+				std::vector<std::string>::iterator it;
+				for(it =matches.begin();it!=matches.end();it++) {
+					completion_list += " ";
+					completion_list += *it;
+				}
+				chat_.add_message("",completion_list);
+				chat_.update_textbox(chat_textbox_);
+			}
+			entry_textbox_.set_text(text);
 		}
 
-		network::send_data(data);
-		entry_textbox_.clear();
 	}
 }
 
-void ui::process_network_data(const config& data, const network::connection sock)
+void ui::process_network_data(const config& data, const network::connection /*sock*/)
 {
 	if(data.child("error")) {
 		throw network::error((*data.child("error"))["message"]);
 	} else {
 		if(data.child("message")) {
+			const config& msg = *data.child("message");
+			config* cignore;
+			bool ignored = false;
+			if ((cignore = preferences::get_prefs()->child("ignore"))){
+				for(std::map<std::string,t_string>::const_iterator i = cignore->values.begin();
+				i != cignore->values.end(); ++i){
+					if(msg["sender"] == i->first){
+						if (i->second == "yes"){
+							ignored = true;
+						}
+					}
+				}
+			}
+
+			if (!ignored){
+				sound::play_sound(game_config::sounds::receive_message);
+
+				chat_.add_message(msg["sender"], msg["message"]);
+				chat_.update_textbox(chat_textbox_);
+			}
+		}
+
+		if(data.child("whisper")){
 			sound::play_sound(game_config::sounds::receive_message);
 
-			const config& msg = *data.child("message");
-			chat_.add_message(msg["sender"], msg["message"]);
+			const config& cwhisper = *data.child("whisper");
+			chat_.add_message("whisper: "+cwhisper["sender"], cwhisper["message"]);
 			chat_.update_textbox(chat_textbox_);
 		}
 
@@ -342,7 +408,7 @@ void ui::process_network_error(network::error& error)
 	throw error;
 }
 
-void ui::process_network_connection(const network::connection sock)
+void ui::process_network_connection(const network::connection /*sock*/)
 {
 	LOG_NW << "Caught network connection.\n";
 }
@@ -355,7 +421,7 @@ void ui::hide_children(bool hide)
 	users_menu_.hide(hide);
 }
 
-void ui::layout_children(const SDL_Rect& rect)
+void ui::layout_children(const SDL_Rect& /*rect*/)
 {
 	title_.set_location(xscale(12) + 8, yscale(38) + 8);
 	users_menu_.set_width(xscale(159));
@@ -376,7 +442,11 @@ void ui::gamelist_updated(bool silent)
 	config::child_iterator user;
 	for (user = users.begin(); user != users.end(); ++user) {
 		const std::string prefix = (**user)["available"] == "no" ? "#" : "";
-		user_strings.push_back(prefix + (**user)["name"].str());
+		std::string suffix = "";
+		if(!(**user)["location"].empty()) {
+			suffix = std::string(" (") + (**user)["location"] + std::string(")");
+		}
+		user_strings.push_back(prefix + (**user)["name"].str() + suffix);
 	}
 	set_user_list(user_strings, silent);
 }
@@ -403,4 +473,3 @@ const gui::widget& ui::title() const
 
 
 }
-
