@@ -1,4 +1,4 @@
-/* $Id: multiplayer_ui.hpp 8626 2005-10-21 13:49:07Z j_daniel $ */
+/* $Id: multiplayer_ui.hpp 11763 2006-06-01 01:25:17Z sapient $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -16,11 +16,12 @@
 #include "config.hpp"
 #include "hotkeys.hpp"
 #include "network.hpp"
-#include "preferences.hpp"
+#include "preferences_display.hpp"
 #include "widgets/label.hpp"
 #include "widgets/button.hpp"
 #include "widgets/menu.hpp"
 #include "widgets/textbox.hpp"
+#include "menu_events.hpp"
 
 #include <deque>
 #include <string>
@@ -28,6 +29,10 @@
 class display;
 struct game_state;
 
+namespace {
+static std::vector<std::string> empty_string_vector;
+}
+
 namespace mp {
 
 enum controller { CNTR_NETWORK = 0, CNTR_LOCAL, CNTR_COMPUTER, CNTR_EMPTY, CNTR_LAST };
@@ -67,10 +72,10 @@ private:
 
 //a base class for the different multiplayer base dialogs: game list, create
 //game, wait game, game setup
-class ui : public gui::widget
+class ui : public gui::widget, private events::chat_handler
 {
 public:
-	enum result { CONTINUE, JOIN, OBSERVE, CREATE, PLAY, QUIT };
+	enum result { CONTINUE, JOIN, OBSERVE, CREATE, PREFERENCES, PLAY, QUIT };
 
 	ui(display& d, const std::string& title,
 			const config& cfg, chat& c, config& gamelist);
@@ -111,6 +116,12 @@ protected:
 	virtual void handle_event(const SDL_Event& event);
 	virtual void handle_key_event(const SDL_KeyboardEvent& event);
 
+	// Override chat_handler
+	void send_chat_query(const std::string& args);
+	void add_chat_message(const std::string& speaker, int side, const std::string& message, display::MESSAGE_TYPE type=display::MESSAGE_PRIVATE);
+	void send_chat_message(const std::string& message, bool allies_only=false);
+
+
 	// Processes any pending network data. Called by the public
 	// process_network() method. Overridden by subclasses who add more
 	// behaviour for network.
