@@ -1,4 +1,4 @@
-/* $Id: multiplayer_wait.cpp,v 1.28 2005/06/06 20:12:47 gruikya Exp $ */
+/* $Id: multiplayer_wait.cpp 12673 2006-07-20 17:44:21Z xan $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -16,8 +16,11 @@
 #include "display.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
+#include "marked-up_text.hpp"
+#include "multiplayer.hpp"
 #include "multiplayer_wait.hpp"
 #include "preferences.hpp"
+#include "replay.hpp"
 #include "statistics.hpp"
 #include "util.hpp"
 #include "video.hpp"
@@ -86,7 +89,7 @@ void wait::leader_preview_pane::draw_contents()
 
 		if (utypes.find(leader) != utypes.end()) {
 			leader_name = utypes.find(leader)->second.language_name();
-			image = utypes.find(leader)->second.image();
+			image = utypes.find(leader)->second.image() + "~TC(1," + utypes.find(leader)->second.flag_rgb() + ")";
 		}
 		for(std::vector<std::string>::const_iterator itor = recruit_list.begin();
 				itor != recruit_list.end(); ++itor) {
@@ -141,13 +144,18 @@ std::string wait::leader_preview_pane::get_selected_leader()
 	return leaders_.get_leader();
 }
 
+void wait::leader_preview_pane::join()
+{
+	widget::join();
+	leader_combo_.join();
+}
+
 wait::wait(display& disp, const config& cfg, const game_data& data, mp::chat& c, config& gamelist) :
 	ui(disp, _("Game Lobby"), cfg, c, gamelist),
 
 	cancel_button_(disp.video(), _("Cancel")),
 	start_label_(disp.video(), _("Waiting for game to start..."), font::SIZE_SMALL, font::LOBBY_COLOUR),
-	game_menu_(disp.video(), std::vector<std::string>()),
-
+	game_menu_(disp.video(), std::vector<std::string>(), false, -1, -1, NULL, &gui::menu::bluebg_style),
 	game_data_(data),
 	stop_updates_(false)
 {
@@ -163,7 +171,7 @@ void wait::process_event()
 void wait::join_game(bool observe)
 {
 	for(;;) {
-		network::connection data_res = gui::network_data_dialog(disp(),
+		network::connection data_res = gui::network_receive_dialog(disp(),
 				_("Getting game data..."), level_);
 		check_response(data_res, level_);
 
@@ -202,6 +210,8 @@ void wait::join_game(bool observe)
 		size_t faction_choice = 0;
 
 		if(allow_changes) {
+			events::event_context context;
+
 			const config* era = level_.child("era");
 			if(era == NULL)
 				throw network::error(_("Era not available"));
@@ -225,9 +235,13 @@ void wait::join_game(bool observe)
 					possible_sides);
 			preview_panes.push_back(&leader_selector);
 
-			faction_choice = size_t(gui::show_dialog(disp(), NULL, "",
-						_("Choose your side:"),
-						gui::OK_ONLY, &choices, &preview_panes));
+			const int res = gui::show_dialog2(disp(), NULL, "", _("Choose your side:"),
+						gui::OK_CANCEL, &choices, &preview_panes);
+			if(res < 0) {
+				set_result(QUIT);
+				return;
+			}
+			faction_choice = res;
 			leader_choice = leader_selector.get_selected_leader();
 
 			wassert(faction_choice < possible_sides.size());
@@ -263,6 +277,14 @@ void wait::start_game()
 	LOG_NW << "starting game\n";
 }
 
+game_state& wait::request_snapshot(){
+	config cfg;
+
+	cfg.add_child("snapshot_request");
+
+	return state_;
+}
+
 void wait::layout_children(const SDL_Rect& rect)
 {
 	ui::layout_children(rect);
@@ -273,6 +295,8 @@ void wait::layout_children(const SDL_Rect& rect)
 	game_menu_.set_location(ca.x, ca.y + title().height());
 	game_menu_.set_measurements(ca.w, y - ca.y - title().height()
 			- gui::ButtonVPadding);
+	game_menu_.set_max_width(ca.w);
+	game_menu_.set_max_height(y - ca.y - title().height() - gui::ButtonVPadding);
 	cancel_button_.set_location(ca.x + ca.w - cancel_button_.width(), y);
 	start_label_.set_location(ca.x, y + 4);
 }
@@ -289,6 +313,9 @@ void wait::process_network_data(const config& data, const network::connection so
 {
 	ui::process_network_data(data, sock);
 
+	if(data["message"] != "") {
+		gui::show_dialog(disp(),NULL,_("Response"),data["message"],gui::OK_ONLY);
+	}
 	if(data["failed"] == "yes") {
 		set_result(QUIT);
 		return;
@@ -326,6 +353,10 @@ void wait::generate_menu()
 	for(config::child_list::const_iterator s = sides.begin(); s != sides.end(); ++s) {
 		const config& sd = **s;
 
+		if(sd["allow_player"] == "no") {
+			continue;
+		}
+
 		std::string description = sd["user_description"];
 		const std::string faction_id = sd["id"];
 
@@ -351,7 +382,7 @@ void wait::generate_menu()
 		const game_data::unit_type_map& utypes = game_data_.unit_types;
 		if (utypes.find(leader_type) != utypes.end()) {
 			leader_name = utypes.find(leader_type)->second.language_name();
-			leader_image = utypes.find(leader_type)->second.image();
+			leader_image = utypes.find(leader_type)->second.image() + std::string("~TC(" + sd["side"] + ",") + std::string(utypes.find(leader_type)->second.flag_rgb() + ")");
 		} else {
 			leader_image = leader_list_manager::random_enemy_picture;
 		}
@@ -373,8 +404,17 @@ void wait::generate_menu()
 		// Mark parentheses translatable for languages like Japanese
 		if(!leader_name.empty())
 			str << _("(") << leader_name << _(")");
-		str << COLUMN_SEPARATOR << sd["gold"] << ' ' << sgettext("unit^Gold")
-			<< COLUMN_SEPARATOR << sd["team_name"];
+		str << COLUMN_SEPARATOR << sd["gold"] << ' ' << sgettext("unit^Gold") << COLUMN_SEPARATOR;
+
+		int income_amt = lexical_cast_default<int>(sd["income"], 0);
+		if(income_amt != 0){
+			str << _("(") << _("Income") << ' ';
+			if(income_amt > 0)
+				str << _("+");
+			str << sd["income"] << _(")");
+		}
+
+		str	<< COLUMN_SEPARATOR << sd["team_name"];
 		str << COLUMN_SEPARATOR << get_colour_string(lexical_cast_default<int>(sd["colour"], 0) - 1);
 		details.push_back(str.str());
 	}
@@ -383,8 +423,9 @@ void wait::generate_menu()
 
 	// Uses the actual connected player list if we do not have any
 	// "gamelist" user data
-	if (gamelist().child("user") != NULL)
+	if (gamelist().child("user") == NULL) {
 		set_user_list(playerlist, true);
+	}
 }
 
 }
