@@ -1,6 +1,6 @@
-/* $Id: playcampaign.cpp,v 1.7 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: playcampaign.cpp 12055 2006-06-12 04:46:55Z soliton $ */
 /*
-   Copyright (C) 2003-2005 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003-2005 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
@@ -17,15 +17,20 @@
 #include <map>
 
 #include "playcampaign.hpp"
-#include "playlevel.hpp"
 #include "config.hpp"
+#include "filesystem.hpp"
 #include "gamestatus.hpp"
+#include "map_create.hpp"
+#include "playmp_controller.hpp"
+#include "playsingle_controller.hpp"
 #include "replay.hpp"
+#include "replay_controller.hpp"
 #include "log.hpp"
 #include "preferences.hpp"
 #include "dialogs.hpp"
 #include "gettext.hpp"
 #include "game_errors.hpp"
+#include "sound.hpp"
 #include "wassert.hpp"
 
 #define LOG_G LOG_STREAM(info, general)
@@ -49,9 +54,81 @@ typedef std::map<std::string, player_controller> controller_map;
 
 }
 
+void play_replay(display& disp, game_state& state, const config& game_config,
+		const game_data& units_data, CVideo& video)
+{
+	std::string type = state.campaign_type;
+	if(type.empty())
+		type = "scenario";
+
+	config const* scenario = NULL;
+
+	//'starting_pos' will contain the position we start the game from.
+	config starting_pos;
+
+	recorder.set_save_info(state);
+	if (state.snapshot.child("side") != NULL){
+		state = read_game(units_data, &state.snapshot);
+	}
+
+	//see if we load the scenario from the scenario data -- if there is
+	//no snapshot data available from a save, or if the user has selected
+	//to view the replay from scratch
+	if(!recorder.at_end()) {
+		//if the starting state is specified, then use that,
+		//otherwise get the scenario data and start from there.
+		if(state.starting_pos.empty() == false) {
+			LOG_G << "loading starting position...\n";
+			starting_pos = state.starting_pos;
+			scenario = &starting_pos;
+		} else {
+			LOG_G << "loading scenario: '" << state.scenario << "'\n";
+			scenario = game_config.find_child(type,"id",state.scenario);
+			LOG_G << "scenario found: " << (scenario != NULL ? "yes" : "no") << "\n";
+		}
+	}
+
+	controller_map controllers;
+
+	const std::string current_scenario = state.scenario;
+
+	try {
+		// preserve old label eg. replay
+		if (state.label.empty())
+			state.label = (*scenario)["name"];
+
+		play_replay_level(units_data,game_config,scenario,video,state);
+
+		state.snapshot = config();
+		recorder.clear();
+		state.replay_data.clear();
+
+	} catch(game::load_game_failed& e) {
+		gui::show_error_message(disp, _("The game could not be loaded: ") + e.message);
+	} catch(game::game_error& e) {
+		gui::show_error_message(disp, _("Error while playing the game: ") + e.message);
+	} catch(gamemap::incorrect_format_exception& e) {
+		gui::show_error_message(disp, std::string(_("The game map could not be loaded: ")) + e.msg_);
+	}
+}
+
+void clean_autosaves(const std::string &label)
+{
+	std::vector<save_info> games = get_saves_list();
+	std::string prefix = label + "-" + _("Auto-Save");
+	std::cerr << "Cleaning autosaves with prefix '" << prefix << "'\n";
+	for (std::vector<save_info>::iterator i = games.begin(); i != games.end(); i++) {
+		if (i->name.compare(0,prefix.length(),prefix) == 0) {
+			std::cerr << "Deleting autosave '" << i->name << "'\n";
+			delete_game(i->name);
+		}
+	}
+}
+
 LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_config,
 		const game_data& units_data, CVideo& video,
-		io_type_t io_type)
+		upload_log &log,
+		io_type_t io_type, bool skip_replay)
 {
 	std::string type = state.campaign_type;
 	if(type.empty())
@@ -127,18 +204,83 @@ LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_conf
 		bool save_game_after_scenario = true;
 
 		try {
-			state.label = (*scenario)["name"];
+			// preserve old label eg. replay
+			if (state.label.empty())
+				state.label = (*scenario)["name"];
+
+			//if the entire scenario should be randomly generated
+			if((*scenario)["scenario_generation"] != "") {
+				LOG_G << "randomly generating scenario...\n";
+				const cursor::setter cursor_setter(cursor::WAIT);
+
+				static config scenario2;
+				scenario2 = random_generate_scenario((*scenario)["scenario_generation"], scenario->child("generator"));
+				//level_ = scenario;
+
+				state.starting_pos = scenario2;
+				scenario = &scenario2;
+			}
+
+			std::string map_data = (*scenario)["map_data"];
+			if(map_data == "" && (*scenario)["map"] != "") {
+				map_data = read_map((*scenario)["map"]);
+			}
+
+			//if the map should be randomly generated
+			if(map_data == "" && (*scenario)["map_generation"] != "") {
+				const cursor::setter cursor_setter(cursor::WAIT);
+				map_data = random_generate_map((*scenario)["map_generation"],scenario->child("generator"));
+
+				//since we've had to generate the map, make sure that when we save the game,
+				//it will not ask for the map to be generated again on reload
+				static config new_level;
+				new_level = *scenario;
+				new_level.values["map_data"] = map_data;
+				scenario = &new_level;
+
+				state.starting_pos = new_level;
+				LOG_G << "generated map\n";
+			}
+
+			sound::play_no_music();
+
+			//LEVEL_RESULT res = play_level(units_data,game_config,scenario,video,state,story,log, skip_replay);
+			LEVEL_RESULT res;
+			switch (io_type){
+			case IO_NONE:
+				res = playsingle_scenario(units_data,game_config,scenario,video,state,story,log, skip_replay);
+				break;
+			case IO_SERVER:
+			case IO_CLIENT:
+				res = playmp_scenario(units_data,game_config,scenario,video,state,story,log, skip_replay);
+				break;
+			}
 
-			LEVEL_RESULT res = play_level(units_data,game_config,scenario,video,state,story);
 
 			state.snapshot = config();
-
+			if (res == DEFEAT) {
+				// tell all clients that the campaign won't continue
+				if(io_type == IO_SERVER) {
+					config end;
+					end.add_child("end_scenarios");
+					network::send_data(end);
+				}
+				gui::show_dialog(disp, NULL,
+				                 _("Defeat"),
+				                 _("You have been defeated!"),
+				                 gui::OK_ONLY);
+			}
+			if(res == QUIT && io_type == IO_SERVER) {
+					config end;
+					end.add_child("end_scenarios");
+					network::send_data(end);
+			}
 			//ask to save a replay of the game
 			if(res == VICTORY || res == DEFEAT) {
 				const std::string orig_scenario = state.scenario;
 				state.scenario = current_scenario;
 
-				std::string label = state.label + " replay";
+				std::string label = state.label + _(" replay");
 
 				bool retry = true;
 
@@ -173,10 +315,8 @@ LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_conf
 				res = VICTORY;
 				save_game_after_scenario = false;
 			}
-
-			if(res != VICTORY) {
+			if(res != VICTORY)
 				return res;
-			}
 		} catch(game::load_game_failed& e) {
 			gui::show_error_message(disp, _("The game could not be loaded: ") + e.message);
 			return QUIT;
@@ -196,7 +336,7 @@ LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_conf
 			config cfg;
 			do {
 				cfg.clear();
-				network::connection data_res = gui::network_data_dialog(disp,
+				network::connection data_res = gui::network_receive_dialog(disp,
 						_("Downloading next level..."), cfg);
 				if(!data_res)
 					throw network::error(_("Connection timed out"));
@@ -245,7 +385,7 @@ LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_conf
 				// Adds player information, and other state
 				// information, to the configuration object
 				wassert(cfg.child("next_scenario") != NULL);
-				write_game(state, *cfg.child("next_scenario"), WRITE_SNAPSHOT_ONLY);
+				write_game(state, *cfg.child("next_scenario")/*, WRITE_SNAPSHOT_ONLY*/);
 				network::send_data(cfg);
 
 			} else if(io_type == IO_SERVER && scenario == NULL) {
@@ -255,35 +395,40 @@ LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_conf
 			}
 		}
 
-		//if this isn't the last scenario, then save the game
-		if(scenario != NULL && save_game_after_scenario) {
+		if(scenario != NULL) {
+			// update the label
+			std::string oldlabel = state.label;
 			state.label = (*scenario)["name"];
-			state.starting_pos = config();
 
-			bool retry = true;
+			//if this isn't the last scenario, then save the game
+			if(save_game_after_scenario) {
+				state.starting_pos = config();
 
-			while(retry) {
-				retry = false;
+				bool retry = true;
 
-				const int should_save = dialogs::get_save_name(disp,
-						_("Do you want to save your game?"),
+				while(retry) {
+					retry = false;
+
+					const int should_save = dialogs::get_save_name(disp,
+						_("Do you want to save your game? (Also erases Auto-Save files)"),
 						_("Name:"),
 						&state.label);
 
 
-				if(should_save == 0) {
-					try {
-						save_game(state);
-					} catch(game::save_game_failed&) {
-						gui::show_error_message(disp, _("The game could not be saved"));
-						retry = true;
+					if(should_save == 0) {
+						try {
+							save_game(state);
+							if (!oldlabel.empty())
+								clean_autosaves(oldlabel);
+						} catch(game::save_game_failed&) {
+							gui::show_error_message(disp, _("The game could not be saved"));
+							retry = true;
+						}
 					}
 				}
 			}
-		}
 
-		//update the replay start
-		if(scenario != NULL) {
+			//update the replay start
 			//FIXME: this should only be done if the scenario was not tweaked.
 			state.starting_pos = *scenario;
 		}
