@@ -1,4 +1,4 @@
-/* $Id: preferences.hpp 7648 2005-07-31 20:22:08Z j_daniel $ */
+/* $Id: preferences.hpp 12024 2006-06-10 23:37:20Z rusty $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -15,6 +15,10 @@
 
 class config;
 class display;
+class team;
+struct game_state;
+
+#include "unit.hpp"
 
 #include <string>
 #include <utility>
@@ -37,20 +41,20 @@ namespace preferences {
 		~manager();
 	};
 
-	struct display_manager
-	{
-		display_manager(display* disp);
-		~display_manager();
-	};
+	// low-level, should be seen only by preferences_display ?
+	void set(std::string key, std::string value);
+	std::string get(const std::string key);
+
+	config* get_prefs();
 
 	bool fullscreen();
-	void set_fullscreen(bool ison);
+	void _set_fullscreen(bool ison);
 
 	std::pair<int,int> resolution();
-	void set_resolution(const std::pair<int,int>& res);
+	void _set_resolution(const std::pair<int,int>& res);
 
 	bool turbo();
-	void set_turbo(bool ison);
+	void _set_turbo(bool ison);
 
 	const std::string& language();
 	void set_language(const std::string& s);
@@ -69,17 +73,17 @@ namespace preferences {
 	int music_volume();
 	void set_music_volume(int vol);
 
-	void mute(bool muted);
-	bool is_muted();
-
 	bool adjust_gamma();
-	void set_adjust_gamma(bool val);
+	void _set_adjust_gamma(bool val);
 
 	int gamma();
-	void set_gamma(int gamma);
+	void _set_gamma(int gamma);
 
 	bool grid();
-	void set_grid(bool ison);
+	void _set_grid(bool ison);
+
+	bool lobby_joins();
+	void _set_lobby_joins(bool show);
 
 	const std::string& official_network_host();
 
@@ -94,6 +98,8 @@ namespace preferences {
 
 	int scroll_speed();
 	void set_scroll_speed(int scroll);
+	bool mouse_scroll_enabled();
+	void enable_mouse_scroll(bool value);
 
 	bool turn_bell();
 	void set_turn_bell(bool ison);
@@ -124,6 +130,20 @@ namespace preferences {
 	int turns();
 	void set_turns(int value);
 
+	bool skip_mp_replay();
+	void set_skip_mp_replay(bool value);
+
+	bool countdown();
+	void set_countdown(bool value);
+	int countdown_init_time();
+	void set_countdown_init_time(int value);
+	int countdown_turn_bonus();
+	void set_countdown_turn_bonus(int value);
+	int countdown_reservoir_time();
+	void set_countdown_reservoir_time(int value);
+	int countdown_action_bonus();
+	void set_countdown_action_bonus(int value);
+
 	int village_gold();
 	void set_village_gold(int value);
 
@@ -146,7 +166,7 @@ namespace preferences {
 	bool ask_delete_saves();
 
 	bool use_colour_cursors();
-	void set_colour_cursors(bool value);
+	void _set_colour_cursors(bool value);
 
 	bool show_floating_labels();
 	void set_show_floating_labels(bool value);
@@ -166,6 +186,17 @@ namespace preferences {
 	bool flip_time();
 	void set_flip_time(bool value);
 
+	bool upload_log();
+	void set_upload_log(bool value);
+	const std::string &upload_id();
+
+	// Multiplayer functions
+	bool chat_timestamp();
+	void set_chat_timestamp(bool value);
+
+	int chat_lines();
+	void set_chat_lines(int lines);
+
 	bool compress_saves();
 
 	std::set<std::string> &encountered_units();
@@ -182,16 +213,28 @@ namespace preferences {
 	void set_theme(const std::string& theme);
 	const std::string& theme();
 
-	void show_preferences_dialog(display& disp, const config& game_cfg);
-	bool show_video_mode_dialog(display& disp);
-	// If prefs is non-null, save the hotkeys in that config instead of
-	// the default.
-	void show_hotkeys_dialog (display & disp, config *prefs=NULL);
+	bool compare_resolutions(const std::pair<int,int>& lhs, const std::pair<int,int>& rhs);
 
 	// Ask for end turn confirmation
 	bool yellow_confirm();
 	bool green_confirm();
 	bool confirm_no_moves();
+
+	// proxies for preferences_dialog
+	void load_hotkeys();
+	void save_hotkeys();
+
+	// Add all recruitable units as encountered so that information
+	// about them are displayed to the user in the help system.
+	void encounter_recruitable_units(std::vector<team>& teams);
+	// Add all units that exist at the start to the encountered units so
+	// that information about them are displayed to the user in the help
+	// system.
+	void encounter_start_units(unit_map& units);
+	// Add all units that are recallable as encountred units.
+	void encounter_recallable_units(game_state& gamestate);
+	// Add all terrains on the map as encountered terrains.
+	void encounter_map_terrain(gamemap& map);
 }
 
 #endif
