@@ -1,4 +1,4 @@
-/* $Id: publish_campaign.cpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: publish_campaign.cpp 8762 2005-11-06 23:51:57Z jhinrichs $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -177,7 +177,7 @@ namespace {
 
 bool campaign_name_legal(const std::string& name)
 {
-	if(name == "" || strlen(name.c_str()) == 0 ||
+	if(name == "" || strlen(name.c_str()) == 0 || name == "." ||
 	   std::find(name.begin(),name.end(),'/') != name.end() ||
 	   std::find(name.begin(),name.end(),'\\') != name.end() ||
 	   std::find(name.begin(),name.end(),':') != name.end() ||
@@ -187,3 +187,19 @@ bool campaign_name_legal(const std::string& name)
 		return true;
 	}
 }
+
+bool check_names_legal(const config& dir)
+{
+        const config::child_list& files = dir.get_children("file");
+        for(config::child_list::const_iterator i = files.begin(); i != files.end(); ++i) {
+                if (!campaign_name_legal((**i)["name"])) return false;
+        }
+        const config::child_list& dirs = dir.get_children("dir");
+		{
+			for(config::child_list::const_iterator i = dirs.begin(); i != dirs.end(); ++i) {
+					if (!campaign_name_legal((**i)["name"])) return false;
+					if (!check_names_legal(**i)) return false;
+			}
+		}
+        return true;
+}
