@@ -1,4 +1,4 @@
-/* $Id: race.cpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: race.cpp 11672 2006-05-28 05:06:56Z rusty $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -90,7 +90,7 @@ wide_string markov_generate_name(const markov_prefix_map& prefixes, size_t chain
 		if (i == prefixes.end() || i->second.empty()) {
 			return res;
 		}
-		if (std::find(i->second.begin(), i->second.end(), 0)
+		if (std::find(i->second.begin(), i->second.end(), static_cast<wchar_t>(0))
 				!= i->second.end()) {
 			// This ending is valid.
 			return res;
@@ -113,8 +113,8 @@ unit_race::unit_race() : ntraits_(0), chain_size_(0), not_living_(false), traits
 }
 
 unit_race::unit_race(const config& cfg) : name_(cfg["name"]), ntraits_(atoi(cfg["num_traits"].c_str())),
-                                          not_living_(cfg["not_living"] == "yes"),
-										  traits_(&cfg.get_children("trait")), global_traits_(cfg["ignore_global_traits"] != "yes")
+                                          not_living_(utils::string_bool(cfg["not_living"])),
+										  traits_(&cfg.get_children("trait")), global_traits_(!utils::string_bool(cfg["ignore_global_traits"]))
 {
 	names_[MALE] = utils::split(cfg["male_names"]);
 	names_[FEMALE] = utils::split(cfg["female_names"]);
@@ -144,6 +144,6 @@ const config::child_list& unit_race::additional_traits() const
 	return *traits_;
 }
 
-int unit_race::num_traits() const { return ntraits_; }
+unsigned int unit_race::num_traits() const { return ntraits_; }
 
 bool unit_race::not_living() const { return not_living_; }
