@@ -1,4 +1,4 @@
-/* $Id: replay.cpp 8865 2005-11-17 04:18:36Z brunowolff $ */
+/* $Id: replay.cpp 12695 2006-07-21 16:39:08Z jhinrichs $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -16,13 +16,12 @@
 #include "actions.hpp"
 #include "ai_interface.hpp"
 #include "dialogs.hpp"
+#include "display.hpp"
+#include "filesystem.hpp"
 #include "game_config.hpp"
 #include "game_events.hpp"
 #include "log.hpp"
-#include "network.hpp"
 #include "pathfind.hpp"
-#include "playlevel.hpp"
-#include "playturn.hpp"
 #include "preferences.hpp"
 #include "replay.hpp"
 #include "show_dialog.hpp"
@@ -42,15 +41,19 @@
 #define LOG_NW LOG_STREAM(info, network)
 #define ERR_NW LOG_STREAM(err, network)
 
+std::string replay::last_replay_error;
+
 //functions to verify that the unit structure on both machines is identical
 namespace {
+
 	void verify(const unit_map& units, const config& cfg)
 	{
+		std::stringstream errbuf;
 		LOG_NW << "verifying unit structure...\n";
 
 		const size_t nunits = atoi(cfg["num_units"].c_str());
 		if(nunits != units.size()) {
-			ERR_NW << "SYNC VERIFICATION FAILED: number of units from data source differ: "
+			errbuf << "SYNC VERIFICATION FAILED: number of units from data source differ: "
 			       << nunits << " according to data source. " << units.size() << " locally\n";
 
 			std::set<gamemap::location> locs;
@@ -60,19 +63,19 @@ namespace {
 				locs.insert(loc);
 
 				if(units.count(loc) == 0) {
-					ERR_NW << "data source says there is a unit at "
+					errbuf << "data source says there is a unit at "
 					       << loc << " but none found locally\n";
 				}
 			}
 
 			for(unit_map::const_iterator j = units.begin(); j != units.end(); ++j) {
 				if(locs.count(j->first) == 0) {
-					ERR_NW << "local unit at " << j->first
+					errbuf << "local unit at " << j->first
 					       << " but none in data source\n";
 				}
 			}
-
-			if (!game_config::ignore_replay_errors) throw replay::error();
+			replay::throw_error(errbuf.str());
+			errbuf.clear();
 		}
 
 		const config::child_list& items = cfg.get_children("unit");
@@ -80,10 +83,11 @@ namespace {
 			const gamemap::location loc(**i);
 			const unit_map::const_iterator u = units.find(loc);
 			if(u == units.end()) {
-				ERR_NW << "SYNC VERIFICATION FAILED: data source says there is a '"
+				errbuf << "SYNC VERIFICATION FAILED: data source says there is a '"
 				       << (**i)["type"] << "' (side " << (**i)["side"] << ") at "
 				       << loc << " but there is no local record of it\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				replay::throw_error(errbuf.str());
+				errbuf.clear();
 			}
 
 			config cfg;
@@ -93,7 +97,7 @@ namespace {
 			static const std::string fields[] = {"type","hitpoints","experience","side",""};
 			for(const std::string* str = fields; str->empty() == false; ++str) {
 				if(cfg[*str] != (**i)[*str]) {
-					ERR_NW << "ERROR IN FIELD '" << *str << "' for unit at "
+					errbuf << "ERROR IN FIELD '" << *str << "' for unit at "
 					       << loc << " data source: '" << (**i)[*str]
 					       << "' local: '" << cfg[*str] << "'\n";
 					is_ok = false;
@@ -101,8 +105,9 @@ namespace {
 			}
 
 			if(!is_ok) {
-				ERR_NW << "(SYNC VERIFICATION FAILED)\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				errbuf << "(SYNC VERIFICATION FAILED)\n";
+				replay::throw_error(errbuf.str());
+				errbuf.clear();
 			}
 		}
 
@@ -169,6 +174,14 @@ replay::replay() : pos_(0), current_(NULL), skip_(0)
 replay::replay(const config& cfg) : cfg_(cfg), pos_(0), current_(NULL), skip_(0)
 {}
 
+void replay::throw_error(const std::string& msg)
+{
+	ERR_NW << msg;
+	last_replay_error = msg;
+	if (!game_config::ignore_replay_errors) throw replay::error(msg);
+}
+
+
 config& replay::get_config()
 {
 	return cfg_;
@@ -184,20 +197,14 @@ const game_state& replay::get_save_info() const
 	return saveInfo_;
 }
 
-void replay::set_skip(int turns_to_skip)
+void replay::set_skip(bool skip)
 {
-	skip_ = turns_to_skip;
+	skip_ = skip;
 }
 
-void replay::next_skip()
+bool replay::is_skipping() const
 {
-	if(skip_ > 0)
-		--skip_;
-}
-
-bool replay::skipping() const
-{
-	return at_end() == false && skip_ != 0;
+	return skip_;
 }
 
 void replay::save_game(const std::string& label, const config& snapshot,
@@ -221,9 +228,24 @@ void replay::save_game(const std::string& label, const config& snapshot,
 	saveInfo_.snapshot = config();
 }
 
+void replay::add_unit_checksum(const gamemap::location& loc,config* const cfg)
+{
+	if(! game_config::mp_debug) {
+		return;
+	}
+	wassert(unit_map_ref);
+	config& cc = cfg->add_child("checksum");
+	loc.write(cc);
+	unit_map::const_iterator u = unit_map_ref->find(loc);
+	wassert(u != unit_map_ref->end());
+	std::string chk_value;
+	u->second.write_checksum(chk_value);
+	cc["value"] = chk_value;
+}
+
 void replay::add_start()
 {
-	config* const cmd = add_command();
+	config* const cmd = add_command(true);
 	cmd->add_child("start");
 }
 
@@ -270,17 +292,33 @@ void replay::add_disband(int value)
 	cmd->add_child("disband",val);
 }
 
+void replay::add_countdown_update(int value, int team)
+{
+	config* const cmd = add_command();
+	config val;
+
+	val["value"] = lexical_cast_default<std::string>(value);
+	val["team"] = lexical_cast_default<std::string>(team);
+
+	cmd->add_child("countdown_update",val);
+}
+
+
 void replay::add_movement(const gamemap::location& a,const gamemap::location& b)
 {
 	add_pos("move",a,b);
 }
 
-void replay::add_attack(const gamemap::location& a, const gamemap::location& b, int weapon)
+void replay::add_attack(const gamemap::location& a, const gamemap::location& b, int att_weapon, int def_weapon)
 {
 	add_pos("attack",a,b);
 	char buf[100];
-	snprintf(buf,sizeof(buf),"%d",weapon);
+	snprintf(buf,sizeof(buf),"%d",att_weapon);
 	current_->child("attack")->values["weapon"] = buf;
+	snprintf(buf,sizeof(buf),"%d",def_weapon);
+	current_->child("attack")->values["defender_weapon"] = buf;
+	add_unit_checksum(a,current_);
+	add_unit_checksum(b,current_);
 }
 
 void replay::add_pos(const std::string& type,
@@ -329,6 +367,16 @@ void replay::add_label(const std::string& text, const gamemap::location& loc)
 	cmd->add_child("label",val);
 }
 
+void replay::clear_labels()
+{
+	config* const cmd = add_command(false);
+
+	(*cmd)["undo"] = "no";
+
+
+	cmd->add_child("clear_labels");
+}
+
 void replay::add_rename(const std::string& name, const gamemap::location& loc)
 {
 	config* const cmd = add_command(false);
@@ -344,7 +392,21 @@ void replay::end_turn()
 	config* const cmd = add_command();
 	cmd->add_child("end_turn");
 }
-
+void replay::add_event(const std::string& name)
+{
+	config* const cmd = add_command();
+	config& ev = cmd->add_child("fire_event");
+	ev["raise"] = name;
+	(*cmd)["undo"] = "no";
+}
+void replay::add_checksum_check(const gamemap::location& loc)
+{
+	if(! game_config::mp_debug) {
+		return;
+	}
+	config* const cmd = add_command();
+	add_unit_checksum(loc,cmd);
+}
 void replay::speak(const config& cfg)
 {
 	config* const cmd = add_command(false);
@@ -459,9 +521,8 @@ config* replay::get_next_action()
 
 void replay::pre_replay()
 {
-	while(pos_ < commands().size() && commands()[pos_]->child("start") != NULL) {
-		if(get_next_action() == NULL)
-			return;
+	if ((rng::random() == NULL) && (commands().size() > 0)){
+		set_random(commands()[pos_]);
 	}
 }
 
@@ -502,6 +563,7 @@ void replay::add_config(const config& cfg, MARK_SENT mark)
 }
 
 namespace {
+
 replay* replay_src = NULL;
 
 struct replay_source_manager
@@ -531,6 +593,32 @@ replay& get_replay_source()
 	}
 }
 
+namespace {
+void check_checksums(display& disp,const unit_map& units,const config& cfg)
+{
+	if(! game_config::mp_debug) {
+		return;
+	}
+	for(config::child_list::const_iterator ci = cfg.get_children("checksum").begin(); ci != cfg.get_children("checksum").end(); ++ci) {
+		gamemap::location loc(**ci);
+		unit_map::const_iterator u = units.find(loc);
+		if(u == units.end()) {
+			std::stringstream message;
+			message << "non existant unit to checksum at " << loc.x+1 << "," << loc.y+1 << "!";
+			disp.add_chat_message("verification",1,message.str(),display::MESSAGE_PRIVATE);
+			continue;
+		}
+		std::string check;
+		u->second.write_checksum(check);
+		if(check != (**ci)["value"]) {
+			std::stringstream message;
+			message << "checksum mismatch at " << loc.x+1 << "," << loc.y+1 << "!";
+			disp.add_chat_message("verification",1,message.str(),display::MESSAGE_PRIVATE);
+		}
+	}
+}
+}
+
 bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
                unit_map& units,
 	       std::vector<team>& teams, int team_num, const gamestatus& state,
@@ -538,16 +626,17 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 {
 	log_scope("do replay");
 
-	const replay_source_manager replay_manager(obj);
+	const replay_source_manager replaymanager(obj);
 
 	replay& replayer = (obj != NULL) ? *obj : recorder;
 
-	clear_shroud(disp,state,map,gameinfo,units,teams,team_num-1);
-	disp.recalculate_minimap();
+	if (!replayer.is_skipping()){
+		disp.recalculate_minimap();
+	}
 
 	const set_random_generator generator_setter(&replayer);
 
-	update_locker lock_update(disp.video(),replayer.skipping());
+	update_locker lock_update(disp.video(),replayer.is_skipping());
 
 	//a list of units that have promoted from the last attack
 	std::deque<gamemap::location> advancing_units;
@@ -564,8 +653,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 		//if we are expecting promotions here
 		if(advancing_units.empty() == false) {
 			if(cfg == NULL) {
-				ERR_NW << "promotion expected, but none found\n";
-				throw replay::error();
+				replay::throw_error("promotion expected, but none found\n");
 			}
 
 			//if there is a promotion, we process it and go onto the next command
@@ -589,9 +677,10 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			}
 		}
 
+
 		//if there is nothing more in the records
 		if(cfg == NULL) {
-			replayer.set_skip(0);
+			//replayer.set_skip(false);
 			return false;
 		}
 
@@ -601,31 +690,40 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 		} else if((child = cfg->child("speak")) != NULL) {
 			const std::string& team_name = (*child)["team_name"];
 			if(team_name == "" || teams[disp.viewing_team()].team_name() == team_name) {
-				if(preferences::message_bell()) {
-					if(!replayer.skipping())
-						sound::play_sound(game_config::sounds::receive_message);
-				}
+				bool is_lobby_join = ((*child)["description"] == "server"
+							&& (*child)["message"].value().find("has logged into the lobby") != std::string::npos);
 
-				const int side = lexical_cast_default<int>((*child)["side"].c_str());
-				disp.add_chat_message((*child)["description"],side,(*child)["message"],
-				                      team_name == "" ? display::MESSAGE_PUBLIC : display::MESSAGE_PRIVATE);
+				if(!replayer.is_skipping() && (!is_lobby_join || preferences::lobby_joins())) {
+					if(preferences::message_bell()) {
+						sound::play_sound(game_config::sounds::receive_message);
+					}
+
+					const int side = lexical_cast_default<int>((*child)["side"].c_str(),0);
+					disp.add_chat_message((*child)["description"],side,(*child)["message"],
+										  team_name == "" ? display::MESSAGE_PUBLIC : display::MESSAGE_PRIVATE);
+				}
 			}
 		} else if((child = cfg->child("label")) != NULL) {
 			const gamemap::location loc(*child);
 			const std::string& text = (*child)["text"];
 
-			disp.labels().set_label(loc,text);
+			if (!replayer.is_skipping()){
+				disp.labels().set_label(loc,text);
+			}
+		} else if((child = cfg->child("clear_labels")) != NULL) {
+			disp.labels().clear();
 		}
 
 		else if((child = cfg->child("rename")) != NULL) {
 			const gamemap::location loc(*child);
 			const std::string& name = (*child)["name"];
 
-			std::map<gamemap::location,unit>::iterator u = units.find(loc);
+			unit_map::iterator u = units.find(loc);
 
 			if(u->second.unrenamable()) {
-				ERR_NW << "renaming unrenamable unit " << u->second.name() << "\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				std::stringstream errbuf;
+				errbuf << "renaming unrenamable unit " << u->second.name() << "\n";
+				replay::throw_error(errbuf.str());
 			}
 
 			u->second.rename(name);
@@ -633,8 +731,6 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 		//if there is an end turn directive
 		else if(cfg->child("end_turn") != NULL) {
-			replayer.next_skip();
-
 			child = cfg->child("verify");
 			if(child != NULL) {
 				verify_units(*child);
@@ -651,32 +747,41 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 			const std::set<std::string>& recruits = current_team.recruits();
 
-			if(val < 0 || val >= recruits.size()) {
-				ERR_NW << "recruitment index is illegal: " << val
+			if(val < 0 || (size_t)val >= recruits.size()) {
+				std::stringstream errbuf;
+				errbuf << "recruitment index is illegal: " << val
 				       << " while this side only has " << recruits.size()
 				       << " units available for recruitment\n";
-				throw replay::error();
+				replay::throw_error(errbuf.str());
 			}
 
 			std::set<std::string>::const_iterator itor = recruits.begin();
 			std::advance(itor,val);
 			const std::map<std::string,unit_type>::const_iterator u_type = gameinfo.unit_types.find(*itor);
 			if(u_type == gameinfo.unit_types.end()) {
-				ERR_NW << "recruiting illegal unit: '" << *itor << "'\n";
-				throw replay::error();
+				std::stringstream errbuf;
+				errbuf << "recruiting illegal unit: '" << *itor << "'\n";
+				replay::throw_error(errbuf.str());
 			}
 
-			unit new_unit(&(u_type->second),team_num,true);
+			bool old_replay = false;
+			float version = atof(state_of_game.version.substr(0, 3).c_str());
+			if (version < 1.1){
+				old_replay = true;
+			}
+			unit new_unit(&gameinfo,&units,&map,&state,&teams,&(u_type->second),team_num,true, old_replay);
 			const std::string& res = recruit_unit(map,team_num,units,new_unit,loc);
 			if(!res.empty()) {
-				ERR_NW << "cannot recruit unit: " << res << "\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				std::stringstream errbuf;
+				errbuf << "cannot recruit unit: " << res << "\n";
+				replay::throw_error(errbuf.str());
 			}
 
 			if(u_type->second.cost() > current_team.gold()) {
-				ERR_NW << "unit '" << u_type->second.id() << "' is too expensive to recruit: "
+				std::stringstream errbuf;
+				errbuf << "unit '" << u_type->second.id() << "' is too expensive to recruit: "
 				       << u_type->second.cost() << "/" << current_team.gold() << "\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				replay::throw_error(errbuf.str());
 			}
 			LOG_NW << "recruit: team=" << team_num << " '" << u_type->second.id() << "' at (" << loc
 			       << ") cost=" << u_type->second.cost() << " from gold=" << current_team.gold() << ' ';
@@ -686,14 +791,14 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 			current_team.spend_gold(u_type->second.cost());
 			LOG_NW << "-> " << (current_team.gold()) << "\n";
-			fix_shroud = true;
-		}
+			fix_shroud = !replayer.is_skipping();
+			check_checksums(disp,units,*cfg);
+}
 
 		else if((child = cfg->child("recall")) != NULL) {
 			player_info* player = state_of_game.get_player(current_team.save_id());
 			if(player == NULL) {
-				ERR_NW << "illegal recall\n";
-				throw replay::error();
+				replay::throw_error("illegal recall\n");
 			}
 
 			sort_units(player->available_units);
@@ -705,21 +810,21 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 			if(val >= 0 && val < int(player->available_units.size())) {
 				statistics::recall_unit(player->available_units[val]);
+				player->available_units[val].set_game_context(&gameinfo,&units,&map,&state,&teams);
 				recruit_unit(map,team_num,units,player->available_units[val],loc);
 				player->available_units.erase(player->available_units.begin()+val);
 				current_team.spend_gold(game_config::recall_cost);
 			} else {
-				ERR_NW << "illegal recall\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				replay::throw_error("illegal recall\n");
 			}
-			fix_shroud = true;
+			fix_shroud = !replayer.is_skipping();
+			check_checksums(disp,units,*cfg);
 		}
 
 		else if((child = cfg->child("disband")) != NULL) {
 			player_info* const player = state_of_game.get_player(current_team.save_id());
 			if(player == NULL) {
-				ERR_NW << "illegal disband\n";
-				throw replay::error();
+				replay::throw_error("illegal disband\n");
 			}
 
 			sort_units(player->available_units);
@@ -729,73 +834,92 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			if(val >= 0 && val < int(player->available_units.size())) {
 				player->available_units.erase(player->available_units.begin()+val);
 			} else {
-				ERR_NW << "illegal disband\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				replay::throw_error("illegal disband\n");
+			}
+		}
+		else if((child = cfg->child("countdown_update")) != NULL) {
+			const std::string& num = (*child)["value"];
+			const int val = lexical_cast_default<int>(num);
+			const std::string& tnum = (*child)["team"];
+			const int tval = lexical_cast_default<int>(tnum,-1);
+			if ( (tval<0)  || ((size_t)tval > teams.size()) ) {
+				std::stringstream errbuf;
+				errbuf << "Illegal countdown update \n" << "Received update for :" << tval << " Current user :" << team_num << "\n" << " Updated value :" << val;
+				replay::throw_error(errbuf.str());
+			} else {
+				teams[tval-1].set_countdown_time(val);
 			}
 		}
-
 		else if((child = cfg->child("move")) != NULL) {
 
 			const config* const destination = child->child("destination");
 			const config* const source = child->child("source");
 
 			if(destination == NULL || source == NULL) {
-				ERR_NW << "no destination/source found in movement\n";
-				throw replay::error();
+				replay::throw_error("no destination/source found in movement\n");
 			}
 
 			const gamemap::location src(*source);
 			const gamemap::location dst(*destination);
 
-			std::map<gamemap::location,unit>::iterator u = units.find(dst);
+			unit_map::iterator u = units.find(dst);
 			if(u != units.end()) {
-				ERR_NW << "destination already occupied: "
+				std::stringstream errbuf;
+				errbuf << "destination already occupied: "
 				       << dst << '\n';
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				replay::throw_error(errbuf.str());
 			}
 			u = units.find(src);
 			if(u == units.end()) {
-				ERR_NW << "unfound location for source of movement: "
+				std::stringstream errbuf;
+				errbuf << "unfound location for source of movement: "
 				       << src << " -> " << dst << '\n';
-				throw replay::error();
+				replay::throw_error(errbuf.str());
 			}
 
-			const bool ignore_zocs = u->second.type().is_skirmisher();
-			const bool teleport = u->second.type().teleports();
+			const bool ignore_zocs = u->second.get_ability_bool("skirmisher",u->first);
+			const bool teleport = u->second.get_ability_bool("teleport",u->first);
 
-			paths paths_list(map,state,gameinfo,units,src,teams,ignore_zocs,teleport);
-			paths_wiper wiper(disp);
-
-			unit current_unit = u->second;
+			paths paths_list(map,state,gameinfo,units,src,teams,ignore_zocs,teleport,current_team);
 
 			std::map<gamemap::location,paths::route>::iterator rt = paths_list.routes.find(dst);
 			if(rt == paths_list.routes.end()) {
 
+				std::stringstream errbuf;
 				for(rt = paths_list.routes.begin(); rt != paths_list.routes.end(); ++rt) {
-					ERR_NW << "can get to: " << rt->first << '\n';
+					errbuf << "can get to: " << rt->first << '\n';
 				}
 
-				ERR_NW << "src cannot get to dst: " << current_unit.movement_left() << ' '
+				errbuf << "src cannot get to dst: " << u->second.movement_left() << ' '
 				       << paths_list.routes.size() << ' ' << src << " -> " << dst << '\n';
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				replay::throw_error(errbuf.str());
 			}
 
 			rt->second.steps.push_back(dst);
 
-			if(!replayer.skipping() && unit_display::unit_visible_on_path(disp,map,rt->second.steps,current_unit,state.get_time_of_day(),units,teams)) {
-				disp.set_paths(&paths_list);
+			if(!replayer.is_skipping() && unit_display::unit_visible_on_path(disp,rt->second.steps,u->second,units,teams)) {
 
 				disp.scroll_to_tiles(src.x,src.y,dst.x,dst.y);
 			}
 
-			units.erase(u);
-
-			if(!replayer.skipping()) {
-				unit_display::move_unit(disp,map,rt->second.steps,current_unit,state.get_time_of_day(),units,teams);
+			if(!replayer.is_skipping()) {
+				u->second.set_hidden(true);
+				unit_display::move_unit(disp,map,rt->second.steps,u->second,units,teams);
+				u->second.set_hidden(false);
 			}
+			else{
+				//unit location needs to be updated
+				u->second.set_goto(*(rt->second.steps.end() - 1));
+			}
+			u->second.set_movement(rt->second.move_left);
 
-			current_unit.set_movement(rt->second.move_left);
-			u = units.insert(std::pair<gamemap::location,unit>(dst,current_unit)).first;
+			std::pair<gamemap::location,unit> *up = units.extract(u->first);
+			up->first = dst;
+			units.add(up);
+			u = units.find(dst);
+			check_checksums(disp,units,*cfg);
+			// Get side now, in case game events change the unit.
+			int u_team = u->second.side()-1;
 			if(map.is_village(dst)) {
 				const int orig_owner = village_owner(dst,teams) + 1;
 				if(orig_owner != team_num) {
@@ -804,28 +928,33 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 				}
 			}
 
-			if(!replayer.skipping()) {
-				disp.draw_tile(dst.x,dst.y);
-				disp.update_display();
+			if(!replayer.is_skipping()) {
+				disp.invalidate(dst);
+				disp.draw();
 			}
 
 			game_events::fire("moveto",dst);
-			//FIXME: what's special about team 1?
+			//FIXME: what's special about team 1? regroup it with next block
 			if(team_num != 1 && (teams.front().uses_shroud() || teams.front().uses_fog()) && !teams.front().fogged(dst.x,dst.y)) {
 				game_events::fire("sighted",dst);
 			}
+			for(std::vector<team>::iterator my_team = teams.begin() ; my_team != teams.end() ; my_team++) {
+				if((my_team->uses_shroud() || my_team->uses_fog()) && !my_team->fogged(dst.x,dst.y)) {
+					my_team->see(u_team);
+				}
+			}
 
-			fix_shroud = true;
+			fix_shroud = !replayer.is_skipping();
+			disp.unhighlight_reach();
 		}
 
 		else if((child = cfg->child("attack")) != NULL) {
-
 			const config* const destination = child->child("destination");
 			const config* const source = child->child("source");
+			check_checksums(disp,units,*cfg);
 
 			if(destination == NULL || source == NULL) {
-				ERR_NW << "no destination/source found in attack\n";
-				throw replay::error();
+				replay::throw_error("no destination/source found in attack\n");
 			}
 
 			const gamemap::location src(*source);
@@ -834,32 +963,33 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			const std::string& weapon = (*child)["weapon"];
 			const int weapon_num = atoi(weapon.c_str());
 
-			std::map<gamemap::location,unit>::iterator u = units.find(src);
+			const std::string& def_weapon = (*child)["defender_weapon"];
+			int def_weapon_num = -1;
+			if (def_weapon.empty()) {
+				// Let's not gratuitously destroy backwards compat.
+				ERR_NW << "Old data, having to guess weapon\n";
+			} else {
+				def_weapon_num = atoi(def_weapon.c_str());
+			}
+
+			unit_map::iterator u = units.find(src);
 			if(u == units.end()) {
-				ERR_NW << "unfound location for source of attack\n";
-				throw replay::error();
+				replay::throw_error("unfound location for source of attack\n");
 			}
 
 			if(size_t(weapon_num) >= u->second.attacks().size()) {
-				ERR_NW << "illegal weapon type in attack\n";
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				replay::throw_error("illegal weapon type in attack\n");
 			}
 
-			std::map<gamemap::location,unit>::const_iterator tgt = units.find(dst);
+			unit_map::const_iterator tgt = units.find(dst);
 
 			if(tgt == units.end()) {
-				ERR_NW << "unfound defender for attack: " << src << " -> " << dst << '\n';
-				if (!game_config::ignore_replay_errors) throw replay::error();
+				std::stringstream errbuf;
+				errbuf << "unfound defender for attack: " << src << " -> " << dst << '\n';
+				replay::throw_error(errbuf.str());
 			}
 
-			game_events::fire("attack",src,dst);
-
-			u = units.find(src);
-			tgt = units.find(dst);
-
-			if(u != units.end() && tgt != units.end()) {
-				attack(disp, map, teams, src, dst, weapon_num, units, state, gameinfo);
-			}
+			attack(disp, map, teams, src, dst, weapon_num, def_weapon_num, units, state, gameinfo, !replayer.is_skipping());
 
 			u = units.find(src);
 			tgt = units.find(dst);
@@ -877,16 +1007,25 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			if(advancing_units.empty()) {
 				check_victory(units,teams);
 			}
-			fix_shroud = true;
+			fix_shroud = !replayer.is_skipping();
+		} else if((child = cfg->child("fire_event")) != NULL) {
+			for(config::child_list::const_iterator v = child->get_children("set_variable").begin(); v != child->get_children("set_variable").end(); ++v) {
+				state_of_game.set_variable((**v)["name"],(**v)["value"]);
+			}
+			game_events::fire((*child)["raise"]);
 		} else {
-			ERR_NW << "unrecognized action\n";
-			if (!game_config::ignore_replay_errors) throw replay::error();
+			if(! cfg->child("checksum")) {
+				replay::throw_error("unrecognized action\n");
+			} else {
+				check_checksums(disp,units,*cfg);
+			}
 		}
 
 		//Check if we should refresh the shroud, and redraw the minimap/map tiles.
 		//This is needed for shared vision to work properly.
-		if(fix_shroud && clear_shroud(disp,state,map,gameinfo,units,teams,team_num-1)) {
+		if(fix_shroud && clear_shroud(disp,state,map,gameinfo,units,teams,team_num-1) && !recorder.is_skipping()) {
 			disp.recalculate_minimap();
+			disp.invalidate_game_status();
 			disp.invalidate_all();
 		}
 
