@@ -1,4 +1,4 @@
-/* $Id: reports.cpp 8821 2005-11-12 19:23:56Z brunowolff $ */
+/* $Id: reports.cpp 12634 2006-07-18 21:30:30Z xan $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -14,11 +14,11 @@
 #include "global.hpp"
 
 #include "actions.hpp"
-#include "font.hpp"
 #include "game_config.hpp"
 #include "gamestatus.hpp"
 #include "gettext.hpp"
 #include "language.hpp"
+#include "marked-up_text.hpp"
 #include "reports.hpp"
 #include "wassert.hpp"
 #include "preferences.hpp"
@@ -29,13 +29,16 @@
 #include <sstream>
 
 namespace {
-	const std::string report_names[] = { "unit_description", "unit_type", "unit_level",
-		"unit_traits","unit_status","unit_alignment","unit_abilities","unit_hp","unit_xp",
-		"unit_moves","unit_weapons","unit_image","unit_profile","time_of_day",
+	const std::string report_names[] = { "unit_description", "unit_type",
+		"unit_level","unit_amla","unit_traits","unit_status",
+		"unit_alignment","unit_abilities","unit_hp","unit_xp",
+		"unit_advancement_options","unit_moves","unit_weapons",
+		"unit_image","unit_profile","time_of_day",
 		"turn","gold","villages","num_units","upkeep", "expenses",
-		 "income", "terrain", "position", "side_playing", "observers",
-		 "report_clock",
-		 "selected_terrain","edit_left_button_function"};
+		"income", "terrain", "position", "side_playing", "observers",
+		"report_countdown", "report_clock",
+		"selected_terrain","edit_left_button_function"
+	};
 	std::map<reports::TYPE, std::string> report_contents;
 }
 
@@ -71,31 +74,26 @@ void report::add_image(const std::string& image, const std::string& tooltip) {
 	this->push_back(element("",image,tooltip));
 }
 
-report generate_report(TYPE type, const gamemap& map, const unit_map& units,
+report generate_report(TYPE type, const gamemap& map, unit_map& units,
                        const std::vector<team>& teams, const team& current_team,
                        int current_side, int playing_side,
                        const gamemap::location& loc, const gamemap::location& mouseover,
                        const gamestatus& status, const std::set<std::string>& observers)
 {
-	unit_map::const_iterator u = units.end();
+	unit_map::iterator u = units.end();
+	SDL_Color HPC;
+	SDL_Color XPC;
+
 
 	if(int(type) >= int(UNIT_REPORTS_BEGIN) && int(type) < int(UNIT_REPORTS_END) || type == POSITION) {
 
-		if(!current_team.fogged(mouseover.x,mouseover.y)) {
-			u = find_visible_unit(units,mouseover,
-					map,
-					status.get_time_of_day().lawful_bonus,
-					teams,current_team);
-		}
-
+		u = find_visible_unit(units,mouseover,
+							  map,
+							  teams,current_team);
 		if(u == units.end()) {
-			if(!current_team.fogged(loc.x,loc.y)) {
-				u = find_visible_unit(units,loc,
-						map,
-						status.get_time_of_day().lawful_bonus,
-						teams,current_team);
-			}
-
+			u = find_visible_unit(units,loc,
+								  map,
+								  teams,current_team);
 			if(u == units.end() && type != POSITION) {
 				return report();
 			}
@@ -106,12 +104,20 @@ report generate_report(TYPE type, const gamemap& map, const unit_map& units,
 
 	switch(type) {
 	case UNIT_DESCRIPTION:
-		return report(u->second.description());
+		return report(u->second.description(),"",u->second.description());
 	case UNIT_TYPE:
-		return report(u->second.type().language_name(),"",u->second.unit_description());
+	        return report(u->second.language_name(),"",u->second.unit_description());
 	case UNIT_LEVEL:
-		str << u->second.type().level();
+		str << u->second.level();
 		break;
+	case UNIT_AMLA: {
+	  report res;
+	  const std::vector<std::pair<std::string,std::string> > & amla_icons=u->second.amla_icons();
+	  for(std::vector<std::pair<std::string,std::string> >::const_iterator i=amla_icons.begin();i!=amla_icons.end();i++){
+	    res.add_image(i->first,i->second);
+	  }
+	  return(res);
+	}
 	case UNIT_TRAITS:
 		return report(u->second.traits_description(),"",u->second.modification_description("trait"));
 	case UNIT_STATUS: {
@@ -119,24 +125,24 @@ report generate_report(TYPE type, const gamemap& map, const unit_map& units,
 		std::stringstream tooltip;
 		report res;
 
-		if(map.on_board(loc) && u->second.invisible(map.underlying_terrain(map[loc.x][loc.y]),status.get_time_of_day().lawful_bonus,loc,units,teams)) {
+		if(map.on_board(loc) && u->second.invisible(loc,units,teams)) {
 			unit_status << "misc/invisible.png";
 			tooltip << _("invisible: ") << _("This unit is invisible. It cannot be seen or attacked by enemy units.");
 			res.add_image(unit_status,tooltip);
 		}
-		if(u->second.has_flag("slowed")) {
+		if(utils::string_bool(u->second.get_state("slowed"))) {
 			unit_status << "misc/slowed.png";
-			tooltip << _("slowed: ") << _("This unit has been slowed. It moves at half normal speed and receives one less attack than normal in combat.");
+			tooltip << _("slowed: ") << _("This unit has been slowed. It will only deal half its normal damage when attacking.");
 			res.add_image(unit_status,tooltip);
 		}
-		if(u->second.has_flag("poisoned")) {
+		if(utils::string_bool(u->second.get_state("poisoned"))) {
 			unit_status << "misc/poisoned.png";
 			tooltip << _("poisoned: ") << _("This unit is poisoned. It will lose 8 HP every turn until it can seek a cure to the poison in a village or from a friendly unit with the 'cures' ability.\n\
 \n\
 Units cannot be killed by poison alone. The poison will not reduce it below 1 HP.");
 			res.add_image(unit_status,tooltip);
 		}
-		if(u->second.has_flag("stone")) {
+		if(utils::string_bool(u->second.get_state("stoned"))) {
 			unit_status << "misc/stone.png";
 			tooltip << _("stone: ") << _("This unit has been turned to stone. It may not move or attack.");
 			res.add_image(unit_status,tooltip);
@@ -145,51 +151,62 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 		return res;
 	}
 	case UNIT_ALIGNMENT: {
-		const std::string& align = unit_type::alignment_description(u->second.type().alignment());
-		const std::string& align_id = unit_type::alignment_id(u->second.type().alignment());
+		const std::string& align = unit_type::alignment_description(u->second.alignment());
+		const std::string& align_id = unit_type::alignment_id(u->second.alignment());
 		return report(align, "", string_table[align_id + "_description"]);
 	}
 	case UNIT_ABILITIES: {
 		report res;
 		std::stringstream tooltip;
-		const std::vector<std::string>& abilities = u->second.type().abilities();
+		const std::vector<std::string>& abilities = u->second.ability_tooltips(u->first);
 		for(std::vector<std::string>::const_iterator i = abilities.begin(); i != abilities.end(); ++i) {
 			str << gettext(i->c_str());
-			if(i+1 != abilities.end())
+			if(i+2 != abilities.end())
 				str << ",";
-
-			tooltip << string_table[*i + "_description"];
+			++i;
+			tooltip << i->c_str();//string_table[*i + "_description"];
 			res.add_text(str,tooltip);
 		}
 
 		return res;
 	}
-	case UNIT_HP:
-		if(u->second.hitpoints() <= u->second.max_hitpoints()/3)
-			str << font::BAD_TEXT;
-		else if(u->second.hitpoints() > 2*(u->second.max_hitpoints()/3))
-			str << font::GOOD_TEXT;
+	case UNIT_HP: {
+	  HPC=u->second.hp_color();
+	  str << "<" << (int) HPC.r << "," << (int) HPC.g << "," << (int) HPC.b << ">"
+	      << u->second.hitpoints() << "/" << u->second.max_hitpoints();
+	  break;
+	}
+	case UNIT_XP: {
+	  XPC=u->second.xp_color();
+	  str << "<" << (int) XPC.r << "," << (int) XPC.g << "," << (int) XPC.b << ">"
+	      << u->second.experience() << "/" << u->second.max_experience();
+	  break;
+	}
+	case UNIT_ADVANCEMENT_OPTIONS: {
+	  report res;
+	  const std::map<std::string,std::string>& adv_icons=u->second.advancement_icons();
+	  for(std::map<std::string,std::string>::const_iterator i=adv_icons.begin();i!=adv_icons.end();i++){
+	    res.add_image(i->first,i->second);
+	  }
+	  return(res);
 
-		str << u->second.hitpoints()
-		    << "/" << u->second.max_hitpoints() << "\n";
+	}
+	case UNIT_MOVES: {
+	  unsigned int movement_left = u->second.movement_left();
+	  if (u->second.side() != playing_side){
+		  movement_left = u->second.total_movement();
+	  }
 
+	  int x = 180;
+	  if(utils::string_bool(u->second.get_state("stoned"))){
+	    x = 128;
+	  }else{
+	    x = (int)(128 + (255-128)*((float)movement_left/u->second.total_movement()));
+	  }
+	  str << "<" << x << "," << x << "," << x <<">";
+	  str << movement_left << "/" << u->second.total_movement();
 		break;
-	case UNIT_XP:
-		if(u->second.can_advance() == false) {
-			str << u->second.experience() << "/-";
-		} else {
-			//if killing a unit of the same level as us lets us advance, display in 'good' colour
-			if(u->second.max_experience() - u->second.experience() <= game_config::kill_experience*u->second.type().level()) {
-				str << font::GOOD_TEXT;
-			}
-
-			str << u->second.experience() << "/" << u->second.max_experience();
-		}
-
-		break;
-	case UNIT_MOVES:
-		str << u->second.movement_left() << "/" << u->second.total_movement();
-		break;
+	}
 	case UNIT_WEAPONS: {
 		report res;
 		std::stringstream tooltip;
@@ -200,24 +217,62 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			return res;
 		}
 
-		const std::vector<attack_type>& attacks = u->second.attacks();
-		for(std::vector<attack_type>::const_iterator at_it = attacks.begin();
+		std::vector<attack_type>& attacks = u->second.attacks();
+		for(std::vector<attack_type>::iterator at_it = attacks.begin();
 		    at_it != attacks.end(); ++at_it) {
+			at_it->set_specials_context(u->first,u->second);
 			const std::string& lang_type = gettext(at_it->type().c_str());
+			str.str("");
+			str << "<245,230,193>";
+			if(utils::string_bool(u->second.get_state("slowed"))) {
+				str << round_damage(at_it->damage(),1,2) << "-" ;
+			} else {
+				str << at_it->damage() << "-" ;
+			}
+			int nattacks = at_it->num_attacks();
+			// compute swarm attacks;
+			unit_ability_list swarm = at_it->get_specials("attacks");
+			if(!swarm.empty()) {
+				int swarm_max_attacks = swarm.highest("attacks_max",nattacks).first;
+				int swarm_min_attacks = swarm.highest("attacks_min").first;
+				int hitp = u->second.hitpoints();
+				int mhitp = u->second.max_hitpoints();
 
-			str << at_it->name() << " (" << lang_type << ")\n";
+				nattacks = swarm_min_attacks + swarm_max_attacks * hitp / mhitp;
 
-			tooltip << at_it->name() << " (" << lang_type << ")\n";
+			} else {
+				nattacks = at_it->num_attacks();
+			}
+			str << nattacks;
+			str << " " << at_it->name();
+			tooltip << at_it->name() << "\n";
+			if(utils::string_bool(u->second.get_state("slowed"))) {
+				tooltip << round_damage(at_it->damage(),1,2) << " " << _("damage") << ", ";
+			} else {
+				tooltip << at_it->damage() << " " << _("damage") << ", ";
+			}
+			tooltip << nattacks;
+			tooltip << " " << _("attacks");
 
+			str<<"\n";
+			res.add_text(str,tooltip);
+
+			str << "<166,146,117>  ";
+			std::string range = _(at_it->range().c_str());
+			str << range << "--" << lang_type << "\n";
+			str<<"\n";
+
+			tooltip << _("weapon range: ") << range <<"\n";
+			tooltip << _("damage type: ") << lang_type << "\n";
 			//find all the unit types on the map, and show this weapon's bonus against all the different units
 			std::set<std::string> seen_units;
 			std::map<int,std::vector<std::string> > resistances;
 			for(unit_map::const_iterator u_it = units.begin(); u_it != units.end(); ++u_it) {
 				if(teams[team_index].is_enemy(u_it->second.side()) && !current_team.fogged(u_it->first.x,u_it->first.y) &&
-				   seen_units.count(u_it->second.type().id()) == 0) {
-					seen_units.insert(u_it->second.type().id());
-					const int resistance = u_it->second.type().movement_type().resistance_against(*at_it) - 100;
-					resistances[resistance].push_back(u_it->second.type().language_name());
+				   seen_units.count(u_it->second.id()) == 0) {
+					seen_units.insert(u_it->second.id());
+					const int resistance = u_it->second.resistance_against(*at_it,false,u_it->first) - 100;
+					resistances[resistance].push_back(u_it->second.language_name());
 				}
 			}
 
@@ -231,37 +286,42 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 
 					tooltip << *i;
 				}
-
 				tooltip << "\n";
 			}
 
 			res.add_text(str,tooltip);
 
-			if (!at_it->special().empty()) {
-				str << gettext(at_it->special().c_str()) << "\n";
-				tooltip << string_table["weapon_special_" + at_it->special() + "_description"];
+
+			const std::vector<std::string>& specials = at_it->special_tooltips();
+
+			if(! specials.empty()) {
+				for(std::vector<std::string>::const_iterator sp_it = specials.begin(); sp_it != specials.end(); ++sp_it) {
+					str << "<166,146,117>  ";
+					str << gettext(sp_it->c_str());
+					str<<"\n";
+					++sp_it;
+					tooltip << gettext(sp_it->c_str());
+				}
 				res.add_text(str,tooltip);
 			}
-
-			str << at_it->damage() << "-" << at_it->num_attacks() << " -- "
-		        << (at_it->range() == attack_type::SHORT_RANGE ?
-		            _("melee") :
-					_("ranged"));
-			tooltip << at_it->damage() << " " << _("damage") << ", "
-					<< at_it->num_attacks() << " " << _("attacks");
-
-			str << "\n";
-			res.add_text(str,tooltip);
 		}
 
 		return res;
 	}
 	case UNIT_IMAGE:
-		return report("",u->second.type().image(),"");
+	{
+		const std::vector<Uint32>& old_rgb = u->second.team_rgb_range();
+		color_range new_rgb = team::get_side_color_range(u->second.side());
+	    return report("",image::locator(u->second.absolute_image(), new_rgb, old_rgb),"");
+	}
 	case UNIT_PROFILE:
-		return report("",u->second.type().image_profile(),"");
+		return report("",u->second.profile(),"");
 	case TIME_OF_DAY: {
-		const time_of_day& tod = timeofday_at(status,units,mouseover);
+		time_of_day tod = timeofday_at(status,units,mouseover,map);
+		// don't show illuminated time on fogged/shrouded tiles
+		if (current_team.fogged(mouseover.x,mouseover.y) || current_team.shrouded(mouseover.x,mouseover.y)) {
+			tod = status.get_time_of_day(false,mouseover);
+		}
 		std::stringstream tooltip;
 
 		tooltip << tod.name << "\n"
@@ -315,7 +375,7 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			break;
 
 		const gamemap::TERRAIN terrain = map.get_terrain(mouseover);
-		const std::string& underlying = map.underlying_terrain(terrain);
+		const std::string& underlying = map.underlying_union_terrain(terrain);
 
 		if(map.is_village(mouseover)) {
 			const int owner = village_owner(mouseover,teams)+1;
@@ -328,25 +388,22 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			} else {
 				str << _("Allied village");
 			}
-
 			str << " ";
 		} else {
-			str << map.get_terrain_info(terrain).name();
+		        str << map.get_terrain_info(terrain).name();
 		}
 
 		if(underlying.size() != 1 || underlying[0] != terrain) {
-			str << " (";
+			str << "(";
 
 			for(std::string::const_iterator i = underlying.begin(); i != underlying.end(); ++i) {
-				str << map.get_terrain_info(*i).name();
+			str << map.get_terrain_info(*i).name();
 				if(i+1 != underlying.end()) {
 					str << ",";
 				}
 			}
-
 			str << ")";
 		}
-
 		break;
 	}
 	case POSITION: {
@@ -361,8 +418,8 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 
 		const gamemap::TERRAIN terrain = map[mouseover.x][mouseover.y];
 
-		const int move_cost = u->second.movement_cost(map,terrain);
-		const int defense = 100 - u->second.defense_modifier(map,terrain);
+		const int move_cost = u->second.movement_cost(terrain);
+		const int defense = 100 - u->second.defense_modifier(terrain);
 
 		if(move_cost > 10) {
 			str << " (-)";
@@ -374,11 +431,28 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 	}
 
 	case SIDE_PLAYING: {
-		char buf[50];
-		snprintf(buf,sizeof(buf),"terrain/flag-team%d-1.png",team::get_side_colour_index(playing_side));
 
+		std::string flag;
+		color_range new_rgb;
+		std::vector<Uint32> old_rgb;
+
+		if(current_team.flag().empty()) {
+			flag = game_config::flag_image;
+			old_rgb = game_config::flag_rgb;
+			new_rgb = team::get_side_color_range(playing_side);
+		} else {
+			flag = current_team.flag();
+		}
+
+		//must recolor flag image
+		animated<image::locator> temp_anim;
+
+		// remove animation stuff we don't care about
+		const std::vector<std::string> items = utils::split(flag);
+		const std::vector<std::string> sub_items = utils::split(items[0], ':');
+		image::locator flag_image(sub_items[0], new_rgb, old_rgb);
 		u = find_leader(units,playing_side);
-		return report("",buf,u != units.end() ? u->second.description() : "");
+		return report("",flag_image,u != units.end() ? u->second.description() : "");
 	}
 
 	case OBSERVERS: {
@@ -414,6 +488,27 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			return report();
 		}
 	}
+	case REPORT_COUNTDOWN: {
+		int min;
+		int sec;
+		if (current_team.countdown_time() > 0){
+			sec = current_team.countdown_time() / 1000;
+
+			if(sec < 60)
+				str << "<200,0,0>";
+			else if(sec < 120)
+				str << "<200,200,0>";
+
+			min = sec / 60;
+			str << min << ":";
+			sec = sec % 60;
+			if (sec < 10) {
+				str << "0";
+			}
+			str << sec;
+		}
+		break;
+		}
 	case REPORT_CLOCK: {
 		time_t t = time(NULL);
 		struct tm *lt=localtime(&t);
@@ -425,7 +520,6 @@ Units cannot be killed by poison alone. The poison will not reduce it below 1 HP
 			return report();
 		}
 	}
-
 	default:
 		wassert(false);
 		break;
