@@ -1,4 +1,4 @@
-/* $Id: preprocessor.cpp 8214 2005-09-12 21:33:34Z gruikya $ */
+/* $Id: preprocessor.cpp 12046 2006-06-11 20:42:17Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -29,12 +29,13 @@
 #define ERR_CF LOG_STREAM(err, config)
 #define LOG_CF LOG_STREAM(info, config)
 
+using std::streambuf;
+
 bool preproc_define::operator==(preproc_define const &v) const {
 	return value == v.value && arguments == v.arguments;
 }
 
-// FIXME
-struct config {
+struct preproc_config {
 	struct error {
 		error(const std::string& msg) : message(msg) {}
 		std::string message;
@@ -61,7 +62,7 @@ public:
 	virtual ~preprocessor();
 };
 
-class preprocessor_streambuf: public std::streambuf
+class preprocessor_streambuf: public streambuf
 {
 	std::string out_buffer_;
 	virtual int underflow();
@@ -90,7 +91,7 @@ preprocessor_streambuf::preprocessor_streambuf(preproc_map *def)
 }
 
 preprocessor_streambuf::preprocessor_streambuf(preprocessor_streambuf const &t)
-	: std::streambuf(), current_(NULL), defines_(t.defines_),
+	: streambuf(), current_(NULL), defines_(t.defines_),
 	  textdomain_(PACKAGE), depth_(t.depth_), quoted_(t.quoted_)
 {
 }
@@ -187,7 +188,7 @@ class preprocessor_data: preprocessor
 	void put(std::string const &);
 public:
 	preprocessor_data(preprocessor_streambuf &, std::istream *,
-	                  std::string const &history,  
+	                  std::string const &history,
 	                  std::string const &name, int line,
 	                  std::string const &dir, std::string const &domain);
 	virtual bool get_chunk();
@@ -372,7 +373,7 @@ bool preprocessor_data::get_chunk()
 		      << token.linenum << ' ' << target_.location_;
 		ERR_CF << error.str() << '\n';
 		pop_token();
-		throw config::error(error.str());
+		throw preproc_config::error(error.str());
 	}
 
 	if (c == '\376') {
@@ -406,7 +407,7 @@ bool preprocessor_data::get_chunk()
 			error << "nested quoted string started at "
 			      << linenum_ << ' ' << target_.location_;
 			ERR_CF << error.str() << '\n';
-			throw config::error(error.str());
+			throw preproc_config::error(error.str());
 		}
 	} else if (c == '{') {
 		if (token.type == '{')
@@ -429,7 +430,7 @@ bool preprocessor_data::get_chunk()
 				error << "no macro name found after #define directive at "
 				      << linenum_ << ' ' << target_.location_;
 				ERR_CF << error.str() << '\n';
-				throw config::error(error.str());
+				throw preproc_config::error(error.str());
 			}
 			std::string symbol = items.front();
 			items.erase(items.begin());
@@ -456,7 +457,7 @@ bool preprocessor_data::get_chunk()
 				error << "unterminated preprocessor definition at "
 				      << linenum << ' ' << target_.location_;
 				ERR_CF << error.str() << '\n';
-				throw config::error(error.str());
+				throw preproc_config::error(error.str());
 			}
 			if (!skipping_) {
 				buffer.erase(buffer.end() - 7, buffer.end());
@@ -487,7 +488,7 @@ bool preprocessor_data::get_chunk()
 				error << "unexpected #else at "
 				      << linenum_ << ' ' << target_.location_;
 				ERR_CF << error.str() << '\n';
-				throw config::error(error.str());
+				throw preproc_config::error(error.str());
 			}
 		} else if (command == "endif") {
 			switch (token.type) {
@@ -500,7 +501,7 @@ bool preprocessor_data::get_chunk()
 				error << "unexpected #endif at "
 				      << linenum_ << ' ' << target_.location_;
 				ERR_CF << error.str() << '\n';
-				throw config::error(error.str());
+				throw preproc_config::error(error.str());
 			}
 			pop_token();
 		} else if (command == "textdomain") {
@@ -515,7 +516,12 @@ bool preprocessor_data::get_chunk()
 			error << "unexpected #enddef at "
 			      << linenum_ << ' ' << target_.location_;
 			ERR_CF << error.str() << '\n';
-			throw config::error(error.str());
+			throw preproc_config::error(error.str());
+		} else if (command == "undef") {
+			skip_spaces();
+			std::string const &symbol = read_word();
+			target_.defines_->erase(symbol);
+			LOG_CF << "undefine macro " << symbol << " (location " << target_.location_ << ")\n";
 		} else
 			comment = true;
 		skip_eol();
@@ -564,7 +570,7 @@ bool preprocessor_data::get_chunk()
 					      << nb_arg << " arguments at " << linenum_
 					      << ' ' << target_.location_;
 					ERR_CF << error.str() << '\n';
-					throw config::error(error.str());
+					throw preproc_config::error(error.str());
 				}
 
 				std::stringstream *buffer = new std::stringstream;
@@ -625,7 +631,7 @@ bool preprocessor_data::get_chunk()
 				std::string const &dir = directory_name(val.location.substr(0, val.location.find(' ')));
 				if (!slowpath_) {
 					LOG_CF << "substituting macro " << symbol << '\n';
-					new preprocessor_data(target_, buffer, val.location, "", 
+					new preprocessor_data(target_, buffer, val.location, "",
 					                      val.linenum, dir, val.textdomain);
 				} else {
 					LOG_CF << "substituting (slow) macro " << symbol << '\n';
