@@ -1,4 +1,4 @@
-/* $Id: string_utils.cpp 8214 2005-09-12 21:33:34Z gruikya $ */
+/* $Id: string_utils.cpp 12503 2006-07-11 20:57:03Z noyga $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -19,10 +19,10 @@
 #include <sstream>
 #include "SDL_types.h"
 
-#include "../gettext.hpp"
-#include "../util.hpp"
-#include "../log.hpp"
-#include "string_utils.hpp"
+#include "gettext.hpp"
+#include "util.hpp"
+#include "log.hpp"
+#include "serialization/string_utils.hpp"
 
 #define ERR_GENERAL LOG_STREAM(err, general)
 
@@ -34,8 +34,9 @@ namespace {
 
 bool two_dots(char a, char b) { return a == '.' && b == '.'; }
 
-void do_interpolation(std::string &res, variable_set& set)
+std::string do_interpolation(const std::string &str, variable_set& set)
 {
+	std::string res = str;
 	//this needs to be able to store negative numbers to check for the while's condition
 	//(which is only false when the previous '$' was at index 0)
 	int rfind_dollars_sign_from = res.size();
@@ -96,28 +97,31 @@ void do_interpolation(std::string &res, variable_set& set)
 			--var_end;
 		}
 
+		const std::string var_name(var_name_begin, var_end);
+
 		if(*var_end == '|') {
 			//It's been used to end this variable name; now it has no more effect.
 			//This can allow use of things like "$$composite_var_name|.x"
 			//(Yes, that's a WML 'pointer' of sorts. They are sometimes useful.)
 			//If there should still be a '|' there afterwards to affect other variable names (unlikely),
 			//just put another '|' there, one matching each '$', e.g. "$$var_containing_var_name||blah"
-			res.erase(var_end);
+			var_end++;
 		}
 
-		const std::string var_name(var_name_begin, var_end);
 
-		//The variable is replaced with its value.
-		//Replace = remove original, and then insert new value, if any.
-		res.erase(var_begin, var_end);
-
-		res.insert(var_begin_loc, set.get_variable_const(var_name));
+		if (var_name == "") {
+			// Allow for a way to have $s in a string.
+			// $| will be replaced by $.
+			res.replace(var_begin, var_end, "$");
+		}
+		else {
+			//The variable is replaced with its value.
+			res.replace(var_begin, var_end,
+			    set.get_variable_const(var_name));
+		}
 	}
 
-	//Remove any remaining '|', which are used to separate variable names,
-	//so that the WML writer doesn't need to worry about whether they're really necessary.
-	//It is also occasionally useful to intentionally put them elsewhere.
-	res.erase(std::remove(res.begin(), res.end(), '|'), res.end());
+	return res;
 }
 
 }
@@ -218,19 +222,26 @@ private:
 
 std::string interpolate_variables_into_string(const std::string &str, const string_map *symbols)
 {
-	std::string res = str;
 	string_map_variable_set set(*symbols);
-	do_interpolation(res, set);
-
-	return res;
+	return do_interpolation(str, set);
 }
 
 std::string interpolate_variables_into_string(const std::string &str, variable_set& variables)
 {
-	std::string res = str;
-	do_interpolation(res, variables);
+	return do_interpolation(str, variables);
+}
 
-	return res;
+// modify a number by string representing integer difference, or optionally %
+int apply_modifier( const int number, const std::string &amount, const int minimum ) {
+	// wassert( amount.empty() == false );
+	int value = atoi(amount.c_str());
+	if(amount[amount.size()-1] == '%') {
+		value = div100rounded(number * value);
+	}
+	value += number;
+	if (( minimum > 0 ) && ( value < minimum ))
+	    value = minimum;
+	return value;
 }
 
 //Prepends a configurable set of characters with a backslash
@@ -270,6 +281,28 @@ std::string &unescape(std::string &str)
 	return str;
 }
 
+bool string_bool(const std::string& str,bool def)
+{
+	if(str == "yes" || str == "on" || str == "true" || lexical_cast_default<int>(str)) {
+		return true;
+	}
+	if(str == "no" || str == "off" || str == "false" || (str != "" && !lexical_cast_default<int>(str))) {
+		return false;
+	}
+	return def;
+}
+
+bool isvalid_username(const std::string& username)
+{
+	//check the username is valid (all alpha-numeric or space (but no space at ends))
+	const size_t alnum = std::count_if(username.begin(),username.end(),isalnum);
+	const size_t underscore = std::count(username.begin(),username.end(),'_');
+	if((alnum + underscore != username.size()) || underscore == username.size() || username.empty() ) {
+		return false;
+	}
+	return true;
+}
+
 std::string join(std::vector< std::string > const &v, char c)
 {
 	std::stringstream str;
@@ -443,14 +476,13 @@ std::string wstring_to_string(const wide_string &src)
 {
 	wchar_t ch;
 	wide_string::const_iterator i;
-	int j;
 	Uint32 bitmask;
 	std::string ret;
 
 	try {
 
 		for(i = src.begin(); i != src.end(); ++i) {
-			int count;
+			unsigned int count;
 			ch = *i;
 
 			/* Determine the bytes required */
@@ -459,22 +491,25 @@ std::string wstring_to_string(const wide_string &src)
 				count++;
 
 			bitmask = 0x800;
-			for(j = 0; j < 5; ++j) {
-				if(ch >= bitmask)
+			for(unsigned int j = 0; j < 5; ++j) {
+				if(static_cast<Uint32>(ch) >= bitmask) {
 					count++;
+				}
+
 				bitmask <<= 5;
 			}
 
-			if(count > 6)
+			if(count > 6) {
 				throw invalid_utf8_exception();
+			}
 
 			if(count == 1) {
-				push_back(ret,ch);
+				push_back(ret,static_cast<char>(ch));
 			} else {
-				for(j = count-1; j >= 0; --j) {
+				for(int j = (int)count-1; j >= 0; --j) {
 					unsigned char c = (ch >> (6*j)) & 0x3f;
 					c |= 0x80;
-					if(j == count-1)
+					if(j == (int)count-1)
 						c |= 0xff << (8 - count);
 					push_back(ret,c);
 				}
@@ -565,7 +600,7 @@ utf8_string capitalize(const utf8_string& s)
 {
 	if(s.size() > 0) {
 		utf8_iterator itor(s);
-#ifdef __APPLE__
+#if defined(__APPLE__) || defined(__AMIGAOS4__)
 		// FIXME: Should we support towupper on recent OSX platforms?
 		wchar_t uchar = *itor;
 		if(uchar >= 0 && uchar < 0x100)
@@ -587,7 +622,7 @@ utf8_string uppercase(const utf8_string& s)
 		std::string res;
 
 		for(;itor != utf8_iterator::end(s); ++itor) {
-#ifdef __APPLE__
+#if defined(__APPLE__) || defined(__AMIGAOS4__)
 			// FIXME: Should we support towupper on recent OSX platforms?
 			wchar_t uchar = *itor;
 			if(uchar >= 0 && uchar < 0x100)
@@ -610,7 +645,7 @@ utf8_string lowercase(const utf8_string& s)
 		std::string res;
 
 		for(;itor != utf8_iterator::end(s); ++itor) {
-#if defined(__APPLE__) || defined(__OpenBSD__)
+#if defined(__APPLE__) || defined(__OpenBSD__) || defined(__AMIGAOS4__)
 			// FIXME: Should we support towupper on recent OSX platforms?
 			wchar_t uchar = *itor;
 			if(uchar >= 0 && uchar < 0x100)
@@ -627,6 +662,59 @@ utf8_string lowercase(const utf8_string& s)
 	return s;
 }
 
+bool wildcard_string_match(const std::string& str, const std::string& match)
+{
+	// match using '*' as any number of characters (including none), and '?' as any one character
+	std::string::const_iterator c = str.begin();
+	std::string::const_iterator m = match.begin();
+	while(c != str.end() && m != match.end()) {
+		if(*m == '?') {
+			++c;
+			++m;
+		} else if(*m == '*') {
+			while(c != str.end()) {
+				if(*c == *(m+1)) {
+					std::string ns,nm;
+					ns.assign(c+1,str.end());
+					nm.assign(m,match.end());
+					if(ns.find(*(m+1)) != std::string::npos) {
+						if(wildcard_string_match(ns,nm)) {
+							return true;
+						}
+					} else {
+						break;
+					}
+				}
+				++c;
+			}
+			++m;
+		} else {
+			if(*c != *m) {
+				return false;
+			}
+			++c;
+			++m;
+		}
+	}
+	if(m == match.end()) {
+		if(c == str.end()) {
+			return true;
+		} else {
+			return false;
+		}
+	} else {
+		if(c == str.end()) {
+			if(*m == '*') {
+				return true;
+			}
+			return false;
+		} else {
+			return false;
+		}
+	}
+}
+
+
 }
 
 std::string vgettext(const char *msgid, const utils::string_map& symbols)
@@ -635,3 +723,4 @@ std::string vgettext(const char *msgid, const utils::string_map& symbols)
 	const std::string msg = utils::interpolate_variables_into_string(orig, &symbols);
 	return msg;
 }
+
