@@ -1,4 +1,4 @@
-/* $Id: game.cpp 8822 2005-11-12 19:40:35Z brunowolff $ */
+/* $Id: game.cpp 12524 2006-07-12 21:16:05Z jhinrichs $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -14,40 +14,52 @@
 #include "../global.hpp"
 
 #include "game.hpp"
+#include "log.hpp"
 #include "../util.hpp"
 #include "../wassert.hpp"
 
 #include <iostream>
+#include <sstream>
+
+#define LOG_SERVER LOG_STREAM(info, general)
 
 int game::id_num = 1;
 
-game::game(const player_map& info) : player_info_(&info), id_(id_num++), sides_taken_(9), side_controllers_(9), started_(false), description_(NULL), end_turn_(0), allow_observers_(true)
+game::game(const player_map& info) : player_info_(&info), id_(id_num++), sides_taken_(9), side_controllers_(9), started_(false), description_(NULL), end_turn_(0), allow_observers_(true), all_observers_muted_(false)
 {}
 
 bool game::is_owner(network::connection player) const
 {
-	wassert(!players_.empty());
-	return player == players_.front();
+	return (!players_.empty() && player == players_.front());
 }
 
 bool game::is_member(network::connection player) const
 {
-	return std::find(players_.begin(),players_.end(),player) != players_.end();
+	return is_player(player) || is_observer(player);
 }
 
 bool game::is_needed(network::connection player) const
 {
-	wassert(!players_.empty());
-	return player == players_.front();
+	return (!players_.empty() && player == players_.front());
 }
 
 bool game::is_observer(network::connection player) const
 {
-	if(is_member(player) == false) {
-		return false;
-	}
+	return std::find(observers_.begin(),observers_.end(),player) != observers_.end();
+}
 
-	return is_needed(player) == false && sides_.count(player) == 0;
+bool game::is_muted_observer(network::connection player) const
+{
+	return std::find(muted_observers_.begin(), muted_observers_.end(), player) != muted_observers_.end();
+}
+
+bool game::is_player(network::connection player) const
+{
+	return std::find(players_.begin(),players_.end(),player) != players_.end();
+}
+
+bool game::all_observers_muted() const{
+	return all_observers_muted_;
 }
 
 bool game::observers_can_label() const
@@ -127,7 +139,7 @@ void game::start_game()
 	//send [observer] tags for all observers that have already joined.
 	//we can do this by re-joining all players that don't have sides, and aren't
 	//the first player (game creator)
-	for(std::vector<network::connection>::const_iterator pl = players_.begin()+1; pl != players_.end(); ++pl) {
+	for(std::vector<network::connection>::const_iterator pl = observers_.begin(); pl != observers_.end(); ++pl) {
 		if(sides_.count(*pl) == 0) {
 			add_player(*pl);
 		}
@@ -146,7 +158,7 @@ bool game::take_side(network::connection player, const config& cfg)
 		if(side_num < 1 || side_num > 9)
 			return false;
 	}
-	catch(bad_lexical_cast& bad_lexical) {
+	catch(bad_lexical_cast&) {
 		return false;
 	}
 
@@ -164,7 +176,7 @@ bool game::take_side(network::connection player, const config& cfg)
 					if(side_num < 1 || side_num > 9)
 						return false;
 				}
-				catch(bad_lexical_cast& bad_lexical) {
+				catch(bad_lexical_cast&) {
 					return false;
 				}
 				// check if the side is taken if not take it
@@ -199,18 +211,21 @@ void game::update_side_data()
 
 	const config::child_itors level_sides = level_.child_range("side");
 
+	const user_vector users = all_game_users();
+	user_vector new_players;
+
+	bool side_found;
 	//for each player:
 	// * Find the player name
 	// * Find the side this player name corresponds to
-	for(std::vector<network::connection>::const_iterator player = players_.begin();
-			player != players_.end(); ++player) {
-
+	for(user_vector::const_iterator player = users.begin(); player != users.end(); ++player) {
 		player_map::const_iterator info = player_info_->find(*player);
 		if (info == player_info_->end()) {
 			std::cerr << "Error: unable to find player info for connection " << *player << "\n";
 			continue;
 		}
 
+		side_found = false;
 		size_t side_num;
 		size_t side_index;
 		config::child_iterator sd;
@@ -220,50 +235,106 @@ void game::update_side_data()
 				if(side_num < 1 || side_num > 9)
 					continue;
 			}
-			catch(bad_lexical_cast& bad_lexical) {
+			catch(bad_lexical_cast&) {
 				continue;
 			}
 			side_index = static_cast<size_t>(side_num - 1);
 
-			if((**sd)["controller"] == "network") {
-				side_controllers_[side_index] = "network";
-				if((**sd)["user_description"] == info->second.name()) {
-					sides_.insert(std::pair<network::connection, size_t>(*player, side_index));
+			if (! sides_taken_[side_index]){
+				if((**sd)["controller"] == "network") {
+					side_controllers_[side_index] = "network";
+					if((**sd)["user_description"] == info->second.name()) {
+						sides_.insert(std::pair<network::connection, size_t>(*player, side_index));
+						sides_taken_[side_index] = true;
+						side_found = true;
+					}
+					else {
+						sides_taken_[side_index] = false;
+					}
+				}
+				else if((**sd)["controller"] == "ai") {
+					side_controllers_[side_index] = "ai";
 					sides_taken_[side_index] = true;
+					side_found = true;
 				}
-				else {
-					sides_taken_[side_index] = false;
+				else if((**sd)["controller"] == "human") {
+					sides_.insert(std::pair<network::connection,size_t>(players_.front(),side_index));
+					sides_taken_[side_index] = true;
+					side_found = true;
+					side_controllers_[side_index] = "human";
 				}
 			}
-			else if((**sd)["controller"] == "ai") {
-				side_controllers_[side_index] = "ai";
-				sides_taken_[side_index] = true;
-			}
-			else if((**sd)["controller"] == "human") {
-				sides_.insert(std::pair<network::connection,size_t>(players_.front(),side_index));
-				sides_taken_[side_index] = true;
-				side_controllers_[side_index] = "human";
+		}
+
+		if (side_found){
+			new_players.push_back(*player);
+		}
+	}
+
+	//check if current players are still left in observers_ and if so remove them there
+	//this may happen in the game lobby if an observer is put into a slot by the host
+	{
+		for(user_vector::const_iterator player = new_players.begin(); player != new_players.end(); ++player) {
+			user_vector::iterator u = find_connection(*player, observers_);
+			if (u != observers_.end()){
+				observers_.erase(u);
+				//if the player is not in players_ add him now
+				if (find_connection(*player, players_) == players_.end()){
+					players_.push_back(*player);
+				}
 			}
 		}
 	}
+
+	//check if all players have a side and if not put them in observers_
+	//this may happen in the game lobby if an observer is put into an occupied slot by the host
+	{
+		//make a copy of players_ so that the iterator does not get corrupted
+		user_vector players_copy;
+		players_copy.insert(players_copy.end(), players_.begin(), players_.end());
+		for (user_vector::iterator player = players_copy.begin(); player != players_copy.end(); player++){
+			if (sides_.find(*player) == sides_.end()) {
+				//if the player doesn't have a side and is not in observers_ yet put him there
+				if (find_connection(*player, observers_) == observers_.end()){
+					observers_.push_back(*player);
+				}
+				user_vector::iterator p = find_connection(*player, players_);
+				players_.erase(p);
+			}
+		}
+	}
+
+	std::cerr << debug_player_info();
 }
 
 const std::string& game::transfer_side_control(const config& cfg)
 {
 	const std::string& player = cfg["player"];
 
-	std::vector<network::connection>::const_iterator i;
-	for(i = players_.begin(); i != players_.end(); ++i) {
-		const player_map::const_iterator pl = player_info_->find(*i);
-		if(pl != player_info_->end() && pl->second.name() == player) {
+	//find the socket for the player that is passed control
+	network::connection sock_entering;
+	bool found = false;
+	for (player_map::const_iterator pl = player_info_->begin(); pl != player_info_->end(); pl++){
+		if (pl->second.name() == player){
+			sock_entering = pl->first;
+			found = true;
 			break;
 		}
 	}
 
-	if(i == players_.end()) {
-		static const std::string notfound = "Player not found";
+	if (!found){
+		static const std::string notfound = "Player/Observer not found";
 		return notfound;
 	}
+
+	user_vector::iterator i = find_connection(sock_entering, observers_);
+	user_vector::iterator j = find_connection(sock_entering, players_);
+
+	if (i == observers_.end() && j == players_.end()){
+		static const std::string notfound = "Player/Observer not found";
+		return notfound;
+	}
+
 	const std::string& side = cfg["side"];
 	static const std::string invalid = "Invalid side number";
 	size_t side_num;
@@ -272,7 +343,7 @@ const std::string& game::transfer_side_control(const config& cfg)
 		if(side_num < 1 || side_num > 9)
 			return invalid;
 	}
-	catch(bad_lexical_cast& bad_lexical) {
+	catch(bad_lexical_cast&) {
 		return invalid;
 	}
 	const size_t nsides = level_.get_children("side").size();
@@ -281,22 +352,73 @@ const std::string& game::transfer_side_control(const config& cfg)
 
 	const size_t side_index = static_cast<size_t>(side_num - 1);
 
-	if(side_controllers_[side_index] == "network" && sides_taken_[side_index]) {
+	if(side_controllers_[side_index] == "network" && sides_taken_[side_index] && cfg["own_side"] != "yes") {
 		static const std::string already = "This side is already controlled by a player";
 		return already;
 	}
 
-	if(cfg["orphan_side"] != "yes") {
-		const size_t active_side = end_turn_/nsides + 1;
-
-		if(lexical_cast_default<size_t>(side,0) == active_side) {
-			static const std::string not_during_turn = "You cannot change a side's controller during its turn";
-			return not_during_turn;
+	//The player owns this side
+	if(cfg["own_side"] == "yes") {
+		//get the socket of the player that issued the command
+		network::connection sock;
+		bool found = false;
+		std::multimap<network::connection, size_t>::iterator oldside;
+		for (std::multimap<network::connection, size_t>::iterator s = sides_.begin(); s != sides_.end(); s++){
+			if (s->second == side_index){
+				sock = s->first;
+				found = true;
+				oldside = s;
+				break;
+			}
 		}
+
+		if (!found){
+			static const std::string player_not_found = "This side is not listed for the game";
+			return player_not_found;
+		}
+
+		//check, if this socket belongs to a player
+		user_vector::iterator p = find_connection(sock, players_);
+		if (p == players_.end()){
+			static const std::string no_player = "The player for this side could not be found";
+			return no_player;
+		}
+
+		//if a player gives up their last side, make them an observer
+		if (sides_.count(sock) == 1){
+			//we need to save this information before the player is erased
+			bool host = is_needed(sock);
+
+			observers_.push_back(*p);
+			players_.erase(p);
+
+			//tell others that the player becomes an observer
+			config cfg_observer = construct_server_message(find_player(sock)->name() + " becomes observer");
+			send_data(cfg_observer);
+
+			//update the client observer list for everyone except player
+			config observer_join;
+			observer_join.add_child("observer").values["name"] = find_player(sock)->name();
+			send_data(observer_join, sock);
+
+			//if this player was the host of the game, transfer game control to another player
+			if (host && transfer_game_control() != NULL){
+				const config& msg = construct_server_message(transfer_game_control()->name() + " has been chosen as new host");
+				send_data(msg);
+			}
+
+			//reiterate because iterators became invalid
+			i = find_connection(sock_entering, observers_);
+			j = find_connection(sock_entering, players_);
+		}
+
+		//clear the sides_ entry
+		sides_.erase(oldside);
 	}
+
 	side_controllers_[side_index] = "network";
 	sides_taken_[side_index] = true;
-	sides_.insert(std::pair<const network::connection,size_t>(*i, side_index));
+	sides_.insert(std::pair<const network::connection,size_t>(sock_entering, side_index));
 
 	// send a response to the host and to the new controller
 	config response;
@@ -308,14 +430,18 @@ const std::string& game::transfer_side_control(const config& cfg)
 	network::queue_data(response,players_.front());
 
 	change["controller"] = "human";
-	network::queue_data(response,*i);
+	network::queue_data(response, sock_entering);
 
-	if(sides_.count(*i) < 2) {
+	if(sides_.count(sock_entering) < 2) {
 		//send everyone a message saying that the observer who is taking the side has quit
 		config observer_quit;
 		observer_quit.add_child("observer_quit").values["name"] = player;
 		send_data(observer_quit);
 	}
+	if (i != observers_.end()){
+		players_.push_back(*i);
+		observers_.erase(i);
+	}
 	static const std::string success = "";
 	return success;
 }
@@ -325,7 +451,8 @@ size_t game::available_slots() const
 	size_t available_slots = 0;
 	const config::child_list& sides = level_.get_children("side");
 	for(config::child_list::const_iterator i = sides.begin(); i != sides.end(); ++i) {
-		std::cerr << "side controller: '" << (**i)["controller"] << "'\n";
+		//std::cerr << "side controller: '" << (**i)["controller"] << "'\n";
+		//std::cerr << "description: '" << (**i)["description"] << "'\n";
 		if((**i)["controller"] == "network" && (**i)["description"].empty()) {
 			++available_slots;
 		}
@@ -341,7 +468,13 @@ bool game::describe_slots()
 
 	const int val = int(available_slots());
 	char buf[50];
-	snprintf(buf,sizeof(buf),"%d",val);
+	int num_sides = level_.get_children("side").size();
+	for(config::child_list::const_iterator it = level_.get_children("side").begin(); it != level_.get_children("side").end(); ++it) {
+		if((**it)["allow_player"] == "no") {
+			num_sides--;
+		}
+	}
+	snprintf(buf,sizeof(buf),"%d/%d",val,num_sides);
 
 	if(buf != (*description())["slots"]) {
 		description()->values["slots"] = buf;
@@ -374,6 +507,21 @@ bool game::player_is_banned(network::connection sock) const
 	return false;
 }
 
+const player* game::mute_observer(network::connection sock){
+	const player* muted_observer = NULL;
+	player_map::const_iterator it = player_info_->find(sock);
+	if (it != player_info_->end() && ! is_muted_observer(sock)){
+		muted_observers_.push_back(sock);
+		muted_observer = &it->second;
+	}
+
+	return muted_observer;
+}
+
+void game::mute_all_observers(bool mute){
+	all_observers_muted_ = mute;
+}
+
 void game::ban_player(network::connection sock)
 {
 	const player_map::const_iterator itor = player_info_->find(sock);
@@ -422,7 +570,7 @@ bool game::end_turn()
 	return true;
 }
 
-void game::add_player(network::connection player)
+void game::add_player(network::connection player, bool observer)
 {
 	//if the game has already started, we add the player as an observer
 	if(started_) {
@@ -444,9 +592,9 @@ void game::add_player(network::connection player)
 		network::queue_data(cfg, player);
 
 		//send observer join of all the observers in the game to player
-		for(std::vector<network::connection>::const_iterator pl = players_.begin()+1; pl != players_.end(); ++pl) {
-			if(sides_.count(*pl) == 0 && *pl != player) {
-				info = player_info_->find(*pl);
+		for(std::vector<network::connection>::const_iterator ob = observers_.begin(); ob != observers_.end(); ++ob) {
+			if(*ob != player) {
+				info = player_info_->find(*ob);
 				if(info != player_info_->end()) {
 					cfg.clear();
 					cfg.add_child("observer").values["name"] = info->second.name();
@@ -457,13 +605,44 @@ void game::add_player(network::connection player)
 	}
 
 	//if the player is already in the game, don't add them.
-	if(std::find(players_.begin(),players_.end(),player) != players_.end()) {
+	user_vector users = all_game_users();
+	if(std::find(users.begin(),users.end(),player) != users.end()) {
 		return;
 	}
 
-	players_.push_back(player);
-
+	//Check first if there are available sides
+	//If not, add the player as observer
+	unsigned int human_sides = 0;
+	if (level_.get_children("side").size() > 0 && !observer){
+		config::child_list sides = level_.get_children("side");
+		for (config::child_list::const_iterator side = sides.begin(); side != sides.end(); side++){
+			if (((**side)["controller"] == "human") || ((**side)["controller"] == "network")){
+				human_sides++;
+			}
+		}
+	}
+	else{
+		if (level_.get_attribute("human_sides") != ""){
+			human_sides = lexical_cast<unsigned int>(level_["human_sides"]);
+		}
+	}
+	LOG_SERVER << debug_player_info();
+	if (human_sides > players_.size()){
+		LOG_SERVER << "adding player...\n";
+		players_.push_back(player);
+	} else{
+		LOG_SERVER << "adding observer...\n";
+		observers_.push_back(player);
+	}
+	LOG_SERVER << debug_player_info();
 	send_user_list();
+	player_map::const_iterator info = player_info_->find(player);
+	if(info != player_info_->end()) {
+		config observer_join;
+		observer_join.add_child("observer").values["name"] = info->second.name();
+		//send observer join to everyone except player
+		send_data(observer_join, player);
+	}
 
 	//send the player the history of the game to-date
 	network::queue_data(history_,player);
@@ -471,12 +650,22 @@ void game::add_player(network::connection player)
 
 void game::remove_player(network::connection player, bool notify_creator)
 {
-	const std::vector<network::connection>::iterator itor =
-	             std::find(players_.begin(),players_.end(),player);
+	LOG_SERVER << "removing player...\n";
+	{
+		const user_vector::iterator itor = std::find(players_.begin(),players_.end(),player);
 
-	if(itor != players_.end())
-		players_.erase(itor);
+		if(itor != players_.end())
+			players_.erase(itor);
+	}
 
+	{
+		const user_vector::iterator itor = std::find(observers_.begin(),observers_.end(),player);
+
+		if(itor != observers_.end())
+			observers_.erase(itor);
+	}
+
+	LOG_SERVER << debug_player_info();
 	bool observer = true;
 	std::pair<std::multimap<network::connection,size_t>::const_iterator,
 	          std::multimap<network::connection,size_t>::const_iterator> sides = sides_.equal_range(player);
@@ -514,8 +703,8 @@ void game::send_user_list()
 	if(started_ == false && description() != NULL) {
 		config cfg;
 		cfg.add_child("gamelist");
-		for(std::vector<network::connection>::const_iterator p = players_.begin(); p != players_.end(); ++p) {
-
+		user_vector users = all_game_users();
+		for(user_vector::const_iterator p = users.begin(); p != users.end(); ++p) {
 			const player_map::const_iterator info = player_info_->find(*p);
 			if(info != player_info_->end()) {
 				config& user = cfg.add_child("user");
@@ -535,8 +724,8 @@ int game::id() const
 
 void game::send_data(const config& data, network::connection exclude)
 {
-	for(std::vector<network::connection>::const_iterator
-	    i = players_.begin(); i != players_.end(); ++i) {
+	const user_vector users = all_game_users();
+	for(user_vector::const_iterator i = users.begin(); i != users.end(); ++i) {
 		if(*i != exclude && (allow_observers_ || is_needed(*i) || sides_.count(*i) == 1)) {
 			network::queue_data(data,*i);
 		}
@@ -569,10 +758,8 @@ void game::send_data_team(const config& data, const std::string& team, network::
 
 void game::send_data_observers(const config& data)
 {
-	for(std::vector<network::connection>::const_iterator i = players_.begin(); i != players_.end(); ++i) {
-		if(is_observer(*i)) {
-			network::queue_data(data,*i);
-		}
+	for(std::vector<network::connection>::const_iterator i = observers_.begin(); i != observers_.end(); ++i) {
+		network::queue_data(data,*i);
 	}
 }
 
@@ -598,17 +785,19 @@ config& game::level()
 
 bool game::empty() const
 {
-	return players_.empty();
+	return players_.empty() && observers_.empty();
 }
 
 void game::disconnect()
 {
-	for(std::vector<network::connection>::iterator i = players_.begin();
-	    i != players_.end(); ++i) {
+	const user_vector users = all_game_users();
+	for(user_vector::const_iterator i = users.begin();
+	    i != users.end(); ++i) {
 		network::queue_disconnect(*i);
 	}
 
 	players_.clear();
+	observers_.clear();
 }
 
 void game::set_description(config* desc)
@@ -621,13 +810,109 @@ config* game::description()
 	return description_;
 }
 
-void game::add_players(const game& other_game)
+void game::add_players(const game& other_game, bool observer)
 {
-	players_.insert(players_.end(),
-	                other_game.players_.begin(),other_game.players_.end());
+	user_vector users = other_game.all_game_users();
+	if (observer){
+		observers_.insert(observers_.end(), users.begin(), users.end());
+	}
+	else{
+		players_.insert(players_.end(), users.begin(), users.end());
+	}
 }
 
 bool game::started() const
 {
 	return started_;
 }
+
+const user_vector game::all_game_users() const{
+	user_vector res;
+
+	res.insert(res.end(), players_.begin(), players_.end());
+	res.insert(res.end(), observers_.begin(), observers_.end());
+
+	return res;
+}
+
+std::string game::debug_player_info() const{
+	std::stringstream result;
+	result << "---------------------------------------\n";
+	result << "players_.size: " << players_.size() << "\n";
+	for (user_vector::const_iterator p = players_.begin(); p != players_.end(); p++){
+		const player_map::const_iterator user = player_info_->find(*p);
+		if (user != player_info_->end()){
+			result << "player: " << user->second.name().c_str() << "\n";
+		}
+		else{
+			result << "player not found\n";
+		}
+	}
+	result << "observers_.size: " << observers_.size() << "\n";
+	for (user_vector::const_iterator o = observers_.begin(); o != observers_.end(); o++){
+		const player_map::const_iterator user = player_info_->find(*o);
+		if (user != player_info_->end()){
+			result << "observer: " << user->second.name().c_str() << "\n";
+		}
+		else{
+			result << "observer not found\n";
+		}
+	}
+	{
+		result << "player_info_: begin\n";
+		for (player_map::const_iterator info = player_info_->begin(); info != player_info_->end(); info++){
+			result << info->second.name().c_str() << "\n";
+		}
+		result << "player_info_: end\n";
+	}
+	result << "---------------------------------------\n";
+	return result.str();
+}
+
+const player* game::find_player(network::connection sock) const{
+	const player* result = NULL;
+	for (player_map::const_iterator info = player_info_->begin(); info != player_info_->end(); info++){
+		if (info->first == sock){
+			result = &info->second;
+		}
+	}
+	return result;
+}
+
+const player* game::transfer_game_control(){
+	const player* result = NULL;
+
+	//search for the next available player in the players list
+	if (players_.size() > 0){
+		result = find_player(players_[0]);
+	}
+	return result;
+}
+
+user_vector::iterator game::find_connection(network::connection sock, user_vector& users){
+	user_vector::iterator p;
+	for (p = users.begin(); p != users.end(); p++){
+		if ((*p) == sock){
+			return p;
+		}
+	}
+	return users.end();
+}
+
+config game::construct_server_message(const std::string& message)
+{
+	config turn;
+	if(started()) {
+		config& cmd = turn.add_child("turn");
+		config& cfg = cmd.add_child("command");
+		config& msg = cfg.add_child("speak");
+		msg["description"] = "server";
+		msg["message"] = message;
+	} else {
+		config& msg = turn.add_child("message");
+		msg["sender"] = "server";
+		msg["message"] = message;
+	}
+
+	return turn;
+}
