@@ -1,4 +1,4 @@
-/* $Id: game.hpp 7532 2005-07-20 07:51:47Z ott $ */
+/* $Id: game.hpp 12524 2006-07-12 21:16:05Z jhinrichs $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -24,6 +24,7 @@
 #include <set>
 #include <vector>
 
+typedef std::vector<network::connection> user_vector;
 typedef std::map<network::connection,player> player_map;
 
 class game
@@ -35,6 +36,9 @@ public:
 	bool is_member(network::connection player) const;
 	bool is_needed(network::connection player) const;
 	bool is_observer(network::connection player) const;
+	bool is_muted_observer(network::connection player) const;
+	bool is_player(network::connection player) const;
+	bool all_observers_muted() const;
 
 	bool observers_can_label() const;
 	bool observers_can_chat() const;
@@ -60,12 +64,15 @@ public:
 
 	bool player_is_banned(network::connection player) const;
 	void ban_player(network::connection player);
+	const player* mute_observer(network::connection player);
+	void mute_all_observers(bool mute);
 
-	void add_player(network::connection player);
+	void add_player(network::connection player, bool observer = false);
 	void remove_player(network::connection player, bool notify_creator=true);
 
 	int id() const;
 
+	config construct_server_message(const std::string& message);
 	void send_data(const config& data, network::connection exclude=0);
 	void send_data_team(const config& data, const std::string& team, network::connection exclude=0);
 	void send_data_observers(const config& data);
@@ -84,7 +91,11 @@ public:
 	void set_description(config* desc);
 	config* description();
 
-	void add_players(const game& other_game);
+	//adds players from one game to another. This is used to add players and
+	//observers from a game to the lobby (which is also implemented as a game),
+	//if that game ends. The second parameter controls, wether the players are
+	//added to the players_ or observers_ vector (default observers_).
+	void add_players(const game& other_game, bool observer = true);
 
 	//function which will process game commands and update the state of the
 	//game accordingly. Will return true iff the game's description changes.
@@ -103,7 +114,19 @@ public:
 		if(termination_.empty()) { termination_ = reason; }
 	}
 
+	//adds players and observers into one vector and returns that
+	const user_vector all_game_users() const;
+
+	const player* find_player(network::connection sock) const;
+
+	const player* transfer_game_control();
+
 private:
+	//returns an iterator on the users vector if sock is found
+	user_vector::iterator find_connection(network::connection sock, user_vector& users);
+
+	//helps debugging player and observer lists
+	std::string debug_player_info() const;
 
 	//function which returns true iff 'player' is on 'team'.
 	bool player_on_team(const std::string& team, network::connection player) const;
@@ -122,7 +145,9 @@ private:
 
 	static int id_num;
 	int id_;
-	std::vector<network::connection> players_;
+	user_vector players_;
+	user_vector observers_;
+	user_vector muted_observers_;
 	std::multimap<network::connection,size_t> sides_;
 	std::vector<bool> sides_taken_;
 	std::vector<std::string> side_controllers_;
@@ -137,6 +162,7 @@ private:
 	int end_turn_;
 
 	bool allow_observers_;
+	bool all_observers_muted_;
 
 	struct ban {
 		ban(const std::string& name, const std::string& address)
