@@ -1,4 +1,4 @@
-/* $Id: server.cpp 8383 2005-09-25 17:54:37Z isaac $ */
+/* $Id: server.cpp 12524 2006-07-12 21:16:05Z jhinrichs $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -46,6 +46,8 @@
 #include <unistd.h>
 #endif
 
+#define LOG_SERVER LOG_STREAM(info, general)
+
 namespace {
 
 config construct_error(const std::string& msg)
@@ -91,9 +93,10 @@ public:
 private:
 	void process_data(network::connection sock, config& data, config& gamelist);
 
-	void process_login(network::connection sock, const config& data, config& gamelist);
-	void process_query(network::connection sock, const config& query, config& gamelist);
-	void process_data_from_player_in_lobby(network::connection sock, config& data, config& gamelist);
+	void process_login(network::connection sock, const config& data);
+	void process_query(network::connection sock, const config& query);
+	void process_whisper(const network::connection sock, const config& whisper);
+	void process_data_from_player_in_lobby(network::connection sock, config& data);
 	void process_data_from_player_in_game(network::connection sock, config& data, config& gamelist);
 
 	std::string process_command(const std::string& cmd);
@@ -128,9 +131,16 @@ private:
 
 	const config& cfg_;
 
+	size_t default_max_messages_;
+	size_t default_time_period_;
+	size_t concurrent_connections_;
+
 	std::set<std::string> accepted_versions_;
 	std::map<std::string,config> redirected_versions_;
 	std::map<std::string,config> proxy_versions_;
+	std::vector<std::string> disallowed_names_;
+
+	bool ip_exceeds_connection_limit(const std::string& ip);
 
 	bool is_ip_banned(const std::string& ip);
 	std::string ban_ip(const std::string& mask);
@@ -138,15 +148,32 @@ private:
 
 	std::string admin_passwd_;
 	std::set<network::connection> admins_;
+
+	std::string motd_;
 };
 
 server::server(int port, input_stream& input, const config& cfg, size_t nthreads) : net_manager_(nthreads), server_(port),
-    not_logged_in_(players_), lobby_players_(players_), last_stats_(time(NULL)), input_(input), cfg_(cfg), admin_passwd_(cfg["passwd"])
+    not_logged_in_(players_), lobby_players_(players_), last_stats_(time(NULL)), input_(input), cfg_(cfg), admin_passwd_(cfg["passwd"]), motd_(cfg["motd"])
 {
 	version_query_response_.add_child("version");
 
 	login_response_.add_child("mustlogin");
 
+	if(cfg_["disallow_names"] == "") {
+		disallowed_names_.push_back("server");
+		disallowed_names_.push_back("ai");
+		disallowed_names_.push_back("player");
+		disallowed_names_.push_back("network");
+		disallowed_names_.push_back("human");
+		disallowed_names_.push_back("admin");
+		disallowed_names_.push_back("computer");
+	} else {
+		disallowed_names_ = utils::split(cfg_["disallow_names"]);
+	}
+	default_max_messages_ = lexical_cast_default<int>(cfg_["max_messages"],4);
+	default_time_period_ = lexical_cast_default<int>(cfg_["messages_time_period"],10);
+	concurrent_connections_ = lexical_cast_default<int>(cfg_["connections_allowed"],5);
+
 	const std::string& versions = cfg_["versions_accepted"];
 	if(versions.empty() == false) {
 		const std::vector<std::string> accepted(utils::split(versions));
@@ -176,16 +203,26 @@ server::server(int port, input_stream& input, const config& cfg, size_t nthreads
 	join_lobby_response_.add_child("join_lobby");
 }
 
+bool server::ip_exceeds_connection_limit(const std::string& ip)
+{
+	size_t connections = 0;
+	for(player_map::const_iterator i = players_.begin(); i != players_.end(); ++i) {
+		if(network::ip_address(i->first) == ip) {
+			++connections;
+		}
+	}
+
+	return connections > concurrent_connections_;
+}
+
 bool server::is_ip_banned(const std::string& ip)
 {
 	for(std::vector<std::string>::const_iterator i = bans_.begin(); i != bans_.end(); ++i) {
 		std::cerr << "comparing for ban '" << *i << "' vs '" << ip << "'\n";
-		const std::string::const_iterator itor = std::mismatch(i->begin(),i->end(),ip.c_str()).first;
-		if(itor == i->end() && i->size() == ip.size() || itor != i->end() && *itor == '*') {
+		if(utils::wildcard_string_match(ip,*i)) {
 			std::cerr << "is banned\n";
 			return true;
 		}
-
 		std::cerr << "not banned\n";
 	}
 
@@ -194,11 +231,6 @@ bool server::is_ip_banned(const std::string& ip)
 
 std::string server::ban_ip(const std::string& mask)
 {
-	const std::string::const_iterator asterisk = std::find(mask.begin(),mask.end(),'*');
-	if(asterisk != mask.end() && asterisk != mask.end()-1) {
-		return "'*' may only appear at the end of the mask";
-	}
-
 	bans_.push_back(mask);
 
 	return "";
@@ -255,7 +287,7 @@ std::string server::process_command(const std::string& cmd)
 		out << "---";
 	} else if(command == "metrics") {
 		out << metrics_;
-	} else if(command == "ban") {
+	} else if(command == "ban" || command == "kban") {
 
 		if(i == cmd.end()) {
 			out << "BAN LIST\n---\n";
@@ -264,7 +296,20 @@ std::string server::process_command(const std::string& cmd)
 			}
 			out << "---";
 		} else {
-			const std::string mask(i+1,cmd.end());
+			std::string mask(i+1,cmd.end());
+
+			for(player_map::const_iterator j = players_.begin(); j != players_.end(); ++j) {
+				if(j->second.name() == mask) {
+					const std::string nick = mask;
+					mask = network::ip_address(j->first);
+					if(command == "kban") {
+						network::queue_disconnect(j->first);
+						out << "Kicked " << nick << ", ";
+					}
+					break;
+				}
+			}
+
 			const std::string& diagnostic = ban_ip(mask);
 			if(diagnostic != "") {
 				out << "Could not ban '" << mask << "': " << diagnostic;
@@ -296,12 +341,29 @@ std::string server::process_command(const std::string& cmd)
 
 		for(player_map::const_iterator j = players_.begin(); j != players_.end(); ++j) {
 			if(j->second.name() == nick) {
-				throw network::error("",j->first);
+				network::queue_disconnect(j->first);
+				return "kicked " + nick;
 			}
 		}
+
+		out << "could not find user '" << nick << "'\n";
+	} else if(command == "motd") {
+		if(i == cmd.end()) {
+			if(motd_ != "") {
+				out << "message of the day: " << motd_;
+				return out.str();
+			} else {
+				return "no message of the day set";
+			}
+		}
+
+		const std::string motd(i+1,cmd.end());
+		motd_ = motd;
+		out << "message of the day set: " << motd_;
+
 	} else {
 		out << "command '" << command << "' is not recognized";
-		out << "available commands are: msg <message>, status, metrics, ban [<nick>], unban <nick>, kick <nick>";
+		out << "available commands are: msg <message>, motd <message>, status, metrics, ban [<nick>], unban <nick>, kick <nick>, kban <nick> ";
 	}
 
 	return out.str();
@@ -336,13 +398,20 @@ void server::run()
 			network::process_send_queue();
 
 			network::connection sock = network::accept_connection();
-			if(sock && is_ip_banned(network::ip_address(sock))) {
-				std::cerr << "rejected banned user '" << network::ip_address(sock) << "'\n";
-				network::send_data(construct_error("You are banned."),sock);
-				network::disconnect(sock);
-			} else if(sock) {
-				network::send_data(version_query_response_,sock);
-				not_logged_in_.add_player(sock);
+			if(sock) {
+				const std::string& ip = network::ip_address(sock);
+				if(is_ip_banned(ip)) {
+					std::cerr << "rejected banned user '" << ip << "'\n";
+					network::send_data(construct_error("You are banned."),sock);
+					network::disconnect(sock);
+				} else if(ip_exceeds_connection_limit(ip)) {
+					std::cerr << "rejected ip '" << ip << "' due to excessive connections\n";
+					network::send_data(construct_error("Too many connections from your host."),sock);
+					network::disconnect(sock);
+				} else {
+					network::send_data(version_query_response_,sock);
+					not_logged_in_.add_player(sock);
+				}
 			}
 
 			config data;
@@ -361,6 +430,7 @@ void server::run()
 				std::cerr << "socket closed: " << e.message << "\n";
 
 				const std::map<network::connection,player>::iterator pl_it = players_.find(e.socket);
+				const std::string pl_name = pl_it != players_.end() ? pl_it->second.name() : "";
 				if(pl_it != players_.end()) {
 					const config::child_list& users = initial_response_.get_children("user");
 					const size_t index = std::find(users.begin(),users.end(),pl_it->second.config_address()) - users.begin();
@@ -377,7 +447,10 @@ void server::run()
 						delete_game(i);
 						e.socket = 0;
 						break;
-					} else {
+					} else if(i->is_member(e.socket)) {
+						if(pl_name != "" && i->is_player(e.socket)) {
+							i->send_data(construct_server_message(pl_name + " has disconnected",*i));
+						}
 						i->remove_player(e.socket);
 					}
 				}
@@ -406,48 +479,71 @@ void server::run()
 
 void server::process_data(const network::connection sock, config& data, config& gamelist)
 {
+	//std::cerr << "in server::process_data...\n";
 	if(proxy::is_proxy(sock)) {
 		proxy::received_data(sock,data);
 	}
 	//if someone who is not yet logged in is sending
 	//login details
-	else if(not_logged_in_.is_member(sock)) {
-		process_login(sock,data,gamelist);
+	else if(not_logged_in_.is_observer(sock)) {
+		process_login(sock,data);
 	} else if(const config* query = data.child("query")) {
-		process_query(sock,*query,gamelist);
-	} else if(lobby_players_.is_member(sock)) {
-		process_data_from_player_in_lobby(sock,data,gamelist);
+		process_query(sock,*query);
+	} else if(const config* whisper = data.child("whisper")) {
+		process_whisper(sock,*whisper);
+	} else if(lobby_players_.is_observer(sock)) {
+		process_data_from_player_in_lobby(sock,data);
 	} else {
 		process_data_from_player_in_game(sock,data,gamelist);
 	}
 }
 
-void server::process_login(const network::connection sock, const config& data, config& gamelist)
+
+void server::process_login(const network::connection sock, const config& data)
 {
 	//see if client is sending their version number
 	const config* const version = data.child("version");
 	if(version != NULL) {
 		const std::string& version_str = (*version)["version"];
 
-		if(accepted_versions_.count(version_str)) {
+		bool accepted = false;
+		for(std::set<std::string>::const_iterator ver_it = accepted_versions_.begin(); ver_it != accepted_versions_.end(); ++ver_it) {
+			if(utils::wildcard_string_match(version_str,*ver_it)) {
+				accepted = true;
+				break;
+			}
+		}
+		if(accepted) {
 			std::cerr << "player joined using accepted version " << version_str << ": telling them to log in\n";
 			network::send_data(login_response_,sock);
 		} else {
-			const std::map<std::string,config>::const_iterator i = redirected_versions_.find(version_str);
-			if(i != redirected_versions_.end()) {
+			std::map<std::string,config>::const_iterator redirect = redirected_versions_.end();
+			for(std::map<std::string,config>::const_iterator red_it = redirected_versions_.begin(); red_it != redirected_versions_.end(); ++red_it) {
+				if(utils::wildcard_string_match(version_str,red_it->first)) {
+					redirect = red_it;
+					break;
+				}
+			}
+			if(redirect != redirected_versions_.end()) {
 				std::cerr << "player joined using version " << version_str << ": redirecting them to "
-				          << i->second["host"] << ":" << i->second["port"] << "\n";
+				          << redirect->second["host"] << ":" << redirect->second["port"] << "\n";
 				config response;
-				response.add_child("redirect",i->second);
+				response.add_child("redirect",redirect->second);
 				network::send_data(response,sock);
 			} else {
-				const std::map<std::string,config>::const_iterator i = proxy_versions_.find(version_str);
+				std::map<std::string,config>::const_iterator proxy = proxy_versions_.end();
+				for(std::map<std::string,config>::const_iterator prox_it = proxy_versions_.begin(); prox_it != proxy_versions_.end(); ++prox_it) {
+					if(utils::wildcard_string_match(version_str,prox_it->first)) {
+						proxy = prox_it;
+						break;
+					}
+				}
 
-				if(i != proxy_versions_.end()) {
+				if(proxy != proxy_versions_.end()) {
 					std::cerr << "player joined using version " << version_str << ": connecting them by proxy to "
-					          << i->second["host"] << ":" << i->second["port"] << "\n";
+					          << proxy->second["host"] << ":" << proxy->second["port"] << "\n";
 
-					proxy::create_proxy(sock,i->second["host"],lexical_cast_default<int>(i->second["port"],15000));
+					proxy::create_proxy(sock,proxy->second["host"],lexical_cast_default<int>(proxy->second["port"],15000));
 				} else {
 
 					std::cerr << "player joined using unknown version " << version_str << ": rejecting them\n";
@@ -478,27 +574,26 @@ void server::process_login(const network::connection sock, const config& data, c
 		return;
 	}
 
-	//check the username is valid (all alpha-numeric or space)
+	//check the username is valid (all alpha-numeric or space (but no space at ends))
 	std::string username = (*login)["username"];
-	utils::strip(username);
-	const size_t alnum = std::count_if(username.begin(),username.end(),isalnum);
-	const size_t spaces = std::count(username.begin(),username.end(),' ');
-	if((alnum + spaces != username.size()) || spaces == username.size() || username.empty()) {
+	if(!utils::isvalid_username(username)) {
 		network::send_data(construct_error(
 		                   "This username is not valid"),sock);
 		return;
 	}
 
-	if(username.size() > 32) {
+	if(username.size() > 18) {
 		network::send_data(construct_error(
 		                   "This username is too long"),sock);
 		return;
 	}
 
-	if(username == "server") {
-		network::send_data(construct_error(
-		                   "The nick 'server' is reserved and can not be used by players"),sock);
-		return;
+	for(std::vector<std::string>::const_iterator d_it = disallowed_names_.begin(); d_it != disallowed_names_.end(); ++d_it) {
+		if(utils::wildcard_string_match(utils::lowercase(username),utils::lowercase(*d_it))) {
+			network::send_data(construct_error(
+							"The nick '" + username + "' is reserved and can not be used by players"),sock);
+			return;
+		}
 	}
 
 	//check the username isn't already taken
@@ -519,7 +614,7 @@ void server::process_login(const network::connection sock, const config& data, c
 
 	config* const player_cfg = &initial_response_.add_child("user");
 
-	const player new_player(username,*player_cfg);
+	const player new_player(username,*player_cfg,default_max_messages_,default_time_period_);
 
 	players_.insert(std::pair<network::connection,player>(sock,new_player));
 
@@ -531,6 +626,10 @@ void server::process_login(const network::connection sock, const config& data, c
 	//send the new player the entire list of games and players
 	network::send_data(initial_response_,sock);
 
+	if(motd_ != "") {
+		network::send_data(construct_server_message(motd_,lobby_players_),sock);
+	}
+
 	//send other players in the lobby the update that the player has joined
 	lobby_players_.send_data(sync_initial_response(),sock);
 
@@ -541,7 +640,7 @@ void server::process_login(const network::connection sock, const config& data, c
 	}
 }
 
-void server::process_query(const network::connection sock, const config& query, config& gamelist)
+void server::process_query(const network::connection sock, const config& query)
 {
 	//process queries from clients in here
 	std::ostringstream response;
@@ -551,8 +650,26 @@ void server::process_query(const network::connection sock, const config& query,
 	} else if(admin_passwd_.empty() == false && query["type"] == admin_passwd_) {
 		admins_.insert(sock);
 		response << "You are now recognized as an administrator";
+		std::cerr << "New Admin recognized:\n";
+		std::cerr << "\tIP: "<< network::ip_address(sock)<<"\n";
+		std::map<network::connection, player>::iterator temp = players_.find(sock);
+		std::cerr << "\tnick: "<< temp->second.name()<<"\n";
+		std::cerr << std::endl;
 	} else if(admins_.count(sock) != 0) {
 		response << process_command(query["type"]);
+		std::cerr << "Admin Command:\n";
+		std::cerr << "\ttype:" << query["type"]<<"\n";
+		std::cerr << "\tIP: "<< network::ip_address(sock)<<"\n";
+		std::map<network::connection, player>::iterator temp = players_.find(sock);
+		std::cerr << "\tnick: "<< temp->second.name()<<"\n";
+		std::cerr << std::endl;
+	} else if(admin_passwd_.empty() == false) {
+	  std::cerr << "FAILED Admin attempt:\n";
+	  std::cerr << "\tIP: "<< network::ip_address(sock)<<"\n";
+	  std::map<network::connection, player>::iterator temp = players_.find(sock);
+	  std::cerr << "\tnick: "<< temp->second.name()<<"\n";
+	  std::cerr << std::endl;
+	  response << "Error: unrecognized query";
 	} else {
 		response << "Error: unrecognized query";
 	}
@@ -560,33 +677,85 @@ void server::process_query(const network::connection sock, const config& query,
 	network::send_data(construct_server_message(response.str(),lobby_players_),sock);
 }
 
-void server::process_data_from_player_in_lobby(const network::connection sock, config& data, config& gamelist)
+void server::process_whisper(const network::connection sock, const config& whisper)
 {
+	bool sent = false;
+	bool do_send = true;
+	std::vector<game>::iterator g;
+	if ((whisper["receiver"]!="") && (whisper["message"]!="") && (whisper["sender"]!="")){
+		for(player_map::const_iterator i = players_.begin(); i != players_.end(); ++i) {
+			if(i->second.name() == whisper["receiver"]) {
+				for(g = games_.begin(); g != games_.end(); ++g) {
+					if(g->is_player(i->first)) {
+						do_send = false;
+						break;
+					}
+				}
+				if(do_send == false) {
+					break;
+				}
+				config cwhisper;
+				cwhisper.add_child("whisper",whisper);
+				network::send_data(cwhisper,i->first);
+				sent = true;
+				break;
+			}
+		}
+	} else {
+		config msg;
+		config data;
+		msg["message"] = "Invalid number of arguments";
+		msg["sender"] = "server";
+		data.add_child("message", msg);
+		network::send_data(data,sock);
+		sent = true;
+	}
+
+	if (sent == false){
+		config msg;
+		config data;
+		if(do_send == false) {
+			msg["message"] = "You cannot send private messages to players in a game.";
+		} else {
+			msg["message"] = "Can't find player "+whisper["receiver"];
+		}
+		msg["sender"] = "server";
+		data.add_child("message", msg);
+		network::send_data(data,sock);
+	}
+}
+
+void server::process_data_from_player_in_lobby(const network::connection sock, config& data)
+{
+	//std::cerr << "in process_data_from_player_in_lobby...\n";
 	const config* const create_game = data.child("create_game");
+
+	const player_map::iterator pl = players_.find(sock);
+	if(pl == players_.end()) {
+		std::cerr << "ERROR: Could not find player in map\n";
+	}
+
 	if(create_game != NULL) {
 
-		std::cerr << "creating game...\n";
+		//std::cerr << "creating game...\n";
 
 		//create the new game, remove the player from the
 		//lobby and put him/her in the game they have created
 		games_.push_back(game(players_));
+		games_.back().level() = *create_game;
 		lobby_players_.remove_player(sock);
 		games_.back().add_player(sock);
 
 		//store the game data here at the moment
-		games_.back().level() = *create_game;
 		std::stringstream converter;
 		converter << games_.back().id();
 		games_.back().level()["id"] = converter.str();
 
 		//mark the player as unavailable in the lobby
-		const player_map::iterator pl = players_.find(sock);
 		if(pl != players_.end()) {
-			pl->second.mark_available(false);
+			pl->second.mark_available(false,games_.back().level()["name"]);
 
 			lobby_players_.send_data(sync_initial_response());
-		} else {
-			std::cerr << "ERROR: Could not find player in map\n";
 		}
 
 		return;
@@ -596,6 +765,7 @@ void server::process_data_from_player_in_lobby(const network::connection sock, c
 	const config* const join = data.child("join");
 	if(join != NULL) {
 		const std::string& id = (*join)["id"];
+		const std::string& str_observer = (*join)["observe"];
 		const int nid = atoi(id.c_str());
 		const std::vector<game>::iterator it =
 		             std::find_if(games_.begin(),games_.end(),
@@ -620,12 +790,12 @@ void server::process_data_from_player_in_lobby(const network::connection sock, c
 		//send them the game data
 		network::send_data(it->level(),sock);
 
-		it->add_player(sock);
+		it->add_player(sock, str_observer == "yes");
 
 		//mark the player as unavailable in the lobby
 		const player_map::iterator pl = players_.find(sock);
 		if(pl != players_.end()) {
-			pl->second.mark_available(false);
+			pl->second.mark_available(false,(*it->description())["name"]);
 
 			lobby_players_.send_data(sync_initial_response());
 		} else {
@@ -637,18 +807,38 @@ void server::process_data_from_player_in_lobby(const network::connection sock, c
 	//of the sender, and forward it to all players in the lobby
 	config* const message = data.child("message");
 	if(message != NULL) {
-		const player_map::const_iterator p = players_.find(sock);
+		const player_map::iterator p = players_.find(sock);
 		wassert(p != players_.end());
+
+		if(p->second.silenced()) {
+			return;
+		} else if(p->second.is_message_flooding()) {
+			network::send_data(construct_server_message("Warning: you are sending too many messages too fast. Your message has not been relayed.",
+			                                            lobby_players_),p->first);
+			return;
+		}
+
 		(*message)["sender"] = p->second.name();
 
 		truncate_message((*message)["message"]);
 
 		lobby_players_.send_data(data,sock);
 	}
+
+	//player requests update of lobby content, for example when cancelling
+	//the create game dialog
+	config* const refresh = data.child("refresh_lobby");
+	if (refresh != NULL){
+		const player_map::iterator p = players_.find(sock);
+		wassert(p != players_.end());
+
+		network::send_data(initial_response_, sock);
+	}
 }
 
 void server::process_data_from_player_in_game(const network::connection sock, config& data, config& gamelist)
 {
+	//std::cerr << "in process_data_from_player_in_game...\n";
 	std::vector<game>::iterator g;
 	for(g = games_.begin(); g != games_.end(); ++g) {
 		if(g->is_member(sock))
@@ -669,18 +859,82 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 	}
 
 	//if the owner is changing the controller for a side
-	if(g->is_owner(sock) && data.child("change_controller") != NULL) {
+	if (data.child("change_controller") != NULL){
 		const config& change = *data.child("change_controller");
-		const std::string& result = g->transfer_side_control(change);
-		if(result == "") {
-			const config& msg = construct_server_message(change["player"] + " takes control of side " + change["side"],*g);
-			g->send_data(msg);
-		} else {
-			const config& msg = construct_server_message(result,*g);
-			network::send_data(msg,sock);
-		}
+		//the player is either host of the game or gives away his own side
+		if(g->is_owner(sock) || change["own_side"] == "yes") {
+			const std::string& result = g->transfer_side_control(change);
+			if(result == "") {
+				const config& msg = construct_server_message(change["player"] + " takes control of side " + change["side"],*g);
+				g->send_data(msg);
+			} else {
+				const config& msg = construct_server_message(result,*g);
+				network::send_data(msg,sock);
+			}
 
-		return;
+			return;
+		}
+	}
+
+	//if all observers are muted
+	if (g->is_owner(sock) && data.child("muteall") != NULL){
+		if (!g->all_observers_muted()){
+			g->mute_all_observers(true);
+			const config& p_msg = construct_server_message("all observers have been muted",*g);
+			g->send_data(p_msg);
+		}
+		else{
+			g->mute_all_observers(false);
+			const config& p_msg = construct_server_message("mute of all observers is removed",*g);
+			g->send_data(p_msg);
+		}
+	}
+
+	//if an observer is muted
+	if(g->is_owner(sock) && data.child("mute") != NULL) {
+		const config& u = *data.child("mute");
+		std::string name = u["username"];
+		std::string lower_name;
+		lower_name.resize(name.size());
+		std::transform(name.begin(), name.end(), lower_name.begin(), tolower);
+
+		if (!name.empty()){
+			player_map::iterator pl;
+			for(pl = players_.begin(); pl != players_.end(); ++pl) {
+				if(pl->second.name() == name) {
+					break;
+				}
+			}
+			if(pl->first != sock && pl != players_.end() && g->is_observer(pl->first)) {
+				const player* player = g->mute_observer(pl->first);
+				if (player != NULL){
+					const config& msg = construct_server_message("You have been muted",*g);
+					network::send_data(msg, pl->first);
+
+					const config& p_msg = construct_server_message(pl->second.name() + " has been muted",*g);
+					g->send_data(p_msg, pl->first);
+				}
+			}
+		}
+		else{
+			std::string muted_nicks = "";
+			user_vector users = g->all_game_users();
+			const player* player;
+
+			for (user_vector::const_iterator user = users.begin(); user != users.end(); user++){
+				if ((g->all_observers_muted() && g->is_observer(*user))
+					|| (!g->all_observers_muted() && g->is_muted_observer(*user))){
+					player = g->find_player(*user);
+					if (player != NULL){
+						if (muted_nicks != "") { muted_nicks += ", "; }
+						muted_nicks += player->name();
+					}
+				}
+			}
+
+			const config& p_msg = construct_server_message("muted observers: " + muted_nicks,*g);
+			g->send_data(p_msg);
+		}
 	}
 
 	//if the owner is banning someone from the game
@@ -694,6 +948,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 		} else if (data.child("kick") != NULL) {
 			const config& u = *data.child("kick");
 			name = u["username"];
+			std::cerr << "kick: " << u["username"] << "\n";
 			ban = false;
 		}
 
@@ -709,7 +964,12 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 				g->ban_player(pl->first);
 				const config& msg = construct_server_message("You have been banned",*g);
 				network::send_data(msg, pl->first);
+
+				const config& p_msg = construct_server_message(pl->second.name() + " has been banned",*g);
+				g->send_data(p_msg);
 			} else {
+				const config& p_msg = construct_server_message(pl->second.name() + " has been kicked",*g);
+				g->send_data(p_msg);
 				g->remove_player(pl->first);
 			}
 
@@ -721,7 +981,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 			lobby_players_.add_player(pl->first);
 
 			//mark the player as available in the lobby
-			pl->second.mark_available(true);
+			pl->second.mark_available(true,"");
 
 			//send the player who was banned the lobby game list
 			network::send_data(initial_response_,pl->first);
@@ -742,7 +1002,10 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 	//if this is data describing changes to a game.
 	else if(g->is_owner(sock) && data.child("scenario_diff")) {
 		g->level().apply_diff(*data.child("scenario_diff"));
-		g->update_side_data();
+		config* cfg_change = data.child("scenario_diff")->child("change_child");
+		if ((cfg_change != NULL) && (cfg_change->child("side") != NULL)){
+			g->update_side_data();
+		}
 
 		const bool lobby_changes = g->describe_slots();
 		if(lobby_changes) {
@@ -768,17 +1031,26 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 				g->level().values["map_data"] = data["map_data"];
 				g->level().values["map"] = data["map"];
 			}
+
+			g->level().values["mp_era"] = data.child("era") != NULL ? data.child("era")->get_attribute("id") : "";
+			g->level().values["mp_scenario"] = data["id"];
 			g->level().values["mp_use_map_settings"] = data["mp_use_map_settings"];
 			g->level().values["mp_village_gold"] = data["mp_village_gold"];
 			g->level().values["mp_fog"] = data["mp_fog"];
 			g->level().values["mp_shroud"] = data["mp_shroud"];
 			g->level().values["experience_modifier"] = data["experience_modifier"];
+			g->level().values["mp_countdown"] = data["mp_countdown"];
+			g->level().values["mp_countdown_init_time"] = data["mp_countdown_init_time"];
+			g->level().values["mp_countdown_turn_bonus"] = data["mp_countdown_turn_bonus"];
+			g->level().values["mp_countdown_reservoir_time"] = data["mp_countdown_reservoir_time"];
+			g->level().values["mp_countdown_action_bonus"] = data["mp_countdown_action_bonus"];
 
 			//update our config object which describes the
 			//open games, and notifies the game of where its description
 			//is located at
 			config& desc = gamelist.add_child("game",g->level());
 			g->set_description(&desc);
+			desc["hash"] = data["hash"];
 
 			//record the full description of the scenario in g->level()
 			g->level() = data;
@@ -818,23 +1090,30 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 
 	const string_map::const_iterator side = data.values.find("side");
 	if(side != data.values.end()) {
-		const bool res = g->take_side(sock,data);
-		config response;
-		if(res) {
-			std::cerr << "played joined side\n";
-			response["side_secured"] = side->second;
-
-			//update the number of available slots
-			const bool res = g->describe_slots();
-			if(res) {
-				lobby_players_.send_data(sync_initial_response());
-			}
-		} else {
-			response["failed"] = "yes";
+		if (g->is_observer(sock)){
+			const config& p_msg = construct_server_message("Sorry " + g->find_player(sock)->name() + ", someone else entered before you.",*g);
+			network::send_data(p_msg, sock);
+			return;
 		}
+		else{
+			const bool res = g->take_side(sock,data);
+			config response;
+			if(res) {
+				std::cerr << "player joined side\n";
+				response["side_secured"] = side->second;
 
-		network::send_data(response,sock);
-		return;
+				//update the number of available slots
+				const bool res = g->describe_slots();
+				if(res) {
+					lobby_players_.send_data(sync_initial_response());
+				}
+			} else {
+				response["failed"] = "yes";
+			}
+
+			network::send_data(response,sock);
+			return;
+		}
 	}
 
 	if(data.child("start_game")) {
@@ -848,11 +1127,14 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 		return;
 	} else if(data.child("leave_game")) {
 		const bool needed = g->is_needed(sock);
+		bool obs = g->is_observer(sock);
+		g->remove_player(sock);
+		g->describe_slots();
 
-		if(needed) {
+		if(g->nplayers() == 0) {
 
 			//tell all other players the game is over,
-			//because a needed player has left
+			//because the last player has left
 			config cfg;
 			cfg.add_child("leave_game");
 			g->send_data(cfg);
@@ -875,7 +1157,7 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 			//set the availability status for all quitting players
 			for(player_map::iterator pl = players_.begin(); pl != players_.end(); ++pl) {
 				if(g->is_member(pl->first)) {
-					pl->second.mark_available(true);
+					pl->second.mark_available(true,"");
 				}
 			}
 
@@ -889,25 +1171,37 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 			//now sync players in the lobby again, to remove the game
 			lobby_players_.send_data(sync_initial_response());
 		} else {
-
-			g->remove_player(sock);
-			g->describe_slots();
-			lobby_players_.add_player(sock);
-
-			//mark the player as available in the lobby
 			const player_map::iterator pl = players_.find(sock);
-			if(pl != players_.end()) {
-				pl->second.mark_available(true);
+			if(pl != players_.end() && !obs) {
+				const config& msg = construct_server_message(pl->second.name() + " has left the game",*g);
+				g->send_data(msg);
 			} else {
 				std::cerr << "ERROR: Could not find player in map\n";
 			}
 
-			//send the player who has quit the game list
-			network::send_data(initial_response_,sock);
-
-			//send all other players in the lobby the update to the lobby
-			lobby_players_.send_data(sync_initial_response(),sock);
+			if (needed){
+				//transfer game control to another player
+				const player* player = g->transfer_game_control();
+				if (player != NULL){
+					const config& msg = construct_server_message(player->name() + " has been chosen as new host", *g);
+					g->send_data(msg);
+				}
+			}
 		}
+		//mark the player as available in the lobby
+		const player_map::iterator pl = players_.find(sock);
+		if(pl != players_.end()) {
+			pl->second.mark_available(true,"");
+		} else {
+			std::cerr << "ERROR: Could not find player in map\n";
+		}
+		lobby_players_.add_player(sock);
+
+		//send the player who has quit the game list
+		network::send_data(initial_response_,sock);
+
+		//send all other players in the lobby the update to the lobby
+		lobby_players_.send_data(sync_initial_response(),sock);
 
 		return;
 	} else if(data["side_secured"].empty() == false) {
@@ -948,6 +1242,11 @@ void server::process_data_from_player_in_game(const network::connection sock, co
 			//spoofing of messages
 			const player_map::const_iterator pl = players_.find(sock);
 			wassert(pl != players_.end());
+			if ((g->all_observers_muted() && g->is_observer(sock)) || g->is_muted_observer(sock)){
+				const config& msg = construct_server_message("You have been muted, others can't see your message!",*g);
+				network::send_data(msg, pl->first);
+				return;
+			}
 			(*speak)["description"] = pl->second.name();
 
 			if((*speak)["team_name"] == "") {
@@ -1003,8 +1302,6 @@ int main(int argc, char** argv)
 	int port = 15000;
 	size_t nthreads = 5;
 
-	network::set_default_send_size(4096);
-
 	config configuration;
 
 #ifndef FIFODIR
@@ -1031,20 +1328,20 @@ int main(int argc, char** argv)
 				std::cerr << "ERROR: could not read configuration file: '" << e.message << "'\n";
 				return -1;
 			}
-		} else if((val == "--max_packet_size" || val == "-m") && arg+1 != argc) {
-			network::set_default_send_size(size_t(atoi(argv[++arg])));
 		} else if(val == "--verbose" || val == "-v") {
 			lg::set_log_domain_severity("all",2);
 		} else if((val == "--port" || val == "-p") && arg+1 != argc) {
 			port = atoi(argv[++arg]);
 		} else if(val == "--help" || val == "-h") {
 			std::cout << "usage: " << argv[0]
-				<< " [options]\n"
-				<< "  -d  --daemon               Runs wesnothd as a daemon\n"
-				<< "  -m, --max_packet_size n    Sets the maximal packet size to n\n"
-				<< "  -p, --port                 Binds the server to the specified port\n"
-				<< "  -V, --version              Returns the server version\n"
-				<< "  -t, --threads n            Uses n worker threads for network I/O (default: 5)\n";
+				<< " [-dvV] [-c path] [-m n] [-p port] [-t n]\n"
+				<< "  -c  --config path          Tells wesnothd where to find the config file to use.\n"
+				<< "  -d  --daemon               Runs wesnothd as a daemon.\n"
+				<< "  -h  --help                 Shows this usage message.\n"
+				<< "  -p, --port port            Binds the server to the specified port.\n"
+				<< "  -t, --threads n            Uses n worker threads for network I/O (default: 5).\n"
+				<< "  -v  --verbose              Turns on more verbose logging.\n"
+				<< "  -V, --version              Returns the server version.\n";
 			return 0;
 		} else if(val == "--version" || val == "-V") {
 			std::cout << "Battle for Wesnoth server " << game_config::version
