@@ -1,4 +1,4 @@
-/* $Id: team.cpp 8302 2005-09-18 23:32:23Z Sirp $ */
+/* $Id: team.cpp 11534 2006-05-19 08:15:18Z rusty $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -43,6 +43,25 @@ teams_manager::~teams_manager()
 	teams = NULL;
 }
 
+std::vector<team> teams_manager::clone(std::vector<team>& team_list){
+	std::vector<team> result;
+	for (std::vector<team>::iterator t = team_list.begin(); t != team_list.end(); t++){
+		result.push_back(*t);
+	}
+	return result;
+}
+
+bool teams_manager::is_observer()
+{
+	for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
+		if(i->is_human() || i->is_persistent()) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
 team::target::target(const config& cfg)
               : criteria(cfg), value(atof(cfg["value"].c_str()))
 {
@@ -65,6 +84,12 @@ team::team_info::team_info(const config& cfg)
 		}
 	}
 
+	const config::child_list& ai_mems = cfg.get_children("ai_memory");
+	for(config::child_list::const_iterator aimem = ai_mems.begin(); aimem != ai_mems.end(); ++aimem) {
+	  ai_memory_.append(**aimem);
+	}
+
+
 	gold = cfg["gold"];
 	income = cfg["income"];
 	name = cfg["name"];
@@ -78,15 +103,34 @@ team::team_info::team_info(const config& cfg)
 		save_id = cfg["description"];
 	}
 
+	countdown_time = cfg["countdown_time"];
+
+	const std::string& bonus = cfg["action_bonus_count"];
+	if(bonus.empty())
+		action_bonus_count = 0;
+	else
+		action_bonus_count = atoi(bonus.c_str());
+
 	colour = lexical_cast_default<int>(cfg["colour"],-1);
 	if(colour == -1)
 		colour = lexical_cast_default<int>(cfg["side"],-1);
 
+	int side = atoi(cfg["side"].c_str());
+
+	std::vector<Uint32> temp_rgb = string2rgb(cfg["team_rgb"]);
+	std::map<int, color_range>::iterator global_rgb = game_config::team_rgb_range.find(side);
+
+	if(temp_rgb.size()){
+	  team_color_range_[side] = color_range(temp_rgb);
+	}else if(global_rgb != game_config::team_rgb_range.end()){
+	  team_color_range_[side] = global_rgb->second;
+	}
+
 	flag = cfg["flag"];
 
 	description = cfg["description"];
 	objectives = cfg["objectives"];
-	objectives_changed = (cfg["objectives_changed"] == "yes");
+	objectives_changed = utils::string_bool(cfg["objectives_changed"]);
 
 	const std::string& village_income = cfg["village_gold"];
 	if(village_income.empty())
@@ -188,8 +232,8 @@ team::team_info::team_info(const config& cfg)
 	}
 
 	//share_view and share_maps can't both be enabled, so share_view overrides share_maps.
-	share_view = (cfg["share_view"] == "yes");
-	share_maps = !share_view && (cfg["share_maps"] != "no");
+	share_view = utils::string_bool(cfg["share_view"]);
+	share_maps = !share_view && utils::string_bool(cfg["share_maps"],true);
 
 	LOG_NG << "team_info::team_info(...): team_name: " << team_name
 	       << ", share_maps: " << share_maps << ", share_view: " << share_view << ".\n";
@@ -202,6 +246,7 @@ void team::team_info::write(config& cfg) const
 	for(std::vector<config>::const_iterator ai = ai_params.begin(); ai != ai_params.end(); ++ai) {
 		cfg.add_child("ai",*ai);
 	}
+	if(!ai_memory_.empty()) cfg.add_child("ai_memory",ai_memory_);
 	cfg["ai_algorithm"] = ai_algorithm;
 
 	cfg["gold"] = gold;
@@ -213,9 +258,13 @@ void team::team_info::write(config& cfg) const
 	cfg["description"] = description;
 	cfg["objectives"] = objectives;
 	cfg["objectives_changed"] = objectives_changed ? "yes" : "no";
+	cfg["countdown_time"]= countdown_time;
 
 	char buf[50];
-	sprintf(buf,"%d",income_per_village);
+	snprintf(buf,sizeof(buf),"%d",action_bonus_count);
+	cfg["action_bonus_count"]= buf;
+
+	snprintf(buf,sizeof(buf),"%d",income_per_village);
 	cfg["village_gold"] = buf;
 
 	std::stringstream enemies_str;
@@ -236,13 +285,13 @@ void team::team_info::write(config& cfg) const
 	}
 	cfg["persistent"] = persistent ? "1" : "0";
 
-	sprintf(buf,"%d",villages_per_scout);
+	snprintf(buf,sizeof(buf),"%d",villages_per_scout);
 	cfg["villages_per_scout"] = buf;
 
-	sprintf(buf,"%f",leader_value);
+	snprintf(buf,sizeof(buf),"%f",leader_value);
 	cfg["leader_value"] = buf;
 
-	sprintf(buf,"%f",village_value);
+	snprintf(buf,sizeof(buf),"%f",village_value);
 	cfg["village_value"] = buf;
 
 	for(std::vector<target>::const_iterator tg = targets.begin(); tg != targets.end(); ++tg) {
@@ -296,6 +345,9 @@ team::team(const config& cfg, int gold) : gold_(gold), auto_shroud_updates_(true
 	for(config::child_list::const_iterator v = villages.begin(); v != villages.end(); ++v) {
 		villages_.insert(gamemap::location(**v));
 	}
+
+	countdown_time_=lexical_cast_default<int>(cfg["countdown_time"],0);
+	action_bonus_count_=lexical_cast_default<int>(cfg["action_bonus_count"],0);
 }
 
 void team::write(config& cfg) const
@@ -305,7 +357,7 @@ void team::write(config& cfg) const
 	cfg["fog"] = uses_fog() ? "yes" : "no";
 
 	char buf[50];
-	sprintf(buf,"%d",gold_);
+	snprintf(buf,sizeof(buf),"%d",gold_);
 	cfg["gold"] = buf;
 
 	//write village locations
@@ -314,6 +366,10 @@ void team::write(config& cfg) const
 	}
 
 	cfg["shroud_data"] = shroud_.write();
+
+	cfg["countdown_time"] = lexical_cast_default<std::string>(countdown_time_);
+	cfg["action_bonus_count"] = lexical_cast_default<std::string>(action_bonus_count_);
+
 }
 
 bool team::get_village(const gamemap::location& loc)
@@ -406,6 +462,24 @@ void team::set_income(int amount)
 	info_.income = lexical_cast<std::string>(amount);
 }
 
+int team::countdown_time() const
+{
+	return countdown_time_;
+}
+void team::set_countdown_time(int amount)
+{
+	countdown_time_=amount;
+}
+
+int team::action_bonus_count() const
+{
+	return action_bonus_count_;
+}
+void team::set_action_bonus_count(int count)
+{
+	action_bonus_count_=count;
+}
+
 const std::set<std::string>& team::recruits() const
 {
 	return info_.can_recruit;
@@ -562,6 +636,19 @@ const config& team::ai_parameters() const
 	return aiparams_;
 }
 
+const config& team::ai_memory() const
+{
+       return info_.ai_memory_;
+}
+
+void team::set_ai_memory(const config& ai_mem){
+  //would perhaps be more efficient to allow writing to the memory directly,
+  //but this method comparmentalizes the functionality and protects against
+  //accidentally overwriting the memory
+  info_.ai_memory_=ai_mem;
+  return;
+}
+
 void team::make_network()
 {
 	info_.controller = team_info::NETWORK;
@@ -718,7 +805,7 @@ void validate_side(int side)
 	}
 
 	if(side < 1 || side > int(teams->size())) {
-		throw game::game_error("invalid side found in unit definition");
+		throw game::game_error("invalid side(" + lexical_cast_default<std::string>(side) + ") found in unit definition");
 	}
 }
 
@@ -840,29 +927,42 @@ bool team::shroud_map::copy_from(const std::vector<const shroud_map*>& maps)
 	return cleared;
 }
 
-const SDL_Color& team::get_side_colour(int side)
+std::map<int, color_range> team::team_color_range_;
+
+const color_range team::get_side_color_range(int side){
+  size_t index = size_t(get_side_colour_index(side));
+  std::map<int, color_range>::iterator p=team_color_range_.find(index);
+  std::map<int, color_range>::iterator gp=game_config::team_rgb_range.find(index);
+
+  if(p != team_color_range_.end()){
+    return(p->second);
+  }else if(gp != game_config::team_rgb_range.end()){
+    return(gp->second);
+  }
+
+  return(color_range(0x00FF0000,0x00FFFFFF,0x00000000));
+}
+
+const Uint32 team::get_side_rgb(int side){
+  return(get_side_color_range(side).mid());
+}
+
+const Uint32 team::get_side_rgb_max(int side){
+  return(get_side_color_range(side).max());
+}
+
+const Uint32 team::get_side_rgb_min(int side){
+  return(get_side_color_range(side).min());
+}
+
+const SDL_Color team::get_side_colour(int side)
 {
-	size_t index = size_t(get_side_colour_index(side) - 1);
-
-	static const SDL_Color sides[] = { {0xFF,0x00,0x00,0},
-	                                   {0x00,0x00,0xFF,0},
-	                                   {0x00,0xFF,0x00,0},
-	                                   {0xFF,0xFF,0x00,0},
-	                                   {0xFF,0x00,0xFF,0},
-	                                   {0xFF,0x7F,0x00,0},
-	                                   {0x89,0x89,0x89,0},
-	                                   {0xFF,0xFF,0xFF,0},
-	                                   {0x94,0x50,0x27,0},
-	                                   {0x02,0xF5,0xE1,0},
-	                                   {0xFF,0x00,0xFF,0} };
-
-	static const size_t nsides = sizeof(sides)/sizeof(*sides);
-
-	if(index < nsides) {
-		return sides[index];
-	} else {
-		return sides[0];
-	}
+	Uint32 rgb=get_side_rgb(side);
+	SDL_Color color={ (0x00FF0000 & rgb)>>16,
+				       (0x0000FF00 & rgb)>>8,
+				       (0x000000FF & rgb),
+	                               0} ;
+	return color;
 }
 
 int team::get_side_colour_index(int side)
@@ -878,3 +978,11 @@ int team::get_side_colour_index(int side)
 	return side;
 }
 
+void team::log_recruitable(){
+	LOG_NG << "Adding recruitable units: \n";
+	for (std::set<std::string>::const_iterator it = info_.can_recruit.begin();
+		 it != info_.can_recruit.end(); it++) {
+		LOG_NG << *it << std::endl;
+	}
+	LOG_NG << "Added all recruitable units\n";
+}
