@@ -1,4 +1,4 @@
-/* $Id: theme.cpp 7698 2005-08-03 22:19:38Z j_daniel $ */
+/* $Id: theme.cpp 11877 2006-06-05 20:02:25Z soliton $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -34,8 +34,8 @@ namespace {
 	const int YDim = 768;
 
 	const size_t DefaultFontSize = font::SIZE_NORMAL;
+        const Uint32 DefaultFontRGB = 0x00C8C8C8;
 
-	typedef struct { size_t x1,y1,x2,y2; } _rect;
 	_rect ref_rect = { 0, 0, 0, 0 };
 
 	size_t compute(std::string expr, size_t ref1, size_t ref2=0 ) {
@@ -50,6 +50,7 @@ namespace {
 		return ref + atoi(expr.c_str());
 	}
 
+	// if x2 or y2 are not specified, use x1 and y1 values
 	_rect read_rect(const config& cfg) {
 		_rect rect = { 0, 0, 0, 0 };
 		const std::vector<std::string> items = utils::split(cfg["rect"].c_str());
@@ -61,9 +62,13 @@ namespace {
 
 		if(items.size() >= 3)
 			rect.x2 = atoi(items[2].c_str());
+		else
+			rect.x2 = rect.x1;
 
 		if(items.size() >= 4)
 			rect.y2 = atoi(items[3].c_str());
+		else
+			rect.y2 = rect.y1;
 
 		return rect;
 	}
@@ -73,8 +78,8 @@ namespace {
 		const _rect rect = read_rect(cfg);
 		sdlrect.x = rect.x1;
 		sdlrect.y = rect.y1;
-		sdlrect.w = rect.x2 - rect.x1;
-		sdlrect.h = rect.y2 - rect.y1;
+		sdlrect.w = (rect.x2 > rect.x1) ? (rect.x2 - rect.x1) : 0;
+		sdlrect.h = (rect.y2 > rect.y1) ? (rect.y2 - rect.y1) : 0;
 
 		return sdlrect;
 	}
@@ -204,12 +209,12 @@ namespace {
 		}
 		// add all the resolutions
 		const config::child_list& res_list = top_cfg.get_children("resolution");
-		for(config::child_list::const_iterator i = res_list.begin(); i != res_list.end(); ++i) {
-			dst_cfg.add_child("resolution", (**i));
+		for(config::child_list::const_iterator j = res_list.begin(); j != res_list.end(); ++j) {
+			dst_cfg.add_child("resolution", (**j));
 		}
 		// add all the resolved resolutions
-		for(std::vector<config>::const_iterator i = res_cfgs_.begin(); i != res_cfgs_.end(); ++i) {
-			dst_cfg.add_child("resolution", (*i));
+		for(std::vector<config>::const_iterator k = res_cfgs_.begin(); k != res_cfgs_.end(); ++k) {
+			dst_cfg.add_child("resolution", (*k));
 		}
 		return;
 	}
@@ -253,20 +258,23 @@ namespace {
 	}
 }
 
-theme::object::object() : loc_(empty_rect), relative_loc_(empty_rect),
+theme::object::object() : location_modified_(false), loc_(empty_rect), relative_loc_(empty_rect),
                           last_screen_(empty_rect), xanchor_(object::FIXED), yanchor_(object::FIXED)
 {}
 
 theme::object::object(const config& cfg)
-                   : loc_(read_sdl_rect(cfg)), relative_loc_(empty_rect),
+                   : id_(cfg["id"]),
+					 loc_(read_sdl_rect(cfg)), relative_loc_(empty_rect),
 				     last_screen_(empty_rect),
                      xanchor_(read_anchor(cfg["xanchor"])),
 					 yanchor_(read_anchor(cfg["yanchor"]))
-{}
+{
+	location_modified_ = false;
+}
 
 SDL_Rect& theme::object::location(const SDL_Rect& screen) const
 {
-	if(last_screen_ == screen)
+	if(last_screen_ == screen && !location_modified_)
 		return relative_loc_;
 
 	last_screen_ = screen;
@@ -326,6 +334,10 @@ const SDL_Rect& theme::object::get_location(void) const
 	return loc_;
 }
 
+const std::string& theme::object::get_id() const{
+	return id_;
+}
+
 theme::object::ANCHORING theme::object::read_anchor(const std::string& str)
 {
 	static const std::string top_anchor = "top", left_anchor = "left",
@@ -341,15 +353,66 @@ theme::object::ANCHORING theme::object::read_anchor(const std::string& str)
 		return FIXED;
 }
 
+void theme::object::modify_location(const _rect rect){
+	loc_.x = rect.x1;
+	loc_.y = rect.y1;
+	loc_.w = rect.x2 - rect.x1;
+	loc_.h = rect.y2 - rect.y1;
+	location_modified_ = true;
+}
+
+void theme::object::modify_location(std::string rect_str, SDL_Rect ref_rect){
+	_rect rect = { 0, 0, 0, 0 };
+	const std::vector<std::string> items = utils::split(rect_str.c_str());
+	if(items.size() >= 1) {
+		rect.x1 = compute(items[0], ref_rect.x, ref_rect.x + ref_rect.w);
+	}
+	if(items.size() >= 2) {
+		rect.y1 = compute(items[1], ref_rect.y, ref_rect.y + ref_rect.h);
+	}
+	if(items.size() >= 3) {
+		rect.x2 = compute(items[2], ref_rect.x + ref_rect.w, rect.x1);
+	}
+	if(items.size() >= 4) {
+		rect.y2 = compute(items[3], ref_rect.y + ref_rect.h, rect.y1);
+	}
+	modify_location(rect);
+}
+
 theme::label::label()
 {}
 
 theme::label::label(const config& cfg)
       : object(cfg), text_(cfg["prefix"].str() + cfg["text"].str() + cfg["postfix"].str()),
-	    icon_(cfg["icon"]), font_(atoi(cfg["font_size"].c_str()))
+	icon_(cfg["icon"]), font_(atoi(cfg["font_size"].c_str()))
 {
 	if(font_ == 0)
 		font_ = DefaultFontSize;
+
+	font_rgb_ = DefaultFontRGB;
+	font_rgb_set_ = false;
+	if(cfg["font_rgb"].size()){
+	std::vector<std::string> rgb_vec = utils::split(cfg["font_rgb"]);
+	  if(3 <= rgb_vec.size()){
+	    std::vector<std::string>::iterator c=rgb_vec.begin();
+	    int r,g,b;
+	    r = (atoi(c->c_str()));
+	    c++;
+	    if(c != rgb_vec.end()){
+	      g = (atoi(c->c_str()));
+	    }else{
+	      g=0;
+	    }
+	    c++;
+	    if(c != rgb_vec.end()){
+	      b=(atoi(c->c_str()));
+	    }else{
+	      b=0;
+	    }
+	    font_rgb_ = (((r<<16) & 0x00FF0000) + ((g<<8) & 0x0000FF00) + ((b) & 0x000000FF));
+	    font_rgb_set_=true;
+	  }
+	}
 }
 
 const std::string& theme::label::text() const
@@ -372,6 +435,16 @@ size_t theme::label::font_size() const
 	return font_;
 }
 
+Uint32 theme::label::font_rgb() const
+{
+	return font_rgb_;
+}
+
+bool theme::label::font_rgb_set() const
+{
+	return font_rgb_set_;
+}
+
 theme::status_item::status_item(const config& cfg)
         : object(cfg),
 		  prefix_(cfg["prefix"].str() + cfg["prefix_literal"].str()),
@@ -385,6 +458,31 @@ theme::status_item::status_item(const config& cfg)
 	if(label_child != NULL) {
 		label_ = label(*label_child);
 	}
+
+	font_rgb_ = DefaultFontRGB;
+	font_rgb_set_ = false;
+	if(cfg["font_rgb"].size()){
+	  std::vector<std::string> rgb_vec = utils::split(cfg["font_rgb"]);
+	  if(3 <= rgb_vec.size()){
+	    std::vector<std::string>::iterator c=rgb_vec.begin();
+	    int r,g,b;
+	    r = (atoi(c->c_str()));
+	    c++;
+	    if(c != rgb_vec.end()){
+	      g = (atoi(c->c_str()));
+	    }else{
+	      g=0;
+	    }
+	    c++;
+	    if(c != rgb_vec.end()){
+	      b=(atoi(c->c_str()));
+	    }else{
+	      b=0;
+	    }
+	    font_rgb_ = (((r<<16) & 0x00FF0000) + ((g<<8) & 0x0000FF00) + ((b) & 0x000000FF));
+	    font_rgb_set_=true;
+	  }
+	}
 }
 
 const std::string& theme::status_item::prefix() const
@@ -407,6 +505,16 @@ size_t theme::status_item::font_size() const
 	return font_;
 }
 
+Uint32 theme::status_item::font_rgb() const
+{
+	return font_rgb_;
+}
+
+bool theme::status_item::font_rgb_set() const
+{
+	return font_rgb_set_;
+}
+
 theme::panel::panel(const config& cfg) : object(cfg), image_(cfg["image"])
 {}
 
@@ -420,14 +528,19 @@ theme::menu::menu() : context_(false)
 
 theme::menu::menu(const config& cfg) : object(cfg), context_(cfg["is_context_menu"] == "true"),
                                        title_(cfg["title"].str() + cfg["title_literal"].str()),
-				       image_(cfg["image"]),
-				       items_(utils::split(cfg["items"]))
+									   tooltip_(cfg["tooltip"]),
+						image_(cfg["image"]), type_(cfg["type"]),
+						items_(utils::split(cfg["items"]))
 {}
 
 bool theme::menu::is_context() const { return context_; }
 
 const std::string& theme::menu::title() const { return title_; }
 
+const std::string& theme::menu::tooltip() const { return tooltip_; }
+
+const std::string& theme::menu::type() const { return type_; }
+
 const std::string& theme::menu::image() const { return image_; }
 
 const std::vector<std::string>& theme::menu::items() const { return items_; }
@@ -477,19 +590,24 @@ bool theme::set_resolution(const SDL_Rect& screen)
 	menus_.clear();
 
 	const config& cfg = **current;
+	add_object(cfg);
+
+	return result;
+}
+
+theme::object& theme::add_object(const config& cfg){
+	theme::object& result = *(new theme::object());
 
 	const config* const main_map_cfg = cfg.child("main_map");
 	if(main_map_cfg != NULL) {
 		main_map_ = object(*main_map_cfg);
-	} else {
-		main_map_ = object();
+		result = main_map_;
 	}
 
 	const config* const mini_map_cfg = cfg.child("mini_map");
 	if(mini_map_cfg != NULL) {
 		mini_map_ = object(*mini_map_cfg);
-	} else {
-		mini_map_ = object();
+		result = mini_map_;
 	}
 
 	const config* const status_cfg = cfg.child("status");
@@ -499,33 +617,131 @@ bool theme::set_resolution(const SDL_Rect& screen)
 				status_.insert(std::pair<std::string,status_item>(i->first,status_item(**j)));
 			}
 		}
+		const config* const unit_image_cfg = status_cfg->child("unit_image");
+		if (unit_image_cfg != NULL) {
+			unit_image_ = object(*unit_image_cfg);
+		} else {
+			unit_image_ = object();
+		}
+		result = unit_image_;
 	}
 
 	const config::child_list& panel_list = cfg.get_children("panel");
 	for(config::child_list::const_iterator p = panel_list.begin(); p != panel_list.end(); ++p) {
-		panels_.push_back(panel(**p));
+		panel new_panel(**p);
+		set_object_location(new_panel, (**p)["rect"], (**p)["ref"]);
+		panels_.push_back(new_panel);
+		result = new_panel;
 	}
 
 	const config::child_list& label_list = cfg.get_children("label");
 	for(config::child_list::const_iterator lb = label_list.begin(); lb != label_list.end(); ++lb) {
-		labels_.push_back(label(**lb));
+		label new_label(**lb);
+		set_object_location(new_label, (**lb)["rect"], (**lb)["ref"]);
+		labels_.push_back(new_label);
+		result = new_label;
 	}
 
 	const config::child_list& menu_list = cfg.get_children("menu");
 	for(config::child_list::const_iterator m = menu_list.begin(); m != menu_list.end(); ++m) {
-		const menu new_menu(**m);
+		menu new_menu(**m);
 		LOG_DP << "adding menu: " << (new_menu.is_context() ? "is context" : "not context") << "\n";
 		if(new_menu.is_context())
 			context_ = new_menu;
-		else
+		else{
+			set_object_location(new_menu, (**m)["rect"], (**m)["ref"]);
 			menus_.push_back(new_menu);
+		}
 
 		LOG_DP << "done adding menu...\n";
+		result = new_menu;
 	}
 
 	return result;
 }
 
+void theme::remove_object(std::string id){
+	for(std::vector<theme::panel>::iterator p = panels_.begin(); p != panels_.end(); ++p) {
+		if (p->get_id() == id){
+			panels_.erase(p);
+			return;
+		}
+	}
+	for(std::vector<theme::label>::iterator l = labels_.begin(); l != labels_.end(); ++l) {
+		if (l->get_id() == id){
+			labels_.erase(l);
+			return;
+		}
+	}
+	for(std::vector<theme::menu>::iterator m = menus_.begin(); m != menus_.end(); ++m) {
+		if (m->get_id() == id){
+			menus_.erase(m);
+			return;
+		}
+	}
+}
+
+void theme::set_object_location(theme::object& element, std::string rect_str, std::string ref_id){
+	theme::object& ref_element = *(new object());
+	if (ref_id.empty()) {
+		ref_id = element.get_id();
+		ref_element = element;
+	}
+	else {
+		ref_element = find_element(ref_id);
+	}
+	if (ref_element.get_id() == ref_id){
+		SDL_Rect ref_rect = ref_element.get_location();
+		element.modify_location(rect_str, ref_rect);
+	}
+}
+
+void theme::modify(const config* cfg){
+	{
+		//changes to existing theme objects
+		const config::child_list& c = cfg->get_children("change");
+		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
+			std::string id = (**j)["id"];
+			std::string ref_id = (**j)["ref"];
+			theme::object& element = find_element(id);
+			if (element.get_id() == id){
+				set_object_location(element, (**j)["rect"], ref_id);
+			}
+		}
+	}
+	//adding new theme objects
+	{
+		const config::child_list& c = cfg->get_children("add");
+		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
+			add_object(**j);
+		}
+	}
+	//removing existent theme objects
+	{
+		const config::child_list& c = cfg->get_children("remove");
+		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
+			remove_object((**j)["id"]);
+		}
+	}
+}
+
+theme::object& theme::find_element(std::string id){
+	theme::object* res = new theme::object();
+	for (std::vector<theme::panel>::iterator p = panels_.begin(); p != panels_.end(); ++p){
+		if (p->get_id() == id) { res = &(*p); }
+	}
+	for (std::vector<theme::label>::iterator l = labels_.begin(); l != labels_.end(); ++l){
+		if (l->get_id() == id) { res = &(*l); }
+	}
+	for (std::vector<theme::menu>::iterator m = menus_.begin(); m != menus_.end(); ++m){
+		if (m->get_id() == id) { res = &(*m); }
+	}
+	if (id == "main-map") { res = &main_map_; }
+	if (id == "mini-map") { res = &mini_map_; }
+	if (id == "unit-image") { res = &unit_image_; }
+	return *res;
+}
+
 const std::vector<theme::panel>& theme::panels() const
 {
 	return panels_;
@@ -565,6 +781,11 @@ const SDL_Rect& theme::mini_map_location(const SDL_Rect& screen) const
 	return mini_map_.location(screen);
 }
 
+const SDL_Rect& theme::unit_image_location(const SDL_Rect& screen) const
+{
+	return unit_image_.location(screen);
+}
+
 std::map<std::string, config> theme::known_themes;
 void theme::set_known_themes(const config* cfg){
         known_themes.clear();
