@@ -1,4 +1,4 @@
-/* $Id: theme.hpp 7697 2005-08-03 21:56:48Z j_daniel $ */
+/* $Id: theme.hpp 11655 2006-05-27 03:49:39Z jhinrichs $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -22,6 +22,8 @@ class config;
 
 #include "SDL.h"
 
+typedef struct { size_t x1,y1,x2,y2; } _rect;
+
 class theme
 {
 
@@ -33,6 +35,12 @@ class theme
 
 		SDL_Rect& location(const SDL_Rect& screen) const;
 		const SDL_Rect& get_location(void) const;
+		const std::string& get_id() const;
+
+		//this supports relocating of theme elements ingame.
+		//It is needed for [change] tags in theme WML.
+		void modify_location(const _rect rect);
+		void modify_location(std::string rect_str, SDL_Rect rect_ref);
 
 		//all on-screen objects have 'anchoring' in the x and y dimensions
 		//'fixed' means that they have fixed co-ordinates and don't move
@@ -45,6 +53,8 @@ class theme
 		enum ANCHORING { FIXED, TOP_ANCHORED, PROPORTIONAL, BOTTOM_ANCHORED };
 
 	private:
+		bool location_modified_;
+		std::string id_;
 		SDL_Rect loc_;
 		mutable SDL_Rect relative_loc_;
 		mutable SDL_Rect last_screen_;
@@ -56,7 +66,7 @@ class theme
 
 public:
 
-	class label : private object
+	class label : public object
 	{
 	public:
 		label();
@@ -70,13 +80,16 @@ public:
 		bool empty() const;
 
 		size_t font_size() const;
-
+		Uint32 font_rgb() const;
+	        bool font_rgb_set() const;
 	private:
 		std::string text_, icon_;
 		size_t font_;
+	        bool font_rgb_set_;
+		Uint32 font_rgb_;
 	};
 
-	class status_item : private object
+	class status_item : public object
 	{
 	public:
 
@@ -91,14 +104,18 @@ public:
 		const label* get_label() const;
 
 		size_t font_size() const;
+                Uint32 font_rgb() const;
+	        bool font_rgb_set() const;
 
 	private:
 		std::string prefix_, postfix_;
 		label label_;
 		size_t font_;
+	        bool font_rgb_set_;
+		Uint32 font_rgb_;
 	};
 
-	class panel : private object
+	class panel : public object
 	{
 	public:
 		explicit panel(const config& cfg);
@@ -111,7 +128,7 @@ public:
 		std::string image_;
 	};
 
-	class menu : private object
+	class menu : public object
 	{
 	public:
 		menu();
@@ -123,17 +140,22 @@ public:
 
 		const std::string& title() const;
 
+		const std::string& tooltip() const;
+
+		const std::string& type() const;
+
 		const std::string& image() const;
 
 		const std::vector<std::string>& items() const;
 	private:
 		bool context_;
-		std::string title_, image_;
+		std::string title_, tooltip_, image_, type_;
 		std::vector<std::string> items_;
 	};
 
 	explicit theme(const config& cfg, const SDL_Rect& screen);
 	bool set_resolution(const SDL_Rect& screen);
+	void modify(const config* cfg);
 
 	const std::vector<panel>& panels() const;
 	const std::vector<label>& labels() const;
@@ -145,11 +167,17 @@ public:
 
 	const SDL_Rect& main_map_location(const SDL_Rect& screen) const;
 	const SDL_Rect& mini_map_location(const SDL_Rect& screen) const;
+	const SDL_Rect& unit_image_location(const SDL_Rect& screen) const;
 
-        static void set_known_themes(const config* cfg);
-        static std::vector<std::string> get_known_themes();
+    static void set_known_themes(const config* cfg);
+    static std::vector<std::string> get_known_themes();
 
 private:
+	theme::object& find_element(std::string id);
+	theme::object& add_object(const config& cfg);
+	void remove_object(std::string id);
+	void set_object_location(theme::object& element, std::string rect_str, std::string ref_id);
+
 	static std::map<std::string, config> known_themes;
         std::string cur_theme;
         config cfg_;
@@ -161,7 +189,7 @@ private:
 
 	std::map<std::string,status_item> status_;
 
-	object main_map_, mini_map_;
+	object main_map_, mini_map_, unit_image_;
 };
 
 #endif
