@@ -1,4 +1,4 @@
-/* $Id: unit_types.hpp 7953 2005-08-21 17:46:03Z gruikya $ */
+/* $Id: unit_types.hpp 12647 2006-07-19 18:54:27Z xan $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -13,111 +13,106 @@
 #ifndef UNIT_TYPES_H_INCLUDED
 #define UNIT_TYPES_H_INCLUDED
 
-#include "animated.hpp"
+#include "unit_animation.hpp"
 #include "config.hpp"
+#include "color_range.hpp"
 #include "map.hpp"
 #include "race.hpp"
-#include "team.hpp"
 #include "util.hpp"
 
 #include <string>
 #include <vector>
 
-//a class to describe a unit's animation sequence
-class unit_animation
-{
-public:
-	struct frame {
-		frame() : xoffset(0), halo_x(0), halo_y(0) {}
-		explicit frame(const std::string& str) : xoffset(0), image(str),
-		                                         halo_x(0), halo_y(0) {}
-		explicit frame(const config& cfg);
 
-		// int start, end;
-		int xoffset;
-		std::string image;
-		std::string image_diagonal;
-		std::string halo;
-		int halo_x, halo_y;
-	};
+class unit_type;
+class unit;
+class unit_ability_list;
+struct game_data;
+class gamestatus;
+class team;
 
-	unit_animation();
-	explicit unit_animation(const config& cfg);
-
-	enum FRAME_TYPE { UNIT_FRAME, MISSILE_FRAME };
-	enum FRAME_DIRECTION { VERTICAL, DIAGONAL };
-
-	void start_animation(int start_frame, FRAME_TYPE type, int acceleration);
-	void update_current_frames();
-	bool animation_finished() const;
-	const frame& get_current_frame(FRAME_TYPE type=UNIT_FRAME) const;
-	int get_animation_time() const;
-	int get_first_frame_time(FRAME_TYPE type=UNIT_FRAME) const;
-	int get_last_frame_time(FRAME_TYPE type=UNIT_FRAME) const;
-
-	struct sfx {
-		int time;
-		std::string on_hit, on_miss;
-	};
-
-	const std::vector<sfx>& sound_effects() const;
-
-private:
-
-	// std::vector<frame> frames_[2];
-	animated<frame> unit_frames_;
-	animated<frame> missile_frames_;
-
-	std::vector<sfx> sfx_;
-};
-
-//the 'attack type' is the type of attack, how many times it strikes,
 //and how much damage it does.
 class attack_type
 {
 public:
 	enum RANGE { SHORT_RANGE, LONG_RANGE };
 
-	attack_type(const config& cfg);
+	attack_type(const config& cfg, const std::string& id, const std::string& image_fighting);
 	const t_string& name() const;
 	const std::string& id() const;
 	const std::string& type() const;
-	const std::string& special() const;
 	const std::string& icon() const;
-	RANGE range() const;
+	RANGE range_type() const;
+	const std::string& range() const;
 	int damage() const;
 	int num_attacks() const;
 	double attack_weight() const;
 	double defense_weight() const;
 
-	bool backstab() const;
-	bool slow() const;
+	bool get_special_bool(const std::string& special,bool force=false) const;
+	unit_ability_list get_specials(const std::string& special) const;
+	std::vector<std::string> special_tooltips(bool force=false) const;
+	std::string weapon_specials(bool force=false) const;
+	void set_specials_context(const gamemap::location& aloc,const gamemap::location& dloc,
+                              const game_data* gamedata, const unit_map* unitmap,
+							  const gamemap* map, const gamestatus* game_status,
+							  const std::vector<team>* teams,bool attacker,const attack_type* other_attack) const;
+	void set_specials_context(const gamemap::location& loc,const unit& un) const;
 
+	bool has_special_by_id(const std::string& special) const;
 	//this function returns a random animation out of the possible
 	//animations for this attack. It will not return the same attack
 	//each time.
-	const unit_animation& animation(gamemap::location::DIRECTION dir=gamemap::location::NDIRECTIONS) const;
-
-	bool matches_filter(const config& cfg) const;
+	bool matches_filter(const config& cfg,bool self=false) const;
 	bool apply_modification(const config& cfg,std::string* description);
+
+	int movement_used() const;
+
+	const config& get_cfg() const;
+	mutable gamemap::location aloc_,dloc_;
+	mutable bool attacker_;
+	mutable const game_data* gamedata_;
+	mutable const unit_map* unitmap_;
+	mutable const gamemap* map_;
+	mutable const gamestatus* game_status_;
+	mutable const std::vector<team>* teams_;
+	mutable const attack_type* other_attack_;
+	/*
+	 * cfg: a weapon special WML structure
+	 */
+	bool special_active(const config& cfg,bool self,bool report=false) const;
+	bool special_affects_opponent(const config& cfg) const;
+	bool special_affects_self(const config& cfg) const;
+
+	struct attack_animation
+	{
+		typedef enum { HIT, MISS, HIT_OR_MISS } hit_type;
+
+		explicit attack_animation(const config& cfg);
+		explicit attack_animation(const std::string &image,const hit_type for_hit= HIT_OR_MISS):animation(image,-200,100),hits(for_hit) {};
+		int matches(bool hit,gamemap::location::DIRECTION dir=gamemap::location::NDIRECTIONS) const;
+
+		std::vector<gamemap::location::DIRECTION> directions;
+		unit_animation animation;
+		unit_animation missile_animation;
+		hit_type hits;
+
+	};
+	config cfg_;
+	const attack_animation& animation(bool hit,gamemap::location::DIRECTION dir=gamemap::location::NDIRECTIONS) const;
 private:
-	std::vector<unit_animation> animation_;
-	std::vector<unit_animation> direction_animation_[6];
+	std::vector<attack_animation> animation_;
 	t_string description_;
 	std::string id_;
 	std::string type_;
-	std::string special_;
 	std::string icon_;
-	RANGE range_;
-	int hexes_;
+	RANGE range_type_;
+	std::string range_;
 	int damage_;
 	int num_attacks_;
 	double attack_weight_;
 	double defense_weight_;
 
-	//caches whether the unit can backstab and slow. This is important
-	//because the AI queries it alot.
-	bool backstab_, slow_;
 };
 
 class unit_movement_type;
@@ -142,9 +137,13 @@ public:
 	void set_parent(const unit_movement_type* parent);
 
 	bool is_flying() const;
+	const std::map<gamemap::TERRAIN,int>& movement_costs() const;
+	const std::map<gamemap::TERRAIN,int>& defense_mods() const;
 
+	const config& get_cfg() const;
+	const unit_movement_type* get_parent() const;
 private:
-	const config& cfg_;
+	const config cfg_;
 
 	mutable std::map<gamemap::TERRAIN,int> moveCosts_;
 	mutable std::map<gamemap::TERRAIN,int> defenseMods_;
@@ -157,25 +156,24 @@ typedef std::map<std::string,unit_movement_type> movement_type_map;
 class unit_type
 {
 public:
-	//this class assumes that the passed in references will remain valid
-	//for at least as long as the class instance
+	friend class unit;
 	unit_type(const config& cfg, const movement_type_map& movement_types,
 	          const race_map& races, const std::vector<config*>& traits);
 	unit_type(const unit_type& o);
 
 	~unit_type();
 
-        // adds an additional advancement path to a unit type
-        // this is used to implement the [advancefrom] tag
-        void add_advancement(const unit_type &advance_to,int experience);
+    // adds an additional advancement path to a unit type
+    // this is used to implement the [advancefrom] tag
+    void add_advancement(const unit_type &advance_to,int experience);
 
 
 	const unit_type& get_gender_unit_type(unit_race::GENDER gender) const;
 	const unit_type& get_variation(const std::string& name) const;
-        //info on the type of unit that the unit reanimates as
-        const std::string& undead_variation() const;
+    //info on the type of unit that the unit reanimates as
+    const std::string& undead_variation() const;
 
-	int num_traits() const;
+	unsigned int num_traits() const;
 
 	std::string generate_description() const;
 
@@ -187,23 +185,18 @@ public:
 	//where name() is always the user-visible, translated, name.
 	//language_name should eventually be renamed name()
 	// const std::string& name() const;
+
 	const std::string& image() const;
-	const std::string& image_halo() const;
-	const std::string& image_moving() const;
 	const std::string& image_profile() const;
-	const std::string& image_fighting(attack_type::RANGE range) const;
-	const std::string& image_defensive(attack_type::RANGE range) const;
-	const std::string& image_leading() const;
-	const std::string& image_healing() const;
-	const std::string& image_halo_healing() const;
-	const std::string& unit_description() const;
-	const std::string& get_hit_sound() const;
-	const std::string& die_sound() const;
+	const t_string& unit_description() const;
+
+    const std::string& flag_rgb() const;
+
 	int hitpoints() const;
 	std::vector<attack_type> attacks() const;
 	const unit_movement_type& movement_type() const;
 
-	int experience_needed() const;
+	int experience_needed(bool with_acceleration=true) const;
 	std::vector<std::string> advances_to() const;
 	const config::child_list& modification_advancements() const;
 	const std::string& usage() const;
@@ -211,6 +204,7 @@ public:
 	struct experience_accelerator {
 		experience_accelerator(int modifier);
 		~experience_accelerator();
+		static int get_acceleration();
 	private:
 		int old_value_;
 	};
@@ -228,65 +222,48 @@ public:
 	fixed_t alpha() const;
 
 	const std::vector<std::string>& abilities() const;
+	const std::vector<std::string>& ability_tooltips() const;
 
-	//max_unit_healing returns the maximum hitpoints a unit next to this
-	//unit can heal per turn. heals returns the total amount of hitpoints
-	//this unit can heal out of all adjacent units
-	int max_unit_healing() const;
-	int heals() const;
-	bool regenerates() const;
-	bool is_leader() const;
-	bool illuminates() const;
-	bool is_skirmisher() const;
-	bool teleports() const;
-	bool nightvision() const;
-	bool steadfast() const;
 	bool not_living() const;
 	bool can_advance() const;
 
 	bool has_zoc() const;
 
 	bool has_ability(const std::string& ability) const;
+	bool has_ability_by_id(const std::string& ability) const;
 
 	const std::vector<config*>& possible_traits() const;
+	bool has_random_traits() const;
 
 	const std::vector<unit_race::GENDER>& genders() const;
 
 	const std::string& race() const;
 
-	const unit_animation* defend_animation(bool hits, attack_type::RANGE range) const;
-	const unit_animation* teleport_animation() const;
-	const unit_animation* die_animation(const attack_type* attack) const;
-
 private:
 	void operator=(const unit_type& o);
+	const std::string& image_fighting(attack_type::RANGE range) const;
 
 	unit_type* gender_types_[2];
 
 	typedef std::map<std::string,unit_type*> variations_map;
 	variations_map variations_;
 
-	const config& cfg_;
+	config cfg_;
 
 	const unit_race* race_;
 
 	fixed_t alpha_;
 
 	std::vector<std::string> abilities_;
+	std::vector<std::string> ability_tooltips_;
 
 	mutable std::string id_;
 
-	int max_heals_;
-	int heals_;
-	bool regenerates_;
-	bool leadership_;
-	bool illuminates_;
-	bool skirmish_;
-	bool teleport_;
-	bool nightvision_;
-	bool steadfast_;
-        std::vector<std::string> advances_to_;
-        int experience_needed_;
+	bool zoc_;
+
+    std::vector<std::string> advances_to_;
+    int experience_needed_;
+
 
 	ALIGNMENT alignment_;
 
@@ -296,30 +273,22 @@ private:
 
 	std::vector<unit_race::GENDER> genders_;
 
-	struct defensive_animation
-	{
-		explicit defensive_animation(const config& cfg);
-		bool matches(bool hits, attack_type::RANGE range) const;
-
-		enum { HIT, MISS, HIT_OR_MISS } hits;
-		enum { SHORT, LONG, SHORT_OR_LONG } range;
-		unit_animation animation;
-	};
-
 	std::vector<defensive_animation> defensive_animations_;
 
 	std::vector<unit_animation> teleport_animations_;
 
-	struct death_animation
-	{
-		explicit death_animation(const config& cfg);
-		bool matches(const attack_type* attack) const;
-
-		std::vector<std::string> damage_type, special;
-		unit_animation animation;
-	};
+	std::multimap<std::string,unit_animation> extra_animations_;
 
 	std::vector<death_animation> death_animations_;
+
+	std::vector<movement_animation> movement_animations_;
+
+	std::vector<standing_animation> standing_animations_;
+
+	std::vector<leading_animation> leading_animations_;
+
+	std::vector<healing_animation> healing_animations_;
+    std::string flag_rgb_;
 };
 
 struct game_data
