@@ -1,4 +1,4 @@
-/* $Id: menu.cpp 8173 2005-09-06 14:00:05Z ott $ */
+/* $Id: menu.cpp 12604 2006-07-17 09:54:21Z isaac $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -11,27 +11,24 @@
    See the COPYING file for more details.
 */
 
-#include "../global.hpp"
+#include "global.hpp"
 
-#include "menu.hpp"
+#include "widgets/menu.hpp"
 
-#include "../font.hpp"
-#include "../image.hpp"
-#include "../sdl_utils.hpp"
-#include "../util.hpp"
-#include "../video.hpp"
-#include "../wml_separators.hpp"
+#include "language.hpp"
+#include "font.hpp"
+#include "image.hpp"
+#include "marked-up_text.hpp"
+#include "sdl_utils.hpp"
+#include "util.hpp"
+#include "video.hpp"
+#include "wml_separators.hpp"
 #include "serialization/string_utils.hpp"
 
 #include <algorithm>
 #include <cassert>
 #include <numeric>
 
-namespace {
-const size_t menu_font_size = font::SIZE_NORMAL;
-const size_t menu_cell_padding = font::SIZE_NORMAL * 3/5;
-}
-
 namespace gui {
 
 menu::basic_sorter::basic_sorter()
@@ -150,7 +147,7 @@ bool menu::basic_sorter::less(int column, const item& row1, const item& row2) co
 
 menu::menu(CVideo& video, const std::vector<std::string>& items,
            bool click_selects, int max_height, int max_width,
-		   const sorter* sorter_obj)
+		   const sorter* sorter_obj, style *menu_style)
         : scrollarea(video),
           max_height_(max_height), max_width_(max_width), max_items_(-1), item_height_(-1),
 		  heading_height_(-1),
@@ -163,6 +160,8 @@ menu::menu(CVideo& video, const std::vector<std::string>& items,
 	  last_was_doubleclick_(false),
 	  sorter_(sorter_obj), sortby_(-1), sortreversed_(false), highlight_heading_(-1)
 {
+	style_ = (menu_style) ? menu_style : &default_style;
+	style_->init();
 	fill_items(items, true);
 }
 
@@ -294,10 +293,10 @@ void menu::update_size()
 	unsigned int h = heading_height();
 	for(size_t i = get_position(),
 	    i_end = minimum(items_.size(), i + max_items_onscreen());
-	    i != i_end; ++i)
+	    i < i_end; ++i)
 		h += get_item_rect(i).h;
 	h = maximum(h, height());
-	if (max_height_ > 0 && h > max_height_)
+	if (max_height_ > 0 && h > (unsigned)max_height_)
 		h = max_height_;
 
 	std::vector<int> const &widths = column_widths();
@@ -305,7 +304,7 @@ void menu::update_size()
 	if (items_.size() > max_items_onscreen())
 		w += scrollbar_width();
 	w = maximum(w, width());
-	if (max_width_ > 0 && w > max_width_)
+	if (max_width_ > 0 && w > (unsigned)max_width_)
 		w = max_width_;
 
 	update_scrollbar_grip_height();
@@ -525,6 +524,9 @@ void menu::key_press(SDLKey key)
 void menu::handle_event(const SDL_Event& event)
 {
 	scrollarea::handle_event(event);
+	if (hidden())
+		return;
+
 	if(event.type == SDL_KEYDOWN) {
 		// Only pass key events if we have the focus
 		if (focus())
@@ -622,7 +624,7 @@ void menu::set_numeric_keypress_selection(bool value)
 	num_selects_ = value;
 }
 
-void menu::scroll(int)
+void menu::scroll(unsigned int)
 {
 	itemRects_.clear();
 	set_dirty();
@@ -649,49 +651,101 @@ void menu::sort_by(int column)
 	set_dirty();
 }
 
-namespace {
-	SDL_Rect item_size(const std::string& item) {
-		SDL_Rect res = {0,0,0,0};
-		std::vector<std::string> img_text_items = utils::split(item, IMG_TEXT_SEPARATOR);
-		for (std::vector<std::string>::const_iterator it = img_text_items.begin();
-			 it != img_text_items.end(); it++) {
-			if (res.w > 0 || res.h > 0) {
-				// Not the first item, add the spacing.
-				res.w += 5;
+SDL_Rect menu::style::item_size(const std::string& item) const {
+	SDL_Rect res = {0,0,0,0};
+	std::vector<std::string> img_text_items = utils::split(item, IMG_TEXT_SEPARATOR);
+	for (std::vector<std::string>::const_iterator it = img_text_items.begin();
+		 it != img_text_items.end(); it++) {
+		if (res.w > 0 || res.h > 0) {
+			// Not the first item, add the spacing.
+			res.w += 5;
+		}
+		const std::string str = *it;
+		if (!str.empty() && str[0] == IMAGE_PREFIX) {
+			const std::string image_name(str.begin()+1,str.end());
+			surface const img = image::get_image(image_name,image::UNSCALED);
+			if(img != NULL) {
+				res.w += img->w;
+				res.h = maximum<int>(img->h, res.h);
 			}
-			const std::string str = *it;
-			if (!str.empty() && str[0] == IMAGE_PREFIX) {
-				const std::string image_name(str.begin()+1,str.end());
-				surface const img = image::get_image(image_name,image::UNSCALED);
-				if(img != NULL) {
-					res.w += img->w;
-					res.h = maximum<int>(img->h, res.h);
-				}
-			} else {
-				const SDL_Rect area = {0,0,10000,10000};
-				const SDL_Rect font_size =
-					font::draw_text(NULL,area,menu_font_size,font::NORMAL_COLOUR,str,0,0);
-				res.w += font_size.w;
-				res.h = maximum<int>(font_size.h, res.h);
-			}
+		} else {
+			const SDL_Rect area = {0,0,10000,10000};
+			const SDL_Rect font_size =
+				font::draw_text(NULL,area,get_font_size(),font::NORMAL_COLOUR,str,0,0);
+			res.w += font_size.w;
+			res.h = maximum<int>(font_size.h, res.h);
 		}
-		return res;
 	}
+	return res;
 }
 
+void menu::style::draw_row_bg(menu& menu_ref, const size_t /*row_index*/, const SDL_Rect& rect, ROW_TYPE type)
+{
+	menu_ref.bg_restore(rect);
+
+	int rgb = 0;
+	double alpha = 0.0;
+
+	switch(type) {
+	case NORMAL_ROW:
+		rgb = normal_rgb_;
+		alpha = normal_alpha_;
+		break;
+	case SELECTED_ROW:
+		rgb = selected_rgb_;
+		alpha = selected_alpha_;
+		break;
+	case HEADING_ROW:
+		rgb = heading_rgb_;
+		alpha = heading_alpha_;
+		break;
+	}
+
+	draw_solid_tinted_rectangle(rect.x, rect.y, rect.w, rect.h,
+				    (rgb&0xff0000) >> 16,(rgb&0xff00) >> 8,rgb&0xff,alpha,
+				    menu_ref.video().getSurface());
+}
+
+void menu::style::draw_row(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
+{
+	if(rect.w == 0 || rect.h == 0) {
+		return;
+	}
+	draw_row_bg(menu_ref, row_index, rect, type);
+
+	SDL_Rect minirect = rect;
+	if(type != HEADING_ROW) {
+		minirect.x += thickness_;
+		minirect.y += thickness_;
+		minirect.w -= 2*thickness_;
+		minirect.h -= 2*thickness_;
+	}
+	menu_ref.draw_row(row_index, minirect, type);
+}
+
+
+
 void menu::column_widths_item(const std::vector<std::string>& row, std::vector<int>& widths) const
 {
 	for(size_t col = 0; col != row.size(); ++col) {
-		const SDL_Rect res = item_size(row[col]);
+		const SDL_Rect res = style_->item_size(row[col]);
+		size_t text_trailing_space = (item_ends_with_image(row[col])) ? 0 : style_->get_cell_padding();
 
 		if(col == widths.size()) {
-			widths.push_back(res.w + menu_cell_padding);
-		} else if(res.w > widths[col] - menu_cell_padding) {
-			widths[col] = res.w + menu_cell_padding;
+			widths.push_back(res.w + text_trailing_space);
+		} else if(res.w > widths[col] - text_trailing_space) {
+			widths[col] = res.w + text_trailing_space;
 		}
 	}
 }
 
+bool menu::item_ends_with_image(const std::string& item) const
+{
+	std::string::size_type pos = item.find_last_of(IMG_TEXT_SEPARATOR);
+	pos = (pos == std::string::npos) ? 0 : pos+1;
+	return(item.size() > pos && item.at(pos) == IMAGE_PREFIX);
+}
+
 const std::vector<int>& menu::column_widths() const
 {
 	if(column_widths_.empty()) {
@@ -712,49 +766,30 @@ void menu::clear_item(int item)
 	bg_restore(rect);
 }
 
-void menu::draw_row(const std::vector<std::string>& row, const SDL_Rect& rect, ROW_TYPE type)
+void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 {
-	if(rect.w == 0 || rect.h == 0) {
-		return;
-	}
-
-	bg_restore(rect);
-
-	int rgb = 0;
-	double alpha = 0.0;
-
-	switch(type) {
-	case NORMAL_ROW:
-		rgb = 0x000000;
-		alpha = 0.2;
-		break;
-	case SELECTED_ROW:
-		rgb = 0x990000;
-		alpha = 0.6;
-		break;
-	case HEADING_ROW:
-		rgb = 0x333333;
-		alpha = 0.3;
-		break;
-	}
-
-	draw_solid_tinted_rectangle(rect.x, rect.y, rect.w, rect.h,
-				    (rgb&0xff0000) >> 16,(rgb&0xff00) >> 8,rgb&0xff,alpha,
-				    video().getSurface());
-
+	//called from style, draws one row's contents in a generic and adaptable way
+	const std::vector<std::string>& row = (type == HEADING_ROW) ? heading_ : items_[row_index].fields;
 	SDL_Rect const &area = screen_area();
 	SDL_Rect const &loc = inner_location();
-
 	const std::vector<int>& widths = column_widths();
+	bool lang_rtl = current_language_rtl();
+	int dir = (lang_rtl) ? -1 : 1;
+	SDL_Rect column = loc;
 
 	int xpos = rect.x;
+	if(lang_rtl)
+		xpos += rect.w;
 	for(size_t i = 0; i != row.size(); ++i) {
 
+		if(lang_rtl)
+			xpos -= widths[i];
 		if(type == HEADING_ROW && highlight_heading_ == int(i)) {
 			draw_solid_tinted_rectangle(xpos,rect.y,widths[i],rect.h,255,255,255,0.3,video().getSurface());
 		}
 
 		const int last_x = xpos;
+		column.w = widths[i];
 		std::string str = row[i];
 		std::vector<std::string> img_text_items = utils::split(str, IMG_TEXT_SEPARATOR);
 		for (std::vector<std::string>::const_iterator it = img_text_items.begin();
@@ -768,15 +803,20 @@ void menu::draw_row(const std::vector<std::string>& row, const SDL_Rect& rect, R
 				if(img != NULL && (xpos - rect.x) + img->w < max_width
 				   && rect.y + img->h < area.h) {
 					const size_t y = rect.y + (rect.h - img->h)/2;
-					video().blit_surface(xpos,y,img);
-					xpos += img->w + 5;
+					const size_t w = img->w + 5;
+					const size_t x = xpos + ((lang_rtl) ? widths[i] - w : 0);
+					video().blit_surface(x,y,img);
+					if(!lang_rtl)
+						xpos += w;
+					column.w -= w;
 				}
 			} else {
+				column.x = xpos;
 				const std::string to_show = max_width_ > -1 ?
-					font::make_text_ellipsis(str, menu_font_size, loc.w - (xpos - rect.x)) : str;
-				const SDL_Rect& text_size = font::text_area(str,menu_font_size);
+					font::make_text_ellipsis(str, style_->get_font_size(), loc.w - (xpos - rect.x)) : str;
+				const SDL_Rect& text_size = font::text_area(str,style_->get_font_size());
 				const size_t y = rect.y + (rect.h - text_size.h)/2;
-				font::draw_text(&video(),area,menu_font_size,font::NORMAL_COLOUR,to_show,xpos,y);
+				font::draw_text(&video(),column,style_->get_font_size(),font::NORMAL_COLOUR,to_show,xpos,y);
 
 				if(type == HEADING_ROW && sortby_ == int(i)) {
 					const surface sort_img = image::get_image(sortreversed_ ? "misc/sort-arrow.png" :
@@ -788,10 +828,13 @@ void menu::draw_row(const std::vector<std::string>& row, const SDL_Rect& rect, R
 					}
 				}
 
-				xpos += text_size.w + 5;
+				xpos += dir * (text_size.w + 5);
 			}
 		}
-		xpos = last_x + widths[i];
+		if(lang_rtl)
+			xpos = last_x;
+		else
+			xpos = last_x + widths[i];
 	}
 }
 
@@ -799,10 +842,10 @@ void menu::draw_contents()
 {
 	SDL_Rect heading_rect = inner_location();
 	heading_rect.h = heading_height();
-	draw_row(heading_,heading_rect,HEADING_ROW);
+	style_->draw_row(*this,0,heading_rect,HEADING_ROW);
 
 	for(size_t i = 0; i != item_pos_.size(); ++i) {
-		draw_row(items_[item_pos_[i]].fields,get_item_rect(i),item_pos_[i] == selected_ ? SELECTED_ROW : NORMAL_ROW);
+		style_->draw_row(*this,item_pos_[i],get_item_rect(i),item_pos_[i] == selected_ ? SELECTED_ROW : NORMAL_ROW);
 	}
 }
 
@@ -819,13 +862,13 @@ void menu::draw()
 				SDL_Rect heading_rect = inner_location();
 				heading_rect.h = heading_height();
 				bg_restore(heading_rect);
-				draw_row(heading_,heading_rect,HEADING_ROW);
+				style_->draw_row(*this,0,heading_rect,HEADING_ROW);
 				update_rect(heading_rect);
 			} else if(*i >= 0 && *i < int(item_pos_.size())) {
-				const int pos = item_pos_[*i];
+				const unsigned int pos = item_pos_[*i];
 				const SDL_Rect& rect = get_item_rect(*i);
 				bg_restore(rect);
-				draw_row(items_[pos].fields,rect,pos == selected_ ? SELECTED_ROW : NORMAL_ROW);
+				style_->draw_row(*this,pos,rect,pos == selected_ ? SELECTED_ROW : NORMAL_ROW);
 				update_rect(rect);
 			}
 		}
@@ -862,7 +905,7 @@ int menu::hit(int x, int y) const
 	return -1;
 }
 
-int menu::hit_column(int x, int y) const
+int menu::hit_column(int x) const
 {
 	std::vector<int> const &widths = column_widths();
 	x -= location().x;
@@ -883,7 +926,7 @@ std::pair<int,int> menu::hit_cell(int x, int y) const
 		return std::pair<int,int>(-1, -1);
 	}
 
-	const int col = hit_column(x,y);
+	const int col = hit_column(x);
 	if(col < 0) {
 		return std::pair<int,int>(-1, -1);
 	}
@@ -895,8 +938,8 @@ int menu::hit_heading(int x, int y) const
 {
 	const size_t height = heading_height();
 	const SDL_Rect& loc = inner_location();
-	if(y >= loc.y && y < loc.y + height) {
-		return hit_column(x,y);
+	if(y >= loc.y && (size_t)y < loc.y + height) {
+		return hit_column(x);
 	} else {
 		return -1;
 	}
@@ -910,7 +953,7 @@ SDL_Rect menu::get_item_rect(int item) const
 SDL_Rect menu::get_item_rect_internal(size_t item) const
 {
 	const SDL_Rect empty_rect = {0,0,0,0};
-	int first_item_on_screen = get_position();
+	unsigned int first_item_on_screen = get_position();
 	if (item < first_item_on_screen ||
 	    size_t(item) >= first_item_on_screen + max_items_onscreen()) {
 		return empty_rect;
@@ -956,7 +999,7 @@ size_t menu::get_item_height_internal(const std::vector<std::string>& item) cons
 {
 	size_t res = 0;
 	for(std::vector<std::string>::const_iterator i = item.begin(); i != item.end(); ++i) {
-		SDL_Rect rect = item_size(*i);
+		SDL_Rect rect = style_->item_size(*i);
 		res = maximum<int>(rect.h,res);
 	}
 
