@@ -1,4 +1,4 @@
-/* $Id: scrollpane.cpp 7120 2005-06-04 19:16:06Z ott $ */
+/* $Id: scrollpane.cpp 11744 2006-05-30 16:55:16Z sapient $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -11,11 +11,11 @@
    See the COPYING file for more details.
 */
 
-#include "../global.hpp"
+#include "global.hpp"
 
 #include <algorithm>
 
-#include "scrollpane.hpp"
+#include "widgets/scrollpane.hpp"
 
 namespace {
 class widget_finder {
@@ -93,7 +93,7 @@ void scrollpane::remove_widget(widget* w)
 	update_content_size();
 }
 
-void scrollpane::set_inner_location(const SDL_Rect& rect)
+void scrollpane::set_inner_location(const SDL_Rect& /*rect*/)
 {
 	for(widget_map::iterator itor = content_.begin(); itor != content_.end(); ++itor) {
 		itor->second.w->set_clip_rect(client_area());
@@ -105,9 +105,9 @@ void scrollpane::draw()
 	//draws the scrollpane background
 }
 
-void scrollpane::scroll(int pos)
+void scrollpane::scroll(unsigned int pos)
 {
-	if (pos == content_pos_.y)
+	if ((int)pos == content_pos_.y)
 		return;
 
 	content_pos_.y = pos;
@@ -121,7 +121,7 @@ void scrollpane::update_widget_positions()
 	std::vector<bool> hidden(content_.size());
 	int i = 0;
 	for(itor = content_.begin(); itor != content_.end(); ++itor) {
-		hidden[i++] = itor->second.w->hidden();
+		hidden[i++] = (itor->second.w->state_ == HIDDEN);
 		itor->second.w->hide();
 	}
 
@@ -158,8 +158,8 @@ SDL_Rect scrollpane::client_area() const
 
 void scrollpane::update_content_size()
 {
-	int maxx = 0;
-	int maxy = 0;
+	unsigned int maxx = 0;
+	unsigned int maxy = 0;
 
 	for(widget_map::iterator itor = content_.begin(); itor != content_.end(); ++itor) {
 		if(itor->second.x + itor->second.w->width() > maxx) {
