@@ -1,6 +1,6 @@
-/* $Id: slider.cpp,v 1.41 2005/06/04 19:16:06 ott Exp $ */
+/* $Id: slider.cpp 12604 2006-07-17 09:54:21Z isaac $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -11,14 +11,16 @@
    See the COPYING file for more details.
 */
 
-#include "../global.hpp"
+#include "global.hpp"
 
-#include "slider.hpp"
-#include "../image.hpp"
-#include "../video.hpp"
+#include "widgets/slider.hpp"
+#include "font.hpp"
+#include "image.hpp"
+#include "video.hpp"
 
 #include <algorithm>
 #include <iostream>
+#include <sstream>
 
 namespace {
 	const std::string slider_image = "buttons/slider.png";
@@ -35,6 +37,15 @@ slider::slider(CVideo &video)
 {
 }
 
+void slider::enable(bool new_val)
+{
+	if(new_val != enabled())
+	{
+		state_ = NORMAL;
+		widget::enable(new_val);
+	}
+}
+
 void slider::set_location(SDL_Rect const &rect)
 {
 	SDL_Rect r = rect;
@@ -111,9 +122,14 @@ SDL_Rect slider::slider_area() const
 
 void slider::draw_contents()
 {
-	const surface image(state_ != NORMAL ? highlightedImage_ : image_);
+	surface image(state_ != NORMAL ? highlightedImage_ : image_);
 	if (image == NULL)
 		return;
+	SDL_Color line_colour = font::NORMAL_COLOUR;
+	if (!enabled()) {
+		image = greyscale_image(image);
+		line_colour = font::DISABLED_COLOUR;
+	}
 
 	SDL_Rect const &loc = location();
 	if (image->w >= loc.w)
@@ -122,7 +138,8 @@ void slider::draw_contents()
 	surface const screen = video().getSurface();
 
 	SDL_Rect line_rect = { loc.x + image->w / 2, loc.y + loc.h / 2, loc.w - image->w, 1 };
-	SDL_FillRect(screen, &line_rect, SDL_MapRGB(screen->format, 221, 221, 221));
+	SDL_FillRect(screen, &line_rect, SDL_MapRGB(screen->format,
+		line_colour.r, line_colour.g, line_colour.b));
 
 	SDL_Rect const &slider = slider_area();
 	video().blit_surface(slider.x, slider.y, image);
@@ -157,16 +174,17 @@ void slider::mouse_down(const SDL_MouseButtonEvent& event)
 		return;
 
 	state_ = CLICKED;
+	set_focus(true);
 	set_slider_position(event.x);
 }
 
 void slider::handle_event(const SDL_Event& event)
 {
-	if (hidden())
+	if (hidden() || !enabled())
 		return;
-
+	
 	STATE start_state = state_;
-
+	
 	switch(event.type) {
 	case SDL_MOUSEBUTTONUP:
 		state_ = NORMAL;
@@ -177,12 +195,21 @@ void slider::handle_event(const SDL_Event& event)
 	case SDL_MOUSEMOTION:
 		mouse_motion(event.motion);
 		break;
+	case SDL_KEYDOWN:
+		if(focus()) {
+			const SDL_keysym& key = reinterpret_cast<const SDL_KeyboardEvent&>(event).keysym;
+			const int c = key.sym;
+			if(c == SDLK_LEFT) {
+				set_value(value_ - increment_);
+			} else if(c == SDLK_RIGHT) {
+				set_value(value_ + increment_);
+			}
+		}
+		break;
 	default:
 		return;
 	}
-
 	if (start_state != state_)
 		set_dirty(true);
 }
-
-}
+} //end namespace gui
