@@ -1,4 +1,4 @@
-/* $Id: widget.cpp 7943 2005-08-21 12:58:46Z gruikya $ */
+/* $Id: widget.cpp 12603 2006-07-17 09:53:55Z isaac $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -11,10 +11,10 @@
    See the COPYING file for more details.
 */
 
-#include "../global.hpp"
+#include "global.hpp"
 
-#include "widget.hpp"
-#include "../video.hpp"
+#include "widgets/widget.hpp"
+#include "video.hpp"
 
 namespace {
 	const SDL_Rect EmptyRect = {-1234,-1234,0,0};
@@ -24,15 +24,16 @@ namespace gui {
 
 widget::widget(const widget &o)
 	: events::handler(), video_(o.video_), restorer_(o.restorer_), rect_(o.rect_),
-	  focus_(o.focus_), needs_restore_(o.needs_restore_),
-	  state_(o.state_), clip_(o.clip_), clip_rect_(o.clip_rect_), volatile_(o.volatile_),
-	  help_text_(o.help_text_), help_string_(o.help_string_)
+	  focus_(o.focus_), needs_restore_(o.needs_restore_), state_(o.state_),
+	  enabled_(o.enabled_), clip_(o.clip_), clip_rect_(o.clip_rect_), volatile_(o.volatile_),
+	  help_text_(o.help_text_), help_string_(o.help_string_), align_(o.align_)
 {
 }
 
-widget::widget(CVideo& video)
-	: video_(&video), rect_(EmptyRect), focus_(true), needs_restore_(false),
-	  state_(UNINIT), clip_(false), volatile_(false), help_string_(0)
+widget::widget(CVideo& video, bool auto_join)
+	: handler(auto_join), video_(&video), rect_(EmptyRect), focus_(true), needs_restore_(false),
+	  state_(UNINIT), enabled_(true), clip_(false), volatile_(false), help_string_(0),
+	  align_(RIGHT_ALIGN)
 {
 }
 
@@ -96,12 +97,22 @@ void widget::set_height(unsigned h)
 	set_location(rect);
 }
 
+void widget::set_align(ALIGN a)
+{
+	align_ = a;
+}
+
 void widget::set_measurements(unsigned w, unsigned h)
 {
 	SDL_Rect rect = { rect_.x, rect_.y, w, h };
 	set_location(rect);
 }
 
+ALIGN widget::align() const
+{
+	return align_;
+}
+
 unsigned widget::width() const
 {
 	return rect_.w;
@@ -152,7 +163,21 @@ void widget::set_clip_rect(const SDL_Rect& rect)
 
 bool widget::hidden() const
 {
-	return state_ == HIDDEN || state_ == UNINIT;
+	return (state_ == HIDDEN || state_ == UNINIT
+		|| (clip_ && !rects_overlap(clip_rect_, rect_)));
+}
+
+void widget::enable(bool new_val)
+{
+	if (enabled_ != new_val) {
+		enabled_ = new_val;
+		set_dirty();
+	}
+}
+
+bool widget::enabled() const
+{
+	return enabled_;
 }
 
 void widget::set_dirty(bool dirty)
