@@ -1,4 +1,4 @@
-/* $Id: widget.hpp 7534 2005-07-20 08:56:52Z ott $ */
+/* $Id: widget.hpp 12603 2006-07-17 09:53:55Z isaac $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -26,6 +26,8 @@ class CVideo;
 
 namespace gui {
 
+enum ALIGN { LEFT_ALIGN, CENTER_ALIGN, RIGHT_ALIGN };
+
 class widget : public events::handler
 {
 public:
@@ -35,15 +37,19 @@ public:
 	void set_width(unsigned w);
 	void set_height(unsigned h);
 	void set_measurements(unsigned w, unsigned h);
+	void set_align(ALIGN a);
 
 	unsigned width() const;
 	unsigned height() const;
+	ALIGN align() const;
 
 	virtual bool focus() const;
 	void set_focus(bool focus);
 
 	virtual void hide(bool value = true);
 	bool hidden() const;
+	virtual void enable(bool new_val = true);
+	bool enabled() const;
 
 	void set_clip_rect(const SDL_Rect& rect);
 
@@ -66,7 +72,7 @@ public:
 
 protected:
 	widget(widget const &o);
-	widget(CVideo& video);
+	widget(CVideo& video, bool auto_join=true);
 	virtual ~widget();
 
 	// During each relocation, this function should be called to register
@@ -79,7 +85,7 @@ protected:
 
 	CVideo& video() const { return *video_; };
 
-	virtual void handle_event(SDL_Event const &event) {}
+	virtual void handle_event(SDL_Event const &/*event*/) {}
 
 	virtual void draw();
 	virtual void draw_contents() {};
@@ -98,6 +104,7 @@ private:
 	mutable bool needs_restore_; // Have we drawn ourselves, so that if moved, we need to restore the background?
 
 	enum { UNINIT, HIDDEN, DIRTY, DRAWN } state_;
+	bool enabled_;
 	bool clip_;
 	SDL_Rect clip_rect_;
 
@@ -105,6 +112,9 @@ private:
 
 	std::string help_text_;
 	int help_string_;
+	ALIGN align_; //limited support, use position
+
+	friend class scrollpane;
 };
 
 }
